// MODIFICATION HISTORY
//  1.1                     )
//  ...                     ) Writing it and other stuff.
//  1.8                     )
//  1.9  28:feb:02  pws     Add locking for serial port.
//  1.11 28:feb:02  pws     Rename macro to POSIX_NO_FILE_LOCKING.
//  1.12 28:feb:02  pws     And set fd to -1 on failure.

#include <stdlib.h>
#include <assert.h>
#define ASSERT assert
#include <time.h>	/* This pulls in the Posix.1b realtime stuff */
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>

#ifdef linux
#include <sys/time.h>
#include <unistd.h>
#endif

#include "time/hi_res_clock.h"

#include "PosixBCSP.h"

typedef unsigned long ul;

#define NUMPACKETS 10

#define NODPRINTF 1
#define dprintf (NODPRINTF) ? 0 : printf

#include <string.h>
#define CHECK(c)	\
	if (!(c))	\
	{		\
		fprintf (stderr, "Assertion %s failed (%s, %lu); "	\
			"errno %li (%s)\n",				\
			#c, __FILE__, (ul) __LINE__,			\
			(long) errno, strerror (errno));		\
		abort ();						\
	}		\
	else


void PosixEnterCS (void *envState)
{
	dprintf ("Entering CS ... ");
	CHECK (pthread_mutex_lock (&((PosixBCSP *) envState)->mMutexEnv) == 0);
	dprintf ("entered ... ");
}


void PosixLeaveCS (void *envState)
{
	dprintf ("leaving CS ... ");
	CHECK (pthread_mutex_unlock (&((PosixBCSP *) envState)->mMutexEnv) == 0);
	dprintf ("left\n");
}


void PosixSignal (void *envState)
{
	dprintf ("Signalling\n");
	CHECK (pthread_mutex_lock (&((PosixBCSP *) envState)->mMutexThread) == 0);
	((PosixBCSP *) envState)->signalled = true;
	CHECK (pthread_cond_signal (&((PosixBCSP *) envState)->mCondDone) == 0);
	CHECK (pthread_mutex_unlock (&((PosixBCSP *) envState)->mMutexThread) == 0);
}


void PosixOnLinkFailure (void *envState)
{
	if (((PosixBCSP *) envState)->userOnLinkFail != NULL)
	{
		((PosixBCSP *) envState)->userOnLinkFail (((PosixBCSP *) envState)->userOnLinkFailArg);
	}
	else
	{
		fprintf (stderr, "PosixBCSP: link failed\n");
	}
}


void PosixReleasePacket (void *envState, Packet *pkt)
{
	dprintf ("Packet %p released\n", pkt);
	free (pkt);
}


PosixBCSP::PosixBCSP ()
{
	InitHiResClock(NULL);

	mStack = (BCSPStack *) malloc (sizeof (BCSPStack));
	ASSERT (mStack != NULL);
	initialiseStack (mStack);

	for (int c = 0 ; c < NUMPACKETS; ++c)
	{
		Packet *pkt = (Packet *) malloc (sizeof (Packet));
		ASSERT (pkt != NULL);
		BCSPaddPacket (mStack, pkt);
	}

	mBCSPenv = *getEnvironment (mStack);
	mBCSPenv.enterCS = PosixEnterCS;
	mBCSPenv.leaveCS = PosixLeaveCS;
	mBCSPenv.signal = PosixSignal;
	mBCSPenv.onLinkFailure = PosixOnLinkFailure;
	mBCSPenv.releasePacket = PosixReleasePacket;
	mBCSPenv.clockMask = 0xffffffff;
	mBCSPenv.envState = this;
	setEnvironment (mStack, &mBCSPenv);

	fd = -1;
	userOnLinkFail = NULL;

	rxdone = txdone = txbusy = signalled = die = false;
	rxbusy = true; /* so rxThread () immediately starts receiving */

	CHECK (pthread_mutex_init (&mMutexEnv, NULL) == 0);
	CHECK (pthread_mutex_init (&mMutexThread, NULL) == 0);
	CHECK (pthread_cond_init (&mCondDoTx, NULL) == 0);
	CHECK (pthread_cond_init (&mCondDoRx, NULL) == 0);
	CHECK (pthread_cond_init (&mCondDone, NULL) == 0);
}


void PosixBCSP::thread ()
{
	uint32 timeNow, timeWakeup;
	bool didIO;

#ifdef MGRRXTXFPRINTF
	static FILE *rxtxfp = fopen (tempnam ("/tmp", "rxtx"), "w");
#endif

	while (!die)
	{
		timeNow = HiResClockGetMilliSec();

		timeWakeup = scheduler (mStack, timeNow); /* Note it is preferable to overshoot, rather than undershoot */
		didIO = false;

		CHECK (pthread_mutex_lock (&mMutexThread) == 0);

#if !NODPRINTF || defined (MGRSHOWSERIALTIME)
		printf ("now: %lu, wakeup: %lu\n", (ul) timeNow, (ul) timeWakeup);
#endif
#ifdef MGRRXTXFPRINTF
		fprintf (rxtxfp, "now: %lu, wakeup: %lu\n", (ul) timeNow, (ul) timeWakeup);
#endif
/*		dprintf ("nrxfree: %u, ntx: %u\n", 
			numFreeSlotsInReceiveBuffer (mStack),
			numBytesInTransmitBuffer (mStack)); */

		if (!txbusy && (txlen = numBytesInTransmitBuffer (mStack)) != 0)
		{
			ASSERT (txlen <= sizeof (txbuf));
			readFromTransmitBuffer (mStack, txbuf, txlen);
			didIO = true;
#if !NODPRINTF || defined (MGRSHOWSERIALTX)
			printf ("tx:");
			for (size_t j = 0; j < txlen; ++j)
			{
				uint8 o = txbuf[j];
				if (o == 0xc0) printf ("/"); else
				printf ("%02x", o);
			}
			printf (" [%lu]\n", (ul) txlen);
#endif
#ifdef MGRRXTXFPRINTF
			fprintf (rxtxfp, "tx:");
			for (size_t j = 0; j < txlen; ++j)
			{
				uint8 o = txbuf[j];
				if (o == 0xc0) fprintf (rxtxfp, "/"); else
				fprintf (rxtxfp, "%02x", o);
			}
			fprintf (rxtxfp, " [%lu]\n", (ul) txlen);
#endif
			txbusy = true;
			CHECK (pthread_cond_signal (&mCondDoTx) == 0);
		}

		if (!rxbusy)
		{
			size_t rxfree = numFreeSlotsInReceiveBuffer (mStack);
			ASSERT (rxlen != 0); /* else there's a thread bug */
			if (rxlen <= rxfree)
			{
				ASSERT (rxlen <= 256); /* u8 ... */
				writeToReceiveBuffer (mStack, rxbuf, rxlen);
				rxbusy = true;
				CHECK (pthread_cond_signal (&mCondDoRx) == 0);
			}
			else
			{
				ASSERT (rxfree <= 256); /* u8 ... */
				writeToReceiveBuffer (mStack, rxbuf, rxfree);
				memmove (rxbuf, rxbuf + rxfree, rxlen - rxfree);
				rxlen -= rxfree; /* HERE: use index instead? */
			}
			didIO = true;
		}

		if (!didIO && isStackIdle (mStack))
		{
			timeNow = HiResClockGetMilliSec ();

			if (timeNow < timeWakeup)
			{
				struct timespec ts;
				HiResClock now;
				HiResClockGetNanoSec(&now);

				dprintf ("Now is %lu.%09lu s;", (ul) now.tv_sec, (ul) now.tv_nsec);
				ts.tv_sec = now.tv_sec + (timeWakeup - timeNow) / 1000;
				ts.tv_nsec = now.tv_nsec + (timeWakeup - timeNow) % 1000 * 1000000;
				if (ts.tv_nsec > 999999999)
				{
					ts.tv_nsec -= 1000000000;
					ts.tv_sec += 1;
				}
				dprintf (" wait until %lu.%09lu s", (ul) ts.tv_sec, (ul) ts.tv_nsec);
				dprintf (" (%lu ms)\n", (ul) (timeWakeup - timeNow));

				while (!(txdone || rxdone || signalled || die))
				{
					int rc_pthread_cond_timedwait;
					dprintf ("thread: waiting for done or timeout\n");
					if ((rc_pthread_cond_timedwait = pthread_cond_timedwait (&mCondDone, &mMutexThread, &ts) == ETIMEDOUT))
						break;
					CHECK (rc_pthread_cond_timedwait == 0);
				}
				if (txdone)
				{
#ifdef MGRRXTXFPRINTF
					fprintf (rxtxfp, "txed\n");
#endif
					txbusy = false;
					txdone = false;
				}
				if (rxdone)
				{
#if !NODPRINTF || defined (MGRSHOWSERIALRX)
					printf ("rx:");
					for (size_t j = 0; j < rxlen; ++j)
					{
					    uint8 o = rxbuf[j];
					    if (o == 0xc0)
						printf ("/");
					    else
						printf ("%02x", o);
					}
					printf (" [%lu]\n", (ul) rxlen);
#endif
#ifdef MGRRXTXFPRINTF
					fprintf (rxtxfp, "rx:");
					for (size_t j = 0; j < rxlen; ++j)
					{
						uint8 o = rxbuf[j];
						if (o == 0xc0) fprintf (rxtxfp, "/"); else
						fprintf (rxtxfp, "%02x", o);
					}
					fprintf (rxtxfp, " [%lu]\n", (ul) rxlen);
#endif
					rxbusy = false;
					rxdone = false;
				}
				if (signalled)
				{
					dprintf ("Ooooh, signalled!\n");
#ifdef MGRRXTXFPRINTF
					fprintf (rxtxfp, "sig\n");
#endif
					signalled = false;
				}
			}
		}

		CHECK (pthread_mutex_unlock (&mMutexThread) == 0);
	}

	dprintf ("thread: adieu, cruel world!\n");
}


static void unlockMutexThread (void *p)
{
	CHECK (pthread_mutex_unlock ((pthread_mutex_t *) p) == 0);
}


void PosixBCSP::txThread ()
{
	ssize_t n;

	CHECK (pthread_mutex_lock (&mMutexThread) == 0);

	while (1) /* Dies by cancellation */
	{
		pthread_cleanup_push (unlockMutexThread, &mMutexThread);

		while (!txbusy || txdone)
		{
			dprintf ("txthread: waiting for do\n");
			CHECK (pthread_cond_wait (&mCondDoTx, &mMutexThread)
			       == 0);
		}
		ASSERT (txbusy && !txdone);

		pthread_cleanup_pop (0);

		CHECK (pthread_mutex_unlock (&mMutexThread) == 0);
		dprintf ("txthread: about to tx\n");

#ifdef linux
		pthread_testcancel ();
#endif
		if ((n = write (fd, txbuf, txlen)) != (ssize_t) txlen)
		{
#ifdef linux
			pthread_testcancel ();
#endif
			fprintf (stderr, "Failed to write %li bytes (rc %li, errno %i):\n", (long) txlen, (long) n, errno);
			perror (NULL);
			tcflush (fd, TCIOFLUSH); /* otherwise can hang! */
			return;
		}

		dprintf ("txthread: txed\n");
		CHECK (pthread_mutex_lock (&mMutexThread) == 0);

		txdone = true;
		CHECK (pthread_cond_signal (&mCondDone) == 0);
	}
}


void PosixBCSP::rxThread ()
{
	ssize_t n;

	CHECK (pthread_mutex_lock (&mMutexThread) == 0);

	while (1) /* Dies by cancellation */
	{
		pthread_cleanup_push (unlockMutexThread, &mMutexThread);

		while (!rxbusy || rxdone)
		{
			dprintf ("rxthread: waiting for do\n");
			CHECK (pthread_cond_wait (&mCondDoRx, &mMutexThread)
			       == 0);
		}
		ASSERT (rxbusy && !rxdone);

		pthread_cleanup_pop (0);

		CHECK (pthread_mutex_unlock (&mMutexThread) == 0);
		dprintf ("rxthread: waiting for rx\n");

#ifdef linux

/* [From Linux 'man pthread_cancel'.]

BUGS

       POSIX specifies that a number of system calls  (basically,
       all   system  calls  that  may  block,  such  as  read(2),
       write(2), wait(2), etc.) and library  functions  that  may
       call  these  system calls (e.g.  fprintf(3)) are cancella-
       tion points.  LinuxThreads is not  yet  integrated  enough
       with the C library to implement this, and thus none of the
       C library functions is a cancellation point.

       For system calls at least, there is a workaround.  Cancel-
       lation  requests  are  transmitted to the target thread by
       sending it a signal. That signal will interrupt all block-
       ing  system calls, causing them to return immediately with
       the EINTR error. So, checking for  cancellation  during  a
       read  system  call,  for instance, can be achieved as fol-
       lows:


              pthread_testcancel();
              retcode = read(fd, buffer, length);
              pthread_testcancel();
*/

		pthread_testcancel ();
#endif
		while ((n = read (fd, rxbuf, sizeof (rxbuf))) < 1)
		{
#ifdef linux
			pthread_testcancel ();
#endif
			fprintf (stderr,
				 "Failed to read %li bytes "
				 "(rc %li, errno %i):\n",
				 (long) sizeof (rxbuf), (long) n, errno);
			perror (NULL);

			/* Exit the thread, unless the error code was
			 * 'Interrupted', in which case we try the
			 * read again.  I have only seen thia happen
			 * ocasionally, and we are not sure why it
			 * would happen at all? */
			if (errno != EINTR)
				return;
		}
		rxlen = n;

		dprintf ("rxthread: rxed\n");
		CHECK (pthread_mutex_lock (&mMutexThread) == 0);

		rxdone = true;
		CHECK (pthread_cond_signal (&mCondDone) == 0);
	}
}


PosixBCSP::~PosixBCSP ()
{
	die = true;

	if (fd != -1)
	{
		CHECK (pthread_join (threadID, NULL) == 0);
		CHECK (pthread_cancel (txThreadID) == 0);
		CHECK (pthread_join (txThreadID, NULL) == 0);
		CHECK (pthread_cancel (rxThreadID) == 0);
		CHECK (pthread_join (rxThreadID, NULL) == 0);

		CHECK (pthread_mutex_destroy (&mMutexEnv) == 0);
		CHECK (pthread_mutex_destroy (&mMutexThread) == 0);
		CHECK (pthread_cond_destroy (&mCondDoTx) == 0);
		CHECK (pthread_cond_destroy (&mCondDoRx) == 0);
		CHECK (pthread_cond_destroy (&mCondDone) == 0);
	}

	BCSPshutdownStack (mStack);
	free (mStack);
	close ();
}


bool PosixBCSP::open (const char *portName, uint32 bps)
{
/*
 * May wish to extend this to support turning on flow control, having 2 stop
 * bits and/or turning off parity/making it odd.  Hardwire for now.  HERE
 */
	bool	parity = true,
		parityeven = true,	/* Only relevant if parity == true */
		onestop = true,		/* Else two stop bits */
		rtscts = false;
#ifndef POSIX_NO_FILE_LOCKING
	struct flock fl;
#endif

	struct termios tios;
#if !NODPRINTF
	speed_t portSpeed;
	int i;
#endif

	fd = ::open (portName, O_RDWR | O_NOCTTY); /* HERE: do we want to use O_NONBLOCK too? */
	if (fd < 0)
	{
		fprintf (stderr, "Failed to open %s (errno %i):\n", portName, errno);
		perror (NULL);
		return false;
	}

#ifndef POSIX_NO_FILE_LOCKING
	fl.l_type = F_WRLCK;
	fl.l_whence = SEEK_SET;
	fl.l_start = 0;
	fl.l_len = 0;
	if (fcntl(fd, F_SETLK, &fl) == -1)
	{
		fprintf(stderr, "Failed to lock %s for writing (errno %i):\n",
			portName, errno);
		perror(NULL);
		::close(fd);
		fd = -1;
		return false;
	}
#endif

	if (tcgetattr (fd, &tios) != 0)
	{
		fprintf (stderr, "Failed to get terminal attributes (errno %i):\n", errno);
		perror (NULL);
		::close (fd);
		fd = -1;
		return false;
	}

#if !NODPRINTF
	dprintf ("Starting terminal attributes:\n");
	dprintf ("tios.c_iflag = 0x%lx\n", (ul) tios.c_iflag);
	dprintf ("tios.c_oflag = 0x%lx\n", (ul) tios.c_oflag);
	dprintf ("tios.c_cflag = 0x%lx\n", (ul) tios.c_cflag);
	dprintf ("tios.c_lflag = 0x%lx\n", (ul) tios.c_lflag);
	dprintf ("tios.c_cc[%i] = {", NCCS);
	for (i = 0; i < NCCS; ++i)
	{
		dprintf ("%lu%s", (ul) tios.c_cc[i], i == NCCS - 1 ? "" : ", ");
	}
	dprintf ("}\n");
	dprintf ("tios.c_cc[VMIN] = %lu\n", (ul) tios.c_cc[VMIN]);
	dprintf ("tios.c_cc[VTIME] = %lu\n", (ul) tios.c_cc[VTIME]);
#endif

	/*
	 * For all the termios mode entries,
	 * turn off all the Posix-defined flags,
	 * whine gently if any non-Posix flags are set,
	 * then turn on any flags of interest.
	 * In this way no non-Posix flags which an
	 * implementation might need set are affected.
	 */

	tios.c_iflag &= ~ (ul) (BRKINT | ICRNL | IGNBRK | IGNCR | IGNPAR | INLCR | INPCK | ISTRIP | IXOFF | IXON | PARMRK);
	if (tios.c_iflag != 0)
	{
#if defined (__sparc__) && defined (MGR)
		if (tios.c_iflag != 020000) /* IMAXBEL */
#endif
			fprintf (stderr, "Warning: tios.c_iflag contains non-zero non-Posix bits: 0%lo\n", (ul) tios.c_iflag);
	}
	tios.c_iflag |= INPCK | IGNPAR;
	/* Result: breaks read as NUL, and parity/framing errors read as nothing, which should ensure the packet sizes end up wrong, which in turn ensures packets containing them are discarded */

	tios.c_oflag &= ~ (ul) OPOST;
	if (tios.c_oflag != 0)
	{
#if defined (__sparc__) && defined (MGR)
		if (tios.c_oflag != 014004) /* ONLCR | TAB3 */
#endif
#ifdef ONLCR
			if (tios.c_oflag != ONLCR)
#endif
				fprintf (stderr, "Warning: tios.c_oflag contains non-zero non-Posix bits: 0%lo\n", (ul) tios.c_oflag);
	}
	/* There are no bits in c_oflag we want to set */
	/* Result: just send data unadulterated! */

	tios.c_cflag &= ~ (ul) (CLOCAL | CREAD | CSIZE | CSTOPB | HUPCL | PARENB | PARODD);
	if (tios.c_cflag != 0)
	{
#if defined (__sparc__) && defined (MGR)
		if (tios.c_cflag != 015) /* B9600 */
#endif
		fprintf (stderr, "Warning: tios.c_cflag contains non-zero non-Posix bits: 0%lo\n", (ul) tios.c_cflag);
	}
	tios.c_cflag |= CREAD | CS8 | CLOCAL;
	if (parity)
	{
		tios.c_cflag |= PARENB;
		if (!parityeven)
		{
			tios.c_cflag |= PARODD;
		}
	}
	if (!onestop)
	{
		tios.c_cflag |= CSTOPB;
	}
	if (rtscts)
	{
		/*
		 * Posix doesn't support hardware flow control,
		 * so just try to turn on any available flag!
		 * None of this has really been tested.
		 * It doesn't appear to be possible to set
		 * hardware flow control through termios
		 * in HPUX and AIX.
		 */
#ifdef ORTSFL /* SCO (bidirectional) */
		tios.c_cflag |= ORTSFL; /* non-Posix! */
		tios.c_cflag &= ~CTSFLOW; /* non-Posix! */
		tios.c_cflag &= ~RTSFLOW; /* non-Posix! */
#else /* !ORTSFL */
#ifdef CRTSCTS /* Solaris (unidirectional), Linux (bidirectional), Digital Unix (bidirectional) */
		tios.c_cflag |= CRTSCTS; /* non-Posix! */
#else
#ifdef CNEW_RTSCTS /* Irix (bidirectional) */
		tios.c_cflag |= CNEW_RTSCTS; /* non-Posix! */
#else
#ifdef CCTS_OFLOW /* BSD (unidirectional) */
		tios.c_cflag |= CCTS_OFLOW; /* non-Posix! */
#else
		fprintf (stderr, "Warning: tios.c_cflag doesn't appear to support hardware flow control\n");
#endif
#endif
#endif
#ifdef CRTSXOFF /* Solaris (unidirectional) */
		tios.c_cflag |= CRTSXOFF; /* non-Posix! */
#else
#ifdef CRTS_IFLOW /* BSD (unidirectional) */
		tios.c_cflag |= CRTS_IFLOW; /* non-Posix! */
#endif
#endif
#endif
	}
	/* Result: use 8{E/O/N}{1/2} with or without RTS/CTS flow control */
	/* Note: no direct control over DTR, but BC01 doesn't use DTR/DSR */

	tios.c_lflag &= ~ (ul) (ECHO | ECHOE | ECHOK | ECHONL | ICANON | IEXTEN | ISIG | NOFLSH | TOSTOP);
	if (tios.c_lflag != 0)
	{
#if defined (__sparc__) && defined (MGR)
		if (tios.c_lflag != 05000) /* ECHOCTL | ECHOKE */
#endif
#if defined (ECHOCTL) && defined (ECHOKE)
		if (tios.c_lflag != (ECHOCTL | ECHOKE))
#endif
		fprintf (stderr, "Warning: tios.c_lflag contains non-zero non-Posix bits: 0%lo\n", (ul) tios.c_lflag);
	}
	/* There are no bits in c_lflag we want to set */
	/* Result: no local mode monstrosities! */

	tios.c_cc[VMIN] = 1;
	tios.c_cc[VTIME] = 0;
	/* Result: on read, wait for at least one char */

#define SET_BAUD_RATE(X) do { \
		cfsetispeed (&tios, B ## X); \
		cfsetospeed (&tios, B ## X); } while (0)

	switch (bps)
	{
	case 38400:
		SET_BAUD_RATE(38400);
		break;
#ifdef B57600
	case 57600:	      
		SET_BAUD_RATE(57600); 
		break;
#endif
#ifdef B115200
	case 115200:
		SET_BAUD_RATE(115200); 
		break;
#endif
#ifdef B230400
	case 230400:
		SET_BAUD_RATE(230400); 
		break;
#endif
#ifdef B460800
	case 460800:
		SET_BAUD_RATE(460800); 
		break;
#endif
#ifdef B500000
	case 500000:      
		SET_BAUD_RATE(500000); 
		break;
#endif
#ifdef B576000
	case 576000:      
		SET_BAUD_RATE(576000); 
		break;	
#endif
#ifdef B921600
	case 921600:
		SET_BAUD_RATE(921600); 
		break;
#endif
#ifdef B1000000
	case 1000000:
		SET_BAUD_RATE(1000000);	
		break;
#endif
#ifdef B1152000
	case 1152000:
		SET_BAUD_RATE(1152000);	
		break;
#endif
#ifdef B1500000
	case 1500000:
		SET_BAUD_RATE(1500000);	
		break;
#endif
#ifdef B2000000
	case 2000000:
		SET_BAUD_RATE(2000000);	
		break;
#endif
#ifdef B2500000
	case 2500000:
		SET_BAUD_RATE(2500000);	
		break;
#endif
#ifdef B3000000
	case 3000000:
		SET_BAUD_RATE(3000000); 
		break;
#endif
#ifdef B3500000
	case 3500000:
		SET_BAUD_RATE(3500000); 
		break;	
#endif
#ifdef B4000000
	case 4000000:
		SET_BAUD_RATE(4000000); 
		break;
#endif
	    
	default:
		fprintf (stderr, "Sorry, I don't know how to set %lu bps\n",
			 (ul) bps);
		::close (fd);
		fd = -1;
		return false;
	}

	if (tcsetattr (fd, TCSANOW, &tios) != 0)
	{
		fprintf (stderr,
			 "Failed to set terminal attributes (errno %i):\n",
			 errno);
		perror (NULL);
		::close (fd);
		fd = -1;
		return false;
	}

#if !NODPRINTF
	dprintf ("Final terminal attributes:\n");
	dprintf ("tios.c_iflag = 0x%lx\n", (ul) tios.c_iflag);
	dprintf ("tios.c_oflag = 0x%lx\n", (ul) tios.c_oflag);
	dprintf ("tios.c_cflag = 0x%lx\n", (ul) tios.c_cflag);
	dprintf ("tios.c_lflag = 0x%lx\n", (ul) tios.c_lflag);
	dprintf ("tios.c_cc[%i] = {", NCCS);
	for (i = 0; i < NCCS; ++i)
	{
		dprintf ("%lu%s", (ul) tios.c_cc[i],
			 i == NCCS - 1 ? "" : ", ");
	}
	dprintf ("}\n");
	dprintf ("tios.c_cc[VMIN] = %lu\n", (ul) tios.c_cc[VMIN]);
	dprintf ("tios.c_cc[VTIME] = %lu\n", (ul) tios.c_cc[VTIME]);

	portSpeed = cfgetospeed (&tios);
	dprintf ("Output speed: %lu\n", (ul) portSpeed);
	portSpeed = cfgetispeed (&tios);
	dprintf ("Input speed: %lu\n", (ul) portSpeed);

	dprintf ("[115200/57600/38400/9600/0 bps == %lu/%lu/%lu/%lu/%lu]\n",
		 (ul) B115200, (ul) B57600, (ul) B38400, (ul) B9600, (ul) B0);
#endif

	return true;
}


bool PosixBCSP::close ()
{
	bool ok = true;

	if (fd != -1)
	{
		ok = tcflush (fd, TCIOFLUSH) == 0; /* otherwise can hang! */
		::close (fd);
		fd = -1;
	}

	return ok;
}


void *threadFromC (void *that)
{
	/* HERE: mask out signals for this thread */
	((PosixBCSP *) that)->thread ();
	return NULL;
}

void *txThreadFromC (void *that)
{
	/* HERE: mask out signals for this thread */
	((PosixBCSP *) that)->txThread ();
	return NULL;
}

void *rxThreadFromC (void *that)
{
	/* HERE: mask out signals for this thread */
	((PosixBCSP *) that)->rxThread ();
	return NULL;
}


void PosixBCSP::start (bool useLinkEstablishment)
{
	int rc;

	if (!useLinkEstablishment)
	{
		mStack->isUnchoked = true;
	}

	rc = pthread_create (&threadID, NULL, threadFromC, this);
	if (rc == -1)
	{
		fprintf (stderr, "Hm, are you sure you're linking with the pthread library?\n"); /* It seems to be possible not to and still build the executable OK! */
	}
	CHECK (rc == 0);
	CHECK (pthread_create (&txThreadID, NULL, txThreadFromC, this) == 0);
	CHECK (pthread_create (&rxThreadID, NULL, rxThreadFromC, this) == 0);
}


bool PosixBCSP::connect (uint32 timeout)
{
	uint32 timeStart = HiResClockGetMilliSec ();
	struct timespec ts = { 0, 100000000 };

	do
	{
		if (BCSPLinkEstablished (mStack))
		{
			return true;
		}
		dprintf ("Still not connected...\n");
		CHECK (nanosleep (&ts, NULL) == 0);
	} while (HiResClockGetMilliSec () - timeStart <= timeout);

	return false;
}


void PosixBCSP::setOnLinkFail (void (*fn) (void *), void *arg)
{
	userOnLinkFail = fn;
	userOnLinkFailArg = arg;
}


#ifdef POSIXBCSP_MAIN
int main (int argc, char *argv[])
{
	PosixBCSP bcsp;

#define PORTNAME "/dev/term/a"
#define BPS 38400

	printf ("Hello!\n");
	if (!bcsp.open (PORTNAME, BPS))
	{
		fprintf (stderr, "Failed to open!\n");
		exit (EXIT_FAILURE);
	}
	bcsp.start ();
	if (!bcsp.connect (5000))
	{
		fprintf (stderr, "Failed to connect!\n");
	}
	else
	{
		printf ("Wahey, connected!\n");
		sleep (1); /* Wait for any incoming data */
	}
	printf ("Goodbye!\n");

	return EXIT_SUCCESS;
}
#endif
