/****************************************************************************

  Copyright (C) Cambridge Silicon Radio April 2000

FILE
        Win32BCSP.cpp  -  win32 wrapper for BCSP stack

MODIFICATION HISTORY
	1.3  27-apr-00 npm  - added boilerplating
	1.4  27-apr-00 npm  - added more comments, replaced defaultconfigureport 
						  by call to buildcommdcb
	1.5  27-apr-00 npm  - moved default baud rate back to 38.4
	1.6  9-may-00 npm	- fixed potential time wraparound problems
	1.7  9-may-00 npm	- changed default baud rate to 115200
	1.8  11-may-00 npm	- implemented '98/95 BuildCommDCB workaround for configurePort
	1.9  11-may-00 npm	- fixed memory leak - packets were not being deleted 
						  added cast to time comparison in Connect 
						  got rid of warnings
	1.10 12-may-00 npm	- fixed writefile problem.
	1.11 22-june-00 npm - a very nasty hack to attempt to workaround the writefile problem
	1.12 29-june-00 npm - oops - forgot that port name and config strs could be non-static
	1.14 16-nov-00 at   - added ability to use port opened and closed externally
	1.15 27-nov-00 at	- updated the writefile hack for serial port opened by handle
	1.16 29-nov-00 at	- serial port purged before being closed
	1.17 12-dec-00 at	- removed start method's useLinkEstablishment parameter
	1.18 12-jan-01 mr01	- improve data throughput
	1.19 15-jan-01 mr01	- fix substitution of parity-failed characters
	1.20 12-mar-01 at	- globally replaced CancelIo by PurgeComm
	                      added free of mStack during destruction
    1.21 12-mar-01 at   - _beginthreadex used instead of CreateThread

	1.22 20-jan-01 dm	- added changes for Windows CE _WINCE compile option

*/


#ifdef	RCS_STRINGS
static  char    win32bcsp_c_id[]
  = "$Id: //depot/bc/main/devHost/HostStack/BCSP/Win32/Win32UART/Win32BCSP.cpp#35 $";
#endif


#include "stdafx.h"
#include <malloc.h>

#ifndef _WINCE
#include <process.h>
#endif


#include "BCSPStack.h"
#include "win32BCSP.h"

#ifdef _WINCE
#include "winbase.h"
#include "threadfunctions.h"
#endif

#ifdef _WINCE
typedef bool booldm;
static WinCEBuildCommDCB(ichar *, DCB *); /* replace the buildcommdcb with equiv code */
static WinCEgetcommsettings(DCB *);
static ichar* tokentable [] = {II("baud"),II("parity"),II("data"),II("stop"),II("rts"),II("dtr")};
static int toktabsize = 6;
#endif

#ifdef CECLINEDBGON
#define CE_DEBUG(MSG) isprintf MSG
#else
#define CE_DEBUG(x) do { } while(0)
#endif


//make the com error character equal to a SLIP frame marker
#define DCB_ERROR_CHAR ((char)0xc0)

#define READFILE_TIMEOUT 1000 ;

//enums for events
enum{
	UART_XMITDONE =0,
	UART_RCVDONE=1,
	TRANSFER_REQUEST=2,
	DIE_REQUEST=3,
} ;

Win32BCSP::~Win32BCSP()
{
	// Destroy the stack
	if (!mStackDead)
	{
		kill() ;
	}



	if (mComport != INVALID_HANDLE_VALUE) 
	{
		//free com port 
#ifndef _WINCE
		PurgeComm(mComport, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);
		CloseHandle(mComport) ;
#else
		FlushFileBuffers(mComport);
		PurgeComm(mComport, PURGE_TXCLEAR | PURGE_RXCLEAR);
		if (!mOpenedAsOtherInstance)
			CloseHandle(mComport);
		Sleep(10);
#endif
	}
	//free events
	for (int c = 0 ; c < sizeof(mEvents)/sizeof(HANDLE) ; c++)
		CloseHandle(mEvents[c]) ;

	//free up bcsp packet headers...
	for (c= 0 ; c< sizeof(Packets)/sizeof(Packet*); c++)
		if (Packets[c]) free(Packets[c]) ;
	if (mSavedConfigStr) free(mSavedConfigStr) ;
	if (mSavedPortName) free(mSavedPortName) ;

	//clean up the environment
	cleanupEnvironment(&mEnvState) ;

	//free the stack
	free(mStack) ;
}


Win32BCSP::Win32BCSP()
{
	//create the initial stack state

	mComport = INVALID_HANDLE_VALUE ;
	mStackThread = INVALID_HANDLE_VALUE ; 

	mEvents[UART_XMITDONE] = CreateEvent(0,1,0,0) ;
	mEvents[UART_RCVDONE] = CreateEvent(0,1,0,0) ;
	mEvents[TRANSFER_REQUEST] = CreateEvent(0,0,0,0) ;
	mEvents[DIE_REQUEST] = CreateEvent(0,0,0,0) ;
	mXMITBusy =false;
	mRCVBusy =false;
	mRCVBytesAvailable =0;
	mXMITBytesAvailable =0;
	mStackDead = false ;
	mStackMustDie = false ;

#ifndef _WINCE
	mXMITOverlapped.Offset = 0;
	mXMITOverlapped.OffsetHigh = 0;
	mXMITOverlapped.hEvent = mEvents[UART_XMITDONE];

	mRCVOverlapped.Offset = 0;
	mRCVOverlapped.OffsetHigh = 0;
	mRCVOverlapped.hEvent = mEvents[UART_RCVDONE];
#else
	mRxWorkerHandle = INVALID_HANDLE_VALUE;
#endif
	createBCSPStack() ;
	initialiseEnvironment(getEnvironment(mStack),
				&mBCSPenv,&
				mEnvState,
				mEvents[TRANSFER_REQUEST],this) ;
	setEnvironment(mStack,&mBCSPenv) ;
	userOnLinkFail = 0 ;
	memset(Packets,0,sizeof(Packets)) ;
	mSavedConfigStr = 0 ;
	mSavedPortName = 0 ;
}


bool Win32BCSP::open(const ichar * portName, const ichar * configurationStr) 
{
	//attempt to open a com port - returns true if successful, false if not
	//the portname is expected to be of the form "comN" where N=1,2,3 etc
	//an optional configuration string can be passed in which follows
	//the mode command syntax.  See docs for BuildCommDCB for details
	
	//in order to work around the horrible '98 problem, we need to save the name and 
	//configuration string in case we need to reopen the port.

	if (portName)
	{
		mSavedPortName = (ichar*) malloc((istrlen(portName)+1) * sizeof(ichar)) ;
		istrcpy (mSavedPortName,portName) ;
		if (configurationStr)
		{
			mSavedConfigStr = (ichar*) malloc((istrlen(configurationStr)+1) * sizeof(ichar)) ;
			istrcpy(mSavedConfigStr,configurationStr) ;
		}
	}
	else
	{
		//doing an open with name = 0 is a special flag to say that we should use the saved
		//values
		portName = mSavedPortName ;
		configurationStr = mSavedConfigStr ;
	}

#ifndef _WINCE
    mComport = CreateFile(portName,
			GENERIC_READ|GENERIC_WRITE,
			0, //exclusive access
			0, //security
			OPEN_EXISTING,
			 FILE_FLAG_OVERLAPPED,
			0
			) ;

#else
#ifndef UNICODE
#error need to be in UNOCODE for file open.
#endif
    mComport = CreateFile(portName,
			GENERIC_READ|GENERIC_WRITE,
			0, //exclusive access
			0, //security
			OPEN_EXISTING,
//			 FILE_FLAG_OVERLAPPED,
			0,
			0
			) ;
	mOpenedAsOtherInstance = false;

#endif

	if (mComport == INVALID_HANDLE_VALUE) return false;
	
	//enlarge the buffers
#ifndef _WINCE
	if (!SetupComm(mComport,1000,1000)) return false;
#else
	if (!SetupComm(mComport,16,16)) return false; /* this will not work? max buffer size is 16 ? */
#endif	
	//set up the port parameters - baud rate etc
	if (!ConfigurePort(mSavedConfigStr)) 
	{
		return false ;
	}

#ifdef _WINCE	
	createRxWorker();	// create the thread that just blocks on waiting for receive
#endif	
	return true ;
}

bool Win32BCSP::open(HANDLE comport)
{
	// Use an existing com port - just ensure that the buffers are large enough
	// and set the timeouts. To ensure that the port stays open long enough, and
	// to enable identical treatment when the stack is destroyed, a duplicate
	// handle is created.

	// Duplicate the file handle

#ifndef _WINCE
	if (comport == INVALID_HANDLE_VALUE) return false;
	HANDLE process = GetCurrentProcess();
	if (!DuplicateHandle(process, comport, process, &mComport,
		                 GENERIC_READ | GENERIC_WRITE, false, 0))
	{
		return false;
	}
	if (mComport == INVALID_HANDLE_VALUE) return false;
#else
	Sleep(1000);
	PurgeComm(mComport, PURGE_TXCLEAR | PURGE_RXCLEAR);
	Sleep(1000);
	mComport = comport;
	mOpenedAsOtherInstance = true;
#endif

	// Enlarge the buffers
#ifndef _WINCE
	if (!SetupComm(mComport, 1000, 1000)) return false;
#else
	if (!SetupComm(mComport,16,16)) return false; /* this will not work? max buffer size is 16 ? */
#endif

	// Make the com error character equal to a SLIP frame marker
	DCB dcb;
	if (!GetCommState(mComport, &dcb)) return false;

	dcb.ErrorChar = DCB_ERROR_CHAR;
	dcb.fErrorChar = 1 ;
	dcb.fBinary = 1 ;
	// Don't force dcb.fParity = 1 ; this allows applications to
	// not perform parity checking, should they so desire

	if (!SetCommState(mComport, &dcb)) return false;

	// Set the timeouts
	if (!setTimeouts()) return false;

#ifdef _WINCE	
//	createRxWorker();					/* create the thread that just blocks on waiting for */
										/* receive */
#endif

	// Successful if this point reached
	return true;
}


bool Win32BCSP::setTimeouts()
{
	//set the timeouts for the com port. 
	COMMTIMEOUTS sCT ;
	memset(&sCT,0,sizeof(COMMTIMEOUTS)) ;
	/*
	 * The following settings of the read timeouts ensure that
	 * ReadFile () will return immediately if there are any
	 * characters in the input buffer, or wait for a character
	 * to arrive and then return this immediately (or timeout).
	 *
	 * This maximises the throughput without impairing the
	 * responsivity: one can request lots of characters in the
	 * ReadFile () request and get a buffer's worth if that
	 * much is available, whatever's there if less than a buffer's
	 * worth is available, and the first thing to arrive if
	 * nothing is currently available.
	 *
	 * See COMMTIMEOUTS in the MSDN Library help for more details.
	 *
	 * HERE: It would be nice not to ever timeout; perhaps
	 * this could be done with some magic value for
	 * ReadTotalTimeoutConstant.  In the absence of any
	 * documented values, MAXDWORD - 1 might be the safest bet.
	 */
#ifndef _WINCE
	sCT.ReadIntervalTimeout = MAXDWORD;
	sCT.ReadTotalTimeoutMultiplier = MAXDWORD;
	sCT.ReadTotalTimeoutConstant = READFILE_TIMEOUT;

	if(!SetCommTimeouts(mComport, &sCT)) return false;
	return true ;
#else

	sCT.ReadIntervalTimeout = MAXDWORD ;
	sCT.ReadTotalTimeoutConstant = 0;
	sCT.ReadTotalTimeoutMultiplier = 0;	/* return immediately even if no character */


//	sCT.WriteTotalTimeoutMultiplier = 10;  
//	sCT.WriteTotalTimeoutConstant = 1000;    
	sCT.WriteTotalTimeoutMultiplier = 0;  
	sCT.WriteTotalTimeoutConstant = 0;    

	if(!SetCommTimeouts(mComport, &sCT)) return false;
	return true ;
#endif

}


/* 
Configure Port
	
There is a bug in BuildCommDCB under windows 95/98 (described in MS article Q201372)
The function attempts to write to the string - if it's a constant array with read-only
access, this will obviously result in an ACV.  The fix is to copy the string into 
an array.

*/
bool Win32BCSP::ConfigurePort(ichar * cfgStr)
{
	DCB dcb;
	
	ichar workaroundStr[100] ; //anyone passing in a very long config string will be out of luck 
							//since it's going to get truncated.

	if (!cfgStr) 
	{
#ifdef _WINCE 
		//default configuration
		cfgStr = II("baud=57600 parity=e data=8 stop=1 dtr=on rts=on") ;
#else
		
		//default configuration
		cfgStr = II("baud=115200 parity=e data=8 stop=1 dtr=on rts=on") ;
#endif
	}
	
	istrncpy(workaroundStr,cfgStr,sizeof(workaroundStr)/sizeof(ichar)-1) ;
	workaroundStr[sizeof(workaroundStr)/sizeof(ichar)-1] = 0 ; //ensure null-termination even for overlong cfg strs

	memset(&dcb,0,sizeof(DCB)) ;
	dcb.DCBlength = sizeof(DCB) ;
#ifndef _WINCE	
	if (!BuildCommDCB(workaroundStr,&dcb)) return false ;
#else
	if (!WinCEBuildCommDCB(workaroundStr, &dcb)) return false;
#endif
	dcb.ErrorChar = DCB_ERROR_CHAR ;
	dcb.fErrorChar = 1 ;
	dcb.fBinary = 1 ;
	dcb.fParity = 1 ;
	
	if(!SetCommState(mComport,&dcb)) return false;

	return  setTimeouts();
}


UINT callRunStack(LPVOID * p)
{
	Win32BCSP * winStack = (Win32BCSP *) p ;
	winStack->runStack() ;	
	winStack->mStackDead = true ; //we've exited
	return 0 ;
}

void Win32BCSP::spawnStack()
{
#ifndef _WINCE
	mStackThread = reinterpret_cast<HANDLE>
		(_beginthreadex(0,0,(unsigned (__stdcall *)(void *))callRunStack,this,0,&mStackThreadId)) ;
#else
	mStackThread = 
		CreateThread(0,0,(unsigned long (__stdcall *)(void *))callRunStack,this,0,&mStackThreadId) ;
#endif
}

void Win32BCSP::createBCSPStack()
{
#ifndef _WINCE
	mStack = (BCSPStack * ) calloc(1,sizeof(BCSPStack)) ;
#else
	mStack = (BCSPStack * ) malloc(sizeof(BCSPStack)) ;
#endif
	initialiseStack(mStack) ;
	BTRACE0(WINERR,II("WINERR debug enabled\n")) ;
	//add a few packets...
	for (int c = 0 ; c < sizeof(Packets)/sizeof(Packet*) ; c++)
	{
#ifdef _WINCE
		Packet * pkt = (Packet*) malloc(sizeof(Packet)) ;
#else
		Packet * pkt = (Packet*) calloc(1,sizeof(Packet)) ;
#endif
		BCSPaddPacket(mStack,pkt) ;
		Packets[c] = pkt ; //keep track of them so that we can delete them later.
	}
}




void Win32BCSP::runStack()
{
	//here's where we actually run round in a loop calling the scheduler...
	while(!mStackMustDie)
	{
		uint32 timeNow = GetTickCount() ;

		uint32 wakeupTime = scheduler(mStack,timeNow) ;

		//now see if we can do anything with the uart...
		//try sending ...

		if(!mXMITBusy) 
		{
			//copy stuff out of stack output buffer 
			mXMITBytesAvailable = numBytesInTransmitBuffer(mStack) ;
			if (mXMITBytesAvailable)
			{
				readFromTransmitBuffer(mStack,mXMITBuffer,(uint8) mXMITBytesAvailable) ;
				if (mXMITBytesAvailable) 
				{
					BTRACE0(IO,II("-->")) ;
					for (unsigned int c = 0 ; c < mXMITBytesAvailable ; c++) 
						PLAINBTRACE1(IO,II(" %02x"),mXMITBuffer[c]) ;
					PLAINBTRACE0(IO,II("\n")) ;
				}
				mXMITBytesAvailable ;
			}
		}
		//now try loading the stack receive buffer
		if (mRCVBytesAvailable &&(mRCVBytesAvailable <= numFreeSlotsInReceiveBuffer(mStack)))
		{
			if (mRCVBytesAvailable) 
			{
				BTRACE0(IO,II("<--")) ;
				for (unsigned int c = 0 ; c < mRCVBytesAvailable ; c++) 
					PLAINBTRACE1(IO,II(" %02x"),mRCVBuffer[c]);
				PLAINBTRACE0(IO,II("\n")) ;
			}

			writeToReceiveBuffer(mStack,mRCVBuffer,(uint8) mRCVBytesAvailable) ;
			mRCVBytesAvailable = 0 ;
		}

		//now startup I/O  - non-zero indicates that the read or write has returned immediately
		if (!startIO())  
		{
			if (isStackIdle(mStack)) 
				wait(wakeupTime) ;
		}
	}
	//we'll return if we get the stackmustdie signal
}

uint8 Win32BCSP::startIO()
{
	uint8 retval = 0 ;
	int  err ;
	if ((!mXMITBusy) && (mXMITBytesAvailable))
	{
		mXMITBusy = true ;
		mLastXmitTime = GetTickCount() ;
#ifndef _WINCE
		if (WriteFile(mComport,mXMITBuffer,mXMITBytesAvailable,&mXMITbytesWritten,&mXMITOverlapped))
		{
			//function succeeded immediately
			mXMITBusy = false ;
			retval =1 ;
		}
		else
		if ((err = GetLastError()) != ERROR_IO_PENDING)
		{
			mXMITBusy = false ;
			BTRACE1(WINERR,II("WriteFile error %d\n"),err) ;
		}
//	}
#else
		
		if (WriteFile(mComport,mXMITBuffer,mXMITBytesAvailable,&mXMITbytesWritten,NULL))
		{
			//function succeeded immediately

			mXMITBusy = false ;
			retval =1 ;
		}

		else
		if ((err = GetLastError()) != ERROR_IO_PENDING)
		{
			mXMITBusy = false ;
			BTRACE1(WINERR,II("WriteFile error %d\n"),err) ;
			BTRACE1(WINERR,II("bytesavail = %d"),mXMITBytesAvailable);
			BTRACE1(WINERR,II("byteswritten = %d"),mXMITbytesWritten);

		}


#endif

	}

#ifndef _WINCE
	if ((!mRCVBusy)  && (!mRCVBytesAvailable))
	{	
#endif
		//Now try to read some chars from the uart.
		uint16 bytesToGet = numFreeSlotsInReceiveBuffer(mStack) ;
		mRCVBusy = true ;
#ifndef _WINCE
		if (ReadFile(mComport,mRCVBuffer,bytesToGet,&mRCVBytesAvailable,&mRCVOverlapped))
		{
			mRCVBusy = false ;
			retval =1 ;
		}
		else
		if (GetLastError() != ERROR_IO_PENDING)
		{
			mRCVBusy = false ;
			BTRACE1(WINERR,II("ReadFile error %d\n"),err) ;

		}
#else


	/* temporary try bytesToGet = 16 */
		bytesToGet = 16;
		ReadFile(mComport,mRCVBuffer,bytesToGet,&mRCVBytesAvailable,NULL);
		if (mRCVBytesAvailable != 0)
		{

		}
		else
		{

		}
#endif

#ifndef _WINCE
	}
#endif

	return retval ; //indicate that we need to wait for some io to complete or not
}

#define HACK_PERIOD 500

void Win32BCSP::wait(uint32 wakeupTime)
{
	//this function just puts the stack thread into idle until some event (or timeout) 
	//wakes us up.
	int32 timeout = wakeupTime - GetTickCount() ;
	if (timeout <=0) return ;
	
	// <sigh> there appears to be a bug in the WriteFile implementation under Win98. 
	// Occasionally, the WriteFile never sets the completion event.  The extremely ugly
	// workaround seems to be to closedown and then reopen the com port if the writefile hasn't
	// completed after a certain period.
#ifndef _WINCE
	if (mXMITBusy && (GetTickCount() - mLastXmitTime > HACK_PERIOD)) 
	{
		if (mSavedPortName)
		{
			BTRACE3(WINERR,II("Cancelling IO at time %d (%d ms from %d)\n"),GetTickCount(),HACK_PERIOD,mLastXmitTime) ;
			PurgeComm(mComport, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);
			mLastXmitTime = GetTickCount() ;
			BTRACE0(WINERR,II("Now closing and reopening serial port!\n")) ;
			CloseHandle(mComport) ;
			mXMITBusy = false ;
			mRCVBusy = false ;
			if (open(0,0))	BTRACE0(WINERR, II("reopened serial port!\n")) ;
			else BTRACE0(WINERR,II("Couldn't reopen serial port\n")) ; 
			//if we didn't manage to reopen the port, we really have problems but then again,
			//we already know this is ugly
		}
		else
		{
			// If the stack was started using an already opened com port then it is not
			// possible to re-open it by name. Hence, we attempt to duplicate the handle
			// again, which may not be as effective.
			HANDLE oldport = mComport;
			BTRACE3(WINERR, II("Cancelling IO at time %d (%d ms from %d)\n"), GetTickCount(), HACK_PERIOD, mLastXmitTime);
			PurgeComm(mComport, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);
			mLastXmitTime = GetTickCount() ;
			BTRACE0(WINERR, II("Now re-duplicating serial port handle and closing old handle!\n"));
			mXMITBusy = false;
			mRCVBusy = false;
			if (open(oldport))
			{
				BTRACE0(WINERR, II("Re-duplicated serial port handle\n"));
				CloseHandle(oldport);
			}
			else
			{
				BTRACE0(WINERR, II("Couldn't re-duplicate serial port handle\n"));
				mComport = oldport;
			}
		}
		return;
	}
#endif
	//on with the real code...

	DWORD ret = WaitForMultipleObjects(sizeof(mEvents)/sizeof(HANDLE),mEvents,false,timeout) ;
	if (ret !=WAIT_TIMEOUT)
	{
		DWORD i = ret-WAIT_OBJECT_0 ;
		switch (i)
		{
#ifndef _WINCE
		case UART_XMITDONE : //This is the transmit-finished - all we need to do is reset the event
				ResetEvent(mEvents[UART_XMITDONE]) ; 
				mXMITBusy = false ; 
				break ; //allow xmit to go ahead 
		case UART_RCVDONE: //This is a receive from the uart 
				{
					BOOL res = GetOverlappedResult(mComport,&mRCVOverlapped,&mRCVBytesAvailable,true) ;
					ResetEvent(mEvents[UART_RCVDONE]) ; 
					mRCVBusy = false ;
					if (!res) 
					{
						mRCVBytesAvailable = 0 ; //oops - this was a timeout!
					}
				}
				break ;
#else
			/* A seperate thread which just waits for receive data is set up */
			/* which generates a UART_RCVDONE event */

		case UART_RCVDONE:
			ResetEvent(mEvents[UART_RCVDONE]) ; 
			break;
#endif
		case TRANSFER_REQUEST: 
			break ;//write req - no action necessary
		case DIE_REQUEST: //the user has requested that we shutdown
			PurgeComm(mComport, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);
			break ;

		default : break ;//error!
		}
	}
	else BTRACE0(USER0,II("WAIT TIMEOUT\n")) ; 
}

bool Win32BCSP::connect(uint32 timeout)
{
	//wait for timeout or until we have link-establishment
	int32 timeoutTime = GetTickCount() + timeout ;
	do
	{
		if (BCSPLinkEstablished(mStack)) return true ;
		Sleep(10) ;
	} while ( (int32)(timeoutTime-GetTickCount()) > 0) ; 
	return false ;
}
void Win32BCSP::start()
{
	//start the BCSP stack thread
	if (!mStack->configuration->useLinkEstablishmentProtocol) mStack->isUnchoked = true ;
	spawnStack() ;
}

void Win32BCSP::kill()
{
	//kill the BCSP stack thread
	if (mStackThread != INVALID_HANDLE_VALUE)
	{
		mStackMustDie = true ;
		SetEvent(mEvents[DIE_REQUEST]) ;
		while (!mStackDead) Sleep(10) ;
		CloseHandle(mStackThread);
		mStackThread = INVALID_HANDLE_VALUE;
	}
#ifdef _WINCE
	killRxWorker();
#endif

	BCSPshutdownStack(mStack) ;
}

void Win32BCSP::onLinkFail()
{
	//alert the user
	if (userOnLinkFail)	userOnLinkFail(userOnLinkFailParam) ;
}

void Win32BCSP::setOnLinkFail(void (*fn)(void*),void * param)
{
	userOnLinkFail = fn ;
	userOnLinkFailParam = param ;
}

#ifdef _WINCE


static WinCEBuildCommDCB(ichar *strin, DCB *dcb)
{
 
    INT i;
	ichar *token;
	ichar seps[]   = II(" ,\t\n");

	ichar *string = II("baud=1200 parity=N data=8 stop=1");
	int intfound = 0;


	CE_DEBUG((II("todo = %s\n\nTokens:\n"),strin));
    /* Establish string and get the first token: */	
    token = istrtok( strin, seps );
   while( token != NULL )
   {
      /* While there are tokens in "string" */
	  CE_DEBUG((II("new token =%s\n\r"), token));
      CE_DEBUG((II(" %s\n"), token ));
	  intfound = 0;
 
	  for (i=0; i<toktabsize; i++)
	  {
		CE_DEBUG((II("look for %s\n\r"),tokentable[i]));
		if (istrstr(token, tokentable[i]))
		{

			CE_DEBUG((II("found %s in %s\n"),tokentable[i], token));
			CE_DEBUG((II("val = %s\n\r"),token + 1 + istrlen(tokentable[i])));
			intfound = 1;

			switch (i)
			{
			case 0: /* baud */
				CE_DEBUG((II("Setting the dcb baud val = %d\n\r"), iatoi(token + 1 + strlen(tokentable[i]))));
				dcb->BaudRate = iatoi(token + 1 + istrlen(tokentable[i]));
				/* do some more complex handling of the baud rate */
//				dcb->BaudRate = baudfuzz(token + 1 + istrlen(tokentable[i]));
				break;


			case 1: /* parity e,o,n,m,s*/
				CE_DEBUG((II("Setting the parity val .... ")));
				dcb->fParity = TRUE;
				if (*(token + 1 + istrlen(tokentable[i])) == II('e'))
				{
					CE_DEBUG((II("even\n\r")));
					dcb->Parity = EVENPARITY;
				}
				else if (*(token + 1 + istrlen(tokentable[i])) == II('o'))
				{
					CE_DEBUG((II("odd\n\r")));

					dcb->Parity = ODDPARITY;
				}
				else if (*(token + 1 + istrlen(tokentable[i])) == II('m'))
				{
					CE_DEBUG(("mark\n\r"));
					dcb->Parity = MARKPARITY;
				}
				else if (*(token + 1 + istrlen(tokentable[i])) == II('s'))
				{
					CE_DEBUG((II("space\n\r")));
					dcb->Parity = SPACEPARITY;
				}
				else
				{
					CE_DEBUG((II("none\n\r")));
					dcb->Parity = NOPARITY;
					dcb->fParity = FALSE;
				}
				break;

			case 2:	/* data */
				CE_DEBUG((II("data set to %d\n\r"),iatoi(token + 1 + istrlen(tokentable[i]))));
				dcb->ByteSize = iatoi(token + 1 + istrlen(tokentable[i]));
				break;
			case 3: /* stop */
				if (*(token + 1 + istrlen(tokentable[i])) == II('1'))
				{
					CE_DEBUG((II("one stop bit\n\r")));
					dcb->StopBits = ONESTOPBIT;
				}
				else
				{
					CE_DEBUG(("two stop bits\n\r"));
					dcb->StopBits = TWOSTOPBITS;
				}
				break;
			case 4: /* rts */
				if (*(token + 2 + istrlen(tokentable[i])) == II('n'))
				{
					CE_DEBUG((II("rts on\n\r")));
					CE_DEBUG((II("temp to control enable")));
					dcb->fRtsControl  = RTS_CONTROL_ENABLE; /*RTS_CONTROL_HANDSHAKE;*/
				}
				else
				{
					CE_DEBUG((II("rts off\n\r")));
					dcb->fRtsControl  = RTS_CONTROL_ENABLE;
				}
				break;



			case 5: /* DTR */
 				if (*(token + 2 + istrlen(tokentable[i])) == II('n'))
				{
					CE_DEBUG((II("dtr on\n\r")));
					CE_DEBUG((II("temp to control enable")));
					dcb->fDtrControl  = DTR_CONTROL_ENABLE; /*DTR_CONTROL_HANDSHAKE;*/
				}
				else
				{
					CE_DEBUG((II("dtr off\n\r")));
					dcb->fDtrControl  = DTR_CONTROL_ENABLE;
				}
				break; 



			} /* end of switch */
		} /* end of if */
	} /* end of for */

	/* if we could not find the token print out an error message and quit */
//	if (i == toktabsize)
	if (intfound == 0)
	{
		CE_DEBUG((II("Can not find the token\n\r")));
		exit(1);
	}


     /* Get next token: */
      token = istrtok( NULL, seps );
//	  printf("Now get the next token\n\r");

   } /* end of while */

//	iprintf(II("Return 1 from wincebuilcommdcb\n\r"));
	return 1;
}



static WinCEgetcommsettings(DCB *dcb)
{
	DWORD dwtmp;
	BYTE bytmp;

	dwtmp = dcb->BaudRate;

	iprintf(II("val of br = 0x%x"),dwtmp);
	dwtmp = dcb->fBinary;
	iprintf(II("val of fbinary = 0x%x\n\r"),dwtmp);
	dwtmp = dcb->fParity;
	iprintf(II("val of fParity = 0x%x\n\r"),dwtmp);
	dwtmp = dcb->fOutxCtsFlow;
	iprintf(II("val of fOutxCTSFlow = 0x%x\n\r"),dwtmp);
	dwtmp = dcb->fOutxDsrFlow;
	iprintf(II("val of fOutxDsrFlow = 0x%x\n\r"),dwtmp);
	dwtmp = dcb->fDtrControl;
	iprintf(II("val of fDtrControl = 0x%x\n\r"),dwtmp);
	dwtmp = dcb->fDsrSensitivity;
	iprintf(II("val of fDsrSensitivity = 0x%x\n\r"),dwtmp);
	dwtmp = dcb->fTXContinueOnXoff;
	iprintf(II("val of fTxcontinuexonxoff = 0x%x\n\r"),dwtmp);
	dwtmp = dcb->fOutX;
	iprintf(II("val of fOutX = 0x%x\n\r"),dwtmp);
	dwtmp = dcb->fInX;
	iprintf(II("val of fInX = 0x%x\n\r"),dwtmp);
	dwtmp = dcb->ErrorChar;
	iprintf(II("val of fErrorchar = 0x%x\n\r"),dwtmp);
	dwtmp = dcb->fNull;
	iprintf(II("val of fNull = 0x%x\n\r"),dwtmp);
	dwtmp = dcb->fRtsControl;
	iprintf(II("val of rtscontro = 0x%x\n\r"),dwtmp);
	dwtmp = dcb->fAbortOnError;
	iprintf(II("val of abort = 0x%x\n\r"),dwtmp);
	bytmp = dcb->ByteSize;
	iprintf(II("val of bytesize = 0x%x\n\r"),bytmp);
	bytmp = dcb->Parity;
	iprintf(II("val of parity = 0x%x\n\r"),bytmp);
	bytmp = dcb->StopBits;
	iprintf(II("val of stops = 0x%x\n\r"),bytmp);
	return 0;
}


void Win32BCSP::createRxWorker ()
{
	DWORD RxWorkerThreadId;
	mRxWorkerHandle = CreateThread(0,0,(unsigned long (__stdcall *)(void *))RxworkerThread,this,0,&RxWorkerThreadId) ;
}


void Win32BCSP::RxworkerThread ( void * aObject )/* thread which just detects rx busy */
{
    ( (Win32BCSP *) aObject )->waitForRxWork();
}


void Win32BCSP::killRxWorker ()
{
	if (mRxWorkerHandle != INVALID_HANDLE_VALUE)
	{
		TerminateThread(mRxWorkerHandle, 0);
		Sleep(10);
		CloseHandle(mRxWorkerHandle);
		mRxWorkerHandle = INVALID_HANDLE_VALUE;
	}
}


void Win32BCSP::waitForRxWork ()
{
/*
	Waitformultipleobjects on comm events. The book talks of 
using setcommmask and then using waitcommevent to block until any of the events 
 you set up
*/
	
	DWORD dwMask;
	CE_DEBUG((II("Start the wait for rx\n\r")));

	while(!mStackMustDie)
	{
		SetCommMask(mComport, EV_RXCHAR | EV_ERR);
		if (WaitCommEvent(mComport,&dwMask, 0))
		{
			// use the flags returned in dwmask to determine the reason for returning */
			switch(dwMask) 
			{
				case EV_RXCHAR:
					/* set the flags */
					SetEvent(mEvents[UART_RCVDONE]);
					break;
			}
		}
	}
}

#endif
