///////////////////////////////////////////////////////////////////////
//
//  FILE   :  CBluetestEngine.cpp
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  CBluetestEngine
//
//  PURPOSE:  Provide the interface as presented in James' Memo.
//
///////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CBluetestEngine.h"
#include "HCI/HCIFactory.h"
#include "HCI/bc01DeviceController.h"

#include "bccmd/bccmdpdu.h"
#include "hq/hqpdu.h"

#define BLUETEST_ENGINE_RF_TEST_PARAMETER_ARRAY_SIZE (TEST_SIZE + 4)

#include "reportStrings.h"

CBluetestEngine::CBluetestEngine()
{
    testStarted = ::CreateEvent ( 0 , 0 , 0 , 0 );
    pioMask     = ::CreateEvent ( 0 , 0 , 0 , 0 );
    pioLines    = ::CreateEvent ( 0 , 0 , 0 , 0 );
    ::InitializeCriticalSection (&sequenceProtection);

    sequenceNumber = 0;
    mFactory = NULL;
    mChip = NULL;
    mStringCallBackFn = NULL;
    mStringCallBackParameters = NULL;
    mPSSetFunction = NULL;
}

CBluetestEngine::~CBluetestEngine()
{
    CloseHandle (testStarted);
    CloseHandle (pioMask);
    CloseHandle (pioLines);

    resetChip();

    ::DeleteCriticalSection (&sequenceProtection);
}

void CBluetestEngine::setOnString ( void (*onString)( bool verboseNotConcise , TCHAR * string , void * parameters ) , void *parameters )
{
    mStringCallBackFn = onString;
    mStringCallBackParameters = parameters;
}

void CBluetestEngine::setFactory ( BlueCoreControllerFactory * aFactory )
{
    mFactory = aFactory;
}

void CBluetestEngine::setPSSetFunction ( void (*aPSSetFunction) ( PARAMETER_ARRAY , void * ) , void * aParameters )
{
    mPSSetFunction = aPSSetFunction;
    mPSSetParameters = aParameters;
}

bool CBluetestEngine::startChip ()
{
    if ( mStringCallBackFn != NULL )
    {
        mChip = mFactory->spawn();

        mChip->setCallBackOnData  ( 0 , discardHCIData  , this );
        mChip->setCallBackOnEvent ( 0 , discardHCIEvent , this );
        mChip->setupOnTransportFailure ( onLinkFail , this );
        mChip->setupRecvHostToChipResponses ( onBCCMD , this );
        mChip->setupRecvChipToHostRequests ( onHQ, this );
        mChip->setupRecvDebug ( onDebugData , this );

        if ( mChip->getTransport()->start() == OK )
        {
            report ( true, 41 , mFactory->getDeviceName() );
            mChip->createWorker();
            return true;
        }
        else
        {
            delete mChip;
            mChip = NULL;
            report ( true, 42 , mFactory->getDeviceName() );
        }
    }
    return false;
}

void CBluetestEngine::resetChip ()
{
    if (mChip != NULL)
    {
        //  Send a hardware reset.
        PARAMETER_ARRAY reset = new PARAMETER_VALUE[4];

        reset[0] = 4;
        reset[1] = BCCMDPDU_SETREQ;
        reset[2] = nextSequence();
        reset[3] = BCCMDVARID_COLD_RESET;

        mChip->sendChipMgtHostToChip ( reset );
        ::Sleep ( 20 );
	delete [] reset ; //NPM

        //  The chip sends no acknowledgement!
        delete mChip;
        mChip = NULL;
    }
}

bool CBluetestEngine::connected ( void )
{
    bool ok = false;
    if (mChip)
        ok = mChip->isTransportReady();

    if (ok)
    {
        report ( true, (uint8)47 );
        ::Sleep (100);
        readFWVersion();
    }
    else
        report ( true, (uint8)48 );

    return ok;
}

bool CBluetestEngine::runTest ( uint16 aTest[4] )
{
    PARAMETER_ARRAY lTest = new PARAMETER_VALUE [BLUETEST_ENGINE_RF_TEST_PARAMETER_ARRAY_SIZE];

    lTest[0] = BLUETEST_ENGINE_RF_TEST_PARAMETER_ARRAY_SIZE;
    lTest[1] = BCCMDPDU_SETREQ;
    lTest[2] = nextSequence();
    lTest[3] = BCCMDVARID_RADIOTEST;
    lTest[4] = aTest[0];
    lTest[5] = aTest[1];
    lTest[6] = aTest[2];
    lTest[7] = aTest[3];

	bool retval ;
    if ( mChip->sendChipMgtHostToChip ( lTest ) )
    {
	report ( true, (uint8)46 , lTest[3] , lTest[4] , lTest[5] , lTest[6] , lTest[7] );
	retval = true;
    }
    else
    {
        report ( true, (uint8)45 );
	retval = false ;;
    }
    delete [] lTest ; //NPM
    return retval ;
}

bool CBluetestEngine::readFWVersion ( void )
{
    PARAMETER_VALUE getVersion[4] = {4, BCCMDPDU_GETREQ , nextSequence() , BCCMDVARID_BUILDID };
    return mChip->sendChipMgtHostToChip ( getVersion );
}


bool CBluetestEngine::readPIOLines ()
{
    PARAMETER_ARRAY lTest = new PARAMETER_VALUE [5];

    lTest[0] = 5;
    lTest[1] = BCCMDPDU_GETREQ;
    lTest[2] = nextSequence();
    lTest[3] = BCCMDVARID_PIO_MASK;
    lTest[4] = 0;

    bool retval =true ;
    if ( mChip->sendChipMgtHostToChip ( lTest ) )
    {
	int nResponse = ::WaitForSingleObject ( pioMask , 3000 );
	
	if  ( nResponse != WAIT_OBJECT_0 )
            retval = false;
    }
    else
    {
        report ( true, (uint8)45 );
	retval = false ;
    }

    if (retval) 
    {
	lTest[0] = 5;
	lTest[1] = BCCMDPDU_GETREQ;
	lTest[2] = nextSequence();
	lTest[3] = BCCMDVARID_PIO;
	lTest[4] = 0;

	if ( mChip->sendChipMgtHostToChip ( lTest ) )
	{
	    int nResponse = ::WaitForSingleObject ( pioLines , 3000 );
	    if  ( nResponse == WAIT_OBJECT_0 ) retval=true;
	    else retval = false ;
	}
	else
	{
	    report ( true, (uint8)45 );
	    retval= false;
	}
    }
    delete [] lTest ; //NPM
    return retval ;
}

bool CBluetestEngine::setPIOLines ( uint16 mask , uint16 settings )
{
    PARAMETER_ARRAY lTest = new PARAMETER_VALUE [5];

    lTest[0] = 5;
    lTest[1] = BCCMDPDU_SETREQ;
    lTest[2] = nextSequence();
    lTest[3] = BCCMDVARID_PIO_MASK;
    lTest[4] = mask;

	bool retval = true ;

    if ( mChip->sendChipMgtHostToChip ( lTest ) )
    {
	int nResponse = ::WaitForSingleObject ( pioMask , 3000 );

	if  ( nResponse == WAIT_OBJECT_0 )
        {
            lTest[0] = 5;
            lTest[1] = BCCMDPDU_SETREQ;
            lTest[2] = nextSequence();
            lTest[3] = BCCMDVARID_PIO;
            lTest[4] = settings;

            if ( mChip->sendChipMgtHostToChip ( lTest ) )
            {
		int nResponse = ::WaitForSingleObject ( pioLines , 3000 );

		if  ( nResponse == WAIT_OBJECT_0 )
                    retval=true ;
		else
                    retval= false;
            }
            else
            {
                report ( true, (uint8)45 );
		retval= false;
            }

        }
	else
        {
	    retval= false;
        }
    }
    else
    {
        report ( true, (uint8)45 );
		retval= false;
    }
    delete [] lTest ; //NPM
    return retval ;
}

bool CBluetestEngine::provokeFault ( uint16 aFaultId )
{
    PARAMETER_VALUE cmd[5] = { 5 , BCCMDPDU_SETREQ , nextSequence() , BCCMDVARID_PROVOKE_FAULT , 0 };
    cmd[4] = aFaultId;
    return sendChipMgtHostToChip ( cmd );
}

bool CBluetestEngine::sendChipMgtHostToChip ( PARAMETER_ARRAY aParameters )
{
    return mChip->sendChipMgtHostToChip ( aParameters );
}

uint16 CBluetestEngine::nextSequence()
{
    //  Generate a unique sequence number.
    ::EnterCriticalSection (&sequenceProtection);
    sequenceNumber ++;
    ::LeaveCriticalSection (&sequenceProtection);
    return sequenceNumber;
}

void CBluetestEngine::discardHCIData ( uint16 connectionHandle , 
                                       bool packetBoundaryFlag , 
                                       uint8 broadcastFlag , 
                                       uint8 *dataStart , 
                                       uint16 dataLength, 
                                       void * aThis)
{
    delete dataStart;
}

void CBluetestEngine::discardHCIEvent ( uint8 eventcode, 
                                        PARAMETER_ARRAY eventParameters ,
                                        void * aThis)
{
    delete eventParameters;
}

void CBluetestEngine::onHQ ( PARAMETER_ARRAY eventParameters , 
                             void * aThis )
{
    ((CBluetestEngine*) aThis)->processHQ ( eventParameters );
}

void CBluetestEngine::processHQ ( PARAMETER_ARRAY eventParameters )
{
    // decode the packet
    uint16 lLength = (uint16)eventParameters[0];

    if (lLength > 3)
    {
        uint16 lVarid = (uint16)eventParameters[1];
        uint16 lSequenceNumber = (uint16)eventParameters[2];
        uint16 lStatus = (uint16)eventParameters[3];

        // decode the data words.
        switch (lVarid)
        {
        case HQVARID_BOOTED:
            report ( true , (uint8)31);
            break;
        case HQVARID_DELAYED_PANIC:
            report ( true , (uint8)32 , eventParameters[4]);
            report ( false , (uint8)32 , eventParameters[4]);
            break;
        case HQVARID_RSSI:
            report ( true , (uint8)33 , eventParameters[4]);
            report ( false , (uint8)33 , eventParameters[4]);
            break;
        case HQVARID_FAULT:
            if ( lLength > 6 )
            {
                uint16 lFaultId = (uint16)eventParameters[4];
                uint32 lTime    = eventParameters[5];
                uint16 lCount   = (uint16)eventParameters[6];
                if (  lFaultId < FAULT_COUNT )
                {
                    report ( true  , (uint8)66 , faultString[lFaultId] , lTime , lCount );
                    report ( false , (uint8)66 , faultString[lFaultId] , lTime , lCount );
                }
                else
                {
                    report ( true , (uint8)34 , lFaultId , lTime , lCount );
                    report ( false , (uint8)34 , lFaultId , lTime , lCount );
                }
            }
            break;
        case HQVARID_RXPKTSTATS:
            report ( true , (uint8)35 , eventParameters[4] ,
                                eventParameters[5] ,
                                eventParameters[6] ,
                                eventParameters[7] ,
                                eventParameters[8] ? _T("true") : _T("false") );
            if ( eventParameters[4] == 0 )
                report ( false , (uint8)54 , _T("RXPKT STATS") );
            break;
        case HQVARID_LUT_ENTRY:
            report ( true , (uint8)36 , eventParameters[4] ,
                                eventParameters[5] );
            //  Wait for the tenth event to print the message
            if ( eventParameters[4] == 10 )
                report ( false , (uint8)54 , _T("LUT") );
            break;
        case HQVARID_SYNTH_SETTLE:
            report ( true , (uint8)37 , eventParameters[4] ,
                                eventParameters[5] );
            if ( eventParameters[4] == 0 )
                report ( false , (uint8)54 , _T("SETTLE") );
            break;
        case HQVARID_FILTER_RESP:
            report ( true , (uint8)38 , eventParameters[4] ,
                                eventParameters[5] ,
                                eventParameters[6] );
            if ( eventParameters[4] == 0 )
                report ( false , (uint8)54 , _T("FILTER RESP") );
            break;
        case HQVARID_IQ_MATCH:
            report ( true , (uint8)39 , eventParameters[4] ,
                                eventParameters[5] ,
                                eventParameters[6] ,
                                eventParameters[7] );
            if ( eventParameters[4] == 0 )
                report ( false , (uint8)54 , _T("IQ MATCH") );
            break;
        case HQVARID_BIT_ERR:
            //  perform some simple calculations for the previous period
            interpretBitErr( eventParameters[4] , eventParameters[5] , eventParameters[6] );
            break;
        default:
            report ( true , (uint8)29 , (uint32) lVarid );
            break;
        }
    }
    delete[] eventParameters;
}

void CBluetestEngine::onBCCMD ( PARAMETER_ARRAY eventParameters , 
                                void * aThis )
{
    ((CBluetestEngine*) aThis)->processBCCMD ( eventParameters );
}

void CBluetestEngine::processBCCMD ( PARAMETER_ARRAY eventParameters )
{
    // decode the packet
    uint8 reportCode = 255;
    uint16 lLength = (uint16)eventParameters[0];

    if (lLength >= 3)
    {
        uint16 lVarid = (uint16)eventParameters[1];
        uint16 lSequenceNumber = (uint16)eventParameters[2];
        uint16 lStatus = (uint16)eventParameters[3];
        uint16 lTest = 0;

        //  decode the parameters
        switch ( lVarid )
        {
        case BCCMDVARID_RADIOTEST:
            SetEvent (testStarted);
            lTest = (uint16)eventParameters[4];
            switch ( lTest )
            {
            /* Acceptable values for "test". */
            case RADIOTEST_PAUSE:
                reportCode = 0 ;
                break;
            case RADIOTEST_TXSTART:
                reportCode = 1 ;
                break;
            case RADIOTEST_RXSTART1:
                reportCode = 2 ;
                break;
            case RADIOTEST_RXSTART2:
                reportCode = 3 ;
                break;
            case RADIOTEST_TXDATA1:
                reportCode = 4 ;
                break;
            case RADIOTEST_TXDATA2:
                reportCode = 5 ;
                break;
            case RADIOTEST_TXDATA3:
                reportCode = 6 ;
                break;
            case RADIOTEST_TXDATA4:
                reportCode = 7 ;
                break;
            case RADIOTEST_RXDATA1:
                reportCode = 8 ;
                break;
            case RADIOTEST_RXDATA2:
                reportCode = 9 ;
                break;

            /* Built-in self tests */

            case RADIOTEST_DEEP_SLEEP:
                reportCode = 10 ;
                break;
            case RADIOTEST_PCM_LB:
                reportCode = 11 ;
                break;
            case RADIOTEST_SETTLE:
                reportCode = 12 ;
                break;
            case RADIOTEST_IF_RESP:
                reportCode = 13 ;
                break;
            case RADIOTEST_VCOTRIM:
                reportCode = 14 ;
                break;
            case RADIOTEST_RF_IQ_MATCH:
                reportCode = 15 ;
                break;
            case RADIOTEST_IF_IQ_MATCH:
                reportCode = 16 ;
                break;
            case RADIOTEST_BUILD_LUT:
                reportCode = 17 ;
                break;
            case RADIOTEST_READ_LUT:
                reportCode = 18 ;
                break;

            /* Bit error rate, configuration and on-air loopback functions */

            case RADIOTEST_BER1:
                reportCode = 19 ;
                //  Initialise the data interpretation variables
                reportLines = 0;
                firstTime = true;
                bitCount = 0;
                bitErrors = 0;
                cumulativeBitCount = 0;
                cumulativeBitErrors = 0;
                cumulativeExpPkts = 0;
                cumulativeRcdPkts = 0;
                cumulativeSyncErr = 0;
                cumulativeHdrErr = 0;
                cumulativeCRCErr = 0;
                cumulativeUncorrErr = 0;
                break;
            case RADIOTEST_BER2:
                reportCode = 20 ;
                //  Initialise the data interpretation variables
                reportLines = 0;
                firstTime = true;
                bitCount = 0;
                bitErrors = 0;
                cumulativeBitCount = 0;
                cumulativeBitErrors = 0;
                cumulativeExpPkts = 0;
                cumulativeRcdPkts = 0;
                cumulativeSyncErr = 0;
                cumulativeHdrErr = 0;
                cumulativeCRCErr = 0;
                cumulativeUncorrErr = 0;
                break;
            case RADIOTEST_LOOP_BACK:
                reportCode = 21 ;
                break;
            case RADIOTEST_CFG_FREQ:
                reportCode = 22 ;
                break;
            case RADIOTEST_CFG_PKT:
                reportCode = 23 ;
                break;
            case RADIOTEST_CFG_BIT_ERR:
                reportCode = 24 ;
                break;
            case RADIOTEST_RX_LOOP_BACK:
                reportCode = 25 ;
                break;
            case RADIOTEST_BER_LOOP_BACK:
                reportCode = 26 ;
                break;
            case RADIOTEST_CONFIG_TX_IF:
                reportCode = 27 ;
                break;
            case RADIOTEST_CFG_UAP_LAP:
                reportCode = 51 ;
                break;
            case RADIOTEST_PCM_EXT_LB:
                reportCode = 52 ;
                break;
            case RADIOTEST_CFG_XTAL_FTRIM:
                reportCode = 53 ;
                break;
            case RADIOTEST_CFG_ACC_ERRS:
                reportCode = 64 ;
                break;
            case RADIOTEST_CFG_IQ_TRIM:
                reportCode = 63 ;
                break;
            case RADIOTEST_CFG_TX_TRIM:
                reportCode = 67 ;
                break;
            default:
                reportCode = 28 ;
                break;
            }
            //  zero is success
            report ( true , reportCode , ( lStatus == BCCMDPDU_STAT_OK ? _T("successful") : _T("failed") ) );
            report ( false , reportCode , ( lStatus == BCCMDPDU_STAT_OK ? _T("successful") : _T("failed") ) );
            break;
        case BCCMDVARID_PIO_MASK:
            SetEvent(pioMask);
            {
                uint8 data = (uint8) eventParameters[4];
                //  spread the data into a uint32, so we ca display it as hex.
                uint32 spread = (data & 1)           // bit 0 stays there
                              + ( (data & 2) << 3 )  // bit 1 to bit 4
                              + ( (data & 4) << 6 )  // bit 2 to bit 8
                              + ( (data & 8) << 9 )  // ...
                              + ( (data & 16) << 12 )
                              + ( (data & 32) << 15 )
                              + ( (data & 64) << 18 )
                              + ( (data & 128) << 21 );  // bit 7 to bit 28.
                report ( true , 49 , spread );
                report ( false , 49 , spread );
            }
            break;
        case BCCMDVARID_PIO:
            SetEvent(pioLines);
            {
                uint8 data = (uint8) eventParameters[4];
                //  spread the data into a uint32, so we ca display it as hex.
                uint32 spread = (data & 1)           // bit 0 stays there
                              + ( (data & 2) << 3 )  // bit 1 to bit 4
                              + ( (data & 4) << 6 )  // bit 2 to bit 8
                              + ( (data & 8) << 9 )  // ...
                              + ( (data & 16) << 12 )
                              + ( (data & 32) << 15 )
                              + ( (data & 64) << 18 )
                              + ( (data & 128) << 21 );  // bit 7 to bit 28.
                report ( true , 50 , spread );
                report ( false , 50 , spread );
            }
            break;
        case BCCMDVARID_PS:
            if ( mPSSetFunction != NULL )
                mPSSetFunction ( eventParameters , mPSSetParameters );
            report ( true , 61 , eventParameters[4] , ( lStatus == BCCMDPDU_STAT_OK ? _T("successful") : _T("failed") ) );
            break;
        case BCCMDVARID_PSSIZE:
            if ( mPSSetFunction != NULL )
                mPSSetFunction ( eventParameters , mPSSetParameters );
            report ( true , (uint8) 62 , ( lStatus  ? _T("failure") : _T("successfully") ) );
            break;
        case BCCMDVARID_PROVOKE_FAULT:
            report ( true , 65 , eventParameters[4] , ( lStatus == BCCMDPDU_STAT_OK ? _T("successfully") : _T("failure") ) );
            break;
        case BCCMDVARID_BUILDID:
            if ( lStatus == BCCMDPDU_STAT_OK )
                report ( true , 68 , eventParameters[4] );
            break;
        default:
            report ( true , 30 , (uint32) lVarid );
            report ( false , 30 , (uint32) lVarid );
            break;
        }
    }
    delete [] eventParameters ;
}

void CBluetestEngine::onLinkFail ( void * aThis )
{
    ((CBluetestEngine*)aThis)->report ( true , (uint8)43 );
}

void CBluetestEngine::onDebugData ( uint8 *aBuffer , uint32 aLength , void *aThis )
{
    CBluetestEngine * lObject = (CBluetestEngine*) aThis;
}

void CBluetestEngine::interpretBitErr ( uint32 type , uint32 periodValue , uint32 cumulativeValue )
{
    //  perform some simple calculations for the previous period
    if ( type == 0 )
    {
        //  assume the previous report has finished.
        if ( bitCount != 0 && cumulativeBitCount != 0 )
        {
            //  generate a verbose report

            double perBitErr = 100 * (double) bitErrors / bitCount;
            double totBitErr = 100 * (double) cumulativeBitErrors / cumulativeBitCount;

            report ( true , (uint8) 55 , perBitErr );
            report ( true , (uint8) 56 , totBitErr );
            report ( true , (uint8) 57 );

            //  and then the concise report.
            if ( firstTime == true || printHeader == true )
            {
                report ( false , (uint8) 58 );
                printHeader = false;
            }

            uint8 reportValue = 0;
            //  if we have just wrapped around
            if ( periodValue == cumulativeValue && !firstTime )
            {
                //  The same as 59 but with a new line at the end.
                reportValue = 60;
                reportLines ++;
            }
            else
            {
                reportValue = 59;
                firstTime = false;
            }

            report ( false , reportValue , totBitErr ,
                                          cumulativeBitCount ,
                                          cumulativeExpPkts ,
                                          cumulativeRcdPkts ,
                                          cumulativeSyncErr ,
                                          cumulativeHdrErr ,
                                          cumulativeCRCErr ,
                                          cumulativeUncorrErr );

            //  after a doxen lines reprint the header.
            if ( reportLines == 12 )
            {
                reportLines = 0;
                printHeader = true;
            }
        }
        else
            report ( false , (uint8)54 , _T("BIT ERR") );

    }

    //  gather the data accross a period
    switch ( type )
    {
    case 0:
        bitCount = periodValue;
        cumulativeBitCount = cumulativeValue;
        break;
    case 1:
        bitErrors = periodValue;
        cumulativeBitErrors = cumulativeValue;
        break;
    case 2:
        //  access code bits
        break;
    case 3:
        cumulativeRcdPkts = cumulativeValue;
        break;
    case 4:
        cumulativeExpPkts = cumulativeValue;
        break;
    case 5:
        cumulativeHdrErr = cumulativeValue;
        break;
    case 6:
        cumulativeCRCErr = cumulativeValue;
        break;
    case 7:
        cumulativeUncorrErr = cumulativeValue;
        break;
    case 8:
        cumulativeSyncErr = cumulativeValue;
        break;
    default:
        //  No data to collect (?)
        break;
    }

    //  generate the verbose report.
    if ( type < 9 )
    {
        report ( true , (uint8)40 , BitErrorReportTypes [ type ] ,
                            periodValue ,
                            cumulativeValue );
    }

    // draw a line under the set of data
    if ( type == 8 )
    {
        report ( true , (uint8)57 );
    }
}

void CBluetestEngine::report ( bool verboseNotConcise , TCHAR * aString )
{
    this->mStringCallBackFn ( verboseNotConcise , aString , this->mStringCallBackParameters );
}

void CBluetestEngine::report ( bool verboseNotConcise , uint8 aStringNumber )
{
    report ( verboseNotConcise , reportStrings[aStringNumber] );
}

void CBluetestEngine::report ( bool verboseNotConcise , uint8 aStringNumber, uint32 p1 )
{
    uint16 length = _tcsclen ( reportStrings[aStringNumber] ) + 10;
    TCHAR * lString = new TCHAR[length];
    _stprintf ( lString , reportStrings[aStringNumber] , p1 );
    report ( verboseNotConcise , lString );
    delete lString;
}

void CBluetestEngine::report ( bool verboseNotConcise , uint8 aStringNumber, uint32 p1 , uint32 p2)
{
    uint16 length = _tcsclen ( reportStrings[aStringNumber] ) + 20;
    TCHAR * lString = new TCHAR[length];
    _stprintf ( lString , reportStrings[aStringNumber] , p1 , p2 );
    report ( verboseNotConcise , lString );
    delete lString;
}

void CBluetestEngine::report ( bool verboseNotConcise , uint8 aStringNumber, uint32 p1 , TCHAR * p2)
{
    uint16 length = _tcsclen ( reportStrings[aStringNumber] ) + 10 + _tcsclen (p2) ;
    TCHAR * lString = new TCHAR[length];
    _stprintf ( lString , reportStrings[aStringNumber] , p1 , p2 );
    report ( verboseNotConcise , lString );
    delete lString;
}

void CBluetestEngine::report ( bool verboseNotConcise , uint8 aStringNumber, uint32 p1 , uint32 p2 , uint32 p3)
{
    uint16 length = _tcsclen ( reportStrings[aStringNumber] ) + 30;
    TCHAR * lString = new TCHAR[length];
    _stprintf ( lString , reportStrings[aStringNumber] , p1 , p2 , p3 );
    report ( verboseNotConcise , lString );
    delete lString;
}

void CBluetestEngine::report ( bool verboseNotConcise , uint8 aStringNumber, TCHAR * p1 , uint32 p2 , uint32 p3)
{
    uint16 length = _tcsclen ( reportStrings[aStringNumber] ) + _tcsclen ( p1 ) + 20;
    TCHAR * lString = new TCHAR[length];
    _stprintf ( lString , reportStrings[aStringNumber] , p1 , p2 , p3 );
    report ( verboseNotConcise , lString );
    delete lString;
}

void CBluetestEngine::report ( bool verboseNotConcise , uint8 aStringNumber, uint32 p1 , uint32 p2 , uint32 p3 , uint32 p4)
{
    uint16 length = _tcsclen ( reportStrings[aStringNumber] ) + 40;
    TCHAR * lString = new TCHAR[length];
    _stprintf ( lString , reportStrings[aStringNumber] , p1 , p2 , p3 , p4 );
    report ( verboseNotConcise , lString );
    delete lString;
}

void CBluetestEngine::report ( bool verboseNotConcise , uint8 aStringNumber, uint32 p1 , uint32 p2 , uint32 p3 , uint32 p4 , uint32 p5)
{
    uint16 length = _tcsclen ( reportStrings[aStringNumber] ) + 50;
    TCHAR * lString = new TCHAR[length];
    _stprintf ( lString , reportStrings[aStringNumber] , p1 , p2 , p3 , p4 , p5 );
    report ( verboseNotConcise , lString );
    delete lString;
}

void CBluetestEngine::report ( bool verboseNotConcise , uint8 aStringNumber, uint32 p1 , uint32 p2 , uint32 p3 , uint32 p4 , TCHAR * p5)
{
    uint16 length = _tcsclen ( reportStrings[aStringNumber] ) + 40 + _tcsclen(p5);
    TCHAR * lString = new TCHAR[length];
    _stprintf ( lString , reportStrings[aStringNumber] , p1 , p2 , p3 , p4 , p5 );
    report ( verboseNotConcise , lString );
    delete lString;
}

void CBluetestEngine::report ( bool verboseNotConcise , uint8 aStringNumber, TCHAR * p1 )
{
    uint16 length = _tcsclen ( reportStrings[aStringNumber] ) + _tcsclen ( p1 );
    TCHAR * lString = new TCHAR[length];
    _stprintf ( lString , reportStrings[aStringNumber] , p1 );
    report ( verboseNotConcise , lString );
    delete lString;
}

void CBluetestEngine::report ( bool verboseNotConcise , uint8 aStringNumber, double p1 )
{
    uint16 length = _tcsclen ( reportStrings[aStringNumber] ) + 10 ;
    TCHAR * lString = new TCHAR[length];
    _stprintf ( lString , reportStrings[aStringNumber] , p1 );
    report ( verboseNotConcise , lString );
    delete lString;
}

void CBluetestEngine::report ( bool verboseNotConcise , uint8 aStringNumber, double p1 , uint32 p2 , uint32 p3 , uint32 p4 , uint32 p5 , uint32 p6 , uint32 p7 , uint32 p8)
{
    uint16 length = _tcsclen ( reportStrings[aStringNumber] ) + 112;
    TCHAR * lString = new TCHAR[length];
    _stprintf ( lString , reportStrings[aStringNumber] , p1 , p2 , p3 , p4 , p5 , p6 , p7 , p8 );
    report ( verboseNotConcise , lString );
    delete lString;
}
