///////////////////////////////////////////////////////////////////////
//
//  FILE   :  PosixH4Transport.cpp
//
//            Copyright CSR 2000
//
//  AUTHOR :  Mark Marshall
//
//  CLASS  :  PosixH4Transport
//
//  PURPOSE:  transparent class inheriting top API from the
//            HCITransport class, and owning a H4 stack which does
//            all the work.
//
//            This Class also provides BCSP-specific private channels
//            L2CAP etc and puts the debug channel messages to a file
//            in the debug build version.
//
//  MODIFICATION HISTORY:
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/Posix/H4/PosixH4Transport.cpp#4 $
//
///////////////////////////////////////////////////////////////////////

#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <termios.h>
#include <stdio.h>
#include <errno.h>
#include <assert.h>

#include "PosixH4Transport.h"
#include "PosixH4.h"

PosixH4TransportBase::PosixH4TransportBase() :
    mStack(0), mComportName(0), mDataRate(0), mComportHandle(-1)
{ }

void PosixH4TransportBase::creationInit(const ichar *comPort /* =0 */)
{
    //  Do the creation initialisation, in a procedure so it can be called
    //  from both starting points.

    if (comPort != 0)
    {
	mComportName = new ichar[istrlen(comPort) + 1];
	istrcpy(mComportName, comPort);
    }
    
    tunnel = true;
    
    mCommandCallBackInfo.channel = hciCommand;
    mACLCallBackInfo.channel = hciACL;
    mSCOCallBackInfo.channel = hciSCO;
    mCommandCallBackInfo.mObject = this;
    mACLCallBackInfo.mObject = this;
    mSCOCallBackInfo.mObject = this;
}

PosixH4TransportBase::~PosixH4TransportBase()
{
    stop();
    delete[] mComportName;
}

startTransportReturnValue PosixH4TransportBase::start( )
{
    startTransportReturnValue ok = OK;

    if (mStack != NULL)
        return otherProblem;

    //set up H4 stack and callbacks on all channels
    mStack = MakeStack();
    mStack->setupRecvCommand ( &mCommandCallBackInfo , onPacketFromChip );
    mStack->setupRecvACLData ( &mACLCallBackInfo , onPacketFromChip );
    mStack->setupRecvSCOData ( &mSCOCallBackInfo , onPacketFromChip );

    H4Stack::startReturnValue inner;

    if (mComportHandle != -1)
	inner = mStack->start(0, 0, (void *)mComportHandle);
    else if (mComportName != 0)
	inner = mStack->start(mComportName, mDataRate);
    else
	return serialProblem;
	
    switch(inner)
    {
    case H4Stack::OK:
	ok = OK;
	mStack->setupOnSyncLoss(onLinkFail, this);
	break;

    case H4Stack::serialProblem:
	ok = serialProblem;
	break;

    case H4Stack::otherProblem:
	ok = otherProblem;
	break;
    }
    
    return ok ;
}

void PosixH4TransportBase::stop()
{
    if (mStack)
    {
        mStack->stop();
        delete mStack;
	mStack = 0;
    }
    // deleting the stack causes cancelled requests on all the channels.
}


bool PosixH4TransportBase::ready(uint32 timeout)

{
    return linkFailed ? false : mStack->ready(timeout);
}

void PosixH4TransportBase::sendpdu(enum BLUECORE_CHANNELS channel,
				   uint8 *apdu, uint32 aLength,
				   CallBack *aCallBack)
{
    if ( !linkFailed )
    {
        switch ( channel )
        {
        case hciCommand:
            assert ( (II("Command longer than 65535") , aLength < 0x10000) );
            mStack->sendCommand ( apdu , (uint16) aLength , aCallBack );
            break;
        case hciACL:
            mStack->sendACLData ( apdu , aLength , aCallBack );
            break;
        case hciSCO:
            assert ( (II("SCO data longer than 65535") , aLength < 0x10000) );
            mStack->sendSCOData ( apdu , (uint16) aLength , aCallBack );
            break;
        default:
            assert ( (II("Channel unavailable in H4"), 0) );
            break;
        }
    }
    else
        aCallBack->diedIncomplete();
}

void
PosixH4TransportBase::processPacketFromChip(enum BLUECORE_CHANNELS channel,
					    uint8 * apdu, uint32 aLength)
{
  recvData(channel, apdu, aLength);
}

/*static*/ void
PosixH4TransportBase::onPacketFromChip(uint8 *apdu,
				       uint32 aLength,
				       void *parameters)
{
    PosixH4TransportBase::CallBackInfo *info =
	(PosixH4TransportBase::CallBackInfo*)parameters;

    info->mObject->processPacketFromChip(info->channel, apdu, aLength);
}

void PosixH4TransportBase::set_sco_bandwidth(int bytes_per_second,
					     void *additional_info,
					     int additional_info_size)
{
}


PosixH4Transport::PosixH4Transport(const ichar *comPort, uint32 aDataRate)
{
    mDataRate = aDataRate;
    mName = II("PosixH4Transport");
    creationInit(comPort);
}

PosixH4Transport::PosixH4Transport(int comPort)
{
    mComportHandle = comPort;
    mName = II("PosixH4Transport");
    creationInit();
}

PosixH4Base *PosixH4Transport::MakeStack()
{
    return new PosixH4;
}
