// MODIFICATION HISTORY
//   1.3  12:feb:02  pws      Added unsigned int arg for compatibility ---
//                            haven't yet implemented the functionality.
//   1.4  13:feb:02  pws      Implement timeouts for waitOnSignal.
//   1.5  21:feb:02  mm01     Fixes to 'waitOnSignal'.  Linux does not set
//                            errno when a pthread function fails, and we need
//                            to check the status in the loop.
//   1.6  27:feb:02  pws      Handle errors in a thread-safer manner.

// We need to define __EXTENSIONS__ to get timeval defined.
// This is required for gettimeofday(), which is the only portable way
// of getting the time with enough resolution for use with
// pthread_cond_timedwait().
#define __EXTENSIONS__ 1
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <errno.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include "signalFunctions.h"

typedef struct SignalBlob
{
	pthread_mutex_t	mutex;
	pthread_cond_t	cond;
	bool		signalled;
};

static void fatal_error(char *string, int err)
#ifdef __GNUC__
__attribute__((noreturn))
#endif
;

static void fatal_error(char *string, int err)
{
    fprintf(stderr, "%s: %s", string, strerror(err));
    exit(1);
}

void * createSignal()
{
	int err;
	SignalBlob *sb = (SignalBlob *) malloc (sizeof (SignalBlob));
	if ((err = pthread_mutex_init (&sb->mutex, NULL)) != 0)
		fatal_error("Failed to init CS", err);
	if ((err = pthread_cond_init (&sb->cond, NULL)) != 0)
		fatal_error("Failed to init condition", err);
	sb->signalled = false;

	return sb;
}

void killSignal ( void * Signal )
{
	SignalBlob *sb = (SignalBlob *) Signal;
	int err;

	if ((err = pthread_cond_destroy (&sb->cond)) != 0)
		fatal_error("Failed to destroy condition", err);
	if ((err = pthread_mutex_destroy (&sb->mutex)) != 0)
		fatal_error("Failed to destroy mutex", err);
	free (sb);
}

void setSignal ( void * Signal )
{
	SignalBlob *sb = (SignalBlob *) Signal;
	int err;

	if ((err = pthread_mutex_lock (&sb->mutex)) != 0)
		fatal_error("Failed to acquire mutex", err);
	sb->signalled = true;
	if ((err = pthread_cond_signal (&sb->cond)) != 0)
		fatal_error("Failed to set condition", err);
	if ((err = pthread_mutex_unlock (&sb->mutex)) != 0)
		fatal_error("Failed to release mutex", err);
}

void unsetSignal ( void * Signal )
{
	SignalBlob *sb = (SignalBlob *) Signal;
	int err;

	if ((err = pthread_mutex_lock (&sb->mutex)) != 0)
		fatal_error("Failed to acquire mutex", err);
	sb->signalled = false;
	if ((err = pthread_mutex_unlock (&sb->mutex)) != 0)
		fatal_error("Failed to release mutex", err);
}

// I am assuming that this should return either S_SIGANLLED if the signal
// was signalled, or S_TIMEDOUT if not (after the number of millisec's)
// If millisec's is zero then we wait for ever.
//
uint8 waitOnSignal ( void * Signal, unsigned int millisec = 0)
{
	SignalBlob *sb = (SignalBlob *) Signal;
	uint8 ret = S_SIGNALLED;
	int err;

	if ((err = pthread_mutex_lock (&sb->mutex)) != 0)
		fatal_error("Failed to acquire mutex", err);
	while (!sb->signalled && ret != S_TIMEOUT)
	{
	    /* 
	     * The interface here is a bit of a mess.  Posix threads
	     * use an absolute time for the wait, given by a
	     * struct timespec.  The obvious way of getting a clock time
	     * is then via clock_gettime(), which uses the same struct.
	     * However, older systems don't implement that, so
	     * we use the widely implemented gettimeofday(), which
	     * unfortunately uses a struct timeval.  This function used to
	     * be only an X/OPEN extension to Posix, but is now in
	     * IEEE Std 1003.1-2001, and as it's been around in UNIX
	     * for donkey's years we ought to be able to rely on it.
	     */
	    if (millisec)
	    {
		struct timeval day_time;
		struct timespec event_time;
		if (gettimeofday(&day_time, NULL))
		{
		    perror("Failed to get real time clock");
		    exit(1);
		}
		event_time.tv_nsec = day_time.tv_usec * 1000L +
		    (millisec % 1000) * 1000000L;
		event_time.tv_sec = day_time.tv_sec + millisec / 1000;
		if (event_time.tv_nsec > 1000000000L)
		{
		    event_time.tv_nsec -= 1000000000L;
		    event_time.tv_sec++;
		}
		if ((err = pthread_cond_timedwait(&sb->cond, &sb->mutex,
						  &event_time)) != 0)
		{
		    if (err == ETIMEDOUT)
			ret = S_TIMEOUT;
		    else
			fatal_error("Failed to wait for condition with timeout"
				, err);
		}
	    }
	    else
	    {
		if ((err = pthread_cond_wait (&sb->cond, &sb->mutex)) != 0)
			fatal_error("Failed to wait for condition", err);
	    }
	}
	if ((err = pthread_mutex_unlock (&sb->mutex)) != 0)
		fatal_error("Failed to release mutex", err);

	return ret;
}
