///////////////////////////////////////////////////////////////////////
//
//  FILE   :  Win32BCSPTransport.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  Win32BCSPTransport
//
//  PURPOSE:  transparent class inheriting top API from the
//            HCITransport class, and owning a BCSPStack which does
//            all the work.
//
//            This Class also provides BCSP-specific private channels
//            L2CAP etc and puts the debug channel messages to a file
//            in the debug build version.
//
//            Start-up sequence:
//            new Win32BCSPTransport;
//            setupCallBacks (HCITransport functions);
//            start stack;
//            
//            If the stack is started with out the callbacks in place,
//            then data arriving in the intervening time will be lost.
//            Since this is likely to be the HCI initialisation packet,
//            that would be bad...
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/Win32/BCSP/Win32BCSPTransport.cpp#50 $
//
//  MODIFICATION HISTORY:
//  1.12  7:Sep:00  at  Added modification history.
//                      Removed the inappropriate duplication of
//                      BlueCoreTransport::recvData().
//  1.22  26:Oct:00 at  Added setLinkEstablishment() method to enable
//                      link establishment to be disabled.
//  1.23  27:Oct:00 at  Added methods to modify the BCSP stack
//                      configuration.
//  1.26  16:Nov:00 at  Added ability to open and close COM port
//                      externally.
//  1.27  16:Nov:00 at  Corrected support for external opening of	
//                      COM port.
//  1.28  23:Nov:00 ajh corrected deletion of void*s.
//  1.30  07:Dec:00 ajh changed SCO channel to be unreliable.
//  1.31  12:Dec:00 at  Removed parameter from transport's start
//                      method.
//  1.35  12:Mar:01 at  Removed commented out free for mBCSPStack
//                      - the memory is released within Win32BCSP.
//	1.36  28:Mar:01 ckl Added stdafx.h
//	1.36  14:Mar:01	dm	Windows CE additions
//	1.36  06:Jul:01	CKL	Callback checked for.
//	1.45  21:Mar:02	CKL	Added check for OS version for comm port name changes.
///////////////////////////////////////////////////////////////////////


#include "HCIDeviceController.h"
#ifndef _WINCE
#include "stdafx.h"
#include "cassert"
#else
#include <windows.h>
#include <windef.h>
#include <winbase.h>
#include <dbgapi.h>
#define assert ASSERT
#endif
#include "Win32BCSPTransport.h"
#include <stdio.h>
#include <stdlib.h>
#include "memfuncs.h"

Win32BCSPTransport::Win32BCSPTransport( ichar* comPort , uint32 aDataRate )
{
    creationInit ();
    comportPrep ( comPort , aDataRate , 0 );
}

Win32BCSPTransport::Win32BCSPTransport( ichar* comPort , ichar * configString )
{
    creationInit ();
    comportPrep ( comPort , 0 , configString );
}

Win32BCSPTransport::Win32BCSPTransport( HANDLE comPort, uint32 baud)
{
    creationInit ();
	mComportHandle = comPort;
	if (baud)
		comportPrep(NULL, baud, NULL);
}

void Win32BCSPTransport::creationInit()
{
    //  Do the creation initialisation, in a procedure so it can be called
    //  from all starting points.
    mStack = NULL;
    mBCSPStack = 0;
    mComportName = NULL;
    mConfigString = NULL;

    BTRACEDISABLE ( 0xFFFFFFFF );
    mComportHandle = INVALID_HANDLE_VALUE;
    tunnel = false;
    mBufferSize = 4096;
    mName = II("Win32BCSPTransport");
    mUserOnLinkFailFn = NULL;
	mStackCfg = *::getDefaultStackConfiguration();
}

void Win32BCSPTransport::comportPrep  ( ichar* comPort , uint32 aDataRate , ichar * configString )
{
    if ( comPort != NULL )
    {
		OSVERSIONINFO osvi;
		osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
		GetVersionEx (&osvi);
		if (istrncmp(comPort, II("\\\\.\\"), 4) == 0)
			comPort += 4;
		// Strip off the leading "\\.\" for Win98.
		switch (osvi.dwPlatformId)
		{

		case VER_PLATFORM_WIN32_WINDOWS:
		default:
			mComportName = new ichar[istrlen(comPort) + 1];
			istrcpy ( mComportName , comPort );
		    break;
		case VER_PLATFORM_WIN32_NT:
			mComportName = new ichar[istrlen(comPort) + 5];
			istrcpy ( mComportName, II("\\\\.\\"));
			istrcpy ( mComportName + istrlen(mComportName) , comPort );
			break;
		}

    }
    if ( aDataRate != 0 )
    {
        mConfigString = new ichar[60];
        isprintf (mConfigString , II("baud=%d parity=e data=8 stop=1 dtr=on rts=on") , aDataRate );
    }
    else if ( configString != NULL )
    {
        int length = istrlen(configString) + 1;
        mConfigString = new ichar[length];
        istrcpy ( mConfigString , configString );
    }
}

Win32BCSPTransport::~Win32BCSPTransport()
{
    stop();
    delete[] mComportName;
    delete[] mConfigString;
}

startTransportReturnValue Win32BCSPTransport::start()
{
    startTransportReturnValue ok = OK;
    if (mStack != NULL)
        ok = otherProblem;
    else
    {
        //set up bcsp stack and callbacks on all channels
        mStack = new Win32BCSP;
	mBCSPStack = mStack->getBCSPStack() ;

        if ( (mComportName == NULL) && (mComportHandle == INVALID_HANDLE_VALUE) )
            ok = serialProblem;
        else
        {    
            if ( mComportHandle == INVALID_HANDLE_VALUE
				 ? !mStack->open( mComportName , mConfigString )
				 : !mStack->open( mComportHandle) )
            {
				ok = serialProblem ;
			}
			else
            {
                mStack->setOnLinkFail(onLinkFail,(void*) this) ;
		::setStackConfiguration(mBCSPStack, &mStackCfg) ;
                mStack->start() ;

                //  Set up all the channels to receive.
                for (uint8 i = 0 ; i < BLUECORE_CHANNEL_COUNT ; i ++ )
                {
                    buffers[i] = new uint8[mBufferSize];
                    if ( i == hciSCO )
                        BCSPopenChannel ( &channels[i] , mBCSPStack , i , BCSPChannelUnreliable , BCSPCallbackDeferred ) ;
                    else
                        BCSPopenChannel ( &channels[i] , mBCSPStack , i , BCSPChannelReliable , BCSPCallbackDeferred ) ;
                    CallBackInfo *info = new CallBackInfo;
                    info->object = this;
                    info->channelNumber = (BLUECORE_CHANNELS)i;
                    BCinitTransferRequest( &recvReqs[i], buffers[i], mBufferSize, onPacketFromChip , info );
                    BCSPreadPacket( &channels[i] , &recvReqs[i] ) ;
                }
            }
        }
		if (ok == OK && mComportHandle != INVALID_HANDLE_VALUE && mConfigString != NULL)
			ConfigurePort(mConfigString);				 

    }
    return ok ;
}

void Win32BCSPTransport::stop()
{
    if (mStack)
		delete mStack;
	mStack = 0 ;
    // deleting the stack causes cancelled requests on all the channels.
}

bool Win32BCSPTransport::ready ( uint32 timeout )
{
    //  if the link has failed return false
    return ( linkFailed ? false : mStack->connect( timeout ) );
}

void Win32BCSPTransport::getStackConfiguration(StackConfiguration &cfg)
{
	cfg = mStackCfg;
}

void Win32BCSPTransport::setStackConfiguration(const StackConfiguration &cfg)
{
	mStackCfg = cfg;
}

uint16 Win32BCSPTransport::getBufferSize()
{
    return mBufferSize;
}

void Win32BCSPTransport::resetStack()
{
    stop();

    start () ;
}


void Win32BCSPTransport::sendpdu ( BLUECORE_CHANNELS channel, uint8 * apdu , uint32 aLength, CallBack *aCallBack )
{
    if (!linkFailed)
    {
        assert ( (II("pdu bigger than uint16") , aLength < 65536) );
        BCTransferRequest *req = new BCTransferRequest;
        CallBackInfo *info = new CallBackInfo;
        info->callBack = aCallBack;
        info->channelNumber = channel;
        info->object = this;
        BCinitTransferRequest( req , apdu , (uint16)aLength , onPacketToChip , info );
        BCSPwritePacket ( &channels[channel], req );
    }
    else
		if (aCallBack)
			aCallBack->diedIncomplete();
}

void Win32BCSPTransport::onPacketToChip ( BCTransferRequest * req, BCTransferStatus status, void * info )
{
    CallBackInfo* lInfo = (CallBackInfo*)info;
    CallBack * lCallBack = lInfo->callBack;
	if ( lCallBack != NULL)
	{
	    if (status != transferCancelled)
	    {
			lCallBack->execute();
		}
	    else
	    {
			lCallBack->diedIncomplete();
		}
	}

	delete lCallBack;
    delete req;
    delete lInfo;
}

void Win32BCSPTransport::onPacketFromChip (BCTransferRequest *req , BCTransferStatus status, void *info)
{
    CallBackInfo* lInfo = (CallBackInfo*)info;
    switch (status)
    {
    case transferCancelled:
        // delete the buffer
        delete[] req->data;
        delete lInfo;
        break;
    case transferBufferTooSmall:
        assert ( 0 );
        break;
    default:
        lInfo->object->processDataFromChip(req,lInfo->channelNumber);
        break;
    }
    return;
}

void Win32BCSPTransport::processDataFromChip ( BCTransferRequest * req, BLUECORE_CHANNELS ch )
{
    //  collect the data.
    //  Take a copy to pass up to the next level, so as not to expose our request->data buffer.
    uint8* buffer = (uint8*)getpdu(req->length);
    memcpy ( buffer , req->data , req->length );

    recvData (ch, buffer, req->length);

    //  reset the transfer request to the right size.
    BCreloadTransferRequest( req , req->data , mBufferSize) ;
    BCSPreadPacket( &channels[ch] , req );
}

void Win32BCSPTransport::set_sco_bandwidth ( int bytes_per_second, void *additional_info, int additional_info_size )
{
}

bool Win32BCSPTransport::ConfigurePort(ichar * ConfigString)
{
	return mStack->ConfigurePort(ConfigString);
}
