///////////////////////////////////////////////////////////////////////
//
//  FILE   :  Win32H4Transport.cpp
//
//            Copyright CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  Win32H4Transport
//
//  PURPOSE:  transparent class inheriting top API from the
//            HCITransport class, and owning a H4 stack which does
//            all the work.
//
//            This Class also provides BCSP-specific private channels
//            L2CAP etc and puts the debug channel messages to a file
//            in the debug build version.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/Win32/H4/Win32H4Transport.cpp#20 $
//
//  MODIFICATION HISTORY:
//  1.5   7:Sep:00  at  Added modification history.
//                      Added intercept of send callback to delete the
//                      pdu buffer for consistency with the BCSP
//                      transport.
//  1.6   16:Oct:00 at  Restored previous callback behaviour - the
//                      memory deallocation problem was in
//                      BlueCoreTransport.
//  1.7   16:Nov:00 at  Added ability to open and close COM port
//                      externally.
//	1.8	   3:Mar:01	dm  Added Windows CE functionality
//	1.9   28:Mar:01 ckl Added stdafx.h
//
///////////////////////////////////////////////////////////////////////

#ifndef _WINCE
#include "stdafx.h"
#include "assert.h"
#else
#include <windef.h>
#include <winbase.h>
#include <dbgapi.h>
#endif
#include "Win32H4Transport.h"

Win32H4Transport::Win32H4Transport( ichar* comPort , uint32 aDataRate )
{
    creationInit ();
    comportPrep ( comPort , aDataRate );
}

Win32H4Transport::Win32H4Transport( HANDLE comPort)
{
    creationInit ();
	mComportHandle = comPort;
}

void Win32H4Transport::creationInit()
{
    //  Do the creation initialisation, in a procedure so it can be called
    //  from both starting points.
    mStack = NULL;
    mComportName = NULL;

	mComportHandle = INVALID_HANDLE_VALUE;
    mDataRate = 0;
    tunnel = true;
    mName = II("Win32H4Transport");

    mCommandCallBackInfo.channel = hciCommand;
    mACLCallBackInfo.channel = hciACL;
    mSCOCallBackInfo.channel = hciSCO;
    mCommandCallBackInfo.mObject = this;
    mACLCallBackInfo.mObject = this;
    mSCOCallBackInfo.mObject = this;
}

void Win32H4Transport::comportPrep  ( ichar* comPort , uint32 aDataRate )
{
    if ( comPort != NULL )
    {
		OSVERSIONINFO osvi;
		osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
		GetVersionEx (&osvi);
		if (istrncmp(comPort, II("\\\\.\\"), 4) == 0)
			comPort += 4;
		// Leave off the leading "\\.\" for Win98.
		switch (osvi.dwPlatformId)
		{

		case VER_PLATFORM_WIN32_WINDOWS:
			mComportName = new ichar[istrlen(comPort) + 1];
			istrcpy ( mComportName , comPort );
		    break;
		case VER_PLATFORM_WIN32_NT:
		// Add the leading "\\.\" for NT / 2K
		default:
			mComportName = new ichar[istrlen(comPort) + 5];
			istrcpy ( mComportName, II("\\\\.\\"));
			istrcpy ( mComportName + istrlen(mComportName) , comPort );
			break;
		}

    }
    mDataRate = aDataRate;
}

Win32H4Transport::~Win32H4Transport()
{
    stop();
    delete[] mComportName;
}

startTransportReturnValue Win32H4Transport::start( )
{
    startTransportReturnValue ok = OK;
    if (mStack != NULL)
        return otherProblem;
    else
    {
        //set up H4 stack and callbacks on all channels
        mStack = new Win32H4;
        mStack->setupRecvCommand ( &mCommandCallBackInfo , onPacketFromChip );
        mStack->setupRecvACLData ( &mACLCallBackInfo , onPacketFromChip );
        mStack->setupRecvSCOData ( &mSCOCallBackInfo , onPacketFromChip );

	H4Stack::startReturnValue inner;
	if (mComportHandle == INVALID_HANDLE_VALUE)
	{
	    if ( mComportName == NULL )
	        return ( serialProblem );
	    inner = mStack->start ( mComportName , mDataRate );
	}
	else
            inner = mStack->start ( 0, 0, mComportHandle );
		
	switch(inner)
	{
	case H4Stack::OK:
            ok = OK;
            mStack->setupOnSyncLoss ( onLinkFail , this);
            break;
	case H4Stack::serialProblem:
            ok = serialProblem;
            break;
	case H4Stack::otherProblem:
            ok = otherProblem;
            break;
	}
    }
    return ok ;
}

void Win32H4Transport::stop()
{
    if (mStack)
    {
        mStack->stop();
        delete mStack;
	mStack = 0 ;
    }
    // deleting the stack causes cancelled requests on all the channels.
}


bool Win32H4Transport::ready ( uint32 timeout )

{
    return ( linkFailed ? false : mStack->ready( timeout ) );
}

void Win32H4Transport::sendpdu ( enum BLUECORE_CHANNELS channel , uint8 * apdu , uint32 aLength , CallBack * aCallBack )
{
    if ( !linkFailed )
    {
        switch ( channel )
        {
        case hciCommand:
            assert ( (II("Command longer than 65535") , aLength < 0x10000) );
            mStack->sendCommand ( apdu , (uint16) aLength , aCallBack );
            break;
        case hciACL:
            mStack->sendACLData ( apdu , aLength , aCallBack );
            break;
        case hciSCO:
            assert ( (II("SCO data longer than 65535") , aLength < 0x10000) );
            mStack->sendSCOData ( apdu , (uint16) aLength , aCallBack );
            break;
        default:
            assert ( (II("Channel unavailable in H4"), 0) );
            break;
        }
    }
    else
        aCallBack->diedIncomplete();
}

void Win32H4Transport::onPacketFromChip ( uint8 * apdu , uint32 aLength , void * parameters )
{
    CallBackInfoH4 * info = (CallBackInfoH4*) parameters ;

    info->mObject->processPacketFromChip ( info->channel , apdu , aLength );
}

void Win32H4Transport::processPacketFromChip ( enum BLUECORE_CHANNELS channel , uint8 * apdu , uint32 aLength )
{
    recvData ( channel , apdu , aLength );
}

void Win32H4Transport::set_sco_bandwidth ( int bytes_per_second, void *additional_info, int additional_info_size )
{
}

