///////////////////////////////////////////////////////////////////////
//
//  FILE   :  Win32USBTransport.cpp
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  Win32USBTransport
//
//  PURPOSE:  transparent class inheriting top API from the
//            HCITransport class, and owning a BCSPStack which does
//            all the work.
//
//            This Class also provides BCSP-specific private channels
//            L2CAP etc and puts the debug channel messages to a file
//            in the debug build version.
//
//            Start-up sequence:
//            new Win32USBTransport;
//            setupCallBacks (HCITransport functions);
//            start stack;
//            
//            If the stack is started with out the callbacks in place,
//            then data arriving in the intervening time will be lost.
//            Since this is likely to be the HCI initialisation packet,
//            that would be bad...
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/Win32/USB/Win32USBTransport.cpp#20 $
//
//  MODIFICATION HISTORY:
//  1.12  7:Sep:00  at  Added modification history.
//                      Removed the inappropriate duplication of
//                      BlueCoreTransport::recvData().
//	1.7   28:Mar:01 ckl Added stdafx.h
//	1.8	   3:Mar:01	dm  Added Windows CE functionality
//	1.18   4:Nov:02 at  Added ability to use an existing handle.
//
///////////////////////////////////////////////////////////////////////

#include "HCIDeviceController.h"

#ifndef _WINCE
#include "cassert"
#include "stdafx.h"
#else
#include <windef.h>
#include <winbase.h>
#include <dbgapi.h>
#endif
#include "Win32USBTransport.h"
#include "csrbc01_usb.h"

Win32USBTransport::Win32USBTransport( ichar* deviceName , uint32 aDataRate )
{
    creationInit ( deviceName , aDataRate , NULL );
}

Win32USBTransport::Win32USBTransport( ichar* deviceName , ichar * configString )
{
    creationInit ( deviceName , 0 , configString );
}

Win32USBTransport::Win32USBTransport( HANDLE deviceHandle , uint32 aDataRate )
{
    creationInit ( NULL , aDataRate , NULL );
	mDeviceHandle = deviceHandle;
}

void Win32USBTransport::creationInit( ichar* deviceName , uint32 aDataRate , ichar * configString )
{
    //  Do the creation initialisation, in a procedure so it can be called
    //  from all starting points.
    mStack = NULL;
    mDeviceName = NULL;
    mConfigString = NULL;

    linkFailed = false;
    mName = II("Win32USBTransport");
    mUserOnLinkFailFn = NULL;
    mDeviceHandle = INVALID_HANDLE_VALUE;
    tunnel = true;

    if ( deviceName != NULL )
    {
        mDeviceName = new ichar[istrlen(deviceName) + 1];
        istrcpy ( mDeviceName , deviceName );
    }
}

Win32USBTransport::~Win32USBTransport()
{
    stop();
    delete[] mDeviceName;
    delete[] mConfigString;
}

startTransportReturnValue Win32USBTransport::start()
{
    startTransportReturnValue ok = OK;
    if (mStack != false)
        ok = otherProblem;
    else
    {
        //set up usb stack and callbacks on all channels
        mStack = new USBStack();

        if ( ( mDeviceName == NULL ) && (mDeviceHandle == INVALID_HANDLE_VALUE) )
            ok = serialProblem;
        else
        {
            mStack->set_data_callback ( onDataPacketFromChip , this );
            mStack->set_event_callback ( onEventPacketFromChip , this );
            mStack->set_sco_callback ( onScoPacketFromChip , this );
			bool opened = true;
			if (mDeviceHandle != INVALID_HANDLE_VALUE)
				opened = mStack->open_connection( mDeviceHandle );
			else
			{
				int open_attempts = 0;
				do
				{
					if (!opened)
						Sleep(1000);
					opened = mStack->open_connection( mDeviceName );
					open_attempts ++;
				} 	while (!opened && open_attempts <= 3);
			}

            if ( !opened )
            {
                ok = serialProblem;
                delete mStack;
                mStack = 0;
            }
        }
    }
    return ok ;
}

void Win32USBTransport::stop()
{
    if (mStack) 
    {
        delete mStack;
    }
    mStack = 0 ;
    // deleting the stack causes cancelled requests on all the channels.
}

// Are we ready yet
// 
//	if we are not open yet - then try and open us again :-)
//  otherwise its all down to linkFailed, and mStack.

bool Win32USBTransport::ready ( uint32 timeout )
{
    if (!mStack) start();
    return !linkFailed && mStack && mStack->is_open ();
}

void Win32USBTransport::onLinkFail(void * aObject)
{
    ((Win32USBTransport*)aObject)->linkFailed = true;
    //  gibber and panic.
    if ( ((Win32USBTransport*)aObject)->mUserOnLinkFailFn != NULL )
        ((Win32USBTransport*)aObject)->mUserOnLinkFailFn (((Win32USBTransport*)aObject)->mUserOnLinkFailParams);
    return;
}

void Win32USBTransport::sendpdu ( BLUECORE_CHANNELS channel, uint8 * apdu , uint32 aLength, CallBack *aCallBack )
{
    if (!linkFailed)
    {
        assert ( (II("pdu bigger than uint16") , aLength < 65536) );
        switch ( channel )
        {
        case hciCommand:
            mStack->send_hci_command( apdu , aLength );
            break;
        case hciACL:
            mStack->send_acl_data( apdu , aLength );
            break;
        case hciSCO:
            mStack->send_sco_data( apdu , aLength );
            break;
        default:
            assert ((II("bad channel"),0));
            break;
        }
        aCallBack->execute();
        // freepdu (apdu);
    }
    else
	{
        aCallBack->diedIncomplete ();
	}
	delete aCallBack;
}

void Win32USBTransport::set_sco_bandwidth ( int bytes_per_second, void *additional_info, int additional_info_size)
{
	mStack->set_sco_bandwidth(bytes_per_second, additional_info, additional_info_size);
}

void Win32USBTransport::onDataPacketFromChip ( void * aData , unsigned long aLength, void *info)
{
    ((Win32USBTransport *) info)->processDataFromChip ( hciACL , aData , aLength );
}

void Win32USBTransport::onEventPacketFromChip ( void * aData , unsigned long aLength, void *info)
{
    ((Win32USBTransport *) info)->processDataFromChip ( hciCommand , aData , aLength );
}

void Win32USBTransport::onScoPacketFromChip ( void * aData , unsigned long aLength, void *info)
{
    ((Win32USBTransport *) info)->processDataFromChip ( hciSCO , aData , aLength );
}

void Win32USBTransport::processDataFromChip ( enum BLUECORE_CHANNELS channel , void * aData , unsigned long aLength )
{
    //  collect the data.
    //  Take a copy to pass up to the next level, so as not to expose our request->data buffer.
    uint8* buffer = new uint8[aLength];
    memcpy ( buffer , aData , aLength );
    recvData (channel, buffer, aLength);
}


