///////////////////////////////////////////////////////////////////////
//
//  FILE   :  Win32USBWidTransport.cpp
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  Win32USBWidTransport
//
//  PURPOSE:  transparent class inheriting top API from the
//            HCITransport class, and owning a BCSPStack which does
//            all the work.
//
//            This Class also provides BCSP-specific private channels
//            L2CAP etc and puts the debug channel messages to a file
//            in the debug build version.
//
//            Start-up sequence:
//            new Win32USBWidTransport;
//            setupCallBacks (HCITransport functions);
//            start stack;
//            
//            If the stack is started with out the callbacks in place,
//            then data arriving in the intervening time will be lost.
//            Since this is likely to be the HCI initialisation packet,
//            that would be bad...
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/Win32/USB/Win32USBWidTransport.cpp#11 $
//
//  MODIFICATION HISTORY:
//  1.12  7:Sep:00  at  Added modification history.
//                      Removed the inappropriate duplication of
//                      BlueCoreTransport::recvData().
//	1.7   28:Mar:01 ckl Added stdafx.h
//	1.8	   3:Mar:01	dm  Added Windows CE functionality
//  1.8    4:Nov:02 at  Added ability to use an existing handle.
//  1.9    6:Nov:02 at  Device name needed even if existing handle.
//
///////////////////////////////////////////////////////////////////////

#include "HCIDeviceController.h"

#ifndef _WINCE
#include "cassert"
#include "stdafx.h"
#else
#include <windef.h>
#include <winbase.h>
#include <dbgapi.h>
#endif
#include "Win32USBWidTransport.h"
#include "wid_usb.h"

Win32USBWidTransport::Win32USBWidTransport( ichar* deviceName , uint32 aDataRate )
{
    creationInit ( deviceName , aDataRate , NULL );
}

Win32USBWidTransport::Win32USBWidTransport( ichar* deviceName , ichar * configString )
{
    creationInit ( deviceName , 0 , configString );
}

Win32USBWidTransport::Win32USBWidTransport( ichar* deviceName , HANDLE deviceHandle )
{
    creationInit ( deviceName , 0 , NULL );
	mDeviceHandle = deviceHandle;
}

void Win32USBWidTransport::creationInit( ichar* deviceName , uint32 aDataRate , ichar * configString )
{
    //  Do the creation initialisation, in a procedure so it can be called
    //  from both starting points.
    mStack = NULL;
    mDeviceName = NULL;
    mConfigString = NULL;

    linkFailed = false;
    mName = II("Win32USBWidTransport");
    mUserOnLinkFailFn = NULL;
    mDeviceHandle = INVALID_HANDLE_VALUE;
    tunnel = true;

    if ( deviceName != NULL )
    {
        mDeviceName = new ichar[istrlen(deviceName) + 1];
        istrcpy ( mDeviceName , deviceName );
    }
}

Win32USBWidTransport::~Win32USBWidTransport()
{
    stop();
    delete[] mDeviceName;
    delete[] mConfigString;
}

startTransportReturnValue Win32USBWidTransport::start()
{
    startTransportReturnValue ok = OK;
    if (mStack != false)
        ok = otherProblem;
    else
    {
        //set up usb stack and callbacks on all channels
        mStack = new USBWidStack();

        if ( ( mDeviceName == NULL ) && (mDeviceHandle == INVALID_HANDLE_VALUE) )
            ok = serialProblem;
        else
        {
            mStack->set_acl_callback ( onDataPacketFromChip , this );
            mStack->set_evt_callback ( onEventPacketFromChip , this );
            if ( !mStack->open_connection( mDeviceName, mDeviceHandle ) )
            {
                ok = serialProblem;
                delete mStack;
                mStack = 0;
            }
        }
    }
    return ok ;
}

void Win32USBWidTransport::stop()
{
    if (mStack) 
    {
        delete mStack;
    }
	mStack = 0 ;
    // deleting the stack causes cancelled requests on all the channels.
}

// Are we ready yet
// 
//	if we are not open yet - then try and open us again :-)
//  otherwise its all down to linkFailed, and mStack.

bool Win32USBWidTransport::ready ( uint32 timeout )
{
    return !linkFailed && mStack && mStack->is_open ();
}

void Win32USBWidTransport::onLinkFail(void * aObject)
{
    ((Win32USBWidTransport*)aObject)->linkFailed = true;
    //  gibber and panic.
    if ( ((Win32USBWidTransport*)aObject)->mUserOnLinkFailFn != NULL )
        ((Win32USBWidTransport*)aObject)->mUserOnLinkFailFn (((Win32USBWidTransport*)aObject)->mUserOnLinkFailParams);
    return;
}

void Win32USBWidTransport::sendpdu ( BLUECORE_CHANNELS channel, uint8 * apdu , uint32 aLength, CallBack *aCallBack )
{
    if (!linkFailed)
    {
        assert ( (II("pdu bigger than uint16") , aLength < 65536) );
        switch ( channel )
        {
        case hciCommand:
            mStack->send_cmd( apdu , aLength );
            break;
        case hciACL:
            mStack->send_acl( apdu , aLength );
            break;
        case hciSCO:
            mStack->send_sco( apdu , aLength );
            break;
        default:
            assert ((II("bad channel"),0));
            break;
        }
        aCallBack->execute();
        // freepdu (apdu);
    }
    else
	{
        aCallBack->diedIncomplete ();
	}
	delete aCallBack;
}

void Win32USBWidTransport::onDataPacketFromChip ( void * aData , unsigned long aLength, void *info)
{
    ((Win32USBWidTransport *) info)->processDataFromChip ( hciACL , aData , aLength );
}

void Win32USBWidTransport::onEventPacketFromChip ( void * aData , unsigned long aLength, void *info)
{
    ((Win32USBWidTransport *) info)->processDataFromChip ( hciCommand , aData , aLength );
}

void Win32USBWidTransport::processDataFromChip ( enum BLUECORE_CHANNELS channel , void * aData , unsigned long aLength )
{
    //  collect the data.
    //  Take a copy to pass up to the next level, so as not to expose our request->data buffer.
    uint8* buffer = new uint8[aLength];
    memcpy ( buffer , aData , aLength );
    recvData (channel, buffer, aLength);
}

void Win32USBWidTransport::set_sco_bandwidth ( int bytes_per_second, void *additional_info, int additional_info_size )
{
}


