// Module to handle USB transport.
// Copyright (c) 2000-2001 Cambridge Silicon Radio.  All Rights Reserved.


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

#include <windows.h>
#include <winioctl.h>
#include <stdio.h>
#include <assert.h>
#include "csrbc01_usb.h"

// dumps out all sco data sent to the device driver to a file (excluding sco headers)
//#define DUMP_SCO_WRITES_TO_FILE

// 
//#define REMOVE_ASSERTS

// instead of using the samples passed to us, generate our own samples
// (FOR TESTING PURPOSES)
//#define GENERATE_SAMPLES

// defines what type of sample to generate if GENERATE_SAMPLES is defined
#define SAMPLERATE_8BIT
//#define SAMPLERATE_16BIT

#ifdef DUMP_SCO_WRITES_TO_FILE
HANDLE sco_handle;
#endif

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

USBStack::USBStack ()
{
	handle = INVALID_HANDLE_VALUE;
	monitor_events_thread_handle = INVALID_HANDLE_VALUE;
	monitor_data_thread_handle = INVALID_HANDLE_VALUE;
	monitor_sco_thread_handle = INVALID_HANDLE_VALUE;
	wait_for_event = CreateEvent (0, 0, 0, 0);
	wait_for_data = CreateEvent (0, 0, 0, 0);
	wait_for_acl = CreateEvent (0, 0, 0, 0);
	wait_for_command = CreateEvent (0, 0, 0, 0);
	wait_for_recv_sco = CreateEvent (0, 0, 0, 0);
	wait_for_send_sco = CreateEvent (0, 0, 0, 0);
	wait_for_start_sco =  CreateEvent (0, 0, 0, 0);
	kill_threads_event = CreateEvent (0, 0, 0, 0);
	InitializeCriticalSection(&close_handle_crit);

	event_paused = 0;
	data_paused = 0;

	// > event size + interrupt data payload size
	event_buffer = new char[256 + 16];

	// > ACL data size + bulk data payload size
	data_buffer = new char[65536 + 64];
	// 
	sco_buffer = new char[65536];

	event_func = 0;
	data_func = 0;
	sco_func = 0;

	event_func_context = 0;
	data_func_context = 0;
	sco_func_context = 0;

	sco_packet_size = 0;
	monitor_sco_enabled = FALSE;

#ifdef DUMP_SCO_WRITES_TO_FILE
	sco_handle = CreateFile(II("sco.raw"),
							GENERIC_WRITE,
							0,
							0,
							CREATE_ALWAYS,
							FILE_ATTRIBUTE_NORMAL | FILE_FLAG_WRITE_THROUGH,
							0);
#endif
}

///////////////////////////////////////////////////////////////////////////////
///////////////////77////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

USBStack::~USBStack ()
{
    close_connection();

	delete event_buffer;
	delete data_buffer;
	delete sco_buffer;

	CloseHandle (wait_for_event);
	CloseHandle (wait_for_data);

	CloseHandle (wait_for_acl);
	CloseHandle (wait_for_command);
	CloseHandle (kill_threads_event);
	CloseHandle (wait_for_send_sco);
	CloseHandle (wait_for_recv_sco);
	CloseHandle (wait_for_start_sco);


	DeleteCriticalSection(&close_handle_crit);

}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

void USBStack::set_sco_bandwidth(int bytes_per_second, void *additional_info, int additional_info_size)
{
	if(bytes_per_second == 0)
	{
		set_alternate_setting(0, additional_info, additional_info_size);
		return;
	}

	if(bytes_per_second <= ((((9*3)-3)/3)*1000))
	{
		set_alternate_setting(1, additional_info, additional_info_size);
		return;
	}

	if(bytes_per_second <= ((((17*3)-3)/3)*1000))
	{
		set_alternate_setting(2, additional_info, additional_info_size);
		return;
	}
	
	if(bytes_per_second <= ((((25*3)-3)/3)*1000))
	{
		set_alternate_setting(3, additional_info, additional_info_size);
		return;
	}
	
	if(bytes_per_second <= ((((33*3)-3)/3)*1000))
	{
		set_alternate_setting(4, additional_info, additional_info_size);
		return;
	}

	if(bytes_per_second <= ((((49*3)-3)/3)*1000))
	{
		set_alternate_setting(5, additional_info, additional_info_size);
		return;
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// valid num values:
//	- 0 = No active voice channels (for USB compliance)
//	- 1 = One voice channel with 8-bit encoding
//	- 2 = Two voice channels with 8-bit encoding & One voice channel with 16-bit encoding
//	- 3 = Three voice channels with 8-bit encoding
//	- 4 = Two voice channels with 16-bit encoding
//	- 5 = Three voice channels with 16-bit encoding

void USBStack::set_alternate_setting(int num, void *additional_info, int additional_info_size)
{
	static OVERLAPPED overlapped = { 0, 0, 0, 0, 0 };
	unsigned long written;
	unsigned char *buffer;
	int status = 0;

#ifndef REMOVE_ASSERTS
	assert ( handle != INVALID_HANDLE_VALUE );
#endif

	// validate alternate setting number
	if((num < 0) || (num > 5))
	{
		return;
	}
	
	// only start monitoring the sco pipe if sco is enabled and we have the bandwidth
	if((monitor_sco_enabled == FALSE) && (num != 0))
	{
		monitor_sco ();

		buffer = (unsigned char *) malloc(1 + additional_info_size);
		if(buffer == NULL)
		{
#ifndef REMOVE_ASSERTS
			assert(1);
#endif
			return;
		}

		// set this to maximum.
		*buffer = 5;//num;
		memcpy(buffer+1, additional_info, additional_info_size);

		Sleep(1000);

		overlapped.hEvent = wait_for_start_sco;

		status = DeviceIoControl(handle,
								 IOCTL_CSRBC01_START_SCO_DATA,
								 buffer,
								 1+additional_info_size,
								 0,			    
								 0,
								 &written,
								 &overlapped);

		if(!status)
		{
			iprintf(II("Failed to issue START_SCO_DATA IoControlCode\n"));
		}

		free(buffer);
/*
		if((!status) && ((err = GetLastError () == ERROR_IO_PENDING))
		{
			status = GetOverlappedResult(handle, &overlapped, &written, TRUE);
		}
*/
	}
#ifdef DEBUG
	iprintf(II("Set Alternate Setting %d : %08x\n"), num, status);
#endif

	switch(num)
	{
	case 0: sco_packet_size = 0;
			break;
	case 1:	sco_packet_size = 9;
			break;
	case 2:	sco_packet_size = 17;
			break;
	case 3:	sco_packet_size = 25;
			break;
	case 4:	sco_packet_size = 33;
			break;
	case 5:	sco_packet_size = 49;
			break;
	default:
#ifndef REMOVE_ASSERTS
			assert(1);
#endif
			break;
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

bool USBStack::open_connection (ichar *device_name)
{
#ifndef _WINCE
	handle = CreateFile
	(
		device_name,
		GENERIC_READ | GENERIC_WRITE,
		0,
		0,
		OPEN_EXISTING,
		FILE_FLAG_OVERLAPPED,
		0
	);
#else
	handle = INVALID_HANDLE_VALUE;
#endif
	if (handle == INVALID_HANDLE_VALUE)
	{
		return false;
	}

//	set_alternate_setting(2);
//	set_alternate_setting(1);
	monitor_events ();
	monitor_data ();
	return true;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

bool USBStack::open_connection (HANDLE device_handle)
{
#ifndef _WINCE
	if (device_handle == INVALID_HANDLE_VALUE)
	{
		return false;
	}
	HANDLE process = GetCurrentProcess();
	if (!DuplicateHandle(process, device_handle, process, &handle,
		                 GENERIC_READ | GENERIC_WRITE, false, 0))
	{
		return false;
	}
#else
	handle = INVALID_HANDLE_VALUE;
#endif
	if (handle == INVALID_HANDLE_VALUE)
	{
		return false;
	}

//	set_alternate_setting(2);
//	set_alternate_setting(1);
	monitor_events ();
	monitor_data ();
	return true;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

bool USBStack::is_open (void)
{
	return (handle != INVALID_HANDLE_VALUE);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

void USBStack::close_connection (void)
{

	close_connection_on_error();
	if (monitor_events_thread_handle != INVALID_HANDLE_VALUE) 
	{
  	  if (WaitForSingleObject(monitor_events_thread_handle, 300) == WAIT_TIMEOUT)
	    TerminateThread (monitor_events_thread_handle, 0);
	  CloseHandle(monitor_events_thread_handle);
	  monitor_events_thread_handle = INVALID_HANDLE_VALUE;
	}

    if (monitor_data_thread_handle != INVALID_HANDLE_VALUE) 
	{
  	  if (WaitForSingleObject(monitor_data_thread_handle, 300)  == WAIT_TIMEOUT)
	    TerminateThread (monitor_data_thread_handle, 0);
	  CloseHandle(monitor_data_thread_handle);
 	  monitor_data_thread_handle = INVALID_HANDLE_VALUE;
	}

    if (monitor_sco_thread_handle != INVALID_HANDLE_VALUE) 
	{
  	  if (WaitForSingleObject(monitor_sco_thread_handle, 300)  == WAIT_TIMEOUT)
	    TerminateThread (monitor_sco_thread_handle, 0);
	  CloseHandle(monitor_sco_thread_handle);
 	  monitor_sco_thread_handle = INVALID_HANDLE_VALUE;
	}

}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

void USBStack::close_handle(void)
{
	EnterCriticalSection(&close_handle_crit);
	if (handle != INVALID_HANDLE_VALUE)
	{
		CloseHandle (handle);
	}
	handle = INVALID_HANDLE_VALUE;
	LeaveCriticalSection(&close_handle_crit);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

void USBStack::close_connection_on_error (void)
{
#ifdef DEBUG_REPORT_ERRORS
	char * lpMsgBuf;
	DWORD err = GetLastError();
	if (err != 0)
	{
		FormatMessageA( 
			FORMAT_MESSAGE_ALLOCATE_BUFFER | 
			FORMAT_MESSAGE_FROM_SYSTEM | 
			FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL,
			err,
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
			(char *)&lpMsgBuf,
			0,
			NULL 
		);
		// Display the string.
		MessageBoxA( NULL, lpMsgBuf, "Error", MB_OK | MB_ICONINFORMATION );
		// Free the buffer.
		LocalFree( lpMsgBuf );
	}
#endif
	
	if ((monitor_events_thread_handle != INVALID_HANDLE_VALUE) 
		||  (monitor_data_thread_handle != INVALID_HANDLE_VALUE)
		||  (monitor_sco_thread_handle != INVALID_HANDLE_VALUE))
	{
	    monitor_events_thread_terminate = true;
	    monitor_data_thread_terminate = true;
		monitor_sco_thread_terminate = true;
		SetEvent(kill_threads_event);
	}
	Sleep(0);
	close_handle();
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

int USBStack::send_hci_command (void *buffer, unsigned long length)
{
	int
		status;

	unsigned long
		written;

	static OVERLAPPED
		overlapped = { 0, 0, 0, 0, 0 };
	HANDLE handles[2] = {wait_for_command, kill_threads_event};
	DWORD err;

	if (handle != INVALID_HANDLE_VALUE)
	{
		overlapped.hEvent = wait_for_command;

		status = DeviceIoControl
		(
			handle,
			IOCTL_CSRBC01_SEND_HCI_COMMAND,
			buffer,
			length,
			0,
			0,
			&written,
			&overlapped
		);

		if ((!status) && ((err = GetLastError ()) == ERROR_IO_PENDING))
		{
			if (WaitForMultipleObjects(2, handles, false, -1) == WAIT_OBJECT_0 + 1)
				return 0;			
			// WaitForSingleObject (wait_for_command, -1);
			written = 0;
		}
		else if (!status)
		{
			close_connection_on_error  ();
		}
#ifndef _WINCE
		status = GetOverlappedResult (handle, &overlapped, &written, FALSE);
#endif
		if (status == 0)
		{
			written = 0;
		}
	}
	else
	{
		written = 0;
	}

	return written;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

int USBStack::send_acl_data (void *data, unsigned long length)
{
	unsigned int
		status;

	unsigned long
		written;

	static OVERLAPPED
		overlapped = { 0, 0, 0, 0, 0 };
	HANDLE handles[2] = {wait_for_acl, kill_threads_event};
	DWORD err;

	if (handle != INVALID_HANDLE_VALUE)
	{
		overlapped.hEvent = wait_for_acl;

		status = WriteFile (handle, data, length, &written, &overlapped);

		if ((!status) && ((err = GetLastError ()) == ERROR_IO_PENDING))
		{
			if (WaitForMultipleObjects(2, handles, false, -1) == WAIT_OBJECT_0 + 1)
				return 0;			

			written = 0;
		}
		else if (!status)
		{
			close_connection_on_error  ();
		}
#ifndef _WINCE
		status = GetOverlappedResult (handle, &overlapped, &written, FALSE);
#endif
		if (status == 0)
		{
			written = 0;
		}
	}
	else
	{
		written = 0;
	}

	return written;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

int USBStack::get_hci_event (void *buffer, int length)
{
	int
		status;

	unsigned long
		written;

	static OVERLAPPED
		overlapped = { 0, 0, 0, 0, 0 };
	HANDLE handles[2] = {wait_for_event, kill_threads_event};
	DWORD err;

	if (handle != INVALID_HANDLE_VALUE)
	{
		if (event_paused > 0)
		{
			event_paused = 1;

			overlapped.hEvent = wait_for_event;

			status = DeviceIoControl
			(
				handle,
				IOCTL_CSRBC01_BLOCK_HCI_EVENT,
				0,
				0,
				0,
				0,
 				&written,
				&overlapped
			);

			if ((!status) && ((err = GetLastError ()) == ERROR_IO_PENDING))
			{
				// WaitForSingleObject (wait_for_event, -1);

				if (WaitForMultipleObjects(2, handles, false, -1) == WAIT_OBJECT_0 + 1)
					return 0;			
				written = 0;
			}
			else if (!status)
			{
				close_connection_on_error  ();
			}
		}

		overlapped.hEvent = wait_for_event;

		status = DeviceIoControl
		(
			handle,
			IOCTL_CSRBC01_GET_HCI_EVENT,
			0,
			0,
			buffer,
			length,
			&written,
			&overlapped
		);

		if ((!status) && ((err = GetLastError ()) == ERROR_IO_PENDING))
		{
			// WaitForSingleObject (wait_for_event, -1);
			if (WaitForMultipleObjects(2, handles, false, -1) == WAIT_OBJECT_0 + 1)
				return 0;			
#ifndef _WINCE
			status = GetOverlappedResult (handle, &overlapped, &written, FALSE);
#endif
		}
		else if (!status)
		{
			close_connection_on_error  ();
		}

		if (status == 0)
		{
			written = 0;
		}
	}
	else
	{
		written = 0;
	}

	if (written == 0)
	{
		event_paused ++;
	}
	else
	{
		event_paused = 0;
	}

	return written;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

DWORD WINAPI USBStack::monitor_events_thread (LPVOID arg)
{
	USBStack
		*stack;

	int
		offset,
		length,
		size;

	stack = (USBStack *) arg;
	stack->monitor_events_thread_terminate = false;

	// ''offset'' is the number of bytes in the buffer (always)
	// ''length'' is the length of the next event
	// ''size''   is the length of the last read from the driver

	size = 0;

	while (!stack->monitor_events_thread_terminate)
	{
		offset = size;

		// If we only have a small amount of data in the buffer, then
		// we need to read in another chunk before we know the length
		while (offset < 2 && !stack->monitor_events_thread_terminate)
		{
			size = stack->get_hci_event (stack->event_buffer + offset, 16);
			offset += size;
		}

		// We now know the length of the current packet
		length = (unsigned char) stack->event_buffer[1] + 2;

		// Read in the entire packett (plus perhaps the start of the next one)
		while (offset < length && !stack->monitor_events_thread_terminate)
		{
			size = stack->get_hci_event (stack->event_buffer + offset, 16);
			offset += size;
		}

		// Send the packet up
		if (stack->event_func && !stack->monitor_events_thread_terminate)
		{
			stack->event_func (stack->event_buffer, length, stack->event_func_context);
		}

		// If there is more data in the buffer that we have not read,
		// then copy it down, update the remaining size, and loop
		if(offset > length)
		{
			memmove(stack->event_buffer, stack->event_buffer + length, offset - length);
		}
		size = offset - length;
	}

	return 0;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

void USBStack::monitor_events (void)
{
	DWORD
		thread_id;
	// ASSERT(monitor_events_thread_handle == INVALID_HANDLE_VALUE);
	// This should perhaps close down the thread if one already exists
	monitor_events_thread_handle = CreateThread (0, 0, monitor_events_thread, this, 0, &thread_id);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

int USBStack::get_hci_data (void *buffer, int length)
{
	int
		status;

	unsigned long
		written;

	static OVERLAPPED
		overlapped = { 0, 0, 0, 0, 0 };
	HANDLE handles[2] = {wait_for_data, kill_threads_event};
	DWORD err;

	if (handle != INVALID_HANDLE_VALUE)
	{
		if (data_paused > 0)
		{
			data_paused = 1;

			overlapped.hEvent = wait_for_data;

			status = DeviceIoControl
			(
				handle,
				IOCTL_CSRBC01_BLOCK_HCI_DATA,
				0,
				0,
				0,
				0,
				&written,
				&overlapped
			);

			if ((!status) && ((err = GetLastError ()) == ERROR_IO_PENDING))
			{
				// WaitForSingleObject (wait_for_data, -1);
				if (WaitForMultipleObjects(2, handles, false, -1) == WAIT_OBJECT_0 + 1)
  					return 0;
			}
			else if (!status)
			{
				close_connection_on_error  ();
			}
		}

		overlapped.hEvent = wait_for_data;

		status = ReadFile (handle, buffer, length, &written, &overlapped);

		if ((!status) && ((err = GetLastError ()) == ERROR_IO_PENDING))
		{
			// WaitForSingleObject (wait_for_data, -1);
			if (WaitForMultipleObjects(2, handles, false, -1) == WAIT_OBJECT_0 + 1)
				return 0;			
#ifndef _WINCE
			status = GetOverlappedResult (handle, &overlapped, &written, FALSE);
#endif
		}
		else if (!status)
		{
			close_connection_on_error  ();
		}

		if (status == 0)
		{
			written = 0;
		}
	}
	else
	{
		written = 0;
	}

	if (written == 0)
	{
		data_paused ++;
	}
	else
	{
		data_paused = 0;
	}

	return written;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

DWORD WINAPI USBStack::monitor_data_thread (LPVOID arg)
{
	USBStack
		*stack;

	int
		offset,
		length,
		size;

	stack = (USBStack *) arg;

	// ''offset'' is the number of bytes in the buffer (always)
	// ''length'' is the length of the next event
	// ''size''   is the length of the last read from the driver

	size = 0;
	stack->monitor_data_thread_terminate = false;

	while (!stack->monitor_data_thread_terminate)
	{
		offset = size;		

		// If we have read less than 4 bytes then the length is not
		// yet valid.  It is a good idea to read this much before we
		// try to use the length
		while (offset < 4 && !stack->monitor_data_thread_terminate)
			{
			size = stack->get_hci_data (stack->data_buffer + offset, 64);
			offset += size;
		}

		// Get the length of the next block (+4)
		length = (unsigned char) stack->data_buffer[2] + ((unsigned short) stack->data_buffer[3] << 8) + 4;

		// Now we read in the body of the data, we keep going until
		// we have read at least ''length'' bytes.
		while (offset < length && !stack->monitor_data_thread_terminate)
		{
			size = stack->get_hci_data (stack->data_buffer + offset, 64);
			offset += size;
		}

		// send the data up the stack
		if (stack->data_func && !stack->monitor_data_thread_terminate)
		{
			stack->data_func (stack->data_buffer, length, stack->data_func_context);
		}

		// If there is more data in the buffer that we have not read,
		// then copy it down and loop
		if(offset > length)
		{
			memmove(stack->data_buffer, stack->data_buffer + length, offset - length);
		}

		// Set the size of what is left in the buffer
		size = offset - length;
	}

	return 0;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

void USBStack::monitor_data (void)
{
	DWORD
		thread_id;

	// assert(monitor_data_thread_handle == INVALID_HANDLE_VALUE);
	// This should perhaps close down the thread if one already exists
	monitor_data_thread_handle = CreateThread (0, 0, USBStack::monitor_data_thread, this, 0, &thread_id);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

void USBStack::set_event_callback (void (*func)(void *, unsigned long, void *), void *context)
{
	event_func = func;
	event_func_context = context;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

void USBStack::set_data_callback (void (*func)(void *, unsigned long, void *), void *context)
{
	data_func = func;
	data_func_context = context;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

void USBStack::set_sco_callback (void (*func)(void *, unsigned long, void *), void *context)
{
	sco_func = func;
	sco_func_context = context;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

//#define SCO_CACHE_BUFFER_SIZE		( 4096 )
#define SCO_REFORMAT_BUFFER_SIZE	( 4096 )
#define SCO_WRITEIOCTL_THRESHOLD	1
//unsigned char out_buffer[SCO_CACHE_BUFFER_SIZE];
//int out_buffer_offset = 0;
unsigned char fragment_buffer[SCO_REFORMAT_BUFFER_SIZE];
int fragment_buffer_offset = 0;
int fragment_buffer_good_offset = 0;
int fragmentation = 0;
unsigned char sample = 0;
#include <math.h>
unsigned char GenerateSample()
{
	static unsigned int offset = 0;
	static int direction = 0;
	static int byte = 0;
	unsigned char samplebyte;
//	unsigned int x;
//	float x;
#define PI 3.1415927

#ifdef SAMPLWRATE_16BIT
	if(byte == 0)
	{
	if(direction == 0)
	{
		offset++;
		if(offset == 12)
		{
			direction = 1;
		}
	}
	else
	{
		offset--;
		if(offset == 0)
		{
			direction = 0;
		}
	}
	}
	x = offset * 2000;
#endif

#ifdef SAMPLERATE_8BIT
	if(direction == 0)
	{
		offset++;
		if(offset == 12)
		{
			direction = 1;
		}
	}
	else
	{
		offset--;
		if(offset == 0)
		{
			direction = 0;
		}
	}
	samplebyte = offset * 20;
#endif
	//x = ((float) offset / 0xffff) * 2.00 * PI;

	//x = sin(x);

//	x = ((float)offset) * (500*2*PI/8000);
//	int amp = (int)((float)(1<<15) * sin(x));
#ifdef SAMPLERATE_16BIT
	if(byte == 0)
	{
		samplebyte = (((unsigned int)(x * 0xffff)) >> 8) & 0xff;
//		samplebyte = (((unsigned int)amp) >> 8) & 0xff;
		byte = 1;
	}
	else
	{
		samplebyte = ((unsigned int)(x * 0xffff)) & 0xff;
//		samplebyte = ((unsigned int)amp)  & 0xff;

		byte = 0;
//		offset++;
	}
#endif	

	return(samplebyte);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

int USBStack::send_sco_data(void *buffer, unsigned long length)
{
	int	status;
	unsigned long written = 0;
	static OVERLAPPED overlapped = { 0, 0, 0, 0, 0};
	HANDLE handles[2] = {wait_for_command, kill_threads_event};
	// DWORD err;
 
	// do nothing if an alternate interface has not been selected yet
	if(sco_packet_size == 0)
	{
		return 0;
	}

	// if we don't have a valid handle, do nothing
	if(handle != INVALID_HANDLE_VALUE)
	{
		// not interest in the 3 byte header as we generate our own
		int temp_length = length-3;
		unsigned char *temp_buffer = (unsigned char *) buffer;
		// copy over original handle lsb
		unsigned char byte0 = *(((unsigned char *)buffer)+0);
		// copy over original handle msb
		unsigned char byte1 = *((unsigned char *)buffer+1);
		// we send 3 sco packets at a time, -3 due to our generated header
		unsigned char byte2 = (sco_packet_size*3)-3;
		
		// skip the sco header as we have just generated our own
		temp_buffer+=3;

		// reformat packets to suit the usb sco packet size
		// this needs to be done on a per handle basis
		while(temp_length > 0)
		{
#ifndef REMOVE_ASSERTS
			assert((fragment_buffer_offset+(byte2-fragmentation)+3) < SCO_REFORMAT_BUFFER_SIZE);
#endif
			// make sure we don't overflow the buffer, clip if we are going to
			if((fragment_buffer_offset+(byte2-fragmentation)+3) > SCO_REFORMAT_BUFFER_SIZE)
			{
				// need to add somthing here...
				break;
			}

			// create a sco header if we are at the start of a new packet
			if(fragmentation == 0)
			{
				fragment_buffer[fragment_buffer_offset++] = byte0;
				fragment_buffer[fragment_buffer_offset++] = byte1;
				fragment_buffer[fragment_buffer_offset++] = byte2;
			}

			// if the incoming length is greather than the primitive we are generating then truncate it
			if(temp_length >= (byte2 - fragmentation))
			{
				for(int x=0;x < byte2 - fragmentation;x++)
				{
#ifdef GENERATE_SAMPLES
					fragment_buffer[fragment_buffer_offset+x] = GenerateSample();
#else
					fragment_buffer[fragment_buffer_offset+x] = *(temp_buffer+x);
#endif
				}
//				memmove(&fragment_buffer[fragment_buffer_offset], temp_buffer, byte2 - fragmentation);
				fragment_buffer_offset+= (byte2 - fragmentation);
				temp_length-= (byte2 - fragmentation);
				temp_buffer+= (byte2 - fragmentation);
				fragmentation = 0;
				// indicate we have a complete packet
				fragment_buffer_good_offset = fragment_buffer_offset;
			}
			else
			{
				for(int x=0;x < temp_length;x++)
				{
#ifdef GENERATE_SAMPLES
					fragment_buffer[fragment_buffer_offset+x] = GenerateSample();
#else
					fragment_buffer[fragment_buffer_offset+x] = *(temp_buffer+x);
#endif
				}
//				memmove(&fragment_buffer[fragment_buffer_offset], temp_buffer, temp_length);
				fragment_buffer_offset+= temp_length;
				// fragmentation is the number of bytes in the partially complete packet
				fragmentation+= temp_length;
				// all done
				temp_length = 0;
			}
		}
		
		// if we did not get a complete reformated packet, do nothing
		if(fragment_buffer_good_offset < ((sco_packet_size*3)*SCO_WRITEIOCTL_THRESHOLD))
		{
			return 0;
		}

		/*		
		// buffer the reformated sco packets to limit the number of ioctl requests
		if((out_buffer_offset + fragment_buffer_good_offset) < SCO_WRITEIOCTL_THRESHOLD)
		{
			memmove(&out_buffer[out_buffer_offset], fragment_buffer, fragment_buffer_good_offset);
			// update the buffer offset
			out_buffer_offset+= fragment_buffer_good_offset;
			written = length;

			// copy the fragmented buffer to the start of the buffer
			if(fragment_buffer_good_offset != fragment_buffer_offset)
			{
				memmove(&fragment_buffer[0], &fragment_buffer[fragment_buffer_good_offset], fragment_buffer_offset - fragment_buffer_good_offset);
			}

			// reset the offset values to take the copy/shift into consideration
			fragment_buffer_offset -= fragment_buffer_good_offset;
			fragment_buffer_good_offset = 0;
		}
		else
		{
			// if the cached data is not very big, send the fragment buffer data as well
			if(out_buffer_offset < SCO_WRITEIOCTL_THRESHOLD)
			{
				memmove(&out_buffer[out_buffer_offset], fragment_buffer, fragment_buffer_good_offset);
				// update the buffer offset
				out_buffer_offset+= fragment_buffer_good_offset;
				written = length;

				// copy the fragmented buffer to the start of the buffer
				if(fragment_buffer_good_offset != fragment_buffer_offset)
				{
					memmove(&fragment_buffer[0], &fragment_buffer[fragment_buffer_good_offset], fragment_buffer_offset - fragment_buffer_good_offset);
				}

				// reset the offset values to take the copy/shift into consideration
				fragment_buffer_offset -= fragment_buffer_good_offset;
				fragment_buffer_good_offset = 0;
			}
*/			
//			for(int x=0; x < fragment_buffer_good_offset; x+=sco_packet_size)
#ifdef DUMP_SCO_WRITES_TO_FILE
			WriteFile(sco_handle, fragment_buffer+3, fragment_buffer_good_offset-3, &written, 0);
#endif
			{
			// issue an ioctl to transfer the data to our usb device
			overlapped.hEvent = wait_for_send_sco;
			// method = METHOD_BUFFERED
			status = DeviceIoControl(// hDevice 
									 handle,				
									 // dwIoControlCode
									 IOCTL_CSRBC01_SEND_SCO_DATA,
									 // lpInBuffer
									 fragment_buffer,
									 // nInBufferSize
									 fragment_buffer_good_offset,
									 // lpOutBuffer
									 0,
									 // nOutBufferSize
									 0,
									 // lpBytesReturned
									 &written,
									 // lpOverlapped
									 &overlapped);

			// flush out sent packets, leaving paritally complete packets at start of buffer 
			if(fragment_buffer_good_offset != fragment_buffer_offset)
			{
				memmove(fragment_buffer, &fragment_buffer[fragment_buffer_good_offset], fragment_buffer_offset - fragment_buffer_good_offset);
			}
			// update offsets to represent flush
			fragment_buffer_offset-= fragment_buffer_good_offset;
			fragment_buffer_good_offset = 0;
/*
			// its most likely the overlapped i/o operation is in progress
			if((!status) && ((err = GetLastError() == ERROR_IO_PENDING))
			{
				// wait for the event to complete in that case
				if(WaitForMultipleObjects(2, handles, false, -1) == WAIT_OBJECT_0 + 1)
				{
					return 0;	 
				}
				written = 0;
			}
			else if(!status)
			{
				close_connection_on_error();
			}
#ifndef _WINCE
			status = GetOverlappedResult(handle, &overlapped, &written, FALSE);
#endif
			if(status == 0)
			{
				written = 0;
			}
*/  
	}
//		}
	}
	else
	{
		written = 0;
	}

	return written;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

int USBStack::get_sco_data (void *buffer, unsigned long length)
{
	int	status;
	unsigned long written = 0;
	static OVERLAPPED overlapped = { 0, 0, 0, 0, 0 };
	DWORD err;

	if(handle != INVALID_HANDLE_VALUE)
	{
		overlapped.hEvent = wait_for_recv_sco;

		// method = METHOD_BUFFERED
		status = DeviceIoControl(// hDevice
								 handle,
								 // dwIoControlCode
								 IOCTL_CSRBC01_RECV_SCO_DATA,
								 // lpInBuffer
								 0,
								 // nInBufferSize
								 0,
								 // lpOutBuffer
								 buffer,
								 // nOutBufferSize
								 length,
								 // lpBytesReturned
								 &written,
								 // lpOverlapped
								 &overlapped);

		// its possible the request has not completed yet so check if it is in progress
		if ((!status) && ((err = GetLastError ()) == ERROR_IO_PENDING))
		{
			// yes, it's in progress
			WaitForSingleObject (wait_for_recv_sco, -1);
			// so wait for it to complete
#ifndef _WINCE
			status = GetOverlappedResult (handle, &overlapped, &written, TRUE);
#endif
		}
		else if (!status)
		{
			iprintf(II("*** BAD STATUS!\n"));
			close_connection_on_error();
			written=0;
		}
	}
	else
	{
		iprintf(II("invalid handle!\n"));
		// we don't have a valid handle so nothing is written to the provided buffer
		written=0;
	}

	return written;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

DWORD WINAPI USBStack::monitor_sco_thread (LPVOID arg)
{
	USBStack *stack = (USBStack *) arg;
	int	offset;
	int	length;
	int index;
	bool alldone;
	int	size = 0;

	stack->monitor_sco_thread_terminate = false;

	// ''offset'' is the number of bytes in the buffer (always)
	// ''length'' is the length of the next event
	// ''size''   is the length of the last read from the driver

	while (!stack->monitor_sco_thread_terminate)
	{
		Sleep(1);

		// do nothing unless a calling app is interested in the data and an alternate interface
		// has been selected
		if(stack->sco_func == 0 || (stack->sco_packet_size == 0))
		{
			Sleep(0);
			continue;
		}
		
		// start position for writting data to the buffer
		offset = size;

		// keep requesting data until we have at least got a sco header
		while ((offset < 3) && !stack->monitor_sco_thread_terminate)
		{
			size = stack->get_sco_data (stack->sco_buffer + offset, 65536-offset);
			offset += size;
			if(size == 0)
			{
				Sleep(100);
			}
		}

		// We now know the length of the current packet and it's header
		length = ((unsigned char) *(stack->sco_buffer+2))+3;
#ifdef ASSERT_SCO_LENGTHS
		assert(length == (stack->sco_packet_size*3));
		// if the data is invalid, just ignore it and try again
#endif		
		if(length != stack->sco_packet_size*3)
		{
			offset = 0;
			continue;
		}

		// keep requesting data until we have got at least one complete sco primitive
		while ((offset < length) && !stack->monitor_sco_thread_terminate)
		{
			size = stack->get_sco_data (stack->sco_buffer + offset, 65536-offset);
			offset += size;
		}

		// process the primitive plus any other primitives we may have been given
		if (stack->sco_func && !stack->monitor_sco_thread_terminate)
		{
			// go through the data we have been given looking for complete packets
			for(index = 0, alldone = FALSE; (index < offset) && (alldone == FALSE);)
			{
				// send the complete packet up the stack
				stack->sco_func (stack->sco_buffer+index, length, stack->sco_func_context);

				// index to the start of the next protential packet
				index+=length;

				// are there enough bytes in the buffer to contain the next sco header?
				if(((index+3) < offset) &&
				   // and are there enough bytes to contain the next sco payload?
				   ((index+(*(stack->sco_buffer+index+2))+3) <= offset))
				{
					// yes, so get the length of the payload and header
					length = (*(stack->sco_buffer+index+2))+3;
#ifdef ASSERT_SCO_LENGTHS
					assert(length == (stack->sco_packet_size*3));
#endif
				}
				else
				{
					// no more packets so exit out of loop
					alldone = TRUE;
				}
			}
		}

		// if there is a paritally complete packet remaining, copy it to the buffer start
		if(index < offset)
		{
			memmove(stack->sco_buffer, stack->sco_buffer + index, offset - index);
		}

		// update the size of valid data in the buffer
		size = offset - index;
	}

	return 0;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

void USBStack::monitor_sco (void)
{
	DWORD thread_id;
	monitor_sco_enabled	= TRUE;
	monitor_sco_thread_handle = CreateThread (0, 0, USBStack::monitor_sco_thread, this, 0, &thread_id);
}

