///////////////////////////////////////////////////////////////////////
//
//  FILE   :  ACLHandler.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  ACLHandler
//
//  PURPOSE:  handle the throughput of ACL data.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/ACLHandler.cpp#40 $
//
///////////////////////////////////////////////////////////////////////

#include "HCIDeviceController.h"
#include "ACLHandler.h"
#include "HCIPacker.h"
#include "connection.h"
#include "memfuncs.h"
#include "thread/critical_section.h"

#ifndef _WINCE
#include <assert.h>
#else
#include <windef.h>
#include <winbase.h>
#include <dbgapi.h>
#define assert ASSERT
#endif

#include <string.h>

ACLHandler::ACLHandler() : protectTotalDataInBuffer(new CriticalSection())
{
    initialised = false;
	ActiveBroadcastHandle = 0;
	PiconetBroadcastHandle = 0;
    mTotalDataInBuffer = 0;
    nextConnection = 0;
}

ACLHandler::~ACLHandler()
{
	delete PiconetBroadcastHandle;
	delete ActiveBroadcastHandle;
	delete protectTotalDataInBuffer;
}

void ACLHandler::initialise ( ConnectionList * aConnList )
{
    pConnectionList = aConnList;
    //  Initialise the fixed piconet and active broadcast handles.
    //  This'll change as we fully implement the spec.
    ActiveBroadcastHandle = new Connection ();
    ActiveBroadcastHandle->initialise ( ACTIVE_BROADCAST_HANDLE , conn_type_acl , pConnectionList , NULL );
    PiconetBroadcastHandle = new Connection ();
    PiconetBroadcastHandle->initialise ( PICONET_BROADCAST_HANDLE , conn_type_acl , pConnectionList , NULL );
}

bool ACLHandler::queue (uint16 aHandle, uint8 *aData , uint32 aLength , bool aPB , uint8 aBT)
{
    bool ok = true;
    Connection * lConnection = pConnectionList->getConnection ( aHandle );
    if ( lConnection != 0 )
    {
        {
           //  keep track of how much data in the buffer
           CriticalSection::Lock here (protectTotalDataInBuffer);

            //  Add the data to the queue.  This machanism automatically breaks
	    //  breaks the data into bitesize chunks.
	    lConnection->add ( aData , aLength , aPB , aBT );
	    //  attempting to move data per connection: mOutBoundData.add ( aHandle , aData , aLength , aPB , aBT );
	    mTotalDataInBuffer += aLength;
	}
        //  keep track of how much data on the connection.
        //  if the "next connection" is null, set it to this first channel
        if ( nextConnection == 0 )
        {
            pConnectionList->setNextConnectionWithACL ( lConnection );
            nextConnection = lConnection;
        }
        //  set the flag to say "I've got some data!"
        mSentSignal->set();
    }
    else
        ok = false;

    return ok;
}

size_t ACLHandler::flush ( uint16 aHandle )
{
    Connection * lConnection = pConnectionList->getConnection ( aHandle );
    return flush(lConnection);
}

size_t ACLHandler::flush ( Connection * aConn )
{
    uint32 threwOut = 0;
    if ( aConn )
    {
        //  keep track of how much data in the buffer
        CriticalSection::Lock here (protectTotalDataInBuffer);
        threwOut = aConn->flushData();
        mTotalDataInBuffer -= threwOut;
    }
    return threwOut;
}

void ACLHandler::initialiseFlowControl( uint16 maxPackets , uint16 packetSize )
{
    if (!initialised)
    {
        initialised = true;
        mNumberOfBuffers = maxPackets;
        mFlowControl.setFree(maxPackets);
    }
    mSizeOfBuffer = packetSize;
    //  moved per connection: mOutBoundData.initialise( packetSize , acl );
}

void ACLHandler::recvData ( uint8 * apdu , uint32 aLength , void * aThis )
{
    ACLHandler * lObject = (ACLHandler*)aThis;

    Datapdu * data = new Datapdu;
    data->connectionHandle = 0;
    data->mBuffer = apdu;
    data->mLength = aLength;

    lObject->onPduReceipt (data);
}

uint16 ACLHandler::getIncomingData( uint16 * aHandle , uint8 ** aDataPtr , uint8 * aPB , uint8 * aBT )
{
#ifdef _WINCE
    Datapdu * lpdu = (Datapdu *)NULL;
#else
    Datapdu * lpdu = (Datapdu *)NULL;
#endif
    uint16 lLength = 0;
    if ( 0 != ( lpdu = (Datapdu *) mIncomingpdu.Get() ) )
    {
        // fixed to check the lengths of data packets
        uint8* data = analyseHCIDataHeader ( lpdu->mBuffer, lpdu->mLength, false , aHandle , aPB , aBT , &lLength );
        if ( data )
        {
            //  move the data so that it can be deleted by the application.
            *aDataPtr = (uint8*) ::getpdu(lLength);
            memcpy (*aDataPtr,data,lLength);
        }
        else
        {
            //  there is a bad pdu out there...
            lpdu->disownData();
            *aDataPtr = 0;
            lLength = 0;
        }

        //  the pdu was passed up in deletable memory, so delete it.
        delete lpdu;
        //  pass that data up to the next level.
    }
    return lLength;
}

NEXT_PACKET_INFO ACLHandler::peekAtNext ( void )
{
    NEXT_PACKET_INFO result = nothingMore;
    if ( nextConnection )
        result = nextConnection->peekAtNext();
    return result;
}

Genericpdu * ACLHandler::getpdu( )
{
    Datapdu * lpdu;
    if ( nextConnection )
        lpdu = nextConnection->getpdu();
    //  how much data is there really?
    uint32 dataInPacket = lpdu->mLength - HCI_DATA_HEADER_SIZE;
    //  maybe eeek.
    assert ( ( II("removed more data from buffer than there is!") , mTotalDataInBuffer >= dataInPacket ) );

    // Enter Critical Section.
    {
        CriticalSection::Lock here (protectTotalDataInBuffer);
	mTotalDataInBuffer -= dataInPacket;
    }
    nextConnection = setNextConnection();
    return lpdu;
}

void ACLHandler::send (Genericpdu * apdu)
{
    //  Check the connection and increment the outstanding packets on it;
    uint16 connectionHandle = ((Datapdu *) apdu)->connectionHandle;
    Connection * lConnection = pConnectionList->getConnection ( connectionHandle );
    if ( lConnection != NULL )
    {
        lConnection->addOutstandingPackets ( 1 );

        //  set up the tidying object
        SendPathway * lSendPathway = new SendPathway;
        lSendPathway->initialise ( this , apdu );

        //  set up the callback.
        CallBack * lCallBack = new CallBack;
        lCallBack->initialise ( lSendPathway , lSendPathway->doneSend );

        pTransport->sendACLData( apdu->mBuffer , apdu->mLength , lCallBack );
    }
    else
    {
        mFlowControl.clear(1);
        delete apdu;
    }
}

uint16 ACLHandler::getBufferSize()
{
    return mSizeOfBuffer;
}

void ACLHandler::doneSend ( Genericpdu * apdu )
{
    delete apdu;
}

size_t ACLHandler::getHowMuchInBuffer ()
{
    //CriticalSection::Lock here (protectTotalDataInBuffer);
    size_t rv = mTotalDataInBuffer;
    return rv;
}

void ACLHandler::checkCollisionsWithBroadcast ( uint16 handle )
{
    //  Just check for now.
    //  At some stage we'll implement the true spec...
    if ( PiconetBroadcastHandle->getHandle() == handle )
    {
        assert ( (II("same handle as Piconet broadcast"), 0 ) );
    }

    if ( ActiveBroadcastHandle->getHandle() == handle )
    {
        assert ( (II("same handle as Active broadcast"), 0 ) );
    }
}

void ACLHandler::disconnect ( Connection * aConn )
{
    flush(aConn);
    mFlowControl.clear(aConn->getOutstandingPackets());
    nextConnection = setNextConnection();
}

Connection * ACLHandler::setNextConnection ()
{
    return pConnectionList->getNextConnectionWithACL();
}
