///////////////////////////////////////////////////////////////////////
//
//  FILE   :  HCIDeviceController.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  HCIDeviceController
//
//  PURPOSE:  Provide a C++ object API.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/HCIDeviceController.cpp#46 $
//
///////////////////////////////////////////////////////////////////////

#include "HCIDeviceController.h"
#include "hci_def.h"
#include "localTypes.h"
#include "connectionList.h"
#include "connection.h"
#include "commandHandler.h"
#include "ACLHandler.h"
//#include "SCOHandler.h"
#include "thread/critical_section.h"
#include "memfuncs.h"
#include "DCThread.h"

#include <string.h>

HCIDeviceController::HCIDeviceController(HCITransport * aTransport, bool usingBlueStack) :
mSignals ( new SignalBox ) ,
mThreadCloseDown ( new SignalBox ) ,
ch ( new CommandHandler ) ,
ah ( new ACLHandler ) ,
cl ( new ConnectionList ) ,
mDataCallBackProtection ( new CriticalSection ) ,
mEventCallBackProtection ( new CriticalSection ),
mComSentSignal (0),
mEvtRecvSignal (0),
mACLSentSignal (0),
mACLRecvSignal (0),
mBeingDeletedSignal (0),
mBadPDUSignal (0),
mThreadExitedSignal (0),
mThread (0)
{
    withBlueStack = usingBlueStack;
    creationInit();

    deleteTransportWithDeviceController = ( aTransport != 0 );
    if ( deleteTransportWithDeviceController )
        setTransport ( aTransport );
}

void HCIDeviceController::creationInit ()
{
#ifdef _INDEBUG
    // release debugging
    mDebug = new indebug;
    ch->mDebug = mDebug;
    ah->mDebug = mDebug;
    //mSignals->mDebug = mDebug;
    //mThreadCloseDown->mDebug = mDebug;
#endif

    mDataCallBackListStart   = NULL;
    mEventCallBackListStart  = NULL;
    mDefaultDataCallBack     = NULL;
    mDefaultEventCallBack    = NULL;
    mTransportReady = false;

    BadPDUList = new pduList;

    mComSentSignal = new SignalBox::Item(mSignals);
    mEvtRecvSignal = new SignalBox::Item(mSignals);
    mACLSentSignal = new SignalBox::Item(mSignals);
    mACLRecvSignal = new SignalBox::Item(mSignals);
    mBeingDeletedSignal = new SignalBox::Item(mSignals);

    mThreadExitedSignal = new SignalBox::Item(mThreadCloseDown);

    //  Initialise the acl handler to check connection handles.
    ah->initialise ( cl );
#ifdef _INDEBUG
    mDebug->unstring ( II("mBeingDeletedSignal = ") );
    mDebug->unload ( (uint8*) &mBeingDeletedSignal , 4 );
    mDebug->unstring ( II("\n") );
#endif

    //  set the thread exited to true, until we enter the thread...
    mThreadExitedSignal->set();
}

HCIDeviceController::~HCIDeviceController ( )
{
    if ( deleteTransportWithDeviceController )
        delete tr;

#ifdef _INDEBUG
    mDebug->unstring ( II("Setting mBeingDeletedSignal ") );
    mDebug->unload ( (uint8*) &mBeingDeletedSignal , 4 );
    mDebug->unstring ( II("\n") );
#endif
    mBeingDeletedSignal->set();

    while ( mThreadExitedSignal->getState() == false )
        mThreadCloseDown->wait();
    if ( mThread )
    {
        while ( mThread->IsActive() );
        delete mThread;
    }

    //  empty the callback lists.
    while ( mDataCallBackListStart != NULL )
    {
        DataCallBackListEntry * tbd = mDataCallBackListStart;
        mDataCallBackListStart = mDataCallBackListStart->next;
        delete tbd;
    }
    while ( mEventCallBackListStart != NULL )
    {
        EventCallBackListEntry * tbd = mEventCallBackListStart;
        mEventCallBackListStart = mEventCallBackListStart->next;
        delete tbd;
    }
#ifdef _INDEBUG
    // release debugging
    delete mDebug;
#endif

    // Delete the SignalBox::Item's beofre the SignalBox's
    delete mComSentSignal;
    delete mEvtRecvSignal;
    delete mACLSentSignal;
    delete mACLRecvSignal;
    delete mBeingDeletedSignal;
    delete mBadPDUSignal;
    delete mThreadExitedSignal;

    delete ah;
    delete ch;
    delete mSignals;
    delete BadPDUList;
    delete mThreadCloseDown;
    delete mDataCallBackProtection;
    delete mEventCallBackProtection;
    delete cl;
}

void HCIDeviceController::setTransport( HCITransport * transport )
{
    //  Initialise transport pointers.
    tr = transport;

#ifdef _INDEBUG
    // release debugging
    tr->mDebug = mDebug;
#endif

    ch->setup(tr , mComSentSignal , mEvtRecvSignal , mBeingDeletedSignal );
    ah->setup(tr , mACLSentSignal , mACLRecvSignal , mBeingDeletedSignal );
//    sh.setTransport(tr);

    //  Set receive callbacks to transport layer.
    tr->setupRecvCommand ( ch , ch->recvEvent );
    tr->setupRecvACLData ( ah , ah->recvData );
//    tr->setupRecvSCOData ( &sh , sh.recvData )

    if (!withBlueStack)
    {
        PARAMETER_VALUE cmdREAD_BUFFER_SIZE[2] = {2,0x1005};
        this->sendHCICommand( cmdREAD_BUFFER_SIZE , true ); 
    }
}

HCITransport * HCIDeviceController::getTransport ()
{
    return tr;
}

void HCIDeviceController::doWork( void )
{
    if ( mBeingDeletedSignal->getState() )
        return;

    checkQueues();
}

void HCIDeviceController::checkQueues( void )
{
    //  Check there is any point in trying
    if ( tr == NULL )
    {
        return;
    }

    bool done = false;

    //  Process Events, process Commands, process data in, process data out.

    //  Check the events.
    if ( !done && mEvtRecvSignal->getState() )
    {
        bool gotOne = true;
        done = mEvtRecvSignal->unset();
        while (gotOne)
        {
            PARAMETER_ARRAY lAnswers = ch->getEvent( );
            if ( lAnswers != NULL )
            {
                if ( interpretEvent ( lAnswers ) )
                    onHCIEvent ( (uint8)lAnswers[1] , lAnswers );
                else
                    freepdu (lAnswers) ;
            }
            else
                gotOne = false;
        }
    }

    //  Check the commands to send
    if ( !done && mComSentSignal->getState() )
    {
        done = mComSentSignal->unset();
        while ( ch->trySend() );
    }

    //  Check the ACL reception...
    if (!done && mACLRecvSignal->getState() )
    {
        bool gotOne = true;
        done = mACLRecvSignal->unset();
        while (gotOne)
        {


            uint8 lPB = 0 , lBT = 0 , *lBuffer = NULL ;
            uint16 lHandle = 0 ;
            uint16 length = ah->getIncomingData( &lHandle , &lBuffer , &lPB , &lBT );
            if ( length != 0 )
            {
                onHCIData( lHandle , lPB != 0 , lBT , lBuffer , length );
            }
            else
                gotOne = false;
        }
    }

    //  Check the ACL to send
    if ( !done && mACLSentSignal->getState() )
    {
        done = mACLSentSignal->unset();
        while ( ah->trySend() );
    }

    //  Check for bad PDUs
    if ( !done && mBadPDUSignal != 0 && mBadPDUSignal->getState() )
    {
        Badpdu * lEntry = NULL ;
        done = mBadPDUSignal->unset();
        while ( (lEntry = (Badpdu *) BadPDUList->Get()) != NULL )
        {
            BadPDUCallBackFn ( lEntry->mReason , lEntry->mBuffer , lEntry->mLength , BadPDUCallBackParameters );
            //  disown the data buffer so the upper layer has to free it.
            lEntry->disownData();
            delete lEntry;
        }
    }
}

void HCIDeviceController::createWorker ()
{
    if ( mThread == 0 )
    {
        mThread = new HCIThread ( this );
        mThread->Start();
    }
}

void HCIDeviceController::workerThread ( void * aObject )
{
    ( (HCIDeviceController *) aObject )->waitForWork();
}

void HCIDeviceController::waitForWork ()
{
    bool continuing = true;
    mThreadExitedSignal->unset();

    while (continuing)
    {
        bool done = false;

        mSignals->wait();

        //  Check we aren't trying to delete the object.
        if ( mBeingDeletedSignal->getState() )
        {
            done = true;
            continuing = false;
        }
        else
            checkQueues();
    }

    mThreadExitedSignal->set();
}

bool HCIDeviceController::sendHCICommand ( PARAMETER_ARRAY parameters )
{
    if ( parameters[1] == HCI_WRITE_VOICE_SETTING )
    {
        sh.set_potential_sample_size ( (parameters[2] & 0x40) ? 16 : 8 );
    }
    return sendHCICommand( parameters , false );
}

bool HCIDeviceController::sendHCICommand ( PARAMETER_ARRAY parameters , bool internallyGeneratedCommand = false )
{
    return ch->queueCommand ( parameters , internallyGeneratedCommand);
}

bool HCIDeviceController::sendHCIData ( uint16 connectionHandle ,
                                        bool packetBoundaryFlag ,
                                        uint8 broadcastFlag ,
                                        uint8* dataStart ,
                                        uint16 dataLength )
{
    return ah->queue(connectionHandle , dataStart , dataLength , packetBoundaryFlag , broadcastFlag );
}

size_t HCIDeviceController::flushHCIData ( uint16 connectionHandle )
{
    return ah->flush ( connectionHandle );
}

void HCIDeviceController::onHCIData( uint16 connectionHandle ,
                                     bool packetBoundaryFlag ,
                                     uint8 broadcastFlag ,
                                     uint8 *dataStart ,
                                     uint16 dataLength )
{
    DataCallBackListEntry * lEntry;
    //  dump that data if the callback isn't set.
    if ( ( lEntry = getCallBackOnData ( connectionHandle ) ) != NULL  && lEntry->function != NULL)
    {
        lEntry->function ( connectionHandle, packetBoundaryFlag, broadcastFlag, 
                           dataStart, dataLength, lEntry->parameters );
    }
    else
        freepdu(dataStart);
}

bool HCIDeviceController::interpretEvent ( PARAMETER_ARRAY aAnswer )
{
    //  The return value of this function is a boolean, which if false indicates that the
    //  command should be with held from the higher level application.
    uint32 lArraySize = aAnswer[PA_ARRAY_LENGTH];
    bool sendTheEventOn = true;
    uint8 eventCode = (uint8) aAnswer[PA_HCI_EVENT_CODE];

    // look for events we can use
    switch (eventCode)
    {
    case HCI_EV_COMMAND_COMPLETE:
        // carries the number of commands we can currently send to the chip.
        ch->mFlowControl.setFree( (uint16)aAnswer[PA_HCI_EVENT_COMMAND_COMPLETE_NUM_HCI_COMMAND_PACKETS] );
        // set the command flag, so we go and check if there is another command in the queue.
        mComSentSignal->set();
        // now lift out all the DeviceDriver generated commands.
        sendTheEventOn = ch->matchCommandToEvent( (uint16) aAnswer[PA_HCI_EVENT_COMMAND_COMPLETE_COMMAND_OPCODE] );
        //  interpret the command completes that are useful to us.
        useCommandComplete ( aAnswer , sendTheEventOn ) ;
        break;
    case HCI_EV_COMMAND_STATUS:
        // carries the number of commands we can currently send to the chip.
        ch->mFlowControl.setFree( (uint16) aAnswer[PA_HCI_EVENT_COMMAND_STATUS_NUM_HCI_COMMAND_PACKETS] );
        // set the command flag, so we go and check if there is another command in the queue.
        mComSentSignal->set();
        //  if status = 0 (so the command is being carried out).
        if ( aAnswer[PA_HCI_EVENT_COMMAND_STATUS_STATUS] == 0)
        {
            // now lift out all the DeviceDriver generated commands.
            sendTheEventOn = ch->matchCommandToEvent( (uint16)aAnswer[PA_HCI_EVENT_COMMAND_STATUS_COMMAND_OPCODE] );
        }
        break;
    case HCI_EV_NUMBER_COMPLETED_PACKETS:
        {
            //  carries the number of packets completed on each handle;
            for ( uint8 count = 0 ;
                  count < aAnswer[PA_HCI_EVENT_NUMBER_OF_COMPLETED_PACKETS_NUMBER_OF_HANDLES] ;
                  count ++ )
            {
                Connection * lConnection = 
                    cl->getConnection(
                    (uint16) aAnswer[ PA_HCI_EVENT_NUMBER_OF_COMPLETED_PACKETS_CONNECTION_HANDLE(count) ]
                    );
                uint16 packetCount = (uint16) aAnswer[PA_HCI_EVENT_NUMBER_OF_COMPLETED_PACKETS_NUMBER(count)];
                if ( lConnection != NULL )
                {
                    switch ( lConnection->getType() )
                    {
                    case conn_type_sco:
                        //  mSCOHandler;
                        break;
                    case conn_type_acl:
                        ah->mFlowControl.clear(packetCount);
                        lConnection->reduceOutstandingPackets  ( packetCount );
                        break;
                    default:
                        //  lose those tokens...
                        break;
                    }
                }
                else
                {
                    //  we're losing tokens again, because the connection isn't there any more.
                    //  but thats OK, since the disconnect event which would have blown away 
                    //  this connection should have recovered those tokens.
                    int i = 10;
                }
            }
            // set the data flag, so we go and check if there is more data in the queue.
            mACLSentSignal->set();
        }
        break;
    case HCI_EV_CONNECTION_COMPLETE:
        if ( aAnswer[2] == 0)
        {
            Connection * lConn = new Connection();
            uint16 handle = (uint16)aAnswer[3];
            bdaddr bda;
            bda.lap = (uint24) aAnswer[4];
            bda.uap = (uint8) aAnswer[5];
            bda.nap = (uint16) aAnswer[6];
            uint8 type = (uint8) aAnswer[7];
            ah->checkCollisionsWithBroadcast ( handle );
            lConn->initialise ( handle , (CONNECTION_TYPE) type , cl , &bda );
            if ( type == conn_type_sco )
            {
                sh.conn_sco();
                tr->set_sco_bandwidth ( sh.get_bandwidth() , 0 , 0 );
            }
        }
        break;
    case HCI_EV_DISCONNECTION_COMPLETE:
        if (true)
        {
            //  whatever the status, delete the object (if it still exists...)
            uint16 handle = (uint16) aAnswer[3];
            Connection * lConn = cl->getConnection(handle);
            uint16 packetCount = 0;
            if (lConn != NULL)
            {
                //  save any outstanding tokens!!
                switch ( lConn->getType() )
                {
                case conn_type_sco:
                    sh.disc_sco();
                    tr->set_sco_bandwidth ( sh.get_bandwidth() , 0 , 0 );
                    break;
                case conn_type_acl:
                    ah->disconnect(lConn);
                    break;
                default:
                    //  lose those tokens...
                    break;
                }
                delete lConn;
            }
        }
        break;
    case HCI_EV_LOOPBACK_COMMAND:
        //  If we get an event looped back then we must have sent a command out, so increment
        //  the flow control.  We're going to get into trouble with HCI extensions which are
        //  uncontrolled in CommandHandler.
        ch->mFlowControl.clear ( 1 );
        // set the command flag, so we go and check if there is another command in the queue.
        mComSentSignal->set();
        break;
    case HCI_EV_DATA_BUFFER_OVERFLOW:
        // There is nothing we can do...  Let the upper layer know, and leave it the
        // responsibility of reseting and stuff...
        break;
    default:
        //  do nothing, but return the packet to the higher level.
        break;
    }
    //  send the packet up to the higher level.
    return sendTheEventOn;
}

void HCIDeviceController::useCommandComplete( PARAMETER_ARRAY aAnswer , bool passingOn )
{
    //  length , eventcode , num_command_tokens , command_opcode ...
    switch (aAnswer[3])
    {
    case HCI_READ_BD_ADDR:
        //  right size and status zero.
        if ( aAnswer[0] == 8 && aAnswer[4] == 0 )
        {
            localDevice.lap = (uint24) aAnswer[5];
            localDevice.uap = (uint8)  aAnswer[6];
            localDevice.nap = (uint16) aAnswer[7];
        }
        break;
    case HCI_READ_BUFFER_SIZE:
        //  right size and status zero
        if ( aAnswer[0] == 9 && aAnswer[4] == 0 )
        {
            //  number of packets and packet length.  Initialise Flow control only
            //  uses the max packet information the first time it receives it.
            ah->initialiseFlowControl ( (uint16) aAnswer[7] , (uint16) aAnswer[5] );
            cl->useBufferSizes ( (uint16) aAnswer[5] , (uint8) aAnswer[6] ,
                                 (uint16) aAnswer[7] , (uint16) aAnswer[8] );
            // whatever... sh.mFlowControl.setFree ( (uint16) aAnswer[8] );
        }
        break;
    case HCI_READ_VOICE_SETTING:
        if ( aAnswer[0] == 6 && aAnswer[4] == 0 )
        {
            sh.set_sample_size ( (aAnswer[5] & 0x40) ? 16 : 8 );
        }
        break;
    case HCI_WRITE_VOICE_SETTING:
        if ( aAnswer[0] == 5 && aAnswer[4] == 0 )
        {
            sh.potential_was_good();
        }
    default:
        break;
    }
}

void HCIDeviceController::onHCIEvent ( uint8 eventCode , PARAMETER_ARRAY parameters )
{
    EventCallBackListEntry * lEntry;
    //  dump that event if the callback isn't set.
    if ( ( lEntry = getCallBackOnEvent ( eventCode ) ) != NULL )
    {
        lEntry->function ( eventCode , parameters , lEntry->parameters );
    }
    else
        freepdu (parameters);
}

void HCIDeviceController::setCallBackOnData ( uint16 aHandle ,
                                          void (*dataCallBackFn) ( uint16 connectionHandle , 
                                                                   bool packetBoundaryFlag , 
                                                                   uint8 broadcastFlag , 
                                                                   uint8 *dataStart , 
                                                                   uint16 dataLength, 
                                                                   void * parameters),
                                          void * dataCallBackParameters)
{
    DataCallBackListEntry * lEntry = getCallBackOnData( aHandle );

    //  Enter Critical Section
    CriticalSection::Lock here (mDataCallBackProtection);

    if ( lEntry == NULL || lEntry->connectionHandle != aHandle )
    {
        lEntry = new DataCallBackListEntry;
        lEntry->connectionHandle = aHandle;
    }

    lEntry->function = dataCallBackFn;
    lEntry->parameters = dataCallBackParameters;
    
    lEntry->next = mDataCallBackListStart;
    mDataCallBackListStart = lEntry;

    if ( aHandle == DEFAULT_CALL_BACK_HANDLE )
        mDefaultDataCallBack = lEntry;

    //  Leave Critical Section
}

void HCIDeviceController::unsetCallBackOnData ( uint16 aHandle )
{
    DataCallBackListEntry * lEntry;

    //  Enter Critical Section
    CriticalSection::Lock here (mDataCallBackProtection);

    for ( lEntry = mDataCallBackListStart;
          lEntry != NULL && lEntry->next->connectionHandle != aHandle ;
          lEntry = lEntry->next );

    if ( lEntry != NULL )
    {
        DataCallBackListEntry * tbd = lEntry->next;
        lEntry->next = lEntry->next->next;
        delete tbd;

        if ( aHandle == DEFAULT_CALL_BACK_HANDLE )
            mDefaultDataCallBack = NULL;
    }

    //  Leave Critical Section
}

DataCallBackListEntry * HCIDeviceController::getCallBackOnData ( uint16 aHandle )
{
    //  Enter Critical Section
    CriticalSection::Lock here (mDataCallBackProtection);

    DataCallBackListEntry * lEntry = getCallBackOnData_nocs ( aHandle );

    //  Leave Critical Section

    return lEntry;
}

DataCallBackListEntry * HCIDeviceController::getCallBackOnData_nocs ( uint16 aHandle )
{
    if ( aHandle == DEFAULT_CALL_BACK_HANDLE )
        return mDefaultDataCallBack;
    DataCallBackListEntry * lEntry = NULL;

    if ( mDataCallBackListStart != NULL )
    {
        for ( lEntry = mDataCallBackListStart;
              lEntry->next != NULL && lEntry->connectionHandle != aHandle ;
              lEntry = lEntry->next );

        if ( lEntry->connectionHandle != aHandle )
            lEntry = mDefaultDataCallBack;
    }

    return lEntry;
}

void HCIDeviceController::setCallBackOnEvent ( uint16 event,
                                               void (*eventCallBackFn) (uint8 eventcode, 
                                                                        PARAMETER_ARRAY eventParameters ,
                                                                        void * parameters),
                                               void * eventCallBackParameters)
{
    //  Enter Critical Section
    CriticalSection::Lock here (mEventCallBackProtection);

    EventCallBackListEntry * lEntry = this->getCallBackOnEvent_nocs ( event );

    if ( lEntry == NULL || lEntry->eventCode != event )
    {
        lEntry = new EventCallBackListEntry;
        lEntry->eventCode = event;
    }

    lEntry->function = eventCallBackFn;
    lEntry->parameters = eventCallBackParameters;

    lEntry->next = mEventCallBackListStart;
    mEventCallBackListStart = lEntry;

    if ( event == DEFAULT_CALL_BACK_EVENT )
        mDefaultEventCallBack = lEntry;

    //  Leave Critical Section
}

void HCIDeviceController::unsetCallBackOnEvent ( uint16 event )
{
    EventCallBackListEntry * lEntry;

    //  Enter Critical Section
    CriticalSection::Lock here (mEventCallBackProtection);

    for ( lEntry = mEventCallBackListStart;
          lEntry != NULL && lEntry->next->eventCode != event ;
          lEntry = lEntry->next );
          
    if ( lEntry != NULL )
    {
        EventCallBackListEntry * tbd = lEntry->next;
        lEntry->next = lEntry->next->next;
        delete tbd;
    }

    if ( event == DEFAULT_CALL_BACK_EVENT )
        mDefaultEventCallBack = NULL;
    //  Leave Critical Section
}

EventCallBackListEntry * HCIDeviceController::getCallBackOnEvent ( uint16 aEventCode )
{
    //  Enter Critical Section
    CriticalSection::Lock here (mEventCallBackProtection);

    EventCallBackListEntry * lEntry = getCallBackOnEvent_nocs ( aEventCode );

    //  Leave Critical Section

    return lEntry;
}

EventCallBackListEntry * HCIDeviceController::getCallBackOnEvent_nocs ( uint16 aEventCode )
{
    if ( aEventCode == DEFAULT_CALL_BACK_EVENT )
        return mDefaultEventCallBack;
    EventCallBackListEntry * lEntry = NULL;

    if (mEventCallBackListStart != NULL)
    {
        for ( lEntry = mEventCallBackListStart;
              lEntry->next != NULL && lEntry->eventCode != aEventCode ;
              lEntry = lEntry->next );

        if ( lEntry->eventCode != aEventCode )
            lEntry = mDefaultEventCallBack;
    }

    return lEntry;
}

bool HCIDeviceController::isTransportReady()
{
    //  Are we connected at this instant?
    if ( !mTransportReady )
        mTransportReady = tr->ready(0);

    return mTransportReady ;
}

void HCIDeviceController::registerBadPDU ( BadPDUReason aWhy , uint8 * apdu , uint32 aLength , void * aThis )
{
    HCIDeviceController * lObject = (HCIDeviceController *) aThis ;
    //  Take a copy of the bad pdu.  The original should be freed when it is analysed.
    //  The copy is, of course, freed by the upper layer.
    Badpdu * lEntry = new Badpdu;
    lEntry->mBuffer = (uint8 *) getpdu (aLength);
    memcpy (lEntry->mBuffer, apdu, aLength);
    lEntry->mLength = aLength;
    lEntry->mReason = aWhy;
    lObject->BadPDUList->Add ( lEntry );
    if ( lObject->mBadPDUSignal != 0 )
        lObject->mBadPDUSignal->set();
}

void HCIDeviceController::setCallBackOnBadPDU ( void (*callBackFn)( BadPDUReason aWhy ,
                                                                    uint8 * apdu ,
                                                                    uint32 aLength ,
                                                                    void * parameters ) ,
                                                void * parameters )
{
    BadPDUCallBackFn = callBackFn;
    BadPDUCallBackParameters = parameters;
    setBadPDUCallback ( registerBadPDU , this );
    mBadPDUSignal = new SignalBox::Item(mSignals);
}

bool HCIDeviceController::getBadPDU ( BadPDUReason* pWhy , uint8 ** ppdu , uint32* pLength )
{
    if ( BadPDUCallBackFn != NULL )
        return false;

    Badpdu * lEntry = (Badpdu *) BadPDUList->Get();

    if ( lEntry != NULL )
    {
        *pWhy = lEntry->mReason;
        *ppdu = lEntry->mBuffer;
        *pLength = lEntry->mLength;
        lEntry->disownData();
        delete lEntry;
        return true;
    }
    else
        return false;
}

bool HCIDeviceController::getHCIEvent ( PARAMETER_ARRAY * parameters )
{
    if ( getCallBackOnEvent ( DEFAULT_CALL_BACK_EVENT ) == NULL )
    {
        *parameters = ch->getEvent();
        return true;
    }
    else
        return false;
}

bool HCIDeviceController::getHCIData  ( uint16* pConnectionHandle ,
                                        bool* pPacketBoundaryFlag ,
                                        uint8* pBroadcastFlag ,
                                        uint8** pDataStart ,
                                        uint16* pDataLength )
{
    if ( getCallBackOnData ( DEFAULT_CALL_BACK_HANDLE ) == NULL )
    {
        uint8 lPB;
        *pDataLength = ah->getIncomingData ( pConnectionHandle , pDataStart , &lPB , pBroadcastFlag );
        *pPacketBoundaryFlag = ( lPB != 0 );
        return true;
    }
    else
        return false;
}

size_t HCIDeviceController::getSizeOfBufferContent ()
{
    return ah->getHowMuchInBuffer();
}

size_t HCIDeviceController::getSizeOfDataWaitingOnHandle ( uint16 connectionHandle )
{
    Connection * lConnection = cl->getConnection ( connectionHandle );
    size_t rv = 0;
    if (lConnection)
        rv = lConnection->getIncompleteData ();
    return rv;
}

uint16 HCIDeviceController::getNumberOfPacketsOutstandingOnHandle ( uint16 connectionHandle )
{
    Connection * lConnection = cl->getConnection ( connectionHandle );
    uint16 rv = 0;
    if (lConnection)
        rv = lConnection->getOutstandingPackets ();
    return rv;
}
