///////////////////////////////////////////////////////////////////////
//
//  FILE   :  connectionList.h
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  ConnectionList
//
//  PURPOSE:  A "secure" array of connection handles and pointers
//            to connections.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/connectionList.cpp#17 $
//
///////////////////////////////////////////////////////////////////////

#include "connectionList.h"
#include "thread/critical_section.h"

//  DEBUG
#include "connection.h"
#ifndef _WINCE
#include <cassert>
#else
#include <windef.h>
#include <winbase.h>
#include <dbgapi.h>
#define assert ASSERT
#endif

ConnectionList::ConnectionList() : protection(new CriticalSection())
{
    Start = NULL;
    connectionCount = 0;
    aclBufferSize = 0;
    scoBufferSize = 0;
    aclMaxTokens = 0;
    scoMaxTokens = 0;
    nextACLConnection = 0;
}

ConnectionList::~ConnectionList()
{
    delete protection;
}

//  Called by the connection creator function.
bool ConnectionList::addConnection (uint16 connectionHandle,
				    Connection *cpointer)
{
    bool ok = true;
    ConnectionListEntry * entry = new ConnectionListEntry;

    entry->handle = connectionHandle ;
    entry->connection = cpointer ;
    if ( cpointer->getType() == conn_type_acl )
        cpointer->setBufferSize(this->aclBufferSize);
    if ( cpointer->getType() == conn_type_sco )
        cpointer->setBufferSize(this->scoBufferSize);

    //  DEBUG
    assert ( (II("different") , connectionHandle == cpointer->getHandle()) );

    //  Enter Critical Region
    CriticalSection::Lock here (protection);

    if ( find ( connectionHandle ) == NULL )
    {
	entry->next = Start ;
	Start = entry;
        connectionCount ++;
    }
    else
    {
	delete entry;
	ok = false;
    }

    //  Leave Critical Region

    return ok;
}

//  Called by the connection destructor function.
void ConnectionList::removeConnection (uint16 connectionHandle)
{
    bool ok = true;
    if (Start)
    {
	ConnectionListEntry * ePointer;

	//  Enter Critical Region
	CriticalSection::Lock here (protection);

        //  Assignment in if().
	if ( ( ePointer = find ( connectionHandle ) ) != 0 )
	{
	    //  Actually remove the entry!
	    if ( Start == ePointer )
		    Start = ePointer->next;
	    else
	    {
		//  Scan the list for the record which points to
		//  the one are going to remove.
		ConnectionListEntry * previous;

		for ( previous = Start ; 
		      ( previous != 0 ) && ( ePointer != previous->next );
		      previous = previous->next ) ;

		if (previous)
		    previous->next = ePointer->next ;
		else
		    ok = false;
	    }
	    delete ePointer;
	}
	else
	    ok = false;

	//  Leave Critical Region
    }
    connectionCount --;
    return ; //  was "return ok", but at the end of the day it
	     //  returns false if it can't find the record, so
	     //  the handle isn't in use anyhow...
}

bool ConnectionList::validateConnection (uint16 connectionHandle)
{
    if ( getConnection (connectionHandle) != NULL )
        return true;
    else
        return false;
}

Connection * ConnectionList::getConnection (uint16 connectionHandle)
{
    CriticalSection::Lock here (protection);
    ConnectionListEntry * lEntry = find ( connectionHandle );
    if ( lEntry != NULL )
        return ( lEntry->connection );
    else
        return NULL;
}

ConnectionListEntry* ConnectionList::find(uint16 connectionHandle)
{
    //  Critical section must be invoked to call this function.
    ConnectionListEntry * ePointer;

    for ( ePointer = Start ; 
	  ePointer != NULL && connectionHandle != ePointer->handle ;
	  ePointer = ePointer->next )
    {
        assert ( (II("different") , ePointer->handle == ePointer->connection->getHandle()) );
    }

    //  DEBUG
    if ( ePointer != NULL )
        assert ( (II("different") , connectionHandle == ePointer->connection->getHandle()) );

    return ePointer;
}

Connection * ConnectionList::getNextConnectionWithACL ()
{
    if ( nextACLConnection == 0 )
        return 0;

    ConnectionListEntry *guess = circle ( nextACLConnection );
    uint16 counter = 0;
    ConnectionListEntry *bestGuess = 0;
    uint16 bestGuessOutstandingTokens = aclMaxTokens + 1;
    while ( counter++ <= connectionCount )
    {
        if ( guess->connection->peekAtNext() != nothingMore )
        {
            uint16 op = 0;
            if ( (op = guess->connection->getOutstandingPackets()) < bestGuessOutstandingTokens )
            {
                bestGuess = guess;
                bestGuessOutstandingTokens = op;
            }
        }
        guess = circle ( guess );
    }
    nextACLConnection = bestGuess;
    if ( nextACLConnection == 0 )
        return 0;

    return nextACLConnection->connection;
}

ConnectionListEntry * ConnectionList::circle ( ConnectionListEntry * clp )
{
    if ( clp == 0 )
        return 0;

    if ( clp->next == 0 )
        return Start;
    else
        return clp->next;
}

void ConnectionList::setNextConnectionWithACL ( Connection * cpointer )
{
    nextACLConnection = find ( cpointer->getHandle() );
    assert ( (II("nextACLConnection mismatch") , cpointer == nextACLConnection->connection ) );
}

void ConnectionList::useBufferSizes( uint16 aclSize, uint8 scoSize , uint16 aclMax , uint16 scoMax )
{
    aclMaxTokens = aclMax ;
    scoMaxTokens = scoMax ;
    if ( aclSize != aclBufferSize )
    {
        aclBufferSize = aclSize;
        for ( ConnectionListEntry * i = Start ; i != 0 ; i = i->next )
            if ( i->connection->getType() == conn_type_acl )
                i->connection->setBufferSize( aclSize );
    }
    if ( scoSize != scoBufferSize )
    {
        scoBufferSize = scoSize;
        for ( ConnectionListEntry * i = Start ; i != 0 ; i = i->next )
            if ( i->connection->getType() == conn_type_sco )
                i->connection->setBufferSize( scoSize );
    }
}
