///////////////////////////////////////////////////////////////////////
//
//  FILE   :  dataBuffer.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  DataBuffer
//
//  PURPOSE:  Hold data for connections.  Copy data into the buffer.
//            Pass it out as a reference.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/dataBuffer.cpp#20 $
//
///////////////////////////////////////////////////////////////////////

#include "dataBuffer.h"
#include "HCIPacker.h"
#include "thread/critical_section.h"

#include <string.h>

//  use a very simple fifo linked list to implement quickly

DataBuffer::DataBuffer() : dataProtection(new CriticalSection())
{
    Start = 0;
    End = 0;
    //  Stick in a random default packet size...
    dataPacketSize = 40;
    dataPacketType = conn_type_unspecified;
}

DataBuffer::~DataBuffer()
{
    dbListElement *tbd;
    while (Start)
    {
        tbd = Start;
        Start = tbd->next;
	delete tbd->pdu;
        delete tbd;
    }
    delete dataProtection;
}

void DataBuffer::initialise (uint16 packetSize , CONNECTION_TYPE type )
{
    dataPacketType = type ;
    dataPacketSize = packetSize ;
    if (type == conn_type_sco)
        dataHeaderSize = VOICE_HEADER_SIZE;
    if (type == conn_type_acl)
        dataHeaderSize = DATA_HEADER_SIZE;
}

//  Give defaults for PB and BT so that a 2 variable send is possible,  This might be used for SCO,
//  which doesn't care about such things.
bool DataBuffer::add ( uint16 connectionHandle ,
                       const uint8 *data ,
                       uint32 length ,
                       bool packetBoundary = false ,
                       uint8 broadcastType = 0 )
{
    if (dataPacketType == conn_type_unspecified)
        return false;

    // start and end of the multiple packets we are creating.
    dbListElement *first = 0;
    dbListElement *last  = 0;

    // local copies of the data pointer and the amount of data left.
    const uint8* dataIndex = data;
    uint32 lLength = length;

    while ( lLength != 0 )
    {
	    //  Create an entry 
	    dbListElement *entry = new dbListElement;

        if (first == 0)
        {
            first = entry;
            last = entry;
        }

        //  For ACL fill in the boundary and broadcast info.
        entry->pdu = new Datapdu;
        entry->pdu->connectionHandle = connectionHandle;
        entry->next = 0;

        //  leave a small gap at the start to insert an HCI header.
        entry->pdu->size(dataPacketSize + dataHeaderSize);


	    if (lLength > dataPacketSize)
	    {
                //  write the pdu header.
                writeHCIDataHeader ( entry->pdu->mBuffer ,
                                     connectionHandle , 
                                     packetBoundary , 
                                     broadcastType , 
                                     dataPacketSize , 
                                     dataPacketType );

                uint8 *writeHere = entry->pdu->mBuffer + dataHeaderSize;
	        memcpy ( writeHere , dataIndex , dataPacketSize);
	        lLength -= dataPacketSize;
                dataIndex += dataPacketSize;
	    }
	    else
	    {
            //  write the pdu header.
            writeHCIDataHeader ( entry->pdu->mBuffer ,
                                 connectionHandle , 
                                 packetBoundary , 
                                 broadcastType , 
                                 lLength , 
                                 dataPacketType );

            uint8 *writeHere = entry->pdu->mBuffer + dataHeaderSize;
	        memcpy ( writeHere , dataIndex , lLength);
	        entry->pdu->mLength = lLength + dataHeaderSize;
	        lLength = 0;
	    }
        
        //  After the first (sub-)packet, the rest aren't boundaries any more...
        packetBoundary = false;

        //  if there is more than one, string these together.
        if ( entry != first )
        {
            last->next = entry;
            last = entry;
        }
    }

    //  Critical region Starts
    CriticalSection::Lock here (dataProtection);

    //  Add the entries to the end of the list.
    if (Start)
	End->next = first;
    else
	Start = first;
    End = last;

    //  Critical region Ends

    return true;
}

size_t DataBuffer::flush ()
{
    //  Enter Critical Section
    CriticalSection::Lock here ( dataProtection );
    dbListElement *tbd;
    uint32 howMuch = 0;
    while (Start)
    {
        tbd = Start;
        Start = tbd->next;
        howMuch += (tbd->pdu->mLength - HCI_DATA_HEADER_SIZE);
	delete tbd->pdu;
        delete tbd;
    }
    //  Leave Critical Section
    return howMuch;
}

NEXT_PACKET_INFO DataBuffer::peekAtNext ()
{
    NEXT_PACKET_INFO cheat = nothingMore;

    CriticalSection::Lock here (dataProtection);
    if ( Start != 0 )
        cheat = ( ( Start->pdu->flowControlled ) ? flowControlled : notFlowControlled );

    return cheat;
}

Datapdu *DataBuffer::get()
{
    Datapdu *lpdu = 0;

    //  only try if there are things to send
    if (Start)
    {
        dbListElement *exitEl;

        {   // Critical Section
	    CriticalSection::Lock here (dataProtection);
	    exitEl = Start;
	    Start = Start->next;
	}

        lpdu = exitEl->pdu;
//#ifndef _DEBUG
        delete exitEl;
//#endif
    }

    return lpdu;
}


/**********************************************************************
    Streaming circular buffer is the ideal...
    but difficult to implement for a bear of little brain...

DataBuffer::DataBuffer()
{
    //  Initialise at 4k.
    bufferSize = INITIAL_BUFFER_SIZE;
    buffer = new uint8[INITIAL_BUFFER_SIZE];
    //  Allow a bit of space in front of each data packet to
    //  write an HCI header.
    removePoint = exposePoint = addPoint = buffer + DATA_HEADER_SIZE;
}

DataBuffer::~DataBuffer()
{
    delete[] buffer;
}

bool DataBuffer::add(uint8 *data, uint16 length)
{
    CriticalSection::Lock here (dataProtection);
    if ( ( buffer + bufferSize ) < ( addPoint + length ) )
    {
	//  Not enough space to write the data at the end of the buffer.
	//  Check if we can fold the data to the start of the buffer.
	{
	    //  resize and rewrite.
	    bufferSize *= 2;
	    if ( bufferSize > MAX_BUFFER_SIZE )
	    {
		return false;
	    }
	    uint8 *newBuffer = new uint8[bufferSize];
	    memcpy (newBuffer , buffer , bufferSize/2 );
	    delete buffer;
	    addPoint += newBuffer - buffer;
	    buffer = newBuffer;
	}
    }
    dataLength += length;
    memcpy (addPoint , data , length);
    addPoint += length;
    // push back the upper limit of the data
    if (top < addPoint)
	top = addPoint;
    return;
}

uint16 DataBuffer::expose(uint8 **pdu, uint16 length)
{
    //  They are asking for more data than we have.
    //  Tough, give them what we have.
    if (length > dataLength)
	length = dataLength;

    CriticalSection::Lock here (dataProtection);
    //  check that the data is in one block atm.
    if (exposePoint + length > top)
    {
       // panic - regroup!
    }
    else
    {
	// enough data is in one block.  That makes life simple.
	*pdu = exposePoint - DATA_HEADER_SIZE;
	exposePoint += length;
    }
    return length + DATA_HEADER_SIZE;
}

bool DataBuffer::remove(uint8 **pdu, uint16 length)
{
    //  if we've reached the top, then reset everything to the bottom!
    if (exposePoint == top)
    {
	top = addPoint;
	exposePoint = buffer + DATA_HEADER_SIZE;
    }
    if (datalength
}

*/
