///////////////////////////////////////////////////////////////////////
//
//  FILE   :  HCIfactory.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  ControllerFactory,
//            HCIControllerFactory,
//            BlueCoreControllerFactory,
//            ControllerDefinition
//
//  PURPOSE:  A set of classes to create and initialise a Host Side HCI/Serial
//            stack.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/Factory/HCIFactory.cpp#13 $
//
///////////////////////////////////////////////////////////////////////
#include "stdafx.h"

#include "HCIDeviceController.h"
#include "HCIFactory.h"
#include "bc01DeviceController.h"
#include "transportCreatorFunctions.h"

#ifdef _WINCE
#include <windef.h>
#include <winbase.h>
#include <dbgapi.h>
#define assert ASSERT
#else
#include <stdio.h>
#include <cassert>
#include <stdarg.h>
#endif


///////////////////////////////////////////////////////////////////////
//   ControllerDefinition
//
//   An unchangeable configuration array.
///////////////////////////////////////////////////////////////////////
ControllerDefinition::ControllerDefinition ( TransportType t , ichar * TransportDevice , uint32 DataRate , bool tunnel )
:
mType ( t ) ,
mTransportDevice ( new ichar [ (istrlen (TransportDevice) + 1 ) ] ),
mDataRate ( DataRate ),
mTunnel ( tunnel )
{
    istrcpy ( mTransportDevice , TransportDevice );
}
ControllerDefinition::~ControllerDefinition()
{
    delete mTransportDevice;
}

////////////////////////////////////////////////////////////////////////
//  ControllerFactory
//  
//  Base class, dealing with the configuration of the Controllers
////////////////////////////////////////////////////////////////////////
ControllerFactory::ControllerFactory ( bool configurable )
{
    mDefinition = getLocalConfiguration( configurable );
}

ControllerFactory::ControllerFactory ( ControllerDefinition * aDefinition )
{
    mDefinition = aDefinition ;
}

ControllerFactory::~ControllerFactory ()
{
    delete mDefinition;
}

bool ControllerFactory::ready()
{
    return ( mDefinition != NULL );
}

ichar * ControllerFactory::getDeviceName ()
{
    return mDefinition->mTransportDevice;
}

BlueCoreTransport * ControllerFactory::getTransport ( )
{
    assert ( mDefinition );

    BlueCoreTransport * t = NULL ;

    switch ( mDefinition->mType )
    {
    case USB:
        t = createUSB ( mDefinition->mTransportDevice , 0 );
        break;
    case BCSP:
        t = createBCSP ( mDefinition->mTransportDevice , mDefinition->mDataRate , mDefinition->mTunnel );
        break;
    case H4:
        t = createH4 ( mDefinition->mTransportDevice , mDefinition->mDataRate );
        break;
    default:
        assert ( ( II("Corrupt type"), 0 ) );
        break;
    }

    return t;
}

////////////////////////////////////////////////////////////////////////
//  HCIControllerFactory
//  
//  Creates HCIDeviceControllers.
////////////////////////////////////////////////////////////////////////
HCIControllerFactory::HCIControllerFactory ( bool configurable ) : ControllerFactory ( configurable )
{
}

HCIControllerFactory::HCIControllerFactory ( ControllerDefinition * aDefinition ) : ControllerFactory ( aDefinition )
{
}

HCIControllerFactory::~HCIControllerFactory ()
{
}

HCIDeviceController * HCIControllerFactory::spawn()
{
    if ( mDefinition != NULL )
        return new HCIDeviceController ( getTransport() );
    else
        return NULL;
}

////////////////////////////////////////////////////////////////////////
//  BlueCoreControllerFactory
//  
//  Creates BlueCoreDeviceControllers.
////////////////////////////////////////////////////////////////////////
BlueCoreControllerFactory::BlueCoreControllerFactory ( bool configurable ) : ControllerFactory ( configurable )
{
//	assert(sizeof(bool) == sizeof(char));
	assert(this != NULL);
}

BlueCoreControllerFactory::BlueCoreControllerFactory ( ControllerDefinition * aDefinition ) : ControllerFactory ( aDefinition )
{
}

BlueCoreControllerFactory::~BlueCoreControllerFactory ()
{
}

BlueCoreDeviceController * BlueCoreControllerFactory::spawn( bool BlueStack )
{
    if ( mDefinition != NULL )
        return new BlueCoreDeviceController ( getTransport() , BlueStack );
    else
        return NULL;
}

