/****************************************************************************
**
**  File: BCPack.c
**
**            (C) CSR 2000
**
**  Purpose:  contains C procedures for Bluecore private channel packet
**            generation and analysis
**MODIFICATION HISTORY
**	#33     1:mar:04 pws    B-2096: Support BCCMDVARID_REMOTE_SNIFF_LIMITS
**
**  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/Packer/BCPacker/BCPack.c#33 $
**
****************************************************************************/

#include "HCIPacker.h"
#include "BCPack_private.h"
#include "hci_private.h"
#include "memfuncs.h"
#include <string.h>

#ifdef _INDEBUG

void * IDO;

void setBCIDO ( void * obj )
{
    IDO = obj;
}

#endif

/****************************************************************************
NAME
	bccmdGenerate  -  generates wire format commands for hostside hci

PARAMETERS
        the address of a pointer to a buffer (to be pointer at a buffer
        by the procedure) and parameter array.

RETURNS
        The size of the buffer written.  Returns 0 on failure to create
        the command pdu successfully.
****************************************************************************/
uint32 bccmdGenerate( uint8 ** aPointerToBuffer , PARAMETER_ARRAY aAllParameters)
{
    PDU_AND_PARAMETER_HOLDER * lHolder;

    uint32 lLength;
    uint16 extra_words = 0;
    uint8 lRowCounter;

    lHolder = createHolder ( NULL , aAllParameters );

    /*  Initialise the return value */
    lHolder->pduLength = 0;

    if ( lHolder != NULL )
    {
        /*  All commands have at least the 3 header entries, so check the size */
        if (lHolder->parameters[0] >= 3)
        {
            /*  Set the parameter pointer to after the header */
            lHolder->parameterIndex = lHolder->parameters + 4;

            /* Allocate some memory for the command buffer */
            if ( ( lHolder->parameters[3] & 0x1FFF ) < BCCMDVARID_SPECIAL_START)
                lHolder->pduLength = bccmdSize       ( (uint16) lHolder->parameters[3] );
            else
                lHolder->pduLength = bccmdArraySize ( (uint16) lHolder->parameters[3] , lHolder->parameterIndex, &extra_words ) ;

            if (lHolder->pduLength != 0)
            {
                /* Allocate some zeroed space */
                lHolder->pdu = (uint8*) getpdu(lHolder->pduLength);
                memset ( lHolder->pdu , 0 , lHolder->pduLength );

                /*  Write in the header */
                lHolder->pduIndex = lHolder->pdu;

                /* command type */
                WRITE_2BYTES ( lHolder->pduIndex , lHolder->parameters[1] );
                lHolder->pduIndex += 2;
                /* length in uint16s */

		WRITE_2BYTES ( lHolder->pduIndex , (lHolder->pduLength >> 1)  );
                lHolder->pduIndex += 2;
                /* sequence number */
                WRITE_2BYTES ( lHolder->pduIndex , lHolder->parameters[2] );
                lHolder->pduIndex += 2;
                /* varid */
                WRITE_2BYTES ( lHolder->pduIndex , lHolder->parameters[3] );
                lHolder->pduIndex += 2;
                /* status */
                WRITE_2BYTES ( lHolder->pduIndex , (BCCMDPDU_STAT_OK) );
                lHolder->pduIndex += 2;

                /*
                **  Step through the command table to find the format.  
                **  Match on the top 6 bits only for all but the specials.
                */
                for ( lRowCounter = 0 ; lRowCounter < BCCMD_TBL_SIZE ; lRowCounter ++ )
                {
                    if ( bccmd_tbl[lRowCounter].code == lHolder->parameters[3] )
                        break;
                    if (   (bccmd_tbl[lRowCounter].code == (lHolder->parameters[3] & 0x1C00))
                        && (lHolder->parameters[3] & 0x1C00) < BCCMDVARID_SPECIAL_START
                        )
                        break;
                }

                /*  Check that we did find something in the table */
                if (lRowCounter != BCCMD_TBL_SIZE)
                {
                    if ( ( lHolder->parameters[1] == BCCMDPDU_SETREQ )
                        || ((lHolder->parameters[3] & 0x1C00) == BCCMDVARID_SPECIAL_START))
		    {
                        if ( !buildBuffer (bccmd_tbl[lRowCounter].format, lHolder ) )
                            lLength = 0;
			else
			{
			    /* Well, I don't know how else to do this... */
			    while (extra_words &&
				   lHolder->parameterIndex <
				   lHolder->parameters +
				   lHolder->parameterCount)
			    {
				WRITE_2BYTES( lHolder->pduIndex,
					      (*lHolder->parameterIndex &
					       0xFFFF));
				lHolder->pduIndex += 2;
				lHolder->parameterIndex++;
			    }
			}
		    }
                }
                else
                    lHolder->pduLength = 0;

                /*  plug those memory leaks. */
                if (lHolder->pduLength == 0)
                    freepdu(lHolder->pdu);
            }
        }
    }

    *aPointerToBuffer = lHolder->pdu;
    lLength = lHolder->pduLength;

    deleteHolder ( lHolder );

    return lLength;
}

/****************************************************************************
NAME
	bccmdSize
        bccmdArraySize

PARAMETERS
        The bccmd varid, or varid and the data

RETURNS
        The number of bytes in the command pdu generated by bccmd_generate().
****************************************************************************/

uint16 bccmdSize (uint16 varid)
{
    return BC_COMMAND_HEADER_SIZE;
}

uint16 bccmdArraySize (uint16 varid, PARAMETER_ARRAY data, uint16 *extra_words)
{
    uint16 length;
    length = BC_COMMAND_HEADER_SIZE;
    *extra_words = 0;
    switch (varid)
    {
    case BCCMDVARID_MEMORY:
    case BCCMDVARID_PS:
    case BCCMDVARID_E2_APP_DATA:
        length += ( (uint16)data[1] * 2 ) - 2 ;
        break;
    case BCCMDVARID_BUFFER:
        length += ( (uint16)data[2] * 2 ) - 2 ;
        break;
    case BCCMDVARID_I2C_TRANSFER:
        length += (( (uint16)data[1] + (uint16)data[2] ) * 2) + 2;
        break;
    case BCCMDVARID_RADIOTEST:
	/*
	 * For RADIOTEST_CFG_HOPPING_SEQUENCE we have a five-word
	 * argument instead of three, i.e. an extra four bytes.
	 */
	if ((uint16)data[0] == RADIOTEST_CFG_HOPPING_SEQUENCE)
	{
	    *extra_words += 2;
	    length += 4;
	}
	break;
    case BCCMDVARID_PSSIZE:
    case BCCMDVARID_PSNEXT:
    case BCCMDVARID_ADCRES:
    case BCCMDVARID_CRYPTKEYLEN:
    case BCCMDVARID_PICONET_INSTANT:
    case BCCMDVARID_GET_CLR_EVT:
    case BCCMDVARID_GET_NEXT_BUILDDEF:
    case BCCMDVARID_PSCLRS:
    case BCCMDVARID_E2_DEVICE:
    case BCCMDVARID_ENABLE_AFH:
    case BCCMDVARID_BER_THRESHOLD:
    case BCCMDVARID_PS_MEMORY_TYPE:
        /* the packet is just the normal size */
        break;
    case BCCMDVARID_REMOTE_SNIFF_LIMITS:
	length += 4;
	break;
    default:
        /* unknown special case */
        return 0;
        break;
    }

    /* ensure that the minimum length we send is COMMAND_HEADER_SIZE */
    if (length < BC_COMMAND_HEADER_SIZE)
        length = BC_COMMAND_HEADER_SIZE;

    /* ensure an even return value which contains the PDU, so add one if length is odd */
    return length + (length & 1);
}

/****************************************************************************
NAME
	hqAnalyse  -  analyses wire format hc->h commands.

PARAMETERS
        a buffer to be analysed, and a pointer to a PARAMETER_ARRAY,
        which will be allocated in the procedure.  The buffer will be
        amended to make it the response to send back to the host controller,
        so that it is not necessary to create a new response pdu.

RETURNS
	The number of elements in the array.  Zero is a failure.
****************************************************************************/

PARAMETER_ARRAY hqAnalyse( uint8 * pdu_buffer, uint32 length )
{
    return pcAnalyse (pdu_buffer, length, false);
}

/****************************************************************************
NAME
	bccmdAnalyse  -  analyses wire format responses to h->hc commands.

PARAMETERS
        a buffer to be analysed, and a pointer to a PARAMETER_ARRAY,
        which will be allocated in the procedure.

RETURNS
	The number of elements in the array.  Zero is a failure.
****************************************************************************/

PARAMETER_ARRAY bccmdAnalyse( uint8 * pdu_buffer , uint32 length )
{
    return pcAnalyse (pdu_buffer, length , true);
}

/****************************************************************************
NAME
	pcAnalyse  -  analyses wire format pdus.

PARAMETERS
        a buffer to be analysed, a pointer to a PARAMETER_ARRAY, which will
        be allocated in the procedure, and a boolean to say whether the pdu
        is bccmd (true) or hq (false).  For hq, the buffer will be amended
        to make it the response to send back to the host controller, so
        that it is not necessary to create a new response pdu.

RETURNS
	The number of elements in the array.  Zero is a failure.
****************************************************************************/

PARAMETER_ARRAY pcAnalyse( uint8 * pduBuffer , uint32 length, bool out)
{
    PDU_AND_PARAMETER_HOLDER * lHolder;
    PACKER_PDU_TBL *lTable;
    uint8 lTableSize;
    uint16 lSpecialStart;

    uint8 lRowCounter;
    uint8 *lStatusPointer;
    uint16 lType, lpduLength, lSequenceNumber, lVarid, lStatus;
    PARAMETER_ARRAY lEventParameters;

    if (out)
    {
        lTable = bccmd_tbl;
        lTableSize = BCCMD_TBL_SIZE;
        lSpecialStart = BCCMDVARID_SPECIAL_START;
    }
    else
    {
        lTable = hq_tbl;
        lTableSize = HQ_TBL_SIZE;
        lSpecialStart = HQVARID_SPECIAL_START;
    }

    lHolder = createHolder ( pduBuffer , NULL );
    lHolder->pduLength = length;

    if ( lHolder->pduLength > 10 )
    {
        /*
        **  Read the first few bytes to get the size of the packet and
        **  the varid.  For HQ, re-write the packet as a response.
        */
        lType            = READ_2BYTES ( lHolder->pduIndex );
        if (!out) /* hq */
            WRITE_2BYTES ( lHolder->pduIndex , HQPDU_GETRESP );
        lHolder->pduIndex += 2;

        lpduLength       = READ_2BYTES ( lHolder->pduIndex );
        lHolder->pduIndex += 2;

        lSequenceNumber  = READ_2BYTES ( lHolder->pduIndex );
        lHolder->pduIndex += 2;

        lVarid           = READ_2BYTES ( lHolder->pduIndex );
        lHolder->pduIndex += 2;

        lStatus          = READ_2BYTES ( lHolder->pduIndex );
        lStatusPointer = lHolder->pduIndex;
        lHolder->pduIndex += 2;

        /*
        **  Allocate some memory for the outgoing parameter array.
        */
        lHolder->parameterCount = pcResponseSize( lVarid , lpduLength , out );
        lHolder->parameters = (PARAMETER_ARRAY) getpdu ( sizeof(PARAMETER_VALUE) * lHolder->parameterCount );
        lHolder->parameters[0] = lHolder->parameterCount;

        /*
        **  Refer to an external table which tells us what the incoming
        **  format should be.
        */
        for ( lRowCounter = 0 ; lRowCounter < lTableSize ; lRowCounter ++ )
        {
            if (lTable[lRowCounter].code == lVarid)
                break;
            if (   (lTable[lRowCounter].code == (uint16)(lVarid & 0x1C00))
                && ((lVarid & 0x1C00) < lSpecialStart)
                )
                break;
        }

        if (lRowCounter != lTableSize)
        {
            /*
            **  Send the buffer pointer and the param array in by reference.
            **  this is for use in recursion code within the function.
            */
            lHolder->parameters[1] = lVarid;
            lHolder->parameters[2] = lSequenceNumber;
            lHolder->parameters[3] = lStatus;
            lHolder->parameterIndex = lHolder->parameters + 4;
            if (!analyseParameters ( lTable[lRowCounter].format , lHolder ) )
            {
                if (!out)
                    WRITE_2BYTES( lStatusPointer , HQPDU_STAT_BAD_PDU );
                lHolder->parameters[0] = 0;
            }
	    /*
	     * Sigh.  This is the third completely independent point in
	     * the code where we check the size of a PDU for a given
	     * format when all we are trying to do is make all the
	     * information supplied go from btcli to the chip and when
	     * both already perform their own tests on the length of
	     * data.
	     *
	     * We could remove the top of this without changing the
	     * basic algorithm by allocating space based on the
	     * PDU size, but still counting the parameters to go in it
	     * based on the format deduced.  Unfortunately, I simply
	     * can't be bothered.
	     *
	     * Luckily, this version of the code is rapidly going out
	     * of fashion.
	     */
	    if (out && lVarid == BCCMDVARID_RADIOTEST &&
		lHolder->parameterIndex > lHolder->parameters+4 &&
		lHolder->parameters[4] == RADIOTEST_CFG_HOPPING_SEQUENCE)
	    {
		/*
		 * This particular form has extra words.
		 */
		uint16 extra_words = 2, i, new_count;
		PARAMETER_ARRAY new_array;

		/*
		 * Assign a larger parameter holder.
		 */
		new_count = lHolder->parameterCount + extra_words;
		new_array = (PARAMETER_ARRAY)
		    getpdu(sizeof(PARAMETER_VALUE) * new_count);
		for (i = 0; i < lHolder->parameterCount; i++)
		    new_array[i] = lHolder->parameters[i];

		lHolder->parameterIndex +=
		    new_array - lHolder->parameters;
		freepdu(lHolder->parameters);
		lHolder->parameters = new_array;
		lHolder->parameters[0] = lHolder->parameterCount = new_count;

		/*
		 * Now copy the extra parameters.
		 */
		while (extra_words--)
		{
		    if (lHolder->pduIndex >= lHolder->pdu + lHolder->pduLength)
		    {
			onBadPDU(BP_OUT_OF_DATA, lHolder->pdu,
				 lHolder->pduLength);
			lHolder->parameters[0] = 0;
			break;
		    }

		    *lHolder->parameterIndex++ =
			READ_2BYTES(lHolder->pduIndex);
		    lHolder->pduIndex += 2;
		}
	    }
        }
        else
        {
            if (!out)
                WRITE_2BYTES( lStatusPointer , HQPDU_STAT_NO_SUCH_VARID );
            onBadPDU ( BP_NO_SUCH_VARID , lHolder->pdu , lHolder->pduLength );
            lHolder->parameters[0] = 0;
        }

        if ( out && lType != BCCMDPDU_GETRESP )
        {
            onBadPDU ( BP_WRONG_TYPE , lHolder->pdu , lHolder->pduLength );
            lHolder->parameters[0] = 0;
        }

        if ( !out && lType == HQPDU_GETRESP )
        {
            onBadPDU ( BP_WRONG_TYPE , lHolder->pdu , lHolder->pduLength );
            lHolder->parameters[0] = 0;
        }

        if ( lHolder->parameters[0] == 0 )
        {
            freepdu (lHolder->parameters);
            lHolder->parameters = NULL;
        }
        else
            /*  
            **  since the allocated value is really just an estimate, calculate
            **  the data count to put into the first element of the PARAMETER_ARRAY.
            **
            **  data count is the size of the array
            */
            lHolder->parameters[0] = lHolder->parameterIndex - lHolder->parameters;

    }  // if the pdu length is > 10
    else
        onBadPDU ( BP_PC_TOO_SMALL , lHolder->pdu , lHolder->pduLength );

    lEventParameters = lHolder->parameters;
    deleteHolder (lHolder);

    return lEventParameters;
}

uint8 pcResponseSize( uint16 aVarid , uint16 apduLength, bool bccmd )
{
    /*
    **  Parameter Array format is uint32s, first one containing a data count,
    **  and then the subsequent ones containing the data.
    **
    **  Then we get 3 to hold the 5 word header (ignoring type and length),
    **  so that is a basic size of 4 elements.
    **
    **  max pduLength is 64 (words).
    */
    uint8 length;

    length = 4;
    if (bccmd)
    {
        if ( ( aVarid & 0x1C00 ) == BCCMDVARID_SPECIAL_START )
        {
            /*
            **  each word is returned, except the type and length,
            **  and then there is one extra.  But in memory and PS
            **  there is a dummy word.
            */
            switch (aVarid)
            {
            case BCCMDVARID_MEMORY:
                length = (uint8) apduLength - 2;
                break;
            case BCCMDVARID_PS:
            case BCCMDVARID_BUFFER:
            case BCCMDVARID_RADIOTEST:
	    case BCCMDVARID_E2_APP_DATA:
                length = (uint8) apduLength - 1;
                break;
            default:
                length = (uint8) apduLength;
                break;
            }
        }
        else
            /*  Standard sets and reads have 1 data word. */
            length ++;
    }
    else
        /*  Biggest HQ packet is 7 data words.  */
        length += 7;

    return length;
}
