/****************************************************************************
**
**  File: hci.c
**
**            (C) CSR 2000
**
**  Purpose:  contains C procedures for hci packet generation and analysis
**
**  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/Packer/HCIPacker/hci.c#28 $
**
****************************************************************************/

/****************************************************************************
**  Modification history
***************************************************************************** 
**  1.1   8:may:00	ajh	Created.
**  1.2   9:aug:00      ajh     Carl and Neil have agreed a data parameter
**                              format.  Time for a rewrite.
*/

#include "HCIPacker.h"
#include "hci_private.h"


#include "memfuncs.h"

#ifdef _INDEBUG

void * IDO;

void setHIDO ( void * obj )
{
    IDO = obj;
}

#endif

/****************************************************************************
NAME
	hci_cmd_generate  -  generates wire format commands for hostside hci

PARAMETERS
        the address of a pointer to a buffer (to be pointer at a buffer
        by the procedure), the HCI op-code and parameter array.

RETURNS
        The size of the buffer writtern.  Returns 0 on failure to create
        the command pdu successfully.
****************************************************************************/
uint32 generateHCICommand ( uint8 **aBufferPointer , PARAMETER_ARRAY lAllParameters )
{
    PDU_AND_PARAMETER_HOLDER * lHolder;
    uint32 lLength;
    uint16 lHCIOpCode;

    uint8  lRowCounter;

    /*  initialise the pdu holding structure */
    lHolder = createHolder ( NULL , lAllParameters );

    /* initialise return value */
    lLength = 0;

    if ( lHolder != NULL )
    {
        lHolder->parameterCount = lHolder->parameters[0];
        if ( lHolder->parameterCount != 0 )
        {
            lHCIOpCode = (uint16) lHolder->parameters[1];
            lHolder->parameterIndex += 2;

            /*
            **   Step through the command table to find the format.
            */
            for ( lRowCounter = 0 ;
                ( lRowCounter < CMD_TABLE_SIZE) && (cmd_table[lRowCounter].code != lHCIOpCode);
	          lRowCounter ++ );

            if (lRowCounter != CMD_TABLE_SIZE)
            {
                /* Allocate some memory for the command buffer */
                if ( ( lHCIOpCode == HCI_WRITE_STORED_LINK_KEY )
                  || ( lHCIOpCode == HCI_HOST_NUM_COMPLETED_PACKETS )
                  || ( lHCIOpCode == HCI_WRITE_CURRENT_IAC_LAP ) )
                    lHolder->pduLength = hciCommandArraySize ( lHCIOpCode, lHolder->parameterIndex );
                else if (lHCIOpCode == /*HCI_SET_AFH_CHANNEL_CLASSIFICATION*/0x0c3f)
                    lHolder->pduLength = 13;
                else
                    lHolder->pduLength = hciCommandSize ( lHCIOpCode , cmd_table[lRowCounter].format);

                if (lHolder->pduLength != 0)
                {
                    lHolder->pdu = (uint8*) getpdu(lHolder->pduLength);

                    lHolder->pduIndex = lHolder->pdu;
                    WRITE_2BYTES ( (lHolder->pduIndex) , lHCIOpCode );
                    lHolder->pduIndex += 2;
                    WRITE_BYTE ( (lHolder->pduIndex) , (lHolder->pduLength - HCI_COMMAND_HEADER_SIZE) );
                    lHolder->pduIndex ++;

                    if (buildBuffer (cmd_table[lRowCounter].format, lHolder ))
                    {
                        if (lHCIOpCode == HCI_SET_EVENT_FILTER || lHCIOpCode == HCI_TUNNEL_COMMAND )
                        {
                            /* the length for this command is only an estimate,
                               so set it to the correct size */
                            lHolder->pduLength = lHolder->pduIndex - lHolder->pdu;
                            WRITE_BYTE ( (lHolder->pdu + 2) , (lHolder->pduLength - HCI_COMMAND_HEADER_SIZE) );
                        }
                    }
                    else
                        lHolder->pduLength = 0;

                    if ( lHolder->pduLength == 0 )
                        freepdu (lHolder->pdu);
                }
                else
                    lHolder->pduLength = 0;
            }
        }
    }

    lLength = lHolder->pduLength;
    *aBufferPointer = lHolder->pdu;
    deleteHolder ( lHolder );

    return lLength;
}

/****************************************************************************
NAME
	hci_cmd_size

PARAMETERS
        The HCI op-code.

RETURNS
        The number of bytes in the command pdu generated by hci_cmd_generate().
****************************************************************************/

uint32 hciCommandSize (uint16 hci_opcode , uint32 format )
{
    uint32 length;
    uint8 foffset;

    length = HCI_COMMAND_HEADER_SIZE;

    for ( foffset = 0;
        ( foffset < (8 * sizeof(format)) ) && ( ( format >> foffset ) != 0 );
	  foffset += 4)
    {
	switch ( (format >> foffset) & 0xF )
	{
	case 1:
	    length += 1;
	    break;
	case 2:
	    length += 2;
	    break;
	case 3:
	    length += 3;
	    break;
	case 4:
	    length += 4;
	    break;
	case 6:
	    length += 6;
	    break;
        case 7:
            length += 16;
	    break;
        case 8:
            length += 8;
	    break;
        case 0xA:
            length += 248;
	    break;
        /*
        ** C shouldn't happen in a command, and commands with a B are handled separately.
        */
        case 0xD:
            /* sufficient length for all possible parameters */
            length += 9;
            break;
	default:
	    return 0; /* Hmmm, unexpected format */
	    break;
	}
    }

    return length;
}

uint32 hciCommandArraySize (uint16 hci_opcode, PARAMETER_ARRAY data)
{
    uint32 length;
    switch (hci_opcode)
    {
    case HCI_WRITE_STORED_LINK_KEY:
        length = (uint16) (22 * (data[0] & 0xFF));
        break;
    case HCI_HOST_NUM_COMPLETED_PACKETS:
        length = (uint16) (4 * (data[0] & 0xFF));
        break;
    case HCI_WRITE_CURRENT_IAC_LAP:
        length = (uint16) (3 * (data[0] & 0xFF));
        break;
    default:
        length = 0;
        break;
    }
    return HCI_COMMAND_HEADER_SIZE + 1 + length;
}

bool generateHCIEventFilterCommand ( PDU_AND_PARAMETER_HOLDER * aHolder )
{
    uint32 lFormat;
    switch ( *(aHolder->parameterIndex) & 0xFF)
    {
    case 0x00:
        lFormat = 1;
        break;
    case 0x01:
        switch ( *(aHolder->parameterIndex + 1 ) & 0xFF)
        {
        case 0x00:
            lFormat = 0x11;
            break;
        case 0x01:
            lFormat = 0x3311;
            break;
        case 0x02:
            lFormat = 0x611;
            break;
        default:
            return false;
            break;
        }
        break;
    case 0x02:
        switch ( *( aHolder->parameterIndex + 1 ) & 0xFF)
        {
        case 0x00:
            lFormat = 0x111;
            break;
        case 0x01:
            lFormat = 0x13311;
            break;
        case 0x02:
            lFormat = 0x1611;
            break;
        default:
            return false;
            break;
        }
        break;
    default:
        return false;
        break;
    }
    return buildBuffer ( lFormat , aHolder );
}

/****************************************************************************
NAME
	analyseHCIEvent  -  analyses wire format events for hostside hci

RETURNS
	true if successful, false otherwise.
*/

extern PARAMETER_ARRAY analyseHCIEvent ( uint8 * apdu , uint32 aLength )
{
    PDU_AND_PARAMETER_HOLDER * lHolder;

    PARAMETER_ARRAY lEventParameters;

    uint8 lRowCounter;
    uint8 lEventCode, lLength;

    /* 1.1 TEMP FIX - local format variable which we can change if
                      we reckon we are in the 1.1 paradigm. */
    uint32 lFormat;

    lHolder = createHolder ( apdu , NULL );
    lHolder->pduLength = aLength;
    /*
    **  Read the first few bytes to get the size of the packet and
    **  the param_array[0].u8code.
    */
    lEventCode = READ_BYTE ( lHolder->pduIndex );
    lHolder->pduIndex ++;
    lLength = READ_BYTE ( lHolder->pduIndex );
    lHolder->pduIndex ++;

    /*  Sanity check the length */
    if ( lHolder->pduLength == (uint32)( lLength + 2 ) )
    {
        /*
        **  Refer to an external table which tells us what the incoming
        **  format should be.
        */
        for ( lRowCounter = 0 ;
            ( lRowCounter < EVT_TABLE_SIZE) && (evt_table[lRowCounter].code != lEventCode);
	      lRowCounter ++ );
        if (lRowCounter == EVT_TABLE_SIZE)
        {
            onBadPDU ( BP_NO_SUCH_EVENT_CODE , lHolder->pdu , lHolder->pduLength );
            return 0;
        }
        /* 1.1 TEMP FIX */
        else
            lFormat = evt_table[lRowCounter].format;

        /* 1.1 TEMP FIX 
         *
         * We need to support 1.0B and 1.1 HCI events, to accomodate this
         * check the length of the event and adjust accordingly.
         * The two events which have different parameter lengths are
         * HCI_Link_Key_Notification_Event (new key type parameter) and
         * HCI_Inquiry_Complete(Num Responses removed).
         */
        switch (lHolder->pdu[0])
        {
        case 0x01: /* HCI_EV_INQUIRY_COMPLETE=0x1 */
            /*
             * If the Inquiry Complete Length is 0x01 the Num Responses is
             * missing hence we are a 1.1 HCI so change aFormat.
             */
            if (lHolder->pdu[1] == 1)
            {
                lFormat = 0x00000001;
            }
            break;
        case  0x18: /* HCI_EV_LINK_KEY_NOTIFICATION=0x18 */
            /*
             * If the Length is 23 the Link Key Type field is present
             * hence we are a 1.1 HCI so change aFormat.
             */
            if (lHolder->pdu[1] == 23)
            {
                lFormat = 0x00000176;
            }
            break;
        }


        /*
        **  Allocate some memory for the outgoing parameter array.
        **  A word for each of the elements, and an extra word for the length.
        */
        lHolder->parameterCount = 1 + hciEventSize ( lEventCode , lHolder->pduLength , lFormat );
        if (lHolder->pdu[0] == 0x2c)   /* HCI_EV_SYNCHRONOUS_CONNECTION_COMPLETE */
        {
            lHolder->parameterCount++;
        }
        if (lHolder->pdu[0] == 0x21)   /* HCI_EV_FLOW_SPECIFICATION_COMPLETE */
        {
            lHolder->parameterCount += 4;
        }
        lHolder->parameters = (PARAMETER_ARRAY) getpdu 
            (sizeof(PARAMETER_VALUE) * lHolder->parameterCount );
        lHolder->parameterIndex = lHolder->parameters + 1;
        lHolder->parameters[0] = lHolder->parameterCount;
        /*
        **  Send the buffer pointer and the param array in by reference.
        **  this is for use in recursion code within the function.
        */
        lHolder->parameterIndex[0] = lEventCode;
        lHolder->parameterIndex ++;
        if ( analyseParameters ( lFormat , lHolder ) )
            lHolder->parameters[0] = lHolder->parameterCount = (lHolder->parameterIndex - lHolder->parameters);
        else
        {
            freepdu (lHolder->parameters);
            lHolder->parameters = NULL;
            return 0;
        }
    }  /* sanity check the length */
    else
        onBadPDU ( BP_HCI_WRONG_SIZE , lHolder->pdu , lHolder->pduLength );

    if (lHolder->pdu[0] == 0x21) /* HCI_EV_FLOW_SPECIFICATION_COMPLETE */
    {
        lHolder->parameters[lHolder->parameterCount] = lHolder->pdu[20] |
          lHolder->pdu[21] << 8 |
          (uint32) lHolder->pdu[22] << 16 |
          (uint32) lHolder->pdu[23] << 24;
        lHolder->parameterCount++;
        lHolder->parameters[0]++;
    }
    if (lHolder->pdu[0] == 0x2c) /* HCI_EV_SYNCHRONOUS_CONNECTION_COMPLETE */
    {
        lHolder->parameters[lHolder->parameterCount] = lHolder->pdu[18];
        lHolder->parameterCount++;
        lHolder->parameters[0]++;
    }
    lEventParameters = lHolder->parameters;
    deleteHolder ( lHolder );

    return lEventParameters;
}

uint32 hciEventSize( uint8 aEventCode , uint32 aLength , uint32 aFormat )
{
    /*
    **  Parameter Array format is uint32s, first one containing a data count,
    **  and then the subsequent ones containing the data. 
    **
    **  A basic event is  1 (size) + 1 (event code) + parameters (read from the format)
    **
    **  Events may not be more that 255 bytes long, so in the worst case
    **  we have to deal with 255 elements, plus an extra at the start.
    */
    int foffset;
    uint32 length;

    length = 1;
    /*  Awkward cases, ie contain A, B or C  */
    if ( ( aEventCode == HCI_EV_NUMBER_COMPLETED_PACKETS )
      || ( aEventCode == HCI_EV_INQUIRY_RESULT )
      || ( aEventCode == HCI_EV_INQUIRY_RESULT_WITH_RSSI )
      || ( aEventCode == HCI_EV_RETURN_LINK_KEYS )
      || ( aEventCode == HCI_EV_COMMAND_COMPLETE )
      || ( aEventCode == HCI_EV_REMOTE_NAME_REQUEST_COMPLETE )
      || ( aEventCode == HCI_EV_LOOPBACK_COMMAND )
      || ( aEventCode == HCI_EV_MANUFACTURER_EXTENSION ) )
        length = aLength + 1; /* this should cover it */
    else
    {
        /*  Every thing else  */
        for ( foffset = 0;
            ( foffset < (8 * sizeof(aFormat)) ) && ( ( aFormat >> foffset ) != 0 );
	      foffset += 4)
        {
	    switch ( (aFormat >> foffset) & 0xF )
	    {
	    case 1:
	        length += 1;
	        break;
	    case 2:
	        length += 1;
	        break;
	    case 3:
	        length += 1;
	        break;
	    case 4:
	        length += 1;
	        break;
	    case 6:
	        length += 3;
	        break;
            case 7:
                length += 16;
	        break;
            case 8:
                length += 8;
	        break;
            /*
            **  D shouldn't happen in an event, and events with a A, B and C are handled separately.
            */
	    default:
	        return 0; /* Hmmm, unexpected format */
	        break;
	    }
        }
    }

    return length;
}

/****************************************************************************
NAME
	analyseHCICommandCompleteEvent  -  analyses wire format return parameters for
			                   command complete events.

RETURNS
	true if successful, or false if unsuccessful.
*/

bool analyseHCICommandCompleteEvent ( uint16 Opcode,
                                      PDU_AND_PARAMETER_HOLDER * aHolder )
{
    PACKER_PDU_TBL *rpointer;

    /*
    **  Refer to an external table which tells us what the incoming
    **  format should be.  Initially the table is zero terminated, for
    **  simplicity of code.  Eventually we should know the number 
    **  of elements in the table and go that way on.
    */
    
    for ( rpointer = comcom_table ;
          rpointer->code && (rpointer->code != Opcode);
	  rpointer ++ );
    if (rpointer->code == 0)
        return false;

    return analyseParameters ( rpointer->format , aHolder );
}

/****************************************************************************
NAME
	writeHCIDataHeader  -  generates wire format data header

RETURNS
        true (1) if successful, false (0) if something goes wrong.
****************************************************************************/

DEVICECONTROLLER_API void writeHCIDataHeader( uint8 *write_point ,
                                 uint16 connection,
                                 uint8 packet_boundary_flag,
                                 uint8 broadcast_flag ,
                                 uint16 length,
                                 enum CONNECTION_TYPE connectionType)
{
    uint16 first_two_bytes;

    first_two_bytes = (connection & 0x0FFF);

    if (packet_boundary_flag)
	first_two_bytes |= 0x2000;
    else
	first_two_bytes |= 0x1000;

    /* can be 0, 1 or 2 */
    if (broadcast_flag == 1)
	first_two_bytes |= 0x4000;
    else if (broadcast_flag == 2)
	first_two_bytes |= 0x8000;

    WRITE_2BYTES ( write_point , first_two_bytes );
    write_point += 2;

    if (connectionType == conn_type_sco)
        WRITE_BYTE(write_point,length);
    else
        WRITE_2BYTES(write_point,length);
}

/****************************************************************************
NAME
	analyseHCIDataHeader  -  analyses wire format data headers.

RETURNS
	a pointer to some data.  Or null if the header is corrupt.
****************************************************************************/

DEVICECONTROLLER_API uint8 * analyseHCIDataHeader ( uint8 * aPointerToBufferContainingApdu,
                                       uint32 aLengthOfBufferContainingApdu,
                                       uint8 SCO,
                                       uint16 * connection_handle_pointer,
                                       uint8 * packet_boundary_flag_pointer,
                                       uint8 * broadcast_flag_pointer,
                                       uint16 *length_pointer)
{
    uint8 * pdu_buffer = aPointerToBufferContainingApdu;
    uint16 first_two_bytes, total_length;

    /*
    **  Is there a Header...
    */
    if ( aLengthOfBufferContainingApdu >= (uint8)( SCO ? HCI_VOICE_HEADER_SIZE : HCI_DATA_HEADER_SIZE ) )
    {
        /*
        **  Read the handle and stuff
        */
        first_two_bytes = READ_2BYTES (pdu_buffer);
        pdu_buffer += 2;
        /*
        **  find the data packet length
        */
        if (SCO)
        {
            *length_pointer = READ_BYTE (pdu_buffer);
            pdu_buffer ++;
            total_length = *length_pointer + HCI_VOICE_HEADER_SIZE;
        }
        else
        {
            *length_pointer = READ_2BYTES (pdu_buffer);
            pdu_buffer += 2;
            total_length = *length_pointer + HCI_DATA_HEADER_SIZE;
        }
    }
    else
        pdu_buffer = 0;

    /*
    **  analyse and return values if the lengths look OK.
    */
    if ( pdu_buffer && total_length == aLengthOfBufferContainingApdu )
    {
        *connection_handle_pointer = first_two_bytes & 0x0FFF;
        if (!SCO)
        {
            uint8 pb = ( first_two_bytes & 0x3000 ) >> 12;
            uint8 bc = first_two_bytes >> 14;
            if (pb == 2)
                *packet_boundary_flag_pointer = true;
            else if ( pb == 1 )
                *packet_boundary_flag_pointer = false;
            else
            {
                onBadPDU ( BP_HCI_DATA_RESERVED_PBF , aPointerToBufferContainingApdu , aLengthOfBufferContainingApdu );
                pdu_buffer = 0;
            }

            if ( pdu_buffer )
            {
                if ( bc != 0x3 )
                    *broadcast_flag_pointer = bc;
                else
                {
                    onBadPDU ( BP_HCI_DATA_RESERVED_BCF , aPointerToBufferContainingApdu , aLengthOfBufferContainingApdu );
                    pdu_buffer = 0;
                }
            }
        }
    }
    else
    {
        onBadPDU ( BP_HCI_DATA_WRONG_SIZE , aPointerToBufferContainingApdu , aLengthOfBufferContainingApdu );
        pdu_buffer = 0;
    }
    return pdu_buffer;
}
