/****************************************************************************
**
**  FILE:	packer.c
**
**            (C) CSR 2000
**
**  PURPOSE:	Contains generic code shared between HCI packing
**		and BC packing routines.
**
**MODIFICATION HISTORY
**	#25     1:mar:04 pws    B-2096: Support BCCMDVARID_REMOTE_SNIFF_LIMITS
**  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/Packer/packer.c#25 $
**
****************************************************************************/

/****************************************************************************
**  Modification history
***************************************************************************** 
**  1.1  22:june:00	ajh	Created from common code in HCI and BC
**                              packers.
**  1.2   9:aug:00      ajh     Carl and Neil have agreed a data parameter
**                              format.  Time for a rewrite.
*/

#include "HCIPacker.h"
#include "hci_private.h"
#include "packer.h"

#include <stdlib.h> /* NULL */

#include "memfuncs.h"

#ifdef _INDEBUG

void * IDO;

void setPIDO ( void * obj )
{
    IDO = obj;
}

#endif

/****************************************************************************
NAME
	encode_bytes  -  copies bytewise data into the PARAMETER_ARRAY
	                 from a buffer

PARAMETERS
        a buffer, a length and a parameter array list.

RETURNS
        true if things work out, false if there isn't enough space...
****************************************************************************/
DEVICECONTROLLER_API bool  encodeBytes ( uint8 *aDataStart ,
                            uint32 aDataLength ,
                            PARAMETER_ARRAY aParameterArray ,
                            PARAMETER_ARRAY *aCurrentIndex )
{
    uint32 lArrayLength ;
    uint32 lCounter ;
    
    lArrayLength = aParameterArray[0] ;

    /*  Check we have the space for all that data */
    if ( aParameterArray + lArrayLength >= *aCurrentIndex + aDataLength )
    {
        /*  write the data, sparsely placed */
        for ( lCounter = 0 ; lCounter < aDataLength ; lCounter ++ )
            *(*aCurrentIndex)++ = *aDataStart++;
    }
    else
        return false;

    return true;
}

/****************************************************************************
NAME
	decode_bytes  -  copies bytewise data out of the PARAMETER_ARRAY
	                 to a buffer

PARAMETERS
        a buffer and a parameter array list.

RETURNS
        true if things work out, false if it gets confused...
****************************************************************************/
DEVICECONTROLLER_API uint32 decodeBytes ( uint8 **aPointerToBuffer ,
                             uint32 aLengthOfReceivingBuffer ,
                             PARAMETER_ARRAY aParameterArray ,
                             PARAMETER_ARRAY *aCurrentOutdex )
{
    uint32 lHowMuchData;
    uint32 lCounter;

    /* total is how much is in the whole array (in the first element) less */
    /* how far we've read already */
    lHowMuchData = aParameterArray[0] + aParameterArray - *aCurrentOutdex ;

    if ( lHowMuchData != 0 )
    {
        if ( *aPointerToBuffer == NULL || aLengthOfReceivingBuffer == 0 )
        {
            /* allocate the memory */
            *aPointerToBuffer = (uint8*) getpdu ( lHowMuchData );
        }
        if ( lHowMuchData > aLengthOfReceivingBuffer )
            lHowMuchData = aLengthOfReceivingBuffer;
        for ( lCounter = 0 ; lCounter < lHowMuchData ; lCounter++ )
            (*aPointerToBuffer)[lCounter] = (uint8) *(*aCurrentOutdex)++;
    }

    return lHowMuchData;
}

/****************************************************************************
NAME
        createHolder - initialises a structure to holder a pdu and
                       PARAMETER_ARRAY together

PARAMETERS
        a pointer to a pdu OR a parameter array

RETURNS
        A pointer to a holder.
****************************************************************************/
PDU_AND_PARAMETER_HOLDER * createHolder ( uint8 *apdu , PARAMETER_ARRAY aParameters )
{
    PDU_AND_PARAMETER_HOLDER * lNewThing;

    if ( apdu == NULL && aParameters == NULL )
        return NULL;

    lNewThing = (PDU_AND_PARAMETER_HOLDER *)
                    getpdu ( sizeof ( PDU_AND_PARAMETER_HOLDER ) );
    lNewThing->pdu = apdu;
    lNewThing->pduIndex = apdu;
    lNewThing->pduLength = 0;

    lNewThing->parameters = aParameters;
    lNewThing->parameterIndex = aParameters;
    if (aParameters != NULL)
        lNewThing->parameterCount = lNewThing->parameters[0];
    else
        lNewThing->parameterCount = 0;

    return lNewThing;
}

/****************************************************************************
NAME
        deleteHolder - deletes the structure.

PARAMETERS
        A pointer to a holder.

RETURNS
        Void.
****************************************************************************/
void deleteHolder ( PDU_AND_PARAMETER_HOLDER * aHolder )
{
     freepdu (aHolder);
}

/****************************************************************************
GLOBAL VARIABLES - well, file wide variables, any how.
****************************************************************************/
static void (*gBadPDUCallBackFn) ( BadPDUReason a, uint8* b, uint32 c, void * d) = NULL;
static void * gBadPDUCallBackParameters = NULL;

/****************************************************************************
NAME
        setBadPDUCallback - sets the bad pdu callback.

PARAMETERS
        A pointer to a bad pdu call back and some parameters to send it.

RETURNS
        Void.
****************************************************************************/
void setBadPDUCallback ( void (*onBadPDUFunction) ( BadPDUReason , uint8* , uint32 , void * ) , void * onBadPDUParameters )
{
    gBadPDUCallBackFn = onBadPDUFunction;
    gBadPDUCallBackParameters = onBadPDUParameters;
}

/****************************************************************************
NAME
        onBadPDU - calls the bad pdu callback.

PARAMETERS
        A pointer to a bad pdu and its length.

RETURNS
        Void.
****************************************************************************/
void onBadPDU ( BadPDUReason aWhy , uint8 * apdu , uint32 aLength )
{
    if ( gBadPDUCallBackFn != NULL )
        gBadPDUCallBackFn ( aWhy , apdu , aLength , gBadPDUCallBackParameters );
}

/****************************************************************************
NAME
	load_buffer  -  generates wire format commands for hostside hci
                        and bccmd

PARAMETERS
        a format, a buffer and a parameter array list.

RETURNS
        true if things work out, false if it gets confused...
****************************************************************************/
bool buildBuffer ( uint32 aFormat ,
                   PDU_AND_PARAMETER_HOLDER * aHolder )
{
    uint8 foffset;
    uint16 counter, element, max[6];
    int max_count = 0;
    uint32 lArrayLength;

    for ( foffset = 0;
        ( foffset < (8 * sizeof(aFormat)) )
          && ( ( aFormat >> foffset ) != 0 )
          && ( aHolder->parameterIndex < aHolder->parameters + aHolder->parameterCount );
	  foffset += 4)
    {
	switch ((aFormat >> foffset) & 0xF)
	{
	case 1:
		    WRITE_BYTE ( (aHolder->pduIndex) , (*aHolder->parameterIndex & 0xFF) );
		    (aHolder->pduIndex) ++;
            aHolder->parameterIndex ++;
	    break;
	case 9:
            max[max_count] = *aHolder->parameterIndex & 0xFFFF;
            max_count ++;
            /* Fall through */
	case 2:
            WRITE_2BYTES ( (aHolder->pduIndex) , (*aHolder->parameterIndex & 0xFFFF) );
            aHolder->pduIndex += 2;
            aHolder->parameterIndex ++;
	    break;
	case 3:
            WRITE_3BYTES ( (aHolder->pduIndex) , (*aHolder->parameterIndex & 0xFFFFFF) );
            aHolder->pduIndex += 3;
            aHolder->parameterIndex ++;
	    break;
	case 4:
            WRITE_4BYTES ( (aHolder->pduIndex) , (*aHolder->parameterIndex) );
            aHolder->pduIndex += 4;
            aHolder->parameterIndex ++;
	    break;
	case 5:
            WRITE_4BCF_BYTES ( (aHolder->pduIndex) , (*aHolder->parameterIndex) );
            aHolder->pduIndex += 4;
            aHolder->parameterIndex ++;
	    break;
	case 6:
            /* BD_ADDR passed as 3 uint32s */
            WRITE_3BYTES ( (aHolder->pduIndex) , (*aHolder->parameterIndex & 0xFFFFFF) );
            WRITE_BYTE   ( (aHolder->pduIndex+3) , (*(aHolder->parameterIndex+1) & 0xFF) );
            WRITE_2BYTES ( (aHolder->pduIndex+4) , (*(aHolder->parameterIndex+2) & 0xFFFF) );
            aHolder->pduIndex += 6;
            aHolder->parameterIndex += 3 ;
            break;
        case 7:
		    /* a block of 16 bytes, least significant first */
            if ( !decodeBytes(&aHolder->pduIndex, 16 ,aHolder->parameters, &aHolder->parameterIndex) )
                return false;
            aHolder->pduIndex += 16;
	    break;
        case 8:
            /* 8 byte event mask, sent as a block of 8 bytes */
            if (!decodeBytes(&aHolder->pduIndex, 8 ,aHolder->parameters, &aHolder->parameterIndex) )
                return false;
            aHolder->pduIndex += 8;
	    break;
        case 0xA:
            /*  pointer to a buffer which runs to the end of the pdu */
            /*  Length of the buffer = aHolder->parameters + aHolder->parameters[0] - aHolder->parameterIndex */
            lArrayLength = ( aHolder->parameters + aHolder->parameters[0] - aHolder->parameterIndex );
            if ( !decodeBytes (&aHolder->pduIndex,
                                lArrayLength,
                                aHolder->parameters ,
                               &aHolder->parameterIndex) )
                return false;
            aHolder->pduIndex += lArrayLength;
            break;
        case 0xB:
            /*  Array, atm all HCI arrays have a uint8 number for how many. 
                BC arrays have a uint16.
                That number is held in the previous array element. */
            if ( max_count == 0 )
            {
                int max = (uint16)( *(aHolder->parameterIndex - 1) );
                for ( element = 0 ; element < max ; element ++ )
                {
                    if (!buildBuffer ((aFormat >> (foffset+4)) , aHolder ) )
                        return false;
                }
            }
            else
            {
                uint32 lFormat = (aFormat >> (foffset+4));
                for ( counter = 0 ; counter < max_count ; counter ++ )
                    for ( element = 0 ; element < max[counter] ; element ++ )
                    {
                        if (!buildBuffer ( lFormat , aHolder ) )
                            return false;
                    }
            }
            /*  must leave the loop now...  else we write another element */
            foffset = (8 * sizeof(aFormat));
            break;
        case 0xC:
		/* Command complete */
            /* This should only exist in HCI events, not commands */
            return false;
            break;
        case 0xD:
		/* event filter command */
            /* this deserves its own procedure... */
            if ( !generateHCIEventFilterCommand ( aHolder ) )
                return false;
            break;
	    case 0xE:
		    /* padding uint16 */
		    aHolder->pduIndex += 2;
            break;
	    default:
            /*  unknown formatting value */
            return false;
		break;
        } /* switch */
    }/* for */

    return true;
}

/***********************************************************************
NAME - analyse_parameters

FUNCTION - it analyses a buffer according to the format passed in
           as a uint32.
	   The formatting protocol is in the header files
           "hci_tables.h" and "BCPack_tables.h".
RETURNS
    true if everything looks OK, false if the procedure gets confused...
*************************************************************************/
uint8 analyseParameters ( uint32 aFormat ,
                          PDU_AND_PARAMETER_HOLDER * aHolder )
{
    uint8 foffset;
    uint16 recurse_counter, recurse_max[6];
    int recurse_max_counter = 0, max_counter;
    uint8 buffer_step;
	uint32 lFormat;

    for ( foffset = 0;
        ( foffset < (8 * sizeof(aFormat)) ) && ( ( aFormat >> foffset ) != 0 );
	  foffset += 4)
    {
	/* out of data */
	if (aHolder->pduIndex >= aHolder->pdu + aHolder->pduLength)
        {
            onBadPDU ( BP_OUT_OF_DATA , aHolder->pdu , aHolder->pduLength );
	    return false;
        }

	switch ( (uint8)( ( aFormat >> foffset ) & 0xF ) )
	{
	case 1:
	    *aHolder->parameterIndex ++ = READ_BYTE ( (aHolder->pduIndex) );
	    buffer_step = 1;
	    break;
	case 2:
	    *aHolder->parameterIndex ++ = READ_2BYTES ( (aHolder->pduIndex) );
	    buffer_step = 2;
	    break;
	case 3:
	    *aHolder->parameterIndex ++ = READ_3BYTES ( (aHolder->pduIndex) );
	    buffer_step = 3;
	    break;
	case 4:
	    *aHolder->parameterIndex ++ = READ_4BYTES ( (aHolder->pduIndex) );
	    buffer_step = 4;
	    break;
        case 5:
	    *aHolder->parameterIndex ++ = READ_4BCF_BYTES ( (aHolder->pduIndex) );
	    buffer_step = 4;
	    break;
	case 6:
            /* LAP, UAP, NAP */
	    *aHolder->parameterIndex ++ = READ_3BYTES ( (aHolder->pduIndex) );
	    *aHolder->parameterIndex ++ = READ_BYTE ( (aHolder->pduIndex+3) );
	    *aHolder->parameterIndex ++ = READ_2BYTES ( (aHolder->pduIndex+4) );
	    buffer_step = 6;
	    break;
	case 7:
	    buffer_step = 16;
            if ( !encodeBytes( aHolder->pduIndex ,
                               buffer_step ,
                               aHolder->parameters ,
                              &aHolder->parameterIndex ) )
            {
                onBadPDU ( BP_OUT_OF_DATA , aHolder->pdu , aHolder->pduLength );
                return false;
            }
            break;
        case 8:
            buffer_step = 8;
            if ( !encodeBytes( aHolder->pduIndex ,
                               buffer_step ,
                               aHolder->parameters ,
                              &aHolder->parameterIndex ) )
            {
                onBadPDU ( BP_OUT_OF_DATA , aHolder->pdu , aHolder->pduLength );
                return false;
            }
            break;
	case 0x9:
	    *aHolder->parameterIndex ++ = READ_2BYTES ( (aHolder->pduIndex) );
	    buffer_step = 2;
            recurse_max[recurse_max_counter] = (uint16) *( aHolder->parameterIndex - 1 );
            recurse_max_counter ++;
            break;
	case 0xA:
            /*
            **  give the entire rest of the pdu to the processor.
            **  It can never be more that 255 bytes, since the parameter length field
            **  in the header is only a uint8.
            */
            if ( !encodeBytes( aHolder->pduIndex ,
                              (aHolder->pduLength + aHolder->pdu - aHolder->pduIndex) ,
                               aHolder->parameters ,
                              &aHolder->parameterIndex ) )
            {
                onBadPDU ( BP_OUT_OF_DATA , aHolder->pdu , aHolder->pduLength );
                return false;
            }
	    break;
        case 0xB:
            /*
            **  B follows 1's in HCI, or 9's in BC.
            */
            if ( recurse_max_counter == 0 )
            {
                recurse_max[recurse_max_counter] = (uint16) *( aHolder->parameterIndex - 1 );
                recurse_max_counter ++;
            }
            lFormat = (aFormat >> (foffset+4));
            for ( max_counter = 0 ; max_counter < recurse_max_counter ; max_counter ++ )
                for (recurse_counter = 0 ; recurse_counter < recurse_max[max_counter] ; recurse_counter ++)
                    (void) analyseParameters ( lFormat , aHolder );
            buffer_step = 0;
            /* drop out of for loop */
            foffset = (8 * sizeof(aFormat));
            break;
        case 0xC:
            /* OpCode is always the last thing in the parameter array */
            if (!analyseHCICommandCompleteEvent ( (uint16) *( aHolder->parameterIndex - 1 ) , aHolder ) )
            {
                onBadPDU ( BP_NO_SUCH_COMMAND , aHolder->pdu , aHolder->pduLength );
                return false;
            }
            /* drop out of for loop */
            buffer_step = 0;
            foffset = (8 * sizeof(aFormat));
            break;
        case 0xE:
            /* dummy uint16 */
            buffer_step = 2;
            break;
	default:
	    /* unknown format */
            onBadPDU ( BP_WARNING_UNKNOWN_FORMAT , aHolder->pdu , aHolder->pduLength );
	    break;
	}
	aHolder->pduIndex += buffer_step;
    }

    return true;
}

