///////////////////////////////////////////////////////////////////////
//
//  FILE   :  blueCoreTransport.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  BlueCoreTransport
//
//  PURPOSE:  Provide a host-side API for the transport of HCI commands
//            /events, ACL packets, SCO packets and manufacturer
//            extension PDUs supported by BC01. This class inherits the
//            top API from the HCITransport class and adds optional
//            support for HCI tunnelling, controlled by the "tunnel"
//            member variable (disabled by default). It is recommended
//            that non-BCSP transports that use this as a base set this
//            variable within their constructor to enable tunnelling.
//
//            When tunnelling is enabled, manufacturer extension PDUs,
//            i.e. anything other than HCI/ACL/SCO data, are repackaged
//            as HCI commands using OGF = 0x3f and OCF = 0x00 (vendor-
//            specific debug commands). In addition, any Command
//            Complete events corresponding to this opcode are
//            discarded.
//
//            Regardless of whether tunnelling is enabled, any HCI
//            events received with code 0xFF (vendor specific debug
//            events) are decoded as HCI tunnelling from Host Controller
//            to Host, and the payload reprocessed as appropriate.
//
//            BCSP / HCI tunnelling channels are used as an enum to
//            allow the HCI, ACL, SCO and manufacturers extension
//            channels to be uniquely identified. A subset of these
//            constants are used directly within the payload descriptor
//            byte of HCI tunnelling commands/events, so the actual
//            numerical values are important.
//            
//            If the stack is started with out the callbacks in place,
//            then data arriving in the intervening time will be lost.
//            Since this is likely to be the HCI initialisation packet,
//            that would be bad...
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/TransportLayer/blueCoreTransport.cpp#25 $
//
//  MODIFICATION HISTORY:
//  1.4   7:Sep:00  at  Added modification history.
//                      Additional enum constants and send... functions
//                      included for all defined BCSP channels.
//                      BCSP channels can optionally be tunnelled via
//                      manufacturers extension HCI commands/events.
//                      Increased supported channels from 16 (BCSP) to
//                      64 (HCI tunnelling).
//  1.5   7:Sep:00  at  Corrected recognition of dummy Command
//                      Completed events.
//  1.6   7:Sep:00      Fixed Windows 98 bug in thread implementation.
//  1.7   8:Sep:00      Changed names of functions.
//  1.8   26:Sep:00 mr01 Removed non-generic #include "StdAfx.h"
//	1.9   16:Oct:00 at  Corrected memory deallocation for tunnelled
//                      PDUs.
//
///////////////////////////////////////////////////////////////////////

#include "HCIDeviceController.h"

#include "blueCoreTransport.h"
#include "memfuncs.h"
#include <string.h>

BlueCoreTransport::BlueCoreTransport()
{
    tunnel = false;
    discardDummyEvents = true;

    for (uint8 i = 0; i < BLUECORE_HCI_TUNNEL_COUNT; i++)
    {
	    mRecvTunnel[i].pdu = 0;
	    mCallBackObjects[i] = 0;
	    mCallBackFunctions[i] = 0;
    }
    mUserOnLinkFailParams = 0;
    mUserOnLinkFailFn = 0;
    linkFailed = false;
    MagicCallBack = 0;
    MagicParameters = 0;
}

BlueCoreTransport::~BlueCoreTransport()
{
	// Free buffers used for reassembly of HCI tunnelling PDUs
	for (uint8 i = 0; i < BLUECORE_HCI_TUNNEL_COUNT; i++)
	{
		if (mRecvTunnel[i].pdu)
		{
			freepdu (mRecvTunnel[i].pdu);
			mRecvTunnel[i].pdu = 0;
		}
	}
}

void BlueCoreTransport::setupRecvHCReply ( void * aObject,
                        void (*aRecvCallBack) ( uint8 * , uint32 , void * ) )
{
    setupRecvData (h2hc,aObject,aRecvCallBack);
}

void BlueCoreTransport::setupRecvCHEvent ( void * aObject,
                        void (*aRecvCallBack) ( uint8 * , uint32 , void * ) )
{
    setupRecvData (hc2h,aObject,aRecvCallBack);
}

void BlueCoreTransport::setupRecvDebugData ( void * aObject,
                        void (*aRecvCallBack) ( uint8 * , uint32 , void * ) )
{
    setupRecvData (debug,aObject,aRecvCallBack);
}

void BlueCoreTransport::setupRecvData (BLUECORE_CHANNELS channelNumber , void * aObject,
                        void (*aRecvCallBack) ( uint8 * , uint32 , void * ) )
{
    //  Allow NULL objects and NULL functions.
    //  The NULL Function effectively unsets the callback.
    //  A NULL Object is no problem really, especially for the evil people not using objects...
    if (   ( channelNumber >= 0 )
        && ( channelNumber < BLUECORE_HCI_TUNNEL_COUNT ) )
    {
        mCallBackObjects[channelNumber] = aObject;
        mCallBackFunctions[channelNumber] = aRecvCallBack;
    }
}

void BlueCoreTransport::setOnLinkFail( void (*aFunction) (void*) , void * aParameter )
{
    if (aFunction != 0)
    {
        mUserOnLinkFailParams = aParameter;
        mUserOnLinkFailFn = aFunction;
    }
}

void BlueCoreTransport::onLinkFail(void * aObject)
{
    BlueCoreTransport * lObject = (BlueCoreTransport*)aObject;
    //  Bowed to the Moral Majority, and removed lObject->linkFailed = true;
    //  Instead, just gibber and panic.
    if ( lObject->mUserOnLinkFailFn != NULL )
        lObject->mUserOnLinkFailFn ( lObject->mUserOnLinkFailParams );
    return;
}


//  Named Send functions:
//  HCI Commands
void BlueCoreTransport::sendCommand ( uint8 * apdu , uint16 aLength, CallBack *aCallBack )
{
    magicsendpdu(hciCommand,apdu,aLength,aCallBack);
}

//  HCI ACL
void BlueCoreTransport::sendACLData ( uint8 * apdu , uint32 aLength, CallBack *aCallBack )
{
    magicsendpdu(hciACL,apdu,aLength,aCallBack);
}

//  HCI SCO
void BlueCoreTransport::sendSCOData ( uint8 * apdu , uint16 aLength, CallBack *aCallBack )
{
    magicsendpdu(hciSCO,apdu,aLength,aCallBack);
}

//  BC01 bcmgr/bccmd (blue core manager/blue core command module)
void BlueCoreTransport::sendbccmd ( uint8 * apdu , uint16 aLength, CallBack *aCallBack )
{
    sendTunnel(bccmd,apdu,aLength,aCallBack);
}

//  BC01 hq (host queries)
void BlueCoreTransport::sendhq ( uint8 * apdu , uint16 aLength, CallBack *aCallBack )
{
    sendTunnel(hq,apdu,aLength,aCallBack);
}

//  BC01 Device Management (L2CAP)
void BlueCoreTransport::senddeviceMgt( uint8 *apdu, uint32 aLength , CallBack *aCallBack )
{
    sendTunnel(deviceMgt,apdu,aLength,aCallBack);
}

//  L2CAP (currently unused)
void BlueCoreTransport::sendl2cap ( uint8 *apdu, uint32 aLength , CallBack *aCallBack )
{
    sendTunnel(l2cap,apdu,aLength,aCallBack);
}

//  RFCOMM
void BlueCoreTransport::sendrfcomm ( uint8 *apdu, uint32 aLength , CallBack *aCallBack )
{
    sendTunnel(rfcomm,apdu,aLength,aCallBack);
}

//  SDP
void BlueCoreTransport::sendsdp ( uint8 *apdu, uint32 aLength , CallBack *aCallBack )
{
    sendTunnel(sdp,apdu,aLength,aCallBack);
}

//  lowest numbered unused channel, used for debugging info
void BlueCoreTransport::senddebug ( uint8 *apdu, uint32 aLength , CallBack *aCallBack )
{
    sendTunnel(debug,apdu,aLength,aCallBack);
}

//  USB style DFU over BCSP
void BlueCoreTransport::sendupgrade ( uint8 *apdu, uint32 aLength , CallBack *aCallBack )
{
    sendTunnel(upgrade,apdu,aLength,aCallBack);
}

void BlueCoreTransport::sendtcs ( uint8 *apdu, uint32 aLength , CallBack *aCallBack )
{
    sendTunnel(tcs,apdu,aLength,aCallBack);
}

// Send a PDU that could require HCI tunnelling for manufacturer extensions
void BlueCoreTransport::sendTunnel ( BLUECORE_CHANNELS channel, uint8 *apdu, uint32 aLength , CallBack *aCallBack )
{
    if (tunnel)
    {
	// Prepare an object to keep track of fragmentation
        TunnelledPacket * lPacket = new TunnelledPacket ( this , aLength , aCallBack );

        uint32 offset = 0;
        while ( offset < aLength )
        {
	    TunnelSendInfo *info = new TunnelSendInfo;
	    info->p = lPacket;

	    // Calculate the payload length and descriptor
	    uint8 descriptor = channel;
	    if (offset == 0)
                descriptor |= HCI_TUNNEL_FIRST;
	    info->length_sent = aLength - offset;
	    if (HCI_TUNNEL_MAX_PAYLOAD < info->length_sent)
                info->length_sent = HCI_TUNNEL_MAX_PAYLOAD;
	    else
                descriptor |= HCI_TUNNEL_LAST;

	    // Build and send the HCI command for this fragment
            info->pdu_frag = (uint8*)getpdu(info->length_sent + 4);
	    info->pdu_frag[0] = HCI_TUNNEL_COMMAND & 0xff;
	    info->pdu_frag[1] = HCI_TUNNEL_COMMAND >> 8;
	    info->pdu_frag[2] = uint8(info->length_sent + 1);
	    info->pdu_frag[3] = descriptor;
	    memcpy(info->pdu_frag + 4, apdu + offset, info->length_sent);
	    offset += info->length_sent;
	    // Send the first fragment of the PDU via HCI tunnelling
	    lPacket->sendFragment(info);
        }
    }
    else
    {
	// HCI tunnelling not enabled, so rely on HCI transport for all channels
		magicsendpdu(channel, apdu, aLength, aCallBack);
    }
}

// A fragment of a PDU has been sent via HCI tunnelling
void TunnelledPacket::sendTunnelCallback ( bool done, void *aObject )
{
    TunnelSendInfo *info = (TunnelSendInfo *) aObject;

    freepdu(info->pdu_frag);
    if (info->length_sent >= info->p->length_left)
    {
	// last fragment sent - if died then all died.
	if (info->p->callBack != 0)
	{
	    if (done)
		    info->p->callBack->execute();
	    else
		    info->p->callBack->diedIncomplete();
	    delete info->p->callBack;
	}
	delete info->p;
    }
    else
        info->p->length_left -= info->length_sent;
    delete info;
}

// Send the next fragment of a PDU via HCI tunnelling
void TunnelledPacket::sendFragment (TunnelSendInfo *info)
{
    CallBack *callBack = new CallBack;
    callBack->initialise(info, sendTunnelCallback);
    transport->sendCommand(info->pdu_frag, uint16(info->length_sent + 4), callBack);
}

// Check received PDUs for HCI tunnelling or dummy events
void BlueCoreTransport::recvTunnel ( uint8 *apdu, uint32 aLength )
{
	if ((3 <= aLength)
		&& (apdu[0] == HCI_TUNNEL_EVENT)
		&& (apdu[1] == aLength - 2))
	{
		// Reassemble HCI tunnelling packet
		BLUECORE_CHANNELS channel = BLUECORE_CHANNELS(apdu[2] & HCI_TUNNEL_CHANNEL_MASK);
		if ((0 <= channel) && (channel < BLUECORE_HCI_TUNNEL_COUNT))
		{
			TunnelRecvInfo *info = &mRecvTunnel[channel];
			uint8 length = apdu[1] - 1;
			bool first = (apdu[2] & HCI_TUNNEL_FIRST) != 0;
			bool last = (apdu[2] & HCI_TUNNEL_LAST) != 0;

			if (first || info->pdu)
			{
				// Add the new fragment to the buffer
				if (!info->pdu) info->length = 0;
				uint32 newlength = info->length + length;
				if (newlength == 0) newlength = 1;
				uint8 *buffer = (uint8*) getpdu (newlength);
				if (info->length) memcpy(buffer, info->pdu, info->length);
				if (info->pdu) freepdu (info->pdu);
				memcpy(buffer + info->length, apdu + 3, length);
				info->pdu = buffer;
				info->length += length;
			}

			if (last && info->pdu)
			{
				// Last fragment received, so handle the complete PDU
				recvData(channel, info->pdu, info->length);
				info->pdu = 0; // No longer responsible for buffer
			}
		}

		// The data has been copied so free the original PDU buffer
		freepdu (apdu);
	}
	else if (tunnel && discardDummyEvents
		     && (4 <= aLength)
		     && (apdu[0] == HCI_TUNNEL_COMPLETE_EVENT)
			 && (apdu[3] == (HCI_TUNNEL_COMMAND & 0xff))
			 && (apdu[4] == (HCI_TUNNEL_COMMAND >> 8)))
	{
		// Discard dummy Command Completed events if tunnelling enabled
		freepdu(apdu);
	}
	else
	{
		// Everything else is a standard HCI event
		recvCommand(apdu,aLength);
	}
}

void BlueCoreTransport::recvData    ( BLUECORE_CHANNELS channel, uint8 *apdu, uint32 aLength )
{
	if ( MagicCallBack )
		MagicCallBack(upstream,channel,apdu,aLength,MagicParameters);
    switch (channel)
    {
    // pass these cases back to the plain class.
    case hciCommand:
        recvTunnel(apdu,aLength);
        break;
    case hciACL:
        recvACLData(apdu,aLength);
        break;
    case hciSCO:
        recvSCOData(apdu,aLength);
        break;
    // Everyone else deals with this.
    default:
        if ((0 <= channel)
            && (channel < BLUECORE_HCI_TUNNEL_COUNT)
            && (mCallBackFunctions[channel] != 0))
		{
            mCallBackFunctions[channel] ( apdu , aLength , mCallBackObjects[channel] );
		}
        else
		{
            freepdu(apdu);
		}
        break;
    }
}

void BlueCoreTransport::magicsendpdu ( BLUECORE_CHANNELS ch , uint8* b , uint32 l , CallBack * cb )
{
    if ( MagicCallBack )
        MagicCallBack ( downstream , ch , b , l , MagicParameters );
    sendpdu ( ch , b , l , cb );
}

void BlueCoreTransport::setMagicSentThisCallBack ( void (*aCallBack) (direction,BLUECORE_CHANNELS,const uint8*,uint32,void*) , void * parameters )
{
	MagicCallBack = aCallBack;
	MagicParameters = parameters;
}