///////////////////////////////////////////////////////////////////////
//
//  FILE   :  PosixH4
//
//  AUTHOR :  Mark Marshall
//
//  CLASS  :  PosixH4
//
//  PURPOSE:  Posix-specific portion of H4 implementation of HCITransport
//
//            This file is a bit low on comments, so here is a small
//            description.
//
//            We create two worker threads one of which reads from the
//            uart and the other writes to the uart.  There is a third
//            thread which runs the 'serialDaemon' from the H4Stack.
//            We use fairly standard mutex/condition variable things
//            to coordinate things.  There are four events that can be
//            returned by the wait function (that is called by the
//            serialDaemon thread).  these are rxdone, txdone,
//            client_die and client_txreq.  These are signalled by the
//            booleans with similar names.
//
//            We only return rxdone when we have got all of the data
//            that we wanted.  If we fail to TX the data we kill that
//            thread and print an error message (this is very unlikely
//            to happen, and indicates that the UART is up the duff).
//
//            We also supply a critical section to the higher layers
//            for their own particular uses.
//
// MODIFICATION HISTORY
//	1.1   21:nov:2002  pws    Remove #ifdef to get file to compile.
///////////////////////////////////////////////////////////////////////

#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <termios.h>
#include <stdio.h>
#include <errno.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>
#define ASSERT assert

#ifdef linux
#include <sys/time.h>
#include <unistd.h>
#include <sys/uio.h>
#endif

#include "PosixH4.h"

#ifndef DEBUG_H4
#define DB_PRINTF(X)
#else
#define DB_PRINTF(X) printf X
#endif

#define CHECK(c)	\
	if (!(c))	\
	{		\
		fprintf (stderr, "Assertion %s failed (%s, %lu); "	\
			"errno %li (%s)\n",				\
			#c, __FILE__, (unsigned long) __LINE__,	\
			(long) errno, strerror (errno));		\
		abort ();						\
	}		\
	else

class PosixH4Base::Impl
{
public:
    Impl(PosixH4Base &owner);

    /* This is the PosixH4Base that we are the implimentation of.  I
     * need this so that we can callthe serialDaemonEntry function
     * properly. */
    PosixH4Base &owner_;

    int handle;		/* The handle (might be UART or BlueZ) */

    bool txbusy;	/* txThread asked by thread to tx some data */
    bool txdone;	/* txThread has txed the data */
    const uint8 *txbuffer;	/* data to write */
    size_t txlen;	/* Length of data to write */

#ifdef linux
    bool txstuffed;
    uint8 txstuf;
#endif

    bool rxbusy;	/* rxThread asked by thread to rx something */
    bool rxdone;	/* rxThread has rxed something */
    uint8 *rxbuffer;	/* Where to read to */
    size_t rxlen;	/* How much data to read */

//#ifdef linux
    uint8 rxfragbuf[0x10000];
    uint8 *rxfragptr;
    size_t rxfraglen;
//#endif

    /* Flags used for inter-thread communication */
    bool signalled;	/* signaller wants thread to awake */
    bool die;		/* destructor wants thread to die */
    bool client_die;
    bool client_txreq;

    /* The two worker threads */
    pthread_t txThreadID;
    pthread_t rxThreadID;

    /* The serialDaemon Thread */
    pthread_t daemonThreadID;

    /* Mutexes used for inter-thread synchronisation */
    pthread_mutex_t mMutexEnv;		/* For user critical sections */
    pthread_mutex_t mMutexThread;	/* For threads' critical sections */

    /* Conditions used for inter-thread communication */
    pthread_cond_t mCondDone;	/* txThread, rxThread, signaller wakes thread*/
    pthread_cond_t mCondDoTx;	/* thread wakes txThread */
    pthread_cond_t mCondDoRx;	/* thread wakes rxThread */

    /* The two reader/writer thread functions */
    void txThread();
    void rxThread();

    /* Static wrappers */
    static void __unlockMutexThread (void *p);
    static void *__txThread(void *p);
    static void *__rxThread(void *p);
    static void *__serialDaemonThread(void *p);

    /* These are the real functions of this class */

    bool initEvents (void);
    void finalEvents (void);
    H4_EVENT waitForEvent (void);

    bool txData (uint32 len, const uint8 *buf);
    bool rxData (uint32 len, uint8 *buf);

    bool initSerialDaemon (void);
    void notifySerialDaemon (H4_EVENT event);
    void finalSerialDaemon (void);

    bool initCriticalSection (void);
    void enterCriticalSection (void);
    void exitCriticalSection (void);
    void finalCriticalSection (void);
};

PosixH4Base::Impl::Impl(PosixH4Base &owner)
    : owner_(owner),
      handle(-1),
      txbusy(false), txdone(false), txbuffer(0), txlen(0),
#ifdef linux
      txstuffed(false),
#endif
      rxbusy(false), rxdone(false), rxbuffer(0), rxlen(0),
#ifdef linux
      rxfraglen(0),
#endif
     signalled(false), die(false), client_die(false), client_txreq(false)
{ }

bool
PosixH4Base::Impl::initEvents (void)
{
    CHECK (pthread_mutex_init (&mMutexThread, NULL) == 0);
    CHECK (pthread_cond_init (&mCondDoTx, NULL) == 0);
    CHECK (pthread_cond_init (&mCondDoRx, NULL) == 0);
    CHECK (pthread_cond_init (&mCondDone, NULL) == 0);

    CHECK (pthread_create (&txThreadID, NULL, __txThread, this) == 0);
    CHECK (pthread_create (&rxThreadID, NULL, __rxThread, this) == 0);

    return true;
}

void
PosixH4Base::Impl::finalEvents (void)
{
    DB_PRINTF(("finalEvents\n"));

    CHECK (pthread_cancel (txThreadID) == 0);
    CHECK (pthread_join (txThreadID, NULL) == 0);
    CHECK (pthread_cancel (rxThreadID) == 0);
    CHECK (pthread_join (rxThreadID, NULL) == 0);

    CHECK (pthread_mutex_destroy (&mMutexThread) == 0);
    CHECK (pthread_cond_destroy (&mCondDoTx) == 0);
    CHECK (pthread_cond_destroy (&mCondDoRx) == 0);
    CHECK (pthread_cond_destroy (&mCondDone) == 0);
}

PosixH4Base::H4_EVENT
PosixH4Base::Impl::waitForEvent (void)
{
    PosixH4Base::H4_EVENT ret;

    DB_PRINTF(("Entering wait."));

    CHECK (pthread_mutex_lock (&mMutexThread) == 0);

    while (!(txdone || rxdone || client_die || client_txreq))
    {
	DB_PRINTF(("thread: waiting for something\n"));
	CHECK (pthread_cond_wait(&mCondDone, &mMutexThread) == 0);
    }

    if (client_die)
    {
	client_die = false;
	ret = CLIENT_DIEREQ;
    }
    else if (txdone)
    {
	txbusy = false;
	txdone = false;
	ret = UART_TXDONE;
    }
    else if (rxdone)
    {
	rxbusy = false;
	rxdone = false;
	ret = UART_RXDONE;
    }
    else /*if (client_txreq)*/
    {
	client_txreq = false;
	ret = CLIENT_TXREQ;
    }

    CHECK (pthread_mutex_unlock (&mMutexThread) == 0);

    return ret;
}

bool
PosixH4Base::Impl::rxData (uint32 len, uint8 *buf)
{
    bool res = false;

    if (len == 0)
	return false;

    CHECK (pthread_mutex_lock (&mMutexThread) == 0);

    if (!rxbusy)
    {
	DB_PRINTF(("rxData len %d  frag: buf %p ptr %p len %d\n",
		   len, rxfragbuf, rxfragptr, rxfraglen));

#ifdef linux
	if (rxfraglen != 0)
	{
	    if (len <= rxfraglen)
	    {
		memcpy(buf, rxfragptr, len);
		rxfraglen -= len;
		rxfragptr += len;

		CHECK (pthread_mutex_unlock (&mMutexThread) == 0);

		return true;
	    }

	    memcpy(buf, rxfragptr, rxfraglen);
	    len -= rxfraglen;
	    buf += rxfraglen;
	    rxfraglen = 0;
	}
#endif
	rxbuffer = buf;
	rxlen = len;
	rxbusy = true;

	CHECK (pthread_cond_signal (&mCondDoRx) == 0);
    }

    CHECK (pthread_mutex_unlock (&mMutexThread) == 0);

    return res;
}

bool
PosixH4Base::Impl::txData (uint32 len, const uint8 *buf)
{
    bool res = false;

    if (len == 0)
	return false;

    CHECK (pthread_mutex_lock (&mMutexThread) == 0);

    if (!txbusy)
    {
#ifdef linux
	if (len == 1 && txstuffed == false)
	{
	    txstuffed = true;
	    txstuf = buf[0];
	    res = true;
	}
	else
#endif
	{
	    txbuffer = buf;
	    txlen = len;
	    txbusy = true;

	    CHECK (pthread_cond_signal (&mCondDoTx) == 0);
	}
    }

    CHECK (pthread_mutex_unlock (&mMutexThread) == 0);

    return res;
}

/*static*/ void
PosixH4Base::Impl::__unlockMutexThread (void *p)
{
    int status = pthread_mutex_unlock ((pthread_mutex_t *) p);
    if (status != 0)
    {
	printf("mutex_unlock = %d (erno = %d 0x%x)\n",
	       status, errno, errno);
	perror("mutex_unlock");
    }
}

/*static*/ void *
PosixH4Base::Impl::__rxThread(void *p)
{
    PosixH4Base::Impl *_this = (PosixH4Base::Impl *)p;
    _this->rxThread();
    return 0;
}

void
PosixH4Base::Impl::rxThread ()
{
    ssize_t readed;

    CHECK (pthread_mutex_lock (&mMutexThread) == 0);

    while (1) /* Dies by cancellation */
    {
	pthread_cleanup_push (__unlockMutexThread, &mMutexThread);

	while (!rxbusy || rxdone)
	{
	    /*DB_PRINTF(("rxthread: waiting for do\n"));*/
	    CHECK (pthread_cond_wait (&mCondDoRx, &mMutexThread) == 0);
	}
	assert (rxbusy && !rxdone);
	assert (rxlen != 0);
	assert (rxbuffer != 0);

	pthread_cleanup_pop (1);

	DB_PRINTF(("rxthread: waiting for rx (%p %d)\n",
	  rxbuffer, rxlen));

	while (rxlen > 0)
	{
	    pthread_testcancel ();
	    readed = read(handle, rxfragbuf, sizeof(rxfragbuf));
	    pthread_testcancel ();

	    if (readed < 1)
	    {
		fprintf(stderr, "Failed to read %li bytes "
			"(rc %li, errno %i):\n",
			(long)rxlen, (long)readed, errno);
		perror(NULL);
	    
		if (errno != EINTR)
		    return;
	    }
	    else
	    {
		DB_PRINTF(("rxthread: rc = %d\n", readed));

		if (readed >= rxlen)
		{
		    memcpy(rxbuffer, rxfragbuf, rxlen);

		    rxfragptr = rxfragbuf + rxlen;
		    rxfraglen = readed - rxlen;

		    rxlen = 0;
		}
		else
		{
		    memcpy(rxbuffer, rxfragbuf, readed);
		    rxlen -= readed;
		    rxbuffer += readed;
		    
		    rxfragptr = rxfragbuf;
		    rxfraglen = 0;
		}
	    }
	}

	DB_PRINTF(("rxthread: rxed!\n"));
	CHECK (pthread_mutex_lock (&mMutexThread) == 0);

	rxdone = true;
	CHECK (pthread_cond_signal (&mCondDone) == 0);
    }
}

/*static*/ void *
PosixH4Base::Impl::__txThread(void *p)
{
    PosixH4Base::Impl *_this = (PosixH4Base::Impl *)p;
    _this->txThread();
    return 0;
}

void
PosixH4Base::Impl::txThread ()
{
    ssize_t n;

    CHECK (pthread_mutex_lock (&mMutexThread) == 0);

    while (1) /* Dies by cancellation */
    {
	pthread_cleanup_push (__unlockMutexThread, &mMutexThread);

	while (!txbusy || txdone)
	{
	    DB_PRINTF(("txthread: waiting for do\n"));
	    CHECK (pthread_cond_wait (&mCondDoTx, &mMutexThread) == 0);
	}
	assert (txbusy && !txdone);
	assert (txlen != 0);
	assert (txbuffer != 0);

	pthread_cleanup_pop (1);

	DB_PRINTF(("txthread: about to tx (%d %p %d)\n",
		   handle, txbuffer, txlen));

	pthread_testcancel ();
#ifdef linux
	if (txstuffed)
	{
	    struct iovec vec[2];
	    vec[0].iov_base = &txstuf;
	    vec[0].iov_len = 1;
	    vec[1].iov_base = (void *)txbuffer;
	    vec[1].iov_len = txlen;
	    DB_PRINTF(("txthread: Stuffing 0x%02x : %p %d\n",
		       txstuf, txbuffer, txlen));
	    n = ::writev(handle, vec, 2);
	    if (n > 0)
		n--;
	    txstuffed = false;
	}
	else
#endif
	{
	    n = ::write(handle, txbuffer, txlen);
	}
	pthread_testcancel ();

	if (n < 0 || (size_t)n != txlen)
	{
	    fprintf(stderr, "Failed to write %li bytes (rc %li, errno %i):\n",
		    (long) txlen, (long) n, errno);
	    perror (NULL);
	    owner_.flushHandle(handle);
	    return;
	}

	DB_PRINTF(("txthread: txed\n"));
	CHECK (pthread_mutex_lock (&mMutexThread) == 0);

	txdone = true;
	CHECK (pthread_cond_signal (&mCondDone) == 0);
    }
}

/*static*/ void *
PosixH4Base::Impl::__serialDaemonThread(void *p)
{
    PosixH4Base::Impl *_this = (PosixH4Base::Impl *)p;
    _this->owner_.serialDaemonEntry ();
    return 0;
}


bool
PosixH4Base::Impl::initSerialDaemon (void)
{
    CHECK (pthread_create (&daemonThreadID, NULL, __serialDaemonThread,
			   this) == 0);
    return true;
}

void
PosixH4Base::Impl::notifySerialDaemon (H4_EVENT evt)
{
    switch(evt)
    {
    case CLIENT_DIEREQ:
	CHECK (pthread_mutex_lock (&mMutexThread) == 0);
	client_die = true;
	CHECK (pthread_cond_signal (&mCondDone) == 0);
	CHECK (pthread_mutex_unlock (&mMutexThread) == 0);
	break;

    case CLIENT_TXREQ:
	CHECK (pthread_mutex_lock (&mMutexThread) == 0);
	client_txreq = true;
	CHECK (pthread_cond_signal (&mCondDone) == 0);
	CHECK (pthread_mutex_unlock (&mMutexThread) == 0);
	break;

    default:
	/* BUG? */
	break;
    }
}

void
PosixH4Base::Impl::finalSerialDaemon (void)
{
    DB_PRINTF(("finalSerialDaemon\n"));

    notifySerialDaemon (CLIENT_DIEREQ);

    CHECK (pthread_join (daemonThreadID, NULL) == 0);

    DB_PRINTF(("finalSerialDaemon - done\n"));
}


bool
PosixH4Base::Impl::initCriticalSection (void)
{
    CHECK (pthread_mutex_init (&mMutexEnv, NULL) == 0);
    return true;
}

void
PosixH4Base::Impl::enterCriticalSection (void)
{
    CHECK (pthread_mutex_lock (&mMutexEnv) == 0);
}

void
PosixH4Base::Impl::exitCriticalSection (void)
{
    CHECK (pthread_mutex_unlock (&mMutexEnv) == 0);
}

void
PosixH4Base::Impl::finalCriticalSection (void)
{
    CHECK (pthread_mutex_destroy (&mMutexEnv) == 0);
}

/********************************************************************
 * Here are the actual functions that will be called by external code.
 * They just reflect back into the ''Impl'' class functions of the
 * same name.
 */

PosixH4Base::PosixH4Base() :
    impl_(* new PosixH4Base::Impl(*this))
{ }

/*virtual*/ PosixH4Base::~PosixH4Base()
{ delete &impl_; }

/*virtual*/ bool
PosixH4Base::initEvents ()
{ return impl_.initEvents(); }

/*virtual*/ void
PosixH4Base::finalEvents ()
{ return impl_.finalEvents(); }

/*virtual*/ PosixH4Base::H4_EVENT
PosixH4Base::waitForEvent ()
{ return impl_.waitForEvent(); }

/*virtual*/ bool
PosixH4Base::rxData (uint32 len, uint8 *buf)
{ return impl_.rxData(len, buf); }

/*virtual*/ bool
PosixH4Base::txData (uint32 len, const uint8 *buf)
{ return impl_.txData(len, buf); }

/*virtual*/ bool
PosixH4Base::initSerialDaemon (void)
{ return impl_.initSerialDaemon(); }

/*virtual*/ void
PosixH4Base::notifySerialDaemon (H4_EVENT evt)
{ impl_.notifySerialDaemon(evt); }

/*virtual*/ void
PosixH4Base::finalSerialDaemon (void)
{ impl_.finalSerialDaemon(); }

/*virtual*/ bool
PosixH4Base::initCriticalSection (void)
{ return impl_.initCriticalSection(); }

/*virtual*/ void
PosixH4Base::enterCriticalSection (void)
{ impl_.enterCriticalSection(); }

/*virtual*/ void
PosixH4Base::exitCriticalSection (void)
{ impl_.exitCriticalSection(); }

/*virtual*/ void 
PosixH4Base::finalCriticalSection (void)
{ impl_.finalCriticalSection(); }

void
PosixH4Base::setHandle(int h)
{ impl_.handle = h; }

int
PosixH4Base::getHandle()
{ return impl_.handle; }

/*************************************/

H4Stack::startReturnValue
PosixH4::initSerial (const ichar *portName, uint32 datarate)
{
    struct termios tios;
    int handle;

    handle = open(portName, O_RDWR | O_NOCTTY);

    if (handle < 0)
    {
	fprintf (stderr, "Failed to open %s (errno %i):\n",
		 portName, errno);
	perror (NULL);
	handle = -1;
	return serialProblem;
    }

    if (tcgetattr (handle, &tios) != 0)
    {
	fprintf(stderr, "Failed to get terminal attributes (errno %i):\n",
		errno);
	perror (NULL);
	close (handle);
	handle = -1;
	return serialProblem;
    }

    tios.c_iflag &= ~(BRKINT | ICRNL | IGNBRK | IGNCR | IGNPAR | INLCR
		      | INPCK | ISTRIP | IXOFF | IXON | PARMRK);
    if (tios.c_iflag != 0)
    {
	fprintf(stderr, "Warning: tios.c_iflag contains non-zero "
		"non-Posix bits: 0%lo\n", (unsigned long)tios.c_iflag);
    }
    tios.c_iflag |= INPCK | IGNPAR;
    /* Result: breaks read as NUL, and parity/framing errors read as
     * nothing, which should ensure the packet sizes end up wrong,
     * which in turn ensures packets containing them are discarded */

    tios.c_oflag &= ~OPOST;
    if (tios.c_oflag != 0)
    {
#ifdef ONLCR
	if (tios.c_oflag != ONLCR)
#endif
	    fprintf (stderr, "Warning: tios.c_oflag contains non-zero "
		     "non-Posix bits: 0%lo\n", (unsigned long)tios.c_oflag);
    }
    /* There are no bits in c_oflag we want to set */
    /* Result: just send data unadulterated! */
    tios.c_cflag &= ~(CLOCAL | CREAD | CSIZE | CSTOPB | HUPCL
			  | PARENB | PARODD);
    if (tios.c_cflag != 0)
    {
	fprintf (stderr, "Warning: tios.c_cflag contains non-zero "
		 "non-Posix bits: 0%lo\n", (unsigned long)tios.c_cflag);
    }
    tios.c_cflag |= CREAD | CS8 | CLOCAL;

    /*
     * Posix doesn't support hardware flow control, so just try to
     * turn on any available flag!  None of this has really been
     * tested.  It doesn't appear to be possible to set hardware
     * flow control through termios in HPUX and AIX.
     */
#ifdef ORTSFL /* SCO (bidirectional) */
    /*tios.c_cflag |= ORTSFL;*/ /* non-Posix! */
    /*tios.c_cflag &= ~CTSFLOW;*/ /* non-Posix! */
    /*tios.c_cflag &= ~RTSFLOW;*/ /* non-Posix! */
#else /* !ORTSFL */
#ifdef CRTSCTS /* Solaris (unidirectional), Linux (bidirectional), Digital Unix (bidirectional) */
    /*ios.c_cflag |= CRTSCTS;*/ /* non-Posix! */
#else
#ifdef CNEW_RTSCTS /* Irix (bidirectional) */
    /*tios.c_cflag |= CNEW_RTSCTS;*/ /* non-Posix! */
#else
#ifdef CCTS_OFLOW /* BSD (unidirectional) */
    /*tios.c_cflag |= CCTS_OFLOW;*/ /* non-Posix! */
#else
    /*fprintf (stderr, "Warning: tios.c_cflag doesn't appear to support "
      "hardware flow control\n");*/
#endif
#endif
#endif
#ifdef CRTSXOFF /* Solaris (unidirectional) */
    /*tios.c_cflag |= CRTSXOFF;*/ /* non-Posix! */
#else
#ifdef CRTS_IFLOW /* BSD (unidirectional) */
    /*tios.c_cflag |= CRTS_IFLOW;*/ /* non-Posix! */
#endif
#endif
#endif
    
    /* Result: use 8{E/O/N}{1/2} with or without RTS/CTS flow control */
    /* Note: no direct control over DTR, but BC01 doesn't use DTR/DSR */
    
    tios.c_lflag &= ~(ECHO | ECHOE | ECHOK | ECHONL | ICANON
		      | IEXTEN | ISIG | NOFLSH | TOSTOP);
    
    if (tios.c_lflag != 0)
    {
#if defined (__sparc__) && defined (MGR)
	if (tios.c_lflag != 05000) /* ECHOCTL | ECHOKE */
#endif
#if defined (ECHOCTL) && defined (ECHOKE)
	if (tios.c_lflag != (ECHOCTL | ECHOKE))
#endif
	    fprintf (stderr, "Warning: tios.c_lflag contains non-zero "
		     "non-Posix bits: 0%lo\n",
		     (unsigned long)tios.c_lflag);
    }
    /* There are no bits in c_lflag we want to set */
    /* Result: no local mode monstrosities! */

    tios.c_cc[VMIN] = 1;
    tios.c_cc[VTIME] = 0;
    /* Result: on read, wait for at least one char */

#define SET_BAUD_RATE(X) do { \
		cfsetispeed (&tios, B ## X); \
		cfsetospeed (&tios, B ## X); } while(0)

    switch (datarate)
    {
#ifdef B38400
    case 38400:		SET_BAUD_RATE(38400); break;
#endif
#ifdef B57600
    case 57600:		SET_BAUD_RATE(57600); break;
#endif
#ifdef B115200
    case 115200:	SET_BAUD_RATE(115200); break;
#endif
#ifdef B230400
    case 230400:	SET_BAUD_RATE(230400); break;
#endif
#ifdef B460800
    case 460800:	SET_BAUD_RATE(460800); break;
#endif
#ifdef B500000
    case 500000:        SET_BAUD_RATE(500000); break;
#endif
#ifdef B576000
    case 576000:       	SET_BAUD_RATE(576000); break;	
#endif
#ifdef B921600
    case 921600:	SET_BAUD_RATE(921600); break;
#endif
#ifdef B1000000
    case 1000000:	SET_BAUD_RATE(1000000);	break;
#endif
#ifdef B1152000
    case 1152000:	SET_BAUD_RATE(1152000);	break;
#endif
#ifdef B1500000
    case 1500000:	SET_BAUD_RATE(1500000);	break;
#endif
#ifdef B2000000
    case 2000000:	SET_BAUD_RATE(2000000);	break;
#endif
#ifdef B2500000
    case 2500000:	SET_BAUD_RATE(2500000);	break;
#endif
#ifdef B3000000
    case 3000000:	SET_BAUD_RATE(3000000); break;
#endif
#ifdef B3500000
    case 3500000:	SET_BAUD_RATE(3500000); break;	
#endif
#ifdef B4000000
    case 4000000:	SET_BAUD_RATE(4000000); break;
#endif

    default:
	fprintf (stderr, "Sorry, I don't know how to set %lu bps\n",
		 (unsigned long)datarate);
	close (handle);
	handle = -1;
	return serialProblem;
    }

    if (tcsetattr(handle, TCSANOW, &tios) != 0)
    {
	fprintf (stderr, "Failed to set terminal attributes (errno %i):\n",
		 errno);
	perror (NULL);
	close(handle);
	handle = -1;
	return serialProblem;
    }

    /* Inform the PosixH4Base of the handle to use */
    setHandle(handle);

    return OK;
}

H4Stack::startReturnValue
PosixH4::initSerial (void *portHandle)
{
    return serialProblem;
}

void
PosixH4::finalSerial (void)
{
    int handle;

    DB_PRINTF(("finalSerial\n"));

    handle = getHandle();
    setHandle(-1);

    flushHandle(handle);

    close(handle); /* close the com port  */
}

/*virtual*/ void
PosixH4::flushHandle(int handle)
{
    if (handle != -1)
	tcflush (handle, TCIOFLUSH); /* otherwise can hang! */
}
