///////////////////////////////////////////////////////////////////////
//
//  FILE   :  AutoCts
//
//  AUTHOR :  Alexander Thoukydides
//
//  CLASS  :  n/a
//
//  PURPOSE:  Set automatic CTS control for the UART if possible
//
///////////////////////////////////////////////////////////////////////

// Include header file for this module
#include "AutoCts.h"

// Include system header files
#include "winbase.h"
#include "conio.h"


// Definitions from "vcomm.h" of the Windows 98 DDK
#define GETCOMBASEIRQ (10)

// Definitions from "ntddser.h" of the Windows 2000 DDK
#define IOCTL_SERIAL_GET_MODEM_CONTROL (0x001b0094)
#define IOCTL_SERIAL_SET_MODEM_CONTROL (0x001b0098)
#define IOCTL_SERIAL_SET_FIFO_CONTROL (0x001b009c)


// Direct COMM port control
class AutoCtsComm
{
public:

	// Register offsets
	enum
	{
		iir = 2,	// Read only	Interrupt Identification Register
					//		bits 6:7	0	No FIFO
					//					2	FIFO enabled but unusable
					//					3	FIFO enabled
					//		bit 5		64 byte FIFO enabled (16750)
					//		bit 4		Reserved
					//		bit 3		Time-out interrupt pending (16550)
					//		bits 1:2	0	Modem status interrupt
					//					1	Transmitter holding register empty
					//					2	Received data available
					//					3	Receiver line status
					//		bit 0		No interrupt pending (i.e. active low)

		fcr = 2,	// Write only	FIFO Control Register
					//		bits 6:7	Interrupt trigger level
					//						0	1 byte
					//						1	4 bytes
					//						2	8 bytes
					//						3	14 bytes
					//		bit 5		Enable 64 byte FIFO (16750)
					//		bit 4		Reserved
					//		bit 3		DMA mode select
					//		bit 2		Clear transmit FIFO (self resetting)
					//		bit 1		Clear receive FIFO (self resetting)
					//		bit 0		Enable FIFOs

		mcr = 4,	// Read/Write	Modem Control Register
					//		bit 7	Reserved
					//		bit 6	Reserved
					//		bit 5	Autoflow control enabled (16750 or 16550c)
					//		bit 4	Loopback mode
					//		bit 3	Aux output 2
					//		bit 2	Aux output 1
					//		bit 1	Force request to send
					//		bit 0	Force data terminal ready

		scr = 7		// Read/Write	Scratch Register
	};

	// Construction and destruction
	AutoCtsComm(HANDLE comport) {}
	virtual ~AutoCtsComm() {}

	// Register access
	virtual int read(int offset) = 0;
	virtual void write(int offset, int value) = 0;
};

// Direct COMM port control under Windows 95, 98 or ME
class AutoCtsCommWin95 : public AutoCtsComm
{
	unsigned short base_address;

public:

	// Construction and destruction
	AutoCtsCommWin95(HANDLE comport) : AutoCtsComm(comport)
	{
		/*
			EscapeCommFunction returns a boolean result.
			However, some reason codes (such as GETCOMBASEIRQ) return a
			numeric value... which appears to be available in dx.
		*/
		unsigned short base = 0;
		if (EscapeCommFunction(comport, GETCOMBASEIRQ))
			_asm mov base, dx;
		base_address = base;
	}

	// Register access
	virtual int read(int offset)
	{
		return _inp(base_address + offset);
	}

	virtual void write(int offset, int value)
	{
		_outp(base_address + offset, value);
	}
};

// Direct COMM port control under Windows NT 4 or 2000
class AutoCtsCommWinNT4 : public AutoCtsComm
{
	HANDLE handle;
	HANDLE event;
	int value_scr;
	int value_iir;

	// Perform a IOCTL operation
	unsigned long ioctl(unsigned long code, void *in, unsigned long inLength, void *out, unsigned long outLength)
	{
		// Create an overlapped data structure
		OVERLAPPED overlapped;
		overlapped.Offset = 0;
		overlapped.OffsetHigh = 0;
		overlapped.hEvent = event;

		// Attempt to perform the requested operation
		if (!DeviceIoControl(handle, code, in, inLength, out, outLength, 0, &overlapped)
			&& (GetLastError() != ERROR_IO_PENDING))
		{
			DWORD error = GetLastError();
			return 0;
		}

		// Wait for the operation to complete
		if (WaitForSingleObject(event, INFINITE) != WAIT_OBJECT_0)
			return 0;

		// Get the result
		unsigned long bytesReturned;
		if (!GetOverlappedResult(handle, &overlapped, &bytesReturned, false))
			return 0;

		// Return the result
		return bytesReturned;
	}

	void ioctl(unsigned long code, int value)
	{
		unsigned long buffer = value;
		ioctl(code, &buffer, sizeof(buffer), 0, 0);
	}

	int ioctl(unsigned long code)
	{
		unsigned long buffer;
		ioctl(code, 0, 0, &buffer, sizeof(buffer));
		return buffer;
	}

public:

	// Construction and destruction
	AutoCtsCommWinNT4(HANDLE comport) : AutoCtsComm(comport)
	{
		handle = comport;
		event = CreateEvent(0, true, false, 0);
	}

	~AutoCtsCommWinNT4()
	{
		CloseHandle(event);
	}

	// Register access
	virtual int read(int offset)
	{
		int value = 0;
		switch (offset)
		{
		case iir:
			value = value_iir;
			break;

		case mcr:
			value = ioctl(IOCTL_SERIAL_GET_MODEM_CONTROL);
			break;

		case scr:
			value = value_scr;
			break;
		}
		return value;
	}

	virtual void write(int offset, int value)
	{
		switch (offset)
		{
		case fcr:
			// Fake a value of IIR since there doesn't seem to be a way to read it
			value_iir = 0x01 | (value & 0x20) | (value & 0x01 ? 0xc0 : 0x00);
			ioctl(IOCTL_SERIAL_SET_FIFO_CONTROL, value);
			break;

		case mcr:
			ioctl(IOCTL_SERIAL_SET_MODEM_CONTROL, value);
			break;

		case scr:
			// Fake the scratch register also
			value_scr = value;
			break;
		}
	}
};


// Determine the version of Windows
enum WindowsVersion
{
	windows_unknown,
	windows_31,
	windows_95,
	windows_98,
	windows_me,
	windows_nt35,
	windows_nt4,
	windows_2k,
	windows_xp
};
static WindowsVersion GetWindowsVersion()
{
	OSVERSIONINFO info;
	WindowsVersion version = windows_unknown;

	// Attempt to read the operating system version
	memset(&info, 0, sizeof(info));
	info.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if (GetVersionEx(&info))
	{
		switch (info.dwPlatformId)
		{
		case VER_PLATFORM_WIN32s:
			// Win32 on Windows 3.1
			version = windows_31;
			break;

		case VER_PLATFORM_WIN32_WINDOWS:
			// Win32 on Windows 95, 98 or ME
			if ((4 < info.dwMajorVersion)
				|| ((4 == info.dwMajorVersion) && (90 <= info.dwMinorVersion)))
				version = windows_me;
			else if ((4 == info.dwMajorVersion) && (10 <= info.dwMinorVersion))
				version = windows_98;
			else if (4 == info.dwMajorVersion)
				version = windows_95;
			break;

		case VER_PLATFORM_WIN32_NT:
			// Win32 on Windows NT 3.51, NT 4.0, 2000 or XP
			if ((5 < info.dwMajorVersion)
				|| ((5 == info.dwMajorVersion) && (1 <= info.dwMinorVersion)))
				version = windows_xp;
			else if (5 == info.dwMajorVersion)
				version = windows_2k;
			else if (4 == info.dwMajorVersion)
				version = windows_nt4;
			else if (3 == info.dwMajorVersion)
				version = windows_nt35;
			break;
		}
	}

	// Return the version of Windows
	return version;
}


// Determine the UART type
enum UartVersion
{
	uart_unknown,	// Unable to identify type
	uart_8250,		// 8250 or 8250B			- No FIFO or scratch register
	uart_16450,		// 16450 or 8250A			- No FIFO
	uart_16550,		// 16550					- 16 byte FIFO (doesn't work)
	uart_16550a,	// 16550a, 16550b or 16650	- 16 or 32 byte FIFO
	uart_16550c,	// 16550c					- 16 byte FIFO with AutoCTS
	uart_16750		// 16750					- 64 byte FIFO with AutoCTS
};
static UartVersion GetUartVersion(AutoCtsComm &port)
{
	UartVersion version = uart_unknown;

	// Use the FIFO status to recognise the UART type
	port.write(AutoCtsComm::fcr, 0x81);
	switch (port.read(AutoCtsComm::iir) & 0xc0)
	{
	case 0x00:
		// No FIFOs - 8250, 8250A, 8250B or 16450
		port.write(AutoCtsComm::scr, 0x42);
		version = port.read(AutoCtsComm::scr) == 0x42
			      ? uart_16450 : uart_8250;
		break;

	case 0x40:
	case 0x80:
		// Unusable FIFO
		version = uart_16550;
		break;

	case 0xc0:
		// FIFOs enabled and working - 16550a, 16550b, 16550c, 16650 or 16750
		port.write(AutoCtsComm::fcr, 0xa1);
		if (port.read(AutoCtsComm::fcr) & 0x20)
			version = uart_16750;
		else
		{
			// Check for automatic flow control support
			int mcrSaved = port.read(AutoCtsComm::mcr);
			port.write(AutoCtsComm::mcr, mcrSaved | 0x20);
			version = port.read(AutoCtsComm::mcr) & 0x20
				      ? uart_16550c : uart_16550a;
			port.write(AutoCtsComm::mcr, mcrSaved);
		}
		break;
	}

	// Return the type of UART
	return version;
}


// Set auto CTS control for the specific COM port if possible
void AutoCts(HANDLE comport)
{
	// Attempt to construct an appropriate object to access the COMM port
	AutoCtsComm *port = 0;
	switch (GetWindowsVersion())
	{
	case windows_95:
	case windows_98:
	case windows_me:
		port = new AutoCtsCommWin95(comport);
		break;

	case windows_2k:
	case windows_xp:
		port = new AutoCtsCommWinNT4(comport);
		break;

	default:
		// Other platforms do not allow direct hardware access
		break;
	}

	// No further action unless port access if available
	if (port)
	{
		// Set automatic flow control if supported
		switch (GetUartVersion(*port))
		{
		case uart_16550c:
		case uart_16750:
			port->write(AutoCtsComm::mcr, port->read(AutoCtsComm::mcr) | 0x20);
			break;

		default:
			// Not supported by other UART types
			break;
		}

		// Delete the COMM port access object
		delete port;
	}
}