///////////////////////////////////////////////////////////////////////
//
//  FILE   :  Win32H4
//
//  AUTHOR :  Mark RISON
//
//  CLASS  :  Win32H4
//
//  PURPOSE:  Win32-specific portion of H4 implementation of HCITransport
//
///////////////////////////////////////////////////////////////////////


#include "Win32H4.h"

#include <assert.h>
#define ASSERT assert
#include <process.h>

#ifdef NDEBUG
#define DB_PRINTF(x)
#else
#include <stdio.h>
#define DB_PRINTF(x) iprintf x
#endif

bool Win32H4::initEvents (void)
{
#define WIN32_NO_SECURITY_ATTRIBUTES NULL
#define WIN32_EVENT_MANUALRESET true
#define WIN32_EVENT_AUTORESET false
#define WIN32_EVENT_INITNONSIG false
#define WIN32_EVENT_ANONYMOUS NULL

	event[UART_TXDONE] = CreateEvent (WIN32_NO_SECURITY_ATTRIBUTES, WIN32_EVENT_MANUALRESET, WIN32_EVENT_INITNONSIG, WIN32_EVENT_ANONYMOUS);
	ASSERT (event[UART_TXDONE] != NULL);
	if (event[UART_TXDONE] == NULL)
	{
		return false;
	}

	event[UART_RXDONE] = CreateEvent (WIN32_NO_SECURITY_ATTRIBUTES, WIN32_EVENT_MANUALRESET, WIN32_EVENT_INITNONSIG, WIN32_EVENT_ANONYMOUS);
	ASSERT (event[UART_RXDONE] != NULL);
	if (event[UART_RXDONE] == NULL)
	{
		CloseHandle (event[UART_TXDONE]);
		return false;
	}

	event[CLIENT_TXREQ] = CreateEvent (WIN32_NO_SECURITY_ATTRIBUTES, WIN32_EVENT_AUTORESET, WIN32_EVENT_INITNONSIG, WIN32_EVENT_ANONYMOUS);
	ASSERT (event[CLIENT_TXREQ] != NULL);
	if (event[CLIENT_TXREQ] == NULL)
	{
		CloseHandle (event[UART_TXDONE]);
		CloseHandle (event[UART_RXDONE]);
		return false;
	}

	event[CLIENT_DIEREQ] = CreateEvent (WIN32_NO_SECURITY_ATTRIBUTES, WIN32_EVENT_AUTORESET, WIN32_EVENT_INITNONSIG, WIN32_EVENT_ANONYMOUS);
	ASSERT (event[CLIENT_DIEREQ] != NULL);
	if (event[CLIENT_DIEREQ] == NULL)
	{
		CloseHandle (event[UART_TXDONE]);
		CloseHandle (event[UART_RXDONE]);
		CloseHandle (event[CLIENT_TXREQ]);
		return false;
	}

	return true;
}


Win32H4::H4_EVENT Win32H4::waitForEvent (void)
{
#define WIN32_WAIT_ANY false
	DWORD ret = WaitForMultipleObjects (H4_NUMEVENTS, event, WIN32_WAIT_ANY, INFINITE);
	H4_EVENT evt = (H4_EVENT) (ret - WAIT_OBJECT_0);

	switch (evt)
	{
	case UART_RXDONE:

		/* HERE need to check nbytes read as could be less than asked for in case of comms error (see on-line doc, "Communications Errors")? */
		ResetEvent (event[UART_RXDONE]);
		break;

	case UART_TXDONE:

		ResetEvent (event[UART_TXDONE]);
		break;

	case CLIENT_TXREQ:
	case CLIENT_DIEREQ:

		break;

	default:

		ASSERT ((II("WaitForMultipleObjects"), false));
		break;
	}

	return evt;
}


void Win32H4::finalEvents (void)
{
	BOOL success;

	success = CloseHandle (event[UART_TXDONE]);
	ASSERT ((II("CloseHandle (event[UART_TXDONE])"), success));
	success = CloseHandle (event[UART_RXDONE]);
	ASSERT ((II("CloseHandle (event[UART_RXDONE])"), success));
	success = CloseHandle (event[CLIENT_TXREQ]);
	ASSERT ((II("CloseHandle (event[CLIENT_TXREQ])"), success));
	success = CloseHandle (event[CLIENT_DIEREQ]);
	ASSERT ((II("CloseHandle (event[CLIENT_DIEREQ])"), success));
}

H4Stack::startReturnValue Win32H4::initSerial (const ichar *portName, uint32 datarate)
{
	DCB dcb;
	BOOL success;
	COMMTIMEOUTS timeouts;
	ichar cfgStr[] = II("data=8 parity=n stop=1"); /* As specified in H4 */

	ASSERT (portName != NULL);

	/* The on-line doc Technical Article "Serial Communications in Win32" is worth reading */

	DB_PRINTF ((II("H4: %s, %lu bps\n"), portName, (unsigned long) datarate));

#define WIN32_NO_SHARING 0
#define WIN32_NO_TEMPLATE_FILE 0
	comport = CreateFile (portName,
			GENERIC_READ | GENERIC_WRITE,
			WIN32_NO_SHARING, // mandatory
			WIN32_NO_SECURITY_ATTRIBUTES,
			OPEN_EXISTING, // mandatory
			FILE_FLAG_OVERLAPPED,
			WIN32_NO_TEMPLATE_FILE // mandatory
			);
	if (comport == INVALID_HANDLE_VALUE)
	{
		DB_PRINTF ((II("H4: failed to open port\n")));
		return serialProblem;
	}

	memset (&dcb, 0, sizeof (DCB));
	dcb.DCBlength = sizeof (DCB);

	success = BuildCommDCB (cfgStr, &dcb); /* Note BuildCommDCB() turns off both hardware and software flow control */
	ASSERT ((II("BuildCommDCB"), success));
	if (!success)
	{
		CloseHandle (comport);
		return otherProblem;
	}

	dcb.fBinary = 1; // probably superfluous, but the on-line doc isn't clear
	dcb.BaudRate = datarate;
	dcb.fOutxCtsFlow = 1; /* As specified in H4 */
	dcb.fDtrControl = DTR_CONTROL_ENABLE; /* As (not) specified in H4 */
	dcb.fRtsControl = RTS_CONTROL_HANDSHAKE; /* As specified in H4 */

	success = SetCommState (comport, &dcb);
	ASSERT ((II("SetCommState"), success));
	if (!success)
	{
		CloseHandle (comport);
		return otherProblem;
	}

#define SERIAL_RXBUFSIZE 1024
#define SERIAL_TXBUFSIZE 1024
	success = SetupComm (comport, SERIAL_RXBUFSIZE, SERIAL_TXBUFSIZE);
	ASSERT ((II("SetupComm"), success));
	if (!success)
	{
		CloseHandle (comport);
		return otherProblem;
	}

#define WIN32_NO_TIMEOUT 0
	timeouts.ReadIntervalTimeout = WIN32_NO_TIMEOUT;
	timeouts.ReadTotalTimeoutMultiplier = WIN32_NO_TIMEOUT;
	timeouts.ReadTotalTimeoutConstant = WIN32_NO_TIMEOUT;
	timeouts.WriteTotalTimeoutMultiplier = WIN32_NO_TIMEOUT;
	timeouts.WriteTotalTimeoutConstant = WIN32_NO_TIMEOUT;
	success = SetCommTimeouts (comport, &timeouts);
	ASSERT ((II("SetCommTimeouts"), success));
	if (!success)
	{
		CloseHandle (comport);
		return otherProblem;
	}

	txOverlapped.Offset = txOverlapped.OffsetHigh = 0;
	txOverlapped.hEvent = event[UART_TXDONE];
	rxOverlapped.Offset = rxOverlapped.OffsetHigh = 0; /* On-line doc is lying when it claims ("OVERLAPPED") these members are ignored for comms devices -- get error on ReadFile! */
	rxOverlapped.hEvent = event[UART_RXDONE];

	return OK;
}

H4Stack::startReturnValue  Win32H4::initSerial (HANDLE portHandle)
{
	BOOL success;
	COMMTIMEOUTS timeouts;
	
	ASSERT(portHandle != INVALID_HANDLE_VALUE);

	DB_PRINTF ((II("H4: %p\n"), portHandle));

	HANDLE process = GetCurrentProcess();
	success = DuplicateHandle(process, portHandle, process, &comport,
		                      GENERIC_READ | GENERIC_WRITE, false, 0);
	ASSERT ((II("DuplicateHandle"), success));
	if (!success)
	{
		DB_PRINTF ((II("H4: failed to duplicate port handle\n")));
		return serialProblem;
	}

	success = SetupComm (comport, SERIAL_RXBUFSIZE, SERIAL_TXBUFSIZE);
	ASSERT ((II("SetupComm"), success));
	if (!success)
	{
		CloseHandle (comport);
		return otherProblem;
	}

#define WIN32_NO_TIMEOUT 0
	timeouts.ReadIntervalTimeout = WIN32_NO_TIMEOUT;
	timeouts.ReadTotalTimeoutMultiplier = WIN32_NO_TIMEOUT;
	timeouts.ReadTotalTimeoutConstant = WIN32_NO_TIMEOUT;
	timeouts.WriteTotalTimeoutMultiplier = WIN32_NO_TIMEOUT;
	timeouts.WriteTotalTimeoutConstant = WIN32_NO_TIMEOUT;
	success = SetCommTimeouts (comport, &timeouts);
	ASSERT ((II("SetCommTimeouts"), success));
	if (!success)
	{
		CloseHandle (comport);
		return otherProblem;
	}

	txOverlapped.Offset = txOverlapped.OffsetHigh = 0;
	txOverlapped.hEvent = event[UART_TXDONE];
	rxOverlapped.Offset = rxOverlapped.OffsetHigh = 0; /* On-line doc is lying when it claims ("OVERLAPPED") these members are ignored for comms devices -- get error on ReadFile! */
	rxOverlapped.hEvent = event[UART_RXDONE];

	return OK;
}


bool Win32H4::txData (uint32 len, const uint8 *buf)
{
	bool done;
	BOOL success;
	DWORD bytes;

	if (len == 0) /* Be pessimistic and assume WriteFile would choke on zero length */
	{
		done = TRUE;
	}
	else
	{
		success = WriteFile (comport, buf, len, &bytes, &txOverlapped);
		done = success != 0;

		if (!done)
		{
			DWORD txgle = GetLastError ();
			ASSERT (txgle == ERROR_IO_PENDING);
		}
	}

	return done;
}


bool Win32H4::rxData (uint32 len, uint8 *buf)
{
	bool done;
	BOOL success;
	DWORD bytes;

	if (len == 0) /* Be pessimistic and assume ReadFile would choke on zero length */
	{
		done = TRUE;
	}
	else
	{
		success = ReadFile (comport, buf, len, &bytes, &rxOverlapped);
		done = success != 0;

		if (!done)
		{
			DWORD rxgle = GetLastError ();
			// DB_PRINTF ((II("Rx: GLE %lu (len %i, buf %p, bytes %lu, ov %p, comport %lu)\n"), rxgle, len, buf, bytes, &rxOverlapped, (unsigned long) comport));
			ASSERT (rxgle == ERROR_IO_PENDING);
		}
	}

	return done;
}


void Win32H4::finalSerial (void)
{
	BOOL success;

	success = CloseHandle (comport);
	ASSERT ((II("CloseHandle (comport)"), success));
}

	
unsigned int __stdcall startSerialDaemon (void *p)
{
	((H4Stack *) p)->serialDaemonEntry ();

	return 0;
}


bool Win32H4::initSerialDaemon (void)
{
	unsigned int threadID;

#define WIN32_THREAD_DEFAULTSTACKSIZE 0
#define WIN32_THREAD_RUNIMMEDIATELY 0
	serialDaemonThread = reinterpret_cast<HANDLE>(_beginthreadex (WIN32_NO_SECURITY_ATTRIBUTES, WIN32_THREAD_DEFAULTSTACKSIZE, startSerialDaemon, this, WIN32_THREAD_RUNIMMEDIATELY, &threadID));
	ASSERT (serialDaemonThread != NULL);
	if (serialDaemonThread == NULL)
	{
		return false;
	}

	return true;
}


void Win32H4::notifySerialDaemon (H4_EVENT evt)
{
	BOOL success;

	success = SetEvent (event[evt]);
	ASSERT ((II("SetEvent"), success));
}


void Win32H4::finalSerialDaemon (void)
{
	DWORD ret;

	notifySerialDaemon (CLIENT_DIEREQ);
	ret = WaitForSingleObject (serialDaemonThread, INFINITE);
	ASSERT (ret == WAIT_OBJECT_0);
	CloseHandle(serialDaemonThread);
}

	
bool Win32H4::initCriticalSection (void)
{
	InitializeCriticalSection (&critSec);

	return true;
}


void Win32H4::enterCriticalSection (void)
{
	EnterCriticalSection (&critSec);
}


void Win32H4::exitCriticalSection (void)
{
	LeaveCriticalSection (&critSec);
}


void Win32H4::finalCriticalSection (void)
{
	DeleteCriticalSection (&critSec);
}
