///////////////////////////////////////////////////////////////////////
//
//  FILE   :  H4Stack
//
//  AUTHOR :  Mark RISON
//
//  CLASS  :  H4Stack
//
//  PURPOSE:  Generic portion of H4 implementation of HCITransport
//
///////////////////////////////////////////////////////////////////////

#include "H4Stack.h"

#include <cstring> // AJH
#include <stdio.h>

#include <assert.h>
#define ASSERT assert

#ifndef DEBUG_H4
#define DB_PRINTF(x)
#else
#define DB_PRINTF(x) iprintf x
#endif

#ifndef TRUE
#define TRUE 1
#endif

H4Stack::H4Stack ()
{
	started = false;
        //  AJH
        mComPktCallBackObject = NULL ;
        mComPktCallBackFunction = NULL ;
        mComCallBackObject = NULL ;
        mComCallBackFunction = NULL ;
        mACLCallBackObject = NULL ;
        mACLCallBackFunction = NULL ;
        mSCOCallBackObject = NULL ;
        mSCOCallBackFunction = NULL ;
        //  AJH End
}

// AJH removing dependency on HCITransport.
void H4Stack::setupRecvCmdPkt ( void * aObject,
                        void (*aRecvCallBack) ( uint8 * , uint32 , void * ) )
{
    if ( ( aObject != NULL ) && ( aRecvCallBack != NULL ) )
    {
        mComPktCallBackObject = aObject ;
        mComPktCallBackFunction = aRecvCallBack ;
    }
}

void H4Stack::setupRecvCommand ( void * aObject,
                        void (*aRecvCallBack) ( uint8 * , uint32 , void * ) )
{
    if ( ( aObject != NULL ) && ( aRecvCallBack != NULL ) )
    {
        mComCallBackObject = aObject ;
        mComCallBackFunction = aRecvCallBack ;
    }
}

void H4Stack::setupRecvACLData ( void * aObject, 
                        void (*aRecvCallBack) ( uint8 * , uint32 , void * ) )
{
    if ( ( aObject != NULL ) && ( aRecvCallBack != NULL ) )
    {
        mACLCallBackObject = aObject ;
        mACLCallBackFunction = aRecvCallBack ;
    }
}

void H4Stack::setupRecvSCOData ( void * aObject,
                        void (*aRecvCallBack) ( uint8 * , uint32 , void * ) )
{
    if ( ( aObject != NULL ) && ( aRecvCallBack != NULL ) )
    {
        mSCOCallBackObject = aObject ;
        mSCOCallBackFunction = aRecvCallBack ;
    }
}

void H4Stack::recvCmdPkt ( uint8 *apdu, uint32 aLength )
{
    if ( mComPktCallBackFunction != NULL )
        mComPktCallBackFunction ( apdu , aLength , mComPktCallBackObject ) ;
    else
        delete apdu;
}

void H4Stack::recvCommand ( uint8 *apdu, uint32 aLength )
{
    if ( mComCallBackFunction != NULL )
        mComCallBackFunction ( apdu , aLength , mComCallBackObject ) ;
    else
        delete apdu;
}

void H4Stack::recvACLData ( uint8 *apdu, uint32 aLength )
{
    if ( mACLCallBackFunction != NULL )
        mACLCallBackFunction ( apdu , aLength , mACLCallBackObject ) ;
    else
        delete apdu;
}

void H4Stack::recvSCOData ( uint8 *apdu, uint32 aLength )
{
    if ( mSCOCallBackFunction != NULL )
        mSCOCallBackFunction ( apdu , aLength , mSCOCallBackObject ) ;
    else
        delete apdu;
}
//  AJH End

H4Stack::startReturnValue H4Stack::start (const ichar *portName,
					  uint32 datarate, void *handle)
{
    startReturnValue srv;

    ASSERT (!started);

    if (!initQueues ())
    {
	return otherProblem;
    }

    if (!initStateMachines ())
    {
	finalQueues ();
	return otherProblem;
    }

    if (!initEvents ()) /* Do this before initSerial() */
    {
	finalQueues ();
	return otherProblem;
    }

    if (handle) srv = initSerial (handle);
    else srv = initSerial (portName, datarate);
    if (srv != OK)
    {
	finalEvents ();
	finalQueues ();
	return srv;
    }

    if (!initCriticalSection ())
    {
	finalSerial ();
	finalEvents ();
	finalQueues ();
	return otherProblem;
    }

    if (!initSerialDaemon ()) /* Do this last */
    {
	finalCriticalSection ();
	finalSerial ();
	finalEvents ();
	finalQueues ();
	return otherProblem;
    }

    started = true;
    DB_PRINTF ((II("H4: client side initialised\n")));

    return OK;
}


bool H4Stack::initQueues (void)
{
#ifdef H4_MULTIQUEUE
	int i;

	for (i = 0; i < H4_NUMQUEUES; ++i)
	{
		queue[i].head = NULL;
	}

	queue[H4_QUEUE_CMD].pktType = H4_PKTTYPE_CMD;
	queue[H4_QUEUE_ACL].pktType = H4_PKTTYPE_ACL;
	queue[H4_QUEUE_SCO].pktType = H4_PKTTYPE_SCO;
#else
	queue.head = NULL;
#endif

	return true;
}


void H4Stack::finalQueues (void)
{
#ifdef H4_MULTIQUEUE
	int i;

	for (i = 0; i < H4_NUMQUEUES; ++i)
	{
		deleteQueue (queue[i].head);
	}
#else
	deleteQueue (queue.head);
#endif
}


void H4Stack::deleteQueue (H4_PDULINK *pduLink)
{
	H4_PDULINK *nextPduLink;

	while (pduLink != NULL)
	{
		DB_PRINTF ((II("Tx: deleting a pending PDU\n")));
		nextPduLink = pduLink->next;
		pduLink->callback->diedIncomplete ();
		delete pduLink->PDU;
		delete pduLink->callback;
		delete pduLink;
		pduLink = nextPduLink;
	}
}


bool H4Stack::initStateMachines (void)
{
	txState = TXIDLE;
	rxState = RXIDLE;
#ifdef H4_SIMPLERESYNC
	resyncing = false;
	onSyncLoss = NULL;
#endif

	return true;
}


void H4Stack::stop (void)
{
	if (started)
	{
		DB_PRINTF ((II("H4: asking serial daemon to die\n")));
		finalSerialDaemon ();
		started = false;
	}
}


H4Stack::~H4Stack ()
{
	ASSERT (!started);
}


void H4Stack::sendACLData (uint8 *apdu, UINT32 aLength, CallBack *aCallBack)
{
#ifdef H4_MULTIQUEUE
    sendPDU (H4_QUEUE_ACL, apdu, aLength, aCallBack);
#else
    sendPDU (H4_PKTTYPE_ACL, apdu, aLength, aCallBack);
#endif
}


void H4Stack::sendCommand (uint8 *apdu, uint16 aLength, CallBack *aCallBack)
{
#ifdef H4_MULTIQUEUE
    sendPDU (H4_QUEUE_CMD, apdu, aLength, aCallBack);
#else
    sendPDU (H4_PKTTYPE_CMD, apdu, aLength, aCallBack);
#endif
}

void H4Stack::sendEvent (uint8 *apdu, uint16 aLength, CallBack *aCallBack)
{
#ifdef H4_MULTIQUEUE
#pragma message ("MULTIQUEUE Does not support the Event channel!"
//    sendPDU (H4_QUEUE_CMD, apdu, aLength, aCallBack);
#else
    sendPDU (H4_PKTTYPE_EVT, apdu, aLength, aCallBack);
#endif
}



void H4Stack::sendSCOData (uint8 *apdu, uint16 aLength, CallBack *aCallBack)
{
#ifdef H4_MULTIQUEUE
    sendPDU (H4_QUEUE_SCO, apdu, aLength, aCallBack);
#else
    sendPDU (H4_PKTTYPE_SCO, apdu, aLength, aCallBack);
#endif
}


#ifdef H4_MULTIQUEUE
void H4Stack::sendPDU (H4_QUEUENUM queuenum, const uint8 *contents, uint32 length, CallBack *callback)
#else
void H4Stack::sendPDU (H4_PKTTYPE pktType, const uint8 *contents, uint32 length, CallBack *callback)
#endif
{
	H4_PDU *pdu;
	H4_PDULINK *pduLink;

	ASSERT (started);
#ifdef H4_MULTIQUEUE
	ASSERT (queuenum >= 0 && queuenum < H4_NUMQUEUES);
#else
	ASSERT (pktType >= 1 && pktType <= 4);
#endif
	ASSERT (contents != NULL);
	ASSERT (length >= 3); // No-parameter command or zero-length SCO packet
	ASSERT (callback != NULL);

	pdu = new H4_PDU;
	ASSERT (pdu != NULL);
	pdu->length = length;
	pdu->contents = contents;

	pduLink = new H4_PDULINK;
	ASSERT (pduLink != NULL);
	pduLink->next = NULL;
	pduLink->PDU = pdu;
#ifndef H4_MULTIQUEUE
	pduLink->pktType = pktType;
#endif
	pduLink->callback = callback;

	enterCriticalSection ();

#ifdef H4_MULTIQUEUE
	if (queue[queuenum].head == NULL)
	{
		DB_PRINTF ((II("Tx: pkttype %u length %u added to empty queue %u\n"), queue[queuenum].pktType, length, queuenum));
		queue[queuenum].head = queue[queuenum].tail = pduLink;
	}
	else
	{
		DB_PRINTF ((II("Tx: pkttype %u length %u added to tail of queue %u\n"), queue[queuenum].pktType, length, queuenum));
		queue[queuenum].tail = queue[queuenum].tail->next = pduLink;
	}
#else
	if (queue.head == NULL)
	{
		DB_PRINTF ((II("Tx: pkttype %u length %u added to empty queue\n"), pktType, length));
		queue.head = queue.tail = pduLink;
	}
	else
	{
		DB_PRINTF ((II("Tx: pkttype %u length %u added to tail of queue\n"), pktType, length));
		queue.tail = queue.tail->next = pduLink;
	}
#endif

	exitCriticalSection ();

	notifySerialDaemon (CLIENT_TXREQ);
}


bool H4Stack::ready (uint32 timeout)
{
	/* timeout not used */

	return started;
}


void H4Stack::serialDaemon (void)
{
    bool dead = false;

    DB_PRINTF ((II("H4: serial daemon started\n")));

    rxStateMachine (); /* Prime rx */

    while (!dead)
    {
	H4_EVENT event = waitForEvent ();

	switch (event)
	{
	case UART_RXDONE:
	    /* HERE need to check nbytes read as could be less than
	     * asked for in case of comms error (see on-line doc,
	     * "Communications Errors") */
	    rxStateMachine ();
	    break;

	case UART_TXDONE:

	    txStateMachine ();
	    break;

	case CLIENT_TXREQ:

	    if (txState == TXIDLE)
	    {
		txStateMachine ();
	    }
	    break;

	case CLIENT_DIEREQ:
	    finalCriticalSection ();
	    finalSerial ();
	    finalEvents (); /* Do this after finalSerial () */
	    finalQueues ();
	    if (rxPkt)
		delete[] rxPkt;
	    dead = true;
	    break;
	}
    }

    DB_PRINTF ((II("H4: serial daemon terminated\n")));
}


void H4Stack::txStateMachine (void)
{
    bool retrigger;

    do
    {
	switch (txState)
	{
	case TXIDLE:

	    enterCriticalSection ();

	    retrigger = false;

#ifdef H4_MULTIQUEUE
	    for (txActiveQueue = 0; txActiveQueue < H4_NUMQUEUES; ++txActiveQueue)
	    {
		if (queue[txActiveQueue].head != NULL)
		{
		    DB_PRINTF ((II("Tx: sending from head of queue %u (pkttype %u)\n"), txActiveQueue, queue[txActiveQueue].pktType));
		    retrigger = txData (1, &queue[txActiveQueue].pktType);
		    txState = TXPKTTYPE;

		    break;
		}
	    }
#else
	    if (queue.head != NULL)
	    {
		DB_PRINTF ((II("Tx: sending from head of queue (pkttype %u)\n"), queue.head->pktType));
		retrigger = txData (1, &queue.head->pktType);
		txState = TXPKTTYPE;
	    }
#endif

	    exitCriticalSection ();

	    break;

	case TXPKTTYPE:

#ifdef H4_MULTIQUEUE
	{
	    uint32 u;
	    DB_PRINTF ((II("Tx: sending pkt")));
	    for (u = 0; u < queue[txActiveQueue].head->PDU->length; ++u)
	    {
		DB_PRINTF ((II(" %02x"), queue[txActiveQueue].head->PDU->contents[u]));
	    }
	    DB_PRINTF ((II(" [len %u]\n"), queue[txActiveQueue].head->PDU->length));
	}
	retrigger = txData (queue[txActiveQueue].head->PDU->length, queue[txActiveQueue].head->PDU->contents);
#else
	{
	    uint32 u;
	    DB_PRINTF ((II("Tx: sending pkt")));
	    for (u = 0; u < queue.head->PDU->length; ++u)
	    {
		DB_PRINTF ((II(" %02x"), queue.head->PDU->contents[u]));
	    }
	    DB_PRINTF ((II(" [len %u]\n"), queue.head->PDU->length));
	}
	retrigger = txData (queue.head->PDU->length, queue.head->PDU->contents);
#endif
	txState = TXPKT;

	break;

	case TXPKT:
	{
	    H4_PDULINK *pduLink;

	    enterCriticalSection ();
#ifdef H4_MULTIQUEUE
	    pduLink = queue[txActiveQueue].head;
	    queue[txActiveQueue].head = pduLink->next;
#else
	    pduLink = queue.head;
	    queue.head = pduLink->next;
#endif
	    exitCriticalSection ();

	    pduLink->callback->execute ();
	    delete pduLink->PDU;
	    delete pduLink->callback;
	    delete pduLink;

	    DB_PRINTF ((II("Tx: pkt sent\n")));
	    retrigger = true;
	    txState = TXIDLE;
	    break;
	}

	default:

	    ASSERT ((txState, false));

	    break;
	}

    } while (retrigger);
}


#ifdef H4_SAS
static uint8 *rxPktPrev = NULL;
static uint32 rxPktPrevLen;
static bool rxPktShow = false;
#endif

void H4Stack::rxStateMachine (void)
{
#ifdef H4_SAS
    static uint32 badcount = 0;
#endif

    bool retrigger;

    do
    {
	switch (rxState)
	{
	case RXIDLE:

	    DB_PRINTF ((II("Rx: waiting for pkt\n")));
	    rxPkt = NULL;
	    retrigger = rxData (1, &rxPktType);
	    rxState = RXPKTTYPE;

	    break;

	case RXPKTTYPE:

	    switch (rxPktType)
	    {
            case H4_PKTTYPE_CMD:

                DB_PRINTF ((II("Rx: receiving command pkt\n")));
                rxPktHdrLen = 3;
                break;

	    case H4_PKTTYPE_ACL:

		DB_PRINTF ((II("Rx: receiving ACL pkt\n")));
		rxPktHdrLen = 4;
		break;

	    case H4_PKTTYPE_SCO:

		DB_PRINTF ((II("Rx: receiving SCO pkt\n")));
		rxPktHdrLen = 3;
		break;

	    case H4_PKTTYPE_EVT:

		DB_PRINTF ((II("Rx: receiving event pkt\n")));
		rxPktHdrLen = 2;
		break;

	    default:

#ifdef H4_SIMPLERESYNC
#ifdef H4_SAS
		if (badcount++ == 0)
		{
		    size_t i;
		    ifprintf (stderr, II("Rx: lost sync\n"));
		    iprintf (II("Rx: prev:"));
		    if (rxPktPrev)
		    {
			for (i = 0; i < rxPktPrevLen; ++i)
			{
			    iprintf (II(" %02x"), rxPktPrev[i]);
			}
		    }
		    else
		    {
			iprintf (II(" none"));
		    }
		    iprintf (II("\nRx: this:"));
		}
		ifprintf (stderr, II(" %02x"), rxPktType);
#endif
		if (!resyncing)
		{
		    resyncing = true;
		    if (onSyncLoss != NULL)
		    {
			onSyncLoss (onSyncLossArg);
		    }
		    else
		    {
			ifprintf (stderr, II("H4Stack: sync lost\n"));
		    }
		}
		DB_PRINTF ((II("Rx: bad pktType (0x%02x) -- discarding and waiting again\n"), rxPktType));
		rxPktHdrLen = 0;
#else
		ASSERT ((rxPktType, false)); /* HERE Should somehow notify upper layers so they can do the official recovery procedure */
		/* HERE for fancy resync scan for 04 0e 04 xx 03 0c 00 */
#endif
		break;
	    }

#ifdef H4_SIMPLERESYNC
	    if (rxPktHdrLen == 0)
	    {
		retrigger = rxData (1, &rxPktType);
		break;
	    }
	    resyncing = false;
#ifdef H4_SAS
	    if (badcount)
	    {
		ifprintf (stderr, II("\nRx: resynchronised after %lu bad octets\n"), (unsigned long) badcount);
		badcount = 0;
		rxPktShow = true;
	    }
#endif
#endif
	    retrigger = rxData (rxPktHdrLen, rxPktHdr);
	    rxState = RXPKTHDR;

	    break;

	case RXPKTHDR:

	    switch (rxPktType)
	    {
            case H4_PKTTYPE_CMD:

                rxPktPayloadLen = rxPktHdr[2];
                break;

	    case H4_PKTTYPE_ACL:

		rxPktPayloadLen = rxPktHdr[2] | (rxPktHdr[3] << 8);
		break;

	    case H4_PKTTYPE_SCO:

		rxPktPayloadLen = rxPktHdr[2];
		break;

	    case H4_PKTTYPE_EVT:

		rxPktPayloadLen = rxPktHdr[1];
		break;
	    }

	    rxPktTotalLen = rxPktHdrLen + (uint32) rxPktPayloadLen;
	    DB_PRINTF ((II("Rx: pkt total length is %u\n"), rxPktTotalLen));
	    rxPkt = new uint8[rxPktTotalLen];
	    ASSERT (rxPkt != NULL);
	    memcpy (rxPkt, rxPktHdr, rxPktHdrLen);
	    retrigger = rxData (rxPktPayloadLen, rxPkt + rxPktHdrLen);
	    rxState = RXPKT;

	    break;

	case RXPKT:

	{
	    uint32 u;
	    DB_PRINTF ((II("Rx: received packet")));
	    for (u = 0; u < rxPktTotalLen; ++u)
	    {
		DB_PRINTF ((II(" %02x"), rxPkt[u]));
	    }
	    DB_PRINTF ((II(" [len %u]\n"), rxPktTotalLen));
	}

#ifdef H4_SAS
	if (rxPktPrev)
	{
	    free (rxPktPrev);
	}
	rxPktPrev = malloc (rxPktTotalLen);
	memcpy (rxPktPrev, rxPkt, rxPktTotalLen);
	rxPktPrevLen = rxPktTotalLen;
	if (rxPktShow)
	{
	    size_t i;
	    iprintf (II("Rx: next:"));
	    for (i = 0; i < rxPktTotalLen; ++i)
	    {
		iprintf (II(" %02x"), rxPkt[i]);
	    }
	    iprintf (II("\n"));
	    rxPktShow = false;
	}
#endif
	switch (rxPktType)
	{
	case H4_PKTTYPE_CMD:

	    recvCmdPkt (rxPkt, rxPktTotalLen);
	    break;

	case H4_PKTTYPE_ACL:

	    recvACLData (rxPkt, rxPktTotalLen);
	    break;

	case H4_PKTTYPE_SCO:

	    recvSCOData (rxPkt, (uint16 /* Always safe */) rxPktTotalLen);
	    break;

	case H4_PKTTYPE_EVT:

	    recvCommand (rxPkt, (uint16 /* Always safe */) rxPktTotalLen); /* Note use recvCommand for events */
	    break;
	}
	/* Note the recv* routines promise to do a delete[] rxPkt */

	retrigger = TRUE;
	rxState = RXIDLE;

	break;

	default:

	    ASSERT ((rxState, false));

	    break;
	}

    } while (retrigger);
}


#ifdef H4_SIMPLERESYNC
void H4Stack::setupOnSyncLoss (void (*onSyncLoss) (void *), void *onSyncLossArg)
{
	this->onSyncLoss = onSyncLoss;
	this->onSyncLossArg = onSyncLossArg;
}
#endif
