///////////////////////////////////////////////////////////////////////
//
//  FILE   :  H4Stack
//
//  AUTHOR :  Mark RISON
//
//  CLASS  :  H4Stack
//
//  PURPOSE:  Generic portion of H4 implementation of HCITransport
//
///////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////
//  Modification history
///////////////////////////////////////////////////////////////////////
//  1.1   sometime  	mr	Created.
//  1.2   11.sep.00     ajh     pushed into its own DLL and removed
//                              dependency on HCITransport.
//  1.6   16.nov.00     at      Added ability to open com port
//                              externally.

#ifndef __H4STACK_H__
#define __H4STACK_H__

#ifdef WIN32
#include "StdAfx.h"
#endif /*WIN32*/


#include "common/types.h"
#include "common/bool.h"
#include "unicode/ichar.h"

#include "callBack.h"


#define UINT32 uint32 /* HERE change all occurrences to just uint32 when other layers catch up */
#define H4_SIMPLERESYNC /* Don't die if bad pkttype; just wait until good pkttype comes along */
#define H4_DEFAULT_DATARATE 38400
//#define H4_MULTIQUEUE /* Have separate and prioritised queues for each pkttype */
//#define H4_SAS /* Special high-power debugging for Steve and Steve -- needs H4_SIMPLERESYNC */


class H4_API_CLASS H4Stack //  AJH : public HCITransport
{
public:
        H4Stack ();
        virtual ~H4Stack ();

        enum startReturnValue { OK , serialProblem , otherProblem };

        startReturnValue start (const ichar *portName, uint32 datarate = H4_DEFAULT_DATARATE, void *handle = 0);
	virtual bool ready (uint32 timeout);
	virtual void stop (void);

	void sendEvent (uint8 *apdu, uint16 aLength, CallBack *aCallBack);
	void sendCommand (uint8 *apdu, uint16 aLength, CallBack *aCallBack);
	void sendACLData (uint8 *apdu, UINT32 aLength, CallBack *aCallBack);
	void sendSCOData (uint8 *apdu, uint16 aLength, CallBack *aCallBack);

        // AJH removing dependency on HCITransport.
	void setupRecvCmdPkt  ( void * aObject,
				void (*aRecvCallBack) ( uint8 * , uint32 , void * ) ); // HCI Command
        void setupRecvCommand ( void * aObject,
                                void (*aRecvCallBack) ( uint8 * , uint32 , void * ) ); // HCI Event
        void setupRecvACLData ( void * aObject, 
                                void (*aRecvCallBack) ( uint8 * , uint32 , void * ) ); // ACL
        void setupRecvSCOData ( void * aObject,
                                void (*aRecvCallBack) ( uint8 * , uint32 , void * ) ); // SCO
        //  AJH End

	#ifdef H4_SIMPLERESYNC
	void setupOnSyncLoss (void (*onSyncLoss) (void *), void *onSyncLossArg);
	#endif

	void resetStack (void) {}; /* HERE remove this when other layers catch up */

	void serialDaemonEntry (void) { serialDaemon (); } /* Don't override this */

protected:

	enum H4_EVENT
	{
		UART_TXDONE,
		UART_RXDONE,
		CLIENT_TXREQ,
		CLIENT_DIEREQ
	};
	#define H4_NUMEVENTS 4

        // AJH removing dependency on HCITransport.
	    void recvCmdPkt  ( uint8 *apdu, uint32 aLength );
        void recvCommand ( uint8 *apdu, uint32 aLength );
        void recvACLData ( uint8 *apdu, uint32 aLength );
        void recvSCOData ( uint8 *apdu, uint32 aLength );
        //  AJH End

private:

	/* Generic */

	enum H4_PKTTYPE		/* See BT1.0B/H:4;2 */
	{
		H4_PKTTYPE_CMD = 1,
		H4_PKTTYPE_ACL = 2,
		H4_PKTTYPE_SCO = 3,
		H4_PKTTYPE_EVT = 4
	};

	#ifdef H4_MULTIQUEUE
	enum H4_QUEUENUM	/* Highest-priority is lowest-numbered */
	{
		H4_QUEUE_SCO = 0,		/* Always number the first one zero */
		H4_QUEUE_CMD,			/* Always increment by one */
		H4_QUEUE_ACL
	};
	#define H4_NUMQUEUES 3
	#endif

	struct H4_PDU
	{
		uint32 length;
		const uint8 *contents;
	};

	struct H4_PDULINK
	{
		H4_PDULINK *next;		/* Points to NULL if last */
		H4_PDU *PDU;
		#ifndef H4_MULTIQUEUE
		uint8 pktType;			/* Must be of size 8 bits, so can't use H4_PKTTYPE */
		#endif
		CallBack *callback;
	};

	struct H4_QUEUE
	{
		H4_PDULINK *head;		/* Points to NULL if the queue is empty */
		H4_PDULINK *tail;		/* Undefined if the queue is empty (see head) */
		#ifdef H4_MULTIQUEUE
		uint8 pktType;			/* Must be of size 8 bits, so can't use H4_PKTTYPE */
		#endif
	};
	#ifdef H4_MULTIQUEUE
	H4_QUEUE queue[H4_NUMQUEUES];
	#else
	H4_QUEUE queue;
	#endif

	enum H4_TXSTATE
	{
		TXIDLE,
		TXPKTTYPE,
		TXPKT
	};
	H4_TXSTATE txState;

	#ifdef H4_MULTIQUEUE
	int txActiveQueue;
	#endif

	enum H4_RXSTATE
	{
		RXIDLE,
		RXPKTTYPE,
		RXPKTHDR,
		RXPKT
	};
	H4_RXSTATE rxState;

	uint8 rxPktType, rxPktHdr[4], *rxPkt;
	unsigned rxPktHdrLen, rxPktPayloadLen;
	uint32 rxPktTotalLen;
	#ifdef H4_SIMPLERESYNC
	bool resyncing;
	void (*onSyncLoss) (void *p);
	void *onSyncLossArg;
	#endif

	bool started;

	bool initQueues (void);
	void finalQueues (void);
	void deleteQueue (H4_PDULINK *pduLink);

	bool initStateMachines (void);

	#ifdef H4_MULTIQUEUE
	void sendPDU (H4_QUEUENUM queuenum, const uint8 *PDU, uint32 length, CallBack *callback);
	#else
	void sendPDU (H4_PKTTYPE pktType, const uint8 *PDU, uint32 length, CallBack *callback);
	#endif

	void serialDaemon (void);
	void txStateMachine (void);
	void rxStateMachine (void);

	/* Target-specific */

	virtual bool initEvents (void) = 0;
	virtual H4_EVENT waitForEvent (void) = 0;
	virtual void finalEvents (void) = 0;

	virtual startReturnValue initSerial (const ichar *portName, uint32 datarate) = 0;
	virtual startReturnValue initSerial (void *handle) = 0;
	virtual bool txData (uint32 len, const uint8 *buf) = 0;
	virtual bool rxData (uint32 len, uint8 *buf) = 0;
	virtual void finalSerial (void) = 0;

	virtual bool initSerialDaemon (void) = 0;
	virtual void notifySerialDaemon (H4_EVENT event) = 0;
	virtual void finalSerialDaemon (void) = 0;

	virtual bool initCriticalSection (void) = 0;
	virtual void enterCriticalSection (void) = 0;
	virtual void exitCriticalSection (void) = 0;
	virtual void finalCriticalSection (void) = 0;

        //  AJH removing dependency on HCITransport.
        //  Pointers to objects and callbacks.
        //  The callbacks must be declared static in the object,
	    void *mComPktCallBackObject;
	    void (*mComPktCallBackFunction) ( uint8 *, uint32 , void * );
        void *mComCallBackObject;
        void (*mComCallBackFunction) ( uint8 *, uint32 , void * );
        void *mACLCallBackObject;
        void (*mACLCallBackFunction) ( uint8 *, uint32 , void * );
        void *mSCOCallBackObject;
        void (*mSCOCallBackFunction) ( uint8 *, uint32 , void * );
        //  AJH End

};

#endif // __H4STACK_H__
