///////////////////////////////////////////////////////////////////////
//
//  FILE   :  thread.cpp
//
//            (C) CSR 2001
//
//  AUTHOR :  Mark Marshall
//
//  PURPOSE:  The POSIX version of the Threadable class.
//
// MODIFICATION HISTORY
//	1.5  11:apr:02  pws      Define __EXTENSIONS__ to get timeval (SunOS).
//
//  $Id: //depot/bc/main/devHost/util/thread/posix/thread.cpp#7 $
//
///////////////////////////////////////////////////////////////////////

#define __EXTENSIONS__ 1
#include "thread/thread.h"

#include <stdio.h>
#include <stdarg.h>
#include <pthread.h>
#include <stdlib.h>
#include <assert.h>
#include <errno.h>
#include <sys/time.h>

#if 1
#define ERROR_MSG(MSG) do { } while(0)
#else
#define ERROR_MSG(MSG) error_printf MSG
static void error_printf(char const *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    vfprintf(stderr, fmt, ap);
    va_end(ap);
}
#endif

//
// This is the class that holds all of the platform specific
// data (and also the static 'ThreadFunc' function).
//
class Threadable::ThreadData : private NoCopy
{
public:
    // The ID of the thread
    pthread_t	    m_threadId;

    // These provide protection for the data in this structure
    pthread_mutex_t m_mutex;
    pthread_cond_t  m_cond;

    // All of the data below this point is protected by the mutex above.
    // The data items are setup in the constructor, and can each
    // be set at _ONE_ other point.

    // This is the thread exit code.  It is set to 0 at the start.  If the
    // thread is killed (does not exit when stop is called), then it will
    // remain at that value, otherwise it is set to the return value of
    // ''ThreadFunc''
    int 	    m_exitCode;

    // This bool is set to true by the ctor.
    // The ''ThreadFunc'' is expected to check on it status (by
    // calling ''KeepGoing'') to see if it should keep going.  It
    // is set to false when ''Stop'' is called.
    bool            m_keepGoing;

    // This event is set to false in the ctor.  It is set to true when
    // the child thread is started.
    bool	    m_eventStarted;

    // This event is set to false in the ctor.  It is set to true when the
    // child thread ends.
    bool	    m_eventEnded;

    // From the above, it can be seen that the child thread is running when:
    // m_eventStarted && !m_eventEnded

    ThreadData();
    ~ThreadData();

    // Static member functions to do the nasty casting
    // of 'LPVOID' to 'Threadable'
    static void* __ThreadFunker(void* lpThreadParameter);
    static void __ThreadCleanup(void* lpThreadParameter);
};

//
// Constructors are dull sometimes!
//
Threadable::ThreadData::ThreadData() :
    m_threadId(0),
    m_exitCode(0),
    m_keepGoing(true),
    m_eventStarted(false),
    m_eventEnded(false)
 {
    int stat;

    stat = pthread_mutex_init ( &m_mutex, NULL );
    if(stat != 0)
    {
	ERROR_MSG(("Cannot create MUTEX! %x\n", stat));
    }

    stat = pthread_cond_init ( &m_cond , NULL );
    if(stat != 0)
    {
	ERROR_MSG(("Cannot create Condition Variable! %x\n", stat));
    }
}

Threadable::ThreadData::~ThreadData()
{
    int stat;

    stat = pthread_cond_destroy( &m_cond );
    if(stat != 0)
    {
	ERROR_MSG(("Cannot destroy Condition Variable! %x\n", stat));
    }
    stat = pthread_mutex_destroy(&m_mutex);
    if(stat != 0)
    {
	ERROR_MSG(("Cannot destroy MUTEX! %x\n", stat));
    }
}

//
// This is the function that gets called to begin the new thread
// It casts the nasty void* to a 'Threadable', and
// calls the 'ThreadFunc' function.
//
/*static*/
void* Threadable::ThreadData::__ThreadFunker(void* lpThreadParameter)
{
    int exitCode;
    int stat;

    // Get the pointer to the underlying 'Threadable'
    assert(lpThreadParameter != 0);
    Threadable* thread = (Threadable*)lpThreadParameter;

    // This Lock/Unlock sequence is so that the set-up of the bool's in the
    // ''Start'' function is forced to happen.
    //
    stat = pthread_mutex_lock( &thread->m_data->m_mutex );
    if(stat != 0)
    {
	// Problem!!
	ERROR_MSG(("ThreadFunker(%p): "
		   "Failed to lock mutex %x\n",
		   thread, stat));
	exit(0);
    }
    stat = pthread_mutex_unlock( &thread->m_data->m_mutex );
    if(stat != 0)
    {
	// Problem!!
	ERROR_MSG(("ThreadFunker(%p): "
		   "Failed to un-lock mutex %x\n", 
		   thread, stat));
	exit(0);
    }

    // Add the cleanup handler.  This does nothing but set a flag to say
    // that we have finished.  It is usefull to have this sort of info though.
    // After the thread has set this flag, it must exit, without using any
    // of the thread resources.
    pthread_cleanup_push(__ThreadCleanup, lpThreadParameter);

    ERROR_MSG(("Threadable::ThreadData::ThreadFunker(%p): "
	       "Starting user function\n", 
	       thread));

    // Run the ThreadFunc
    exitCode = thread->ThreadFunc();

    ERROR_MSG(("Threadable::ThreadData::ThreadFunker(%p): "
	       "User function over\n", 
	       thread));

    // save the exit code
    thread->m_data->m_exitCode = exitCode;

    // This will cause the cleanup handler to be called.  After this
    // point, the thread must assume that the data structure pointed to
    // by 'thread' does not exist any more!
    pthread_cleanup_pop(1);

    ERROR_MSG(("Threadable - Child thread has ended\n"));

    // Return the exit code
    return (void*)exitCode;
}

/*static*/
void Threadable::ThreadData::__ThreadCleanup(void* lpThreadParameter)
{
    int stat;

    // Get the pointer to the underlying 'Threadable'
    assert(lpThreadParameter != 0);
    Threadable* thread = (Threadable*)lpThreadParameter;

    ERROR_MSG(("ThreadCleanup(%p)\n", thread));

    stat = pthread_mutex_lock( &thread->m_data->m_mutex );
    if(stat != 0)
    {
	// Problem!!
	ERROR_MSG(("ThreadCleanup(%p): "
		   "Failed to lock mutex %x\n",
		   thread, stat));
	exit(0);
    }

    // This thread has now ended!
    thread->m_data->m_eventEnded = true;

    // Signal to any other threads
    stat = pthread_cond_signal( &thread->m_data->m_cond );
    if(stat != 0)
    {
	// Problem!!
	ERROR_MSG(("ThreadCleanup(%p): "
		   "Failed to signal condition variable! %x\n",
		   thread, stat));
	exit(0);
    }

    // This unlock is the last possible use of the 'thread' data
    // structure!
    stat = pthread_mutex_unlock( &thread->m_data->m_mutex );
    if(stat != 0)
    {
	// Problem!!
	ERROR_MSG(("ThreadCleanup(%p): "
		   "Failed to un-lock mutex %x\n",
		   thread, stat));
	exit(0);
    }
}

//
// Create a new 'Threadable' object
//
Threadable::Threadable()
{
    m_data = new ThreadData();
    assert(m_data != 0);

    ERROR_MSG(("Threadable::Threadable(%p)\n", this));
}

//
// This will hopefully stop the thread if it is running, and
// clean up any data that has been allocated
//
Threadable::~Threadable()
{
    assert(m_data != 0);

    Kill();

    /* If we delete this, with the thread still runnning
     * then we will crash.  We might as well wait here for a while then.
     */
    while(IsActive())
	/*EMPTY*/;

    delete m_data;

    ERROR_MSG(("Threadable::~Threadable(%p)\n", this));
}

//
// This will start the new thread running, returns 'true'
// if everything is OK.
//
bool Threadable::Start()
{
    int stat;

    assert(m_data != 0);

    ERROR_MSG(("Threadable::Start\n"));

    stat = pthread_mutex_lock( &m_data->m_mutex );
    if(stat != 0)
    {
	// Problem!!
	ERROR_MSG(("Failed to lock mutex %x\n", stat));
	exit(0);
    }

    assert(m_data->m_keepGoing);
    assert(!m_data->m_eventStarted);
    assert(!m_data->m_eventEnded);

    // Set the event here saying that the thread is running
    m_data->m_eventStarted = true;

    stat = pthread_create ( &m_data->m_threadId,
			    0,
			    ThreadData::__ThreadFunker,
			    (void*)this );
    if(stat != 0)
    {
	// Problem!!
	ERROR_MSG(("Create Thread failed! %x\n", stat));

	// Pretend that the thread has ended already
	m_data->m_eventEnded = true;
    }
    else
    {
	// Is this needed?
	pthread_detach (m_data->m_threadId);
    }

    // Actually run the thread now!
    stat = pthread_mutex_unlock( &m_data->m_mutex );
    if(stat != 0)
    {
	// Problem!!
	ERROR_MSG(("Failed to un-lock mutex %x\n", stat));
	exit(0);
    }

    return true;
}

//
// Returns 'true' if the thread is running
//
bool Threadable::IsActive()
{
    int stat;
    bool isRunning;

    assert(m_data != 0);

    stat = pthread_mutex_lock( &m_data->m_mutex );
    if(stat != 0)
    {
	// Problem!!
	ERROR_MSG(("Threadable::IsActive(%p): "
		   "Failed to lock mutex %x\n",
		   this, stat));
	exit(0);
    }

    isRunning = m_data->m_eventStarted && !m_data->m_eventEnded;

    stat = pthread_mutex_unlock( &m_data->m_mutex );
    if(stat != 0)
    {
	// Problem!!
	ERROR_MSG(("Threadable::IsActive(%p): "
		   "Failed to un-lock mutex %x\n",
		   this, stat));
	exit(0);
    }

    // Are we running the other thread.
    return isRunning;
}

//
// Returns the exit code of the thread
//
int Threadable::GetExitCode()
{
    assert(m_data != 0);

    return m_data->m_exitCode;
}

//
// Should the run function keep going?
//
bool Threadable::KeepGoing()
{
    int stat;
    bool keepGoing;

    assert(m_data != 0);

    stat = pthread_mutex_lock( &m_data->m_mutex );
    if(stat != 0)
    {
	// Problem!!
	ERROR_MSG(("Threadable::KeepGoing(%p): "
		   "Failed to lock mutex %x\n",
		   this, stat));
	exit(0);
    }

    keepGoing = m_data->m_keepGoing;

    stat = pthread_mutex_unlock( &m_data->m_mutex );
    if(stat != 0)
    {
	// Problem!!
	ERROR_MSG(("Threadable::KeepGoing(%p): "
		   "Failed to un-lock mutex %x\n",
		   this, stat));
      	exit(0);
    }

    return keepGoing;
}

//
// Signal to the run function that we should stop soon.
//
void Threadable::Stop()
{
    int stat;

    assert(m_data != 0);

    ERROR_MSG(("Threadable::Stop(%p)\n", this));

    stat = pthread_mutex_lock( &m_data->m_mutex );
    if(stat != 0)
    {
	// Problem!!
	ERROR_MSG(("Threadable::Stop(%p): "
		   "Failed to lock mutex %x\n",
		   this, stat));
        return;
    }
    
    m_data->m_keepGoing = false;

    stat = pthread_mutex_unlock( &m_data->m_mutex );
    if(stat != 0)
    {
	// Problem!!
	ERROR_MSG(("Threadable::Stop(%p): "
		   "Failed to un-lock mutex %x\n",
		   this, stat));
        return;
    }
}

//
// Wait for the other thread to end.
// If it does not stop by itslef after the timeout then we take
// drastic measures (not that drstic on a nice POSIX system)
//
void Threadable::Kill(unsigned millisec /*= 1000*/)
{
    int stat;

    assert(m_data != 0);

    // Should this have been called before?
    Stop();

    stat = pthread_mutex_lock( &m_data->m_mutex );
    if(stat != 0)
    {
	// Problem!!
	ERROR_MSG(("Threadable::Kill(%p): "
		   "Failed to lock mutex %x\n",
		   this, stat));
    }

    if(m_data->m_eventStarted && !m_data->m_eventEnded)
    {
	// If we have started, and not ended, then we will wait until
	// we have.
	if(millisec != 0)
	{
	    // There is a limit to how long we will wait.
	    struct timeval now;
	    gettimeofday(&now, 0);
	    struct timespec timeout;
	    timeout.tv_sec = now.tv_sec + (millisec / 1000);
	    timeout.tv_nsec = now.tv_usec * 1000 + (millisec % 1000) * 1000000;
	    while(timeout.tv_nsec > 1000000000)
	    {
		timeout.tv_nsec -= 1000000000;
		timeout.tv_sec  += 1;
	    }
	    // timeout is when we will stop waiting
	    do
	    {
		ERROR_MSG(("Threadable::Kill(%p): "
			   "Starting cond_timedwait\n",
			   this));

		stat = pthread_cond_timedwait(&m_data->m_cond,
					      &m_data->m_mutex,
					      &timeout);

		ERROR_MSG(("Threadable::Kill(%p): "
			   "Done cond_timedwait %d\n",
			   this, stat));

		if(stat != ETIMEDOUT && stat != 0)
		{
		    ERROR_MSG(("Threadable::Kill(%p): "
			       "Failed to signal cond variable? %d\n",
			       this, stat));
		}
	    } while(!m_data->m_eventEnded && stat != ETIMEDOUT);

	    // At this point, we have either timed out, or we have
	    // been signalled
	}
	else
	{
	    // millisec was zero, so we want to cancel the thread now
	    // we can set stat to ETIMEDOUT, and it will be so
	    stat = ETIMEDOUT;
	}

	if(stat == ETIMEDOUT)
	{
	    // We timed out, so we try harder to kill the thread
	    // (this should always work, and it is not that harsh)
	    stat = pthread_cancel( m_data->m_threadId );
	    if(stat != 0)
	    {
		// Problem!!
		ERROR_MSG(("Threadable::Kill(%p): "
			   "Failed to cancel thread %x\n",
			   this, stat));
		exit(0);
	    }
	}
    }

    stat = pthread_mutex_unlock( &m_data->m_mutex );
    if(stat != 0)
    {
	// Problem!!
	ERROR_MSG(("Threadable::Kill(%p): "
		   "Failed to un-lock mutex %x\n",
		   this, stat));
    }
}


// Lower the threads priority a couple of points
void Threadable::LowPriority()
{
}

// Set priority back to normal
void Threadable::NormalPriority()
{
}

// Make the thread have more priority
void Threadable::HighPriority()
{
}
