// Win32 implememtation of the Mutex thread.  Uses the CreateMutex function
// 
// Modification History
// $Log: system_wide_mutex.cpp,v $
// Revision 1.11  2003/06/05 20:19:29  cl01
// Vain attampt to fix problem with mutexes
//
// Revision 1.10  2003/06/05 17:22:51  cl01
// Added extra checks.
//
// Revision 1.9  2003/06/05 15:41:10  cl01
// Tidied up interface
// Fixed potential race hazard over acquired var in threads.
//
// Revision 1.8  2002/09/23 16:07:14  cl01
// Changed actual mutex object from Semaphore to Mutex
//
// Revision 1.7  2002/06/27 14:31:59  cl01
// Relaxed rules for printing filenames in Mutex Logging
//
// Revision 1.6  2002/06/27 13:40:10  cl01
// Added debug log (to diagnose spi mutex deadlocks)
//
// Revision 1.5  2002/03/13 18:17:43  bcsw
// Changed non-Unicode support
//
// Revision 1.4  2001/12/19 17:37:25  ajh
// Internationalisation route 2:
// Dual interface, without ichar.
//
// Revision 1.3  2001/11/27 09:53:02  at01
// Corrected size of allocated buffer for Unicode strings.
//
// Revision 1.2  2001/11/19 15:14:27  at01
// HostStack and DFUWizard updated to use Unicode characters under Win32 via the unicode/ichar.h support. This will use the Microsoft Layer for Unicode under Windows 9x/ME (hopefully). This initial set of changes is likely to break everything other than DFUWizard until suitable changes are made elsewhere.
//
// Revision 1.1  2001/10/11 16:44:56  bs01
// Rename from mutex to system_wide_mutex (the shorter name could have cause people to use it inappropriately and get unintended interactions between separate processes.)
//
// Revision 1.5  2001/10/11 10:13:52  cl01
// Added code to prefix any mutex name.
//
// Revision 1.4  2001/10/10 17:26:17  cl01
// Added code to prefix any mutex name.
//
// Revision 1.3  2001/10/10 11:53:47  cl01
// Uses Semaphore as underlying sync obj.  Mutex had problems when moved between threads.
//
// Revision 1.2  2001/10/09 18:03:47  cl01
// Added Exclusive Access for Widcomm USB driver.
//
//
// $Id: //depot/bc/main/devHost/util/thread/win32/system_wide_mutex.cpp#13 $

#include "thread/system_wide_mutex.h"
#include <windows.h>
#include <malloc.h>
#include <stdio.h>
#include <time.h>
#include <assert.h>

#include "thread/error_msg.h"


#ifdef MUTEX_LOG
#define LOG_MSG(x) data_->WriteLog(sprintf_static(x))
#else
#define LOG_MSG(x) do { } while(0)
#endif 




class SystemWideMutex::MutexData
{
public:
    MutexData(const char * mutex_name);
    MutexData(const wchar_t * mutex_name);
    ~MutexData();

    HANDLE handle;
	bool m_acquired;

#ifdef MUTEX_LOG
	void WriteLog(const char *string);
	bool enable_logging_;
#endif

private:
    void commonCreation ( const wchar_t *mutex_name );

};


SystemWideMutex::MutexData::MutexData(const char *mutex_name)
	: m_acquired(false)
{
    static const char prefix[] = "csr_mutex_";
    //  this call previously used _alloca.  It is probably better to do so,
    //  but I'm a coward when it comes to new things...  this string gets
    //  deleted at the end of the function.

#ifdef MUTEX_LOG
	enable_logging_ = (strcmp(mutex_name, "spi_app_mutex" ) == 0);
#endif

    char * string = new char [ strlen(mutex_name) + strlen(prefix) + 1 ];
    strcpy(string, prefix);
    strcat(string, mutex_name);

    char *sp = string;
    while (*sp)
    {
        if (strchr(".\\/\"*", *sp))
			*sp = '-';
        sp++;
    }
    handle = CreateMutexA(NULL, 0, string);
	OUTPUT_HANDLE_CREATE(handle);

    // OutputDebugString("Mutex created:");
    // OutputDebugString(string);
    // OutputDebugString("\n");
	// int lasterr = GetLastError();
    // assert(handle);
    delete string;
}

SystemWideMutex::MutexData::MutexData(const wchar_t *mutex_name)
	: m_acquired(false)
{
    static const wchar_t prefix[] = L"csr_mutex_";
    //  this call previously used _alloca.  It is probably better to do so,
    //  but I'm a coward when it comes to new things...  this string gets
    //  deleted at the end of the function.

#ifdef MUTEX_LOG
	enable_logging_ = (wcscmp(mutex_name, L"spi_app_mutex" ) == 0);
#endif

    wchar_t * string = new wchar_t [ wcslen(mutex_name) + wcslen(prefix) + 1 ];
    wcscpy(string, prefix);
    wcscat(string, mutex_name);

    wchar_t *sp = string;
    while (*sp)
    {
	if (wcschr(L".\\/\"*", *sp))
	    *sp = L'-';
	sp++;
    }
    handle = CreateMutexW(NULL, 0, string);
	OUTPUT_HANDLE_CREATE(handle);

    // OutputDebugString("Mutex created:");
    // OutputDebugString(string);
    // OutputDebugString("\n");
	// int lasterr = GetLastError();
    // assert(handle);
    delete string;
}

SystemWideMutex::MutexData::~MutexData()
{
    CloseHandle(handle);
	OUTPUT_HANDLE_CLOSE(handle);

}


#ifdef MUTEX_LOG
void SystemWideMutex::MutexData::WriteLog(const char *string)
{
	FILE *file;
	char buf[128]; 
	if (enable_logging_)
	{
		file = fopen("c:\\spi_app_mutex.log", "ab");
		_strtime(buf);
		fputs(buf, file);
		fputs(" ", file);
		// Prepare a string containing 30 characters containing the end of the exe name + the beginning of the arguments.
		{
			const char *cmd = GetCommandLineA();
			int pos;
			int len = strlen(cmd);
			const char * path_break = strstr(cmd,".exe\"");
			if (!path_break) 
				path_break = strstr(cmd,".EXE\"");
			if (!path_break) 
				path_break = strstr(cmd,"\" ");
			if (!path_break) 
				path_break = strstr(cmd," ");
			if (path_break)
				pos = path_break - cmd - 14;
			else
				pos = len;
			
			if (pos < 0)
				pos = 0;
			char *bufptr = buf;
			for (int i = pos; i < pos + 30; i++)
			{
				if (i < len)
					*bufptr++ = cmd[i];
				else
					*bufptr++ = ' ';
			}
			*bufptr = '\0';
					
			fputs(buf, file);
			fputs(" ", file);
		}
		fputs(string, file);
		fputs("\r\n", file);
		fclose(file);
	}
}
#endif

SystemWideMutex::SystemWideMutex(const char *mutex_name, bool AcquireImmed)
    : data_(new MutexData(mutex_name))
{
	if (AcquireImmed) Acquire(0);
}

SystemWideMutex::SystemWideMutex(const wchar_t *mutex_name, bool AcquireImmed)
    : data_(new MutexData(mutex_name))
{
	if (AcquireImmed) Acquire(0);
}


SystemWideMutex::~SystemWideMutex()
{
	Release();
	delete data_;
}

bool SystemWideMutex::Acquire(uint32 dwMilliseconds)
{
	DWORD wait_result;
	assert(data_->handle != INVALID_HANDLE_VALUE);
	wait_result = WaitForSingleObject(data_->handle, dwMilliseconds);
	if (wait_result != WAIT_OBJECT_0)
		wait_result = WaitForSingleObject(data_->handle, dwMilliseconds);

	data_->m_acquired = (wait_result == WAIT_OBJECT_0);

#ifdef MUTEX_LOG
	if (data_->enable_logging_)
	{
		if (data_->m_acquired)
			LOG_MSG(("Acquired mutex"));
		else
			LOG_MSG(("  Failed to acquire mutex"));
	}
#endif

	return data_->m_acquired;
}

void SystemWideMutex::Release()
{
	data_->m_acquired = false;
	BOOL release_result = ReleaseMutex(data_->handle);

#ifdef MUTEX_LOG
	if (data_->enable_logging_)
	{
		if (release_result)
			LOG_MSG(("Released mutex"));
	}
#endif

}

bool SystemWideMutex::IsAcquired() const
{
	assert(data_);
	return data_->m_acquired;
}

