///////////////////////////////////////////////////////////////////////
//
//  FILE   :  thread.cpp
//
//            (C) CSR 2001
//
//  AUTHOR :  Mark Marshall
//
//  PURPOSE:  The header file for all the WIN32 definitions of
//            the Threadable class.
//
//  $Id: //depot/bc/main/devHost/util/thread/win32/thread.cpp#14 $
//
///////////////////////////////////////////////////////////////////////

#include "thread/thread.h"

#define WIN32_LEAN_AND_MEAN	// Exclude rarely-used stuff from Windows headers
#include <windows.h>
#include <stdio.h>
#include <stdarg.h>
#include <tchar.h>
#ifndef _WINCE
#include <assert.h>
#define ASSERT assert
#else
#include <windef.h>
#include <dbgapi.h>
#endif

#include "../error_msg.h"

//
// This is the class that holds all of the platform specific
// data (and also the static 'ThreadFunc' function).
//
class Threadable::ThreadData : private NoCopy
{
public:
    HANDLE	    m_threadHandle;	// This is the handle of the thread
    DWORD	    m_threadId;		// What is this for?
    DWORD	    m_exitCode;		// This is the thread exit code

    // This bool is set to true before the thread is started.
    // THe ''ThreadFunc'' is expected to check on it status (by
    // calling ''KeepGoing'') to see if it should keep going.
    HANDLE          m_keepGoing;

    HANDLE	    m_eventStarted;	// This event is set while the other thread is running.
    HANDLE	    m_eventEnded;	// This event is set when it ends

    ThreadData();
    ~ThreadData();

    // Static member function to do the nasty casting of 'LPVOID' to 'Threadable'
    static DWORD WINAPI __ThreadFunker(LPVOID lpThreadParameter);
};

//
// Constructors are dull sometimes!
//
Threadable::ThreadData::ThreadData()
{
    m_threadHandle = NULL;
    m_threadId = 0;
    m_exitCode = 0;

    m_eventStarted = CreateEvent(NULL, TRUE, FALSE, NULL);
	OUTPUT_HANDLE_CREATE(m_eventStarted);
    if(m_eventStarted == NULL)
    {
	DWORD err = GetLastError();
	ERROR_MSG((("Cannot create EVENT! %x\n"), err));
    }

    m_eventEnded = CreateEvent(NULL, TRUE, FALSE, NULL);
	OUTPUT_HANDLE_CREATE(m_eventEnded);
    if(m_eventEnded == NULL)
    {
	DWORD err = GetLastError();
	ERROR_MSG((("Cannot create EVENT! %x\n"), err));
    }

    m_keepGoing = CreateEvent(NULL, TRUE, FALSE, NULL);
	OUTPUT_HANDLE_CREATE(m_keepGoing);
    if(m_keepGoing == NULL)
    {
	DWORD err = GetLastError();
	ERROR_MSG((("Cannot create EVENT! %x\n"), err));
    }
}

Threadable::ThreadData::~ThreadData()
{
    if(m_eventStarted != NULL)
	CloseHandle(m_eventStarted);
	OUTPUT_HANDLE_CLOSE(m_eventStarted);

    if(m_eventEnded != NULL)
	CloseHandle(m_eventEnded);
	OUTPUT_HANDLE_CLOSE(m_eventEnded);

    if(m_keepGoing != NULL)
	CloseHandle(m_keepGoing);
	OUTPUT_HANDLE_CLOSE(m_keepGoing);
}

//
// This is the function that gets called to begin the new thread
// It casts the nasty void* pointer to a 'Threadable', and
// calls the 'ThreadFunc' function.
//
/*static*/ DWORD WINAPI Threadable::ThreadData::__ThreadFunker(LPVOID lpThreadParameter)
{
    DWORD exitCode;

    // Get the pointer to the underlying 'Threadable'
    ASSERT(lpThreadParameter != 0);
    Threadable* thread = (Threadable*)lpThreadParameter;

    // Run the ThreadFunc
    exitCode = thread->ThreadFunc();

    // save the exit code
    thread->m_data->m_exitCode = exitCode;

    // Close the handle to the thread
    if(CloseHandle(thread->m_data->m_threadHandle) == 0)
    {
	// Problem!!
	DWORD err = GetLastError();
	ERROR_MSG((("Cannot close thread handle after ThreadFunc has finished! %x\n"), err));
    }
	OUTPUT_HANDLE_CLOSE(thread->m_data->m_threadHandle);
    thread->m_data->m_threadHandle = NULL;

    // Set the event here saying that we have exited
    if(SetEvent(thread->m_data->m_eventEnded) == 0)
    {
	// Problem!!
	DWORD err = GetLastError();
	ERROR_MSG((("Set Event failed! %x\n"), err));
    }

    // Return the exit code
    return exitCode;
}

//
// Create a new 'Threadable' object
//
Threadable::Threadable()
{
    m_data = new ThreadData();

    ERROR_MSG((("Threadable::Threadable\n")));
}

//
// This will hopefully stop the thread if it is running, and
// clean up any data that has been allocated
//
Threadable::~Threadable()
{
    ASSERT(m_data != 0);
    ASSERT(m_data->m_eventEnded != NULL);

    Kill();

    delete m_data;

    ERROR_MSG((("Threadable::~Threadable\n")));
}

//
// This will start the new thread running, returns 'true'
// if everything is OK.
//
bool Threadable::Start()
{
    HANDLE hnd;

    ASSERT(m_data != 0);

    hnd = CreateThread ( NULL, 0, ThreadData::__ThreadFunker, (LPVOID)this, CREATE_SUSPENDED, &m_data->m_threadId );
	OUTPUT_HANDLE_CREATE(hnd);
    if(hnd == NULL)
    {
	// Problem!!
	DWORD err = GetLastError();
	ERROR_MSG((("Create Thread failed! %x\n"), err));
        return false;
    }

    m_data->m_threadHandle = hnd;

    // Set the event here saying that the thread is running
    if(SetEvent(m_data->m_eventStarted) == 0)
    {
	// Problem!!
	DWORD err = GetLastError();
	ERROR_MSG((("Set Event failed! %x\n"), err));
    }

    // Say that we have not ended yet
    if(ResetEvent(m_data->m_eventEnded) == 0)
    {
	// Problem!!
	DWORD err = GetLastError();
	ERROR_MSG((("Set Event failed! %x\n"), err));
    }

    // Say that we should keep going
    if(SetEvent(m_data->m_keepGoing) == 0)
    {
	// Problem!!
	DWORD err = GetLastError();
	ERROR_MSG((("Set Event failed! %x\n"), err));
    }

    // Actually run the thread now!
    if(ResumeThread(m_data->m_threadHandle) == 0xffffffff)
    {
	// Problem!!
	DWORD err = GetLastError();
	ERROR_MSG((("Resume Thread failed! %x\n"), err));
	return false;
    }

    return true;
}

//
// Returns 'true' if the thread is running
//
bool Threadable::IsActive()
{
    DWORD stat;

    ASSERT(m_data != 0);

    // The 0 is the timeout value -> zero means test but dont wait
    stat = WaitForSingleObject(m_data->m_eventStarted, 0);
    if(stat != WAIT_OBJECT_0 && stat != WAIT_TIMEOUT)
    {
	// Problem!!
	DWORD err = GetLastError();
	ERROR_MSG((("WaitForSingleObject failed(IsAvtive.1)! %x\n"), err));
    }

    if(stat == WAIT_TIMEOUT)	// Have we started this thread
	return false;		// No -> we are not still runnning then, are we!

    // The 0 is the timeout value -> zero means test but dont wait
    stat = WaitForSingleObject(m_data->m_eventEnded, 0);
    if(stat != WAIT_OBJECT_0 && stat != WAIT_TIMEOUT)
    {
	// Problem!!
	DWORD err = GetLastError();
	ERROR_MSG((("WaitForSingleObject failed(IsAvtive.2)! %x\n"), err));
    }

    if(stat == WAIT_OBJECT_0)	// Have we signalled that we have exited
	return false;		// Yes -> then we are not active

    // We are running the other thread.
    return true;
}

//
// Returns the exit code of the thread
//
int Threadable::GetExitCode()
{
    ASSERT(m_data != 0);

    return (int)m_data->m_exitCode;
}

//
// Should the run function keep going?
//
bool Threadable::KeepGoing()
{
    DWORD stat;

    ASSERT(m_data != 0);

    stat = WaitForSingleObject(m_data->m_keepGoing, 0);
    if(stat != WAIT_OBJECT_0 && stat != WAIT_TIMEOUT)
    {
	// Problem!!
	DWORD err = GetLastError();
	ERROR_MSG((("WaitForSingleObject failed(IsAvtive.2)! %x\n"), err));
    }

    return stat == WAIT_OBJECT_0;
}

//
// Signal to the run function that we should stop soon.
//
void Threadable::Stop()
{
    ASSERT(m_data != 0);

    if(ResetEvent(m_data->m_keepGoing) == 0)
    {
	// Problem!!
	DWORD err = GetLastError();
	ERROR_MSG((("ResetEvent failed(Stop)! %x\n"), err));
    }
}

//
// Wait for the other thread to end.
// If it does not stop by itslef after the timeout
// Then we take drastic measures
//
void Threadable::Kill(unsigned millisec /*= 1000*/)
{
    DWORD stat;

    ASSERT(m_data != 0);

    // Should this have been called before?
    Stop();

    // The 0 is the timeout value -> zero means test but dont wait
    stat = WaitForSingleObject(m_data->m_eventStarted, 0);
    if(stat != WAIT_OBJECT_0 && stat != WAIT_TIMEOUT)
    {
	// Problem!!
	DWORD err = GetLastError();
	ERROR_MSG((("WaitForSingleObject failed(IsAvtive.1)! %x\n"), err));
    }
    if(stat == WAIT_TIMEOUT)	// Have we started this thread
	return;			// No -> we are not still runnning then, are we!

    // We wait for the event that signals that the child thread has died
    stat = WaitForSingleObject(m_data->m_eventEnded, millisec);
    if(stat != WAIT_OBJECT_0 && stat != WAIT_TIMEOUT)
    {
	// Problem!!
	DWORD err = GetLastError();
	ERROR_MSG((("WaitForSingleObject failed(IsActive.2)! %x\n"), err));
    }

    if(stat == WAIT_TIMEOUT)
    {
	// If we have got here, then this is very bad.  We will try to stomp on
	// the thread, but that might not work, and is very bad practice.
	// Hoefully we are about to close the application anyway?

	// Problem!!
	ERROR_MSG((("Thread did not stop!\n")));

	if(TerminateThread(m_data->m_threadHandle, 0xdead) == 0)
	{
	    // Problem!!
	    DWORD err = GetLastError();
	    ERROR_MSG((("Thread did not terminate (%x)!\n"), err));
	}

 	// This is nasty.  Will this kill the thread?
	if(CloseHandle(m_data->m_threadHandle) == 0)
	{
	    // Problem!!
	    DWORD err = GetLastError();
	    ERROR_MSG((("Cannot close thread handle on destruction! %x\n"), err));
	}
	OUTPUT_HANDLE_CLOSE(m_data->m_threadHandle);
	m_data->m_threadHandle = NULL;

	// Set the event here saying that we have exited
	if(SetEvent(m_data->m_eventEnded) == 0)
	{
	    // Problem!!
	    DWORD err = GetLastError();
	    ERROR_MSG((("Set Event failed! %x\n"), err));
	}
    }
}

void Threadable::LowPriority(void)
{
    ASSERT(m_data != 0);
    SetThreadPriority(m_data->m_threadHandle, THREAD_PRIORITY_LOWEST);
}

void Threadable::NormalPriority(void)
{
    ASSERT(m_data != 0);
    SetThreadPriority(m_data->m_threadHandle, THREAD_PRIORITY_NORMAL);
}

void Threadable::HighPriority(void)
{
    ASSERT(m_data != 0);
    SetThreadPriority(m_data->m_threadHandle, THREAD_PRIORITY_HIGHEST);
}
