/*******************************************************************************

    (C) COPYRIGHT Cambridge Consultants Ltd 1998

    CCL Project Reference C6066 - Lindisfarne

FILE
        $Workfile: l2cap_prim.h $
        $Revision: #9 $
        $Author: bcsw $
    

ORIGINAL AUTHOR
        Chris Swannack

CONTAINS
        L2CAP Interface Primitives.

DESCRIPTION
    The top interface to L2CAP. Multiple protocols will be able to post to this
    interface. Each protocol is responsible for registering a PSM and associated
    handle so that L2CAP knows where to send incoming events. 
	
NOTES/LIMITATIONS
    Based on Draft Version 1.0A of the Bluetooth specification

    Note, the service interface in the L2CAP specification is specified
    as part of a TCI testable interface. This interface is the implementation
    specific interface.
    

REFERENCES
    c6066-s-001 Bluetooth Software Architecture
    c6066-s-002 Link Manager Subsystem design
    c6066-s-003 L2CAP Subsystem design
    c6066-s-004 Device Manager Subsystem design
  

*******************************************************************************/
#ifndef BLUESTACK__L2CAP_PRIM_H     /* Once is enough */
#define BLUESTACK__L2CAP_PRIM_H

#ifdef RCS_STRINGS
static const char* const l2cap_prim_h_id = "$Header: //depot/bc/main/interface/app/bluestack/l2cap_prim.h#9 $";
#endif

/*============================================================================*
                    Header files on which this one depends
 *============================================================================*/
#include "hci.h" /* path modified */ 

/*============================================================================*
                       Interface #defined parameters
 *============================================================================*/

/*------------------------------------------------
 Configuration option defines 
--------------------------------------------------*/
#define MTU_OPTION              0x01
#define FLUSH_OPTION            0x02
#define QOS_OPTION              0x03  

/*------------------------------------------------
 Configuration option header size
--------------------------------------------------*/
#define SIZEOF_OPTION_HEADER    0x02

/*------------------------------------------------
 Configuration option sizes (excluding headers)
--------------------------------------------------*/
#define SIZEOF_MTU_OPTION       0x02
#define SIZEOF_FLUSH_OPTION     0x02
#define SIZEOF_QOS_OPTION       0x16  /* 22 decimal */

/*------------------------------------------
    QOS parameters 
--------------------------------------------*/
#define SERVICE_NO_TRAFFIC              0x00
#define SERVICE_BEST_EFFORT             0x01
#define SERVICE_GUARANTEED              0x02

/*------------------------------------------
    QOS defaults 
--------------------------------------------*/
#define MINIMUM_MTU             48

#define DEFAULT_MTU             672
#define DEFAULT_FLUSH_TO        0xFFFF
#define DEFAULT_SERVICE_TYPE    SERVICE_BEST_EFFORT
#define DEFAULT_TOKEN_RATE      0x00000000
#define DEFAULT_TOKEN_BUCKET    0x00000000
#define DEFAULT_PEAK_BW         0x00000000
#define DEFAULT_LATENCY         0xFFFFFFFF
#define DEFAULT_DELAY_VAR       0xFFFFFFFF

/*-------------------------------------------
 L2CA_CONNECT response codes
---------------------------------------------*/
#define L2CA_CONNECTION_SUCCESSFUL      0x0000
#define L2CA_CONNECTION_PENDING         0x0001
#define L2CA_CONNECTION_REJ_PSM         0x0002
#define L2CA_CONNECTION_REJ_SECURITY    0x0003
#define L2CA_CONNECTION_REJ_RESOURCES   0x0004
/* Not in the specification. Allows L2CAP to reject 
connect requests because LC not initialised and
Buffer_Size_Ind not received by L2CAP*/
#define L2CA_CONNECTION_REJ_NOT_READY   0xdddd
#define L2CA_CONNECTION_FAILED          0xddde
#define L2CA_CONNECTION_TIMEOUT         0xEEEE

/*-------------------------------------------
 L2CA_DISCONNECT_CFM result codes
---------------------------------------------*/
#define L2CA_DISCONNECT_SUCCESSFUL      0x0000
#define L2CA_DISCONNECT_TIMEOUT         0xEEEE

/*-------------------------------------------
 L2CA_CONFIG response codes
---------------------------------------------*/
#define CONFIG_SUCCESS                  0x0000
#define CONFIG_UNACCEPTABLE             0x0001
#define CONFIG_REJECTED                 0x0002
#define CONFIG_UNKNOWN                  0x0003
/* Not in the specification. Allows L2CAP to reject 
config requests locally */
#define CONFIG_REJECTED_LOCALLY         0xdddd
#define CONFIG_TIMEOUT                  0xEEEE

/*-------------------------------------------
 L2CA_PING_CFM result codes
---------------------------------------------*/
#define L2CA_PING_OK                    0x0000
#define L2CA_PING_TIMEOUT               0x0001
/* Not in the specification. Allows L2CAP to reject 
ping requests locally */
#define L2CA_PING_REJECTED              0xdddd

/*--------------------------------------------
 L2CA_GETINFO_CFM result codes
---------------------------------------------*/
#define L2CA_GETINFO_OK                 0x0000
#define L2CA_GETINFO_NOT_SUPPORTED      0x0001
#define L2CA_GETINFO_TIMEOUT            0x0003
/* Not in the specification. Allows L2CAP to reject 
getinfo requests locally */
#define L2CA_GETINFO_REJECTED           0xdddd


/*-------------------------------------------
 L2CA_DATAWRITE_CFM result parameter codes
---------------------------------------------*/
#define WRITE_SUCCESS                   0x0000
#define WRITE_TIMEOUT                   0x0001
#define WRITE_LINK_TERMINATED           0x0002
/* Not in the spec. Rejected by L2CAP locally */
/* No L2CAP connection, channel not yet configured */
#define WRITE_NO_CONNECTION             0xaaaa

/*-------------------------------------------
 L2CA_DATAREAD_IND result parameter codes
---------------------------------------------*/
#define READ_SUCCESS                   0x0000
#define READ_FAIL                      0x0001

/*-------------------------------------------
 Logical Connectionless responses
---------------------------------------------*/
#define L2CA_CONNECTLESS_SUCCESSFUL      0x0000
#define L2CA_CONNECTLESS_FAIL            0x0001

/*-------------------------------------------
 Logical Connectionless responses
---------------------------------------------*/
#define L2CA_GROUP_SUCCESSFUL           0x0000
#define L2CA_GROUP_FAIL                 0x0001

/*-------------------------------------------
 L2CAP Connectionless Reception CID
---------------------------------------------*/
#define L2CA_CL_RX_CID                  0x0002

/*-------------------------------------------
 L2CAP invalid PSM value
---------------------------------------------*/
#define L2CA_PSM_INVALID                0x0000

/*-------------------------------------------------------------
 Start of L2CAP primitives 
---------------------------------------------------------------*/

#define L2CAP_PRIM_BASE    0x0000  


/*-------------------------------------------------------------
L2CAP Primitives
---------------------------------------------------------------*/

/* Add primitives here... */
typedef enum l2cap_prim_tag
{
    ENUM_SEP_L2CA_FIRST_PRIM = L2CAP_PRIM_BASE,
    ENUM_L2CA_REGISTER_REQ,
    ENUM_L2CA_REGISTER_CFM,
    ENUM_L2CA_CONNECT_REQ,
    ENUM_L2CA_CONNECT_RSP,
    ENUM_L2CA_CONNECT_IND,
    ENUM_L2CA_CONNECT_CFM,
    ENUM_L2CA_CONFIG_REQ,
    ENUM_L2CA_CONFIG_RSP,
    ENUM_L2CA_CONFIG_IND,
    ENUM_L2CA_CONFIG_CFM,
    ENUM_L2CA_DATAWRITE_REQ,
    ENUM_L2CA_DATAWRITE_CFM,
    ENUM_L2CA_DATAREAD_IND,
    ENUM_L2CA_DISCONNECT_REQ,
    ENUM_L2CA_DISCONNECT_RSP,
    ENUM_L2CA_DISCONNECT_IND,
    ENUM_L2CA_DISCONNECT_CFM,
    ENUM_L2CA_TIMEOUT_IND,
    ENUM_L2CA_QOSVIOLATION_IND,
    ENUM_L2CA_PING_REQ,
    ENUM_L2CA_PING_CFM,
    ENUM_L2CA_GETINFO_REQ,
    ENUM_L2CA_GETINFO_CFM,
    
    ENUM_L2CA_GROUP_CREATE_REQ,
    ENUM_L2CA_GROUP_CREATE_CFM,
    
    ENUM_L2CA_GROUP_CLOSE_REQ,
    ENUM_L2CA_GROUP_CLOSE_CFM,
    
    ENUM_L2CA_ENABLE_CONNECTIONLESS_REQ,
    ENUM_L2CA_ENABLE_CONNECTIONLESS_CFM,
    
    ENUM_L2CA_DISABLE_CONNECTIONLESS_REQ,
    ENUM_L2CA_DISABLE_CONNECTIONLESS_CFM,
    
    ENUM_SEP_L2CA_LAST_PRIM
}
L2CAP_PRIM_T;

/* ...and reflect here as a type */
#define L2CA_REGISTER_REQ           ((l2cap_prim_t)ENUM_L2CA_REGISTER_REQ)
#define L2CA_REGISTER_CFM           ((l2cap_prim_t)ENUM_L2CA_REGISTER_CFM)
#define L2CA_CONNECT_REQ            ((l2cap_prim_t)ENUM_L2CA_CONNECT_REQ)
#define L2CA_CONNECT_RSP            ((l2cap_prim_t)ENUM_L2CA_CONNECT_RSP)
#define L2CA_CONNECT_IND            ((l2cap_prim_t)ENUM_L2CA_CONNECT_IND)
#define L2CA_CONNECT_CFM            ((l2cap_prim_t)ENUM_L2CA_CONNECT_CFM)
#define L2CA_CONFIG_REQ             ((l2cap_prim_t)ENUM_L2CA_CONFIG_REQ)
#define L2CA_CONFIG_RSP             ((l2cap_prim_t)ENUM_L2CA_CONFIG_RSP)
#define L2CA_CONFIG_IND             ((l2cap_prim_t)ENUM_L2CA_CONFIG_IND)
#define L2CA_CONFIG_CFM             ((l2cap_prim_t)ENUM_L2CA_CONFIG_CFM)
#define L2CA_DATAWRITE_REQ          ((l2cap_prim_t)ENUM_L2CA_DATAWRITE_REQ)
#define L2CA_DATAWRITE_CFM          ((l2cap_prim_t)ENUM_L2CA_DATAWRITE_CFM)
#define L2CA_DATAREAD_IND           ((l2cap_prim_t)ENUM_L2CA_DATAREAD_IND)
#define L2CA_DISCONNECT_REQ         ((l2cap_prim_t)ENUM_L2CA_DISCONNECT_REQ)
#define L2CA_DISCONNECT_RSP         ((l2cap_prim_t)ENUM_L2CA_DISCONNECT_RSP)
#define L2CA_DISCONNECT_IND         ((l2cap_prim_t)ENUM_L2CA_DISCONNECT_IND)
#define L2CA_DISCONNECT_CFM         ((l2cap_prim_t)ENUM_L2CA_DISCONNECT_CFM)
#define L2CA_TIMEOUT_IND            ((l2cap_prim_t)ENUM_L2CA_TIMEOUT_IND)
#define L2CA_QOSVIOLATION_IND       ((l2cap_prim_t)ENUM_L2CA_QOSVIOLATION_IND)
#define L2CA_PING_REQ               ((l2cap_prim_t)ENUM_L2CA_PING_REQ)
#define L2CA_PING_CFM               ((l2cap_prim_t)ENUM_L2CA_PING_CFM)
#define L2CA_GETINFO_REQ            ((l2cap_prim_t)ENUM_L2CA_GETINFO_REQ)
#define L2CA_GETINFO_CFM            ((l2cap_prim_t)ENUM_L2CA_GETINFO_CFM)

#define L2CA_GROUP_CREATE_REQ           ((l2cap_prim_t)ENUM_L2CA_GROUP_CREATE_REQ)
#define L2CA_GROUP_CREATE_CFM           ((l2cap_prim_t)ENUM_L2CA_GROUP_CREATE_CFM)

#define L2CA_GROUP_CLOSE_REQ            ((l2cap_prim_t)ENUM_L2CA_GROUP_CLOSE_REQ)
#define L2CA_GROUP_CLOSE_CFM            ((l2cap_prim_t)ENUM_L2CA_GROUP_CLOSE_CFM)

#define L2CA_ENABLE_CONNECTIONLESS_REQ  ((l2cap_prim_t)ENUM_L2CA_ENABLE_CONNECTIONLESS_REQ)
#define L2CA_ENABLE_CONNECTIONLESS_CFM  ((l2cap_prim_t)ENUM_L2CA_ENABLE_CONNECTIONLESS_CFM)

#define L2CA_DISABLE_CONNECTIONLESS_REQ ((l2cap_prim_t)ENUM_L2CA_DISABLE_CONNECTIONLESS_REQ)
#define L2CA_DISABLE_CONNECTIONLESS_CFM ((l2cap_prim_t)ENUM_L2CA_DISABLE_CONNECTIONLESS_CFM)

    
#define L2CAP_BASE	                (ENUM_SEP_L2CA_FIRST_PRIM + 1)
#define L2CAP_MAX                   (ENUM_SEP_L2CA_LAST_PRIM - 1)

/*---------------------------------- 
 end of #define of primitives 
------------------------------------*/

/*----------------------------------
 primitive type for l2cap 
------------------------------------*/
typedef uint16_t        l2cap_prim_t;

/*----------------------------------
 other parameter typedefs 
------------------------------------*/
typedef uint16_t        mtu_t;
typedef uint16_t        cid_t;
typedef uint16_t        timeout_t;
typedef uint16_t        result_t;
typedef uint16_t        response_t;
typedef uint16_t        status_t;
typedef uint8_t         identifier_t;

typedef uint8_t         service_type_t;
typedef uint32_t        token_rate_t;
typedef uint32_t        token_bucket_t;
typedef uint32_t        bandwidth_t;
typedef uint32_t        latency_t;
typedef uint32_t        variation_t;

typedef uint16_t		options_t;

/*------------------------------------------------
 QOS flow specification structure
--------------------------------------------------*/

typedef struct
{
    service_type_t      service_type;   /* best effort, etc */
    token_rate_t        token_rate;     /* token rate */
    token_bucket_t      token_bucket;   /* token bucket */
    bandwidth_t         peak_bw;        /* peak bandwidth */
    latency_t           latency;        /* latency */
    variation_t         delay_var;      /* delay variation */

}QOS_FLOW_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Register a PSM so that L2CAP can multiplex between protocols. 
 *      In the case where we are operating under the CSR scheduler, 
 *      the phandle is the queue identifier for a particular higher 
 *      layer protocol.
 *      
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;       /* Always L2CA_REGISTER_REQ */
    psm_t               psm;        /* The local PSM */
    phandle_t           phandle;    /* Handle of where to send messages */

} L2CA_REGISTER_REQ_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Confirmation to previous register request 
 *      
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;           /* Always L2CA_REGISTER_CFM */
    phandle_t           phandle;        /* Handle of where to send messages */
    psm_t               psm;            /* The local PSM */

} L2CA_REGISTER_CFM_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Connect request from higher layer protocol to remote device              
 *      Note the addition of a psm_local - otherwise there is no way of
 *      knowing where to send the confirm to. 
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;       /* Always L2CA_CONNECT_REQ */
    psm_t               psm_local;  /* psm of local requesting protocol */
    BD_ADDR_T           bd_addr;    /* Bluetooth address of remote device */
    psm_t               psm;        /* The PSM on the remote device */
    hci_pkt_type_t		packet_type;/* Set to zero for default */

} L2CA_CONNECT_REQ_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Response to a connection indication              
 *      
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;           /* Always L2CA_CONNECT_RSP */
    identifier_t        identifier;     /* Used to identify the connect signal */
    cid_t               cid;            /* Channel identifier */
    response_t          response;       /* Success or failure */

} L2CA_CONNECT_RSP_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Indication of a connect request from a remote device.
 *      Here, the bd_addr is used by the higher layer protocol
 *      to match the cid with a particular device
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;           /* Always L2CA_CONNECT_IND */
    phandle_t           phandle;        /* Destination phandle */
    identifier_t        identifier;     /* Used to identify the connect signal */
    BD_ADDR_T           bd_addr;        /* Bluetooth address of remote device */
    cid_t               cid;            /* Channel identifier */
    psm_t               psm;			/* The PSM on the local device */

} L2CA_CONNECT_IND_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Confirmation to earlier connection request              
 *      
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;           /* Always L2CA_CONNECT_CFM */
    phandle_t           phandle;        /* Destination phandle */
    BD_ADDR_T           bd_addr;        /* Bluetooth address of remote device */
    cid_t               cid;            /* Channel identifier */
    psm_t               psm;			/* The PSM on the local device */
    response_t          response;       /* Logical connection setup codes */                                       

} L2CA_CONNECT_CFM_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Configuration request for a particular channel.
 *		Bitwise AND the options field bit flags to identify which of the 
 *		following three option fields are used.
 *
 *----------------------------------------------------------------------------*/

#define SPECIFY_MTU		((options_t)0x0001)
#define SPECIFY_FLUSH	((options_t)0x0002)
#define SPECIFY_QOS		((options_t)0x0004)

typedef struct
{
    l2cap_prim_t        type;       /* Always L2CA_CONFIG_REQ */
    cid_t               cid;        /* Channel identifier */
	options_t			options;	/* Bit field of options present below */
    mtu_t               in_mtu;     /* Max MTU this channel can accept */
    QOS_FLOW_T          outflow;    /* Local QOS spec */
    timeout_t           outflush_to;/* Local Flush timeout */
	bool_t				more_flag;	/* More to come */

} L2CA_CONFIG_REQ_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Configuration response for a particular channel.
 *      A pointer to a block of options is used to allow
 *      unknown options to be sent though L2CAP. In future, 
 *      remote devices may send unknown options to L2CAP
 *      and there is no conveniant way to deal with them.
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;                   /* Always L2CA_CONFIG_RES */
    cid_t               cid;                    /* Channel identifier */
    identifier_t        identifier;             /* Used to identify the config signal */
    uint16_t            size_options;			/* Size of options */
    uint8_t             *options;				/* The options */
    response_t          response;               /* Configuration response codes */
	bool_t				more_flag;				/* More to come */

} L2CA_CONFIG_RSP_T;


/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Configuration indication for a particular channel.
 *      L2CAP does not attempt to parse the options into a
 *      regular structure form since future compatability allows
 *      unknown options to be sent though L2CAP. To allow for this
 *      possibility a pointer a block of options is used.
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;                   /* Always L2CA_CONFIG_IND */
    phandle_t           phandle;                /* Destination phandle */
    cid_t               cid;                    /* Channel identifier */
    identifier_t        identifier;             /* Used to identify the config signal */
    uint16_t            size_options;			/* Size of options */
    uint8_t             *options;				/* The options */
	bool_t				more_flag;				/* More to come */

} L2CA_CONFIG_IND_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Configuration confirmation for a particular channel              
 *      
 *		This primitive indicates the contents of a L2CA_CONFIG_RES signal from
 *		the remote L2CAP. The options flag field shows which of the standard
 *		options were sent in this signal. The in_mtu, outflow and flush_to
 *		fields show the aggregate options over all L2CA_CONFIG_RES signals
 *		so far. These will be set to default values if no option has been
 *		returned in a signal.
 *		Note that the more_flag indicates that negotiation is on-going and the
 *		L2CAP channel is not yet open.
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;			/* Always L2CA_CONFIG_CFM */
    phandle_t           phandle;        /* Destination phandle */
    cid_t               cid;			/* Channel identifier */
    response_t          response;		/* Configuration response codes */
	options_t			options;		/* Bit field of options present below */
    mtu_t               in_mtu;         /* Max MTU this channel can accept */
    QOS_FLOW_T          outflow;        /* Local QOS spec */
    timeout_t           flush_to;       /* Local Flush timeout */
	bool_t				more_flag;		/* More to come */

} L2CA_CONFIG_CFM_T;


/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Request to send L2CAP packet.
 *      Only one packet can be sent per channel at a time. Higher
 *      layers may only transmit another packet when they receive
 *      a datawrite_cfm primitive.
 *
 *		NOTE:
 *
 *		The data field may be used in two ways:
 *
 *		(1) If the length field is NON-ZERO, then the data field points to a
 *			pmalloc'ed block containing the packet who's length is indicated
 *			by the length field.
 *
 *		(2) If length is ZERO, then the data field points to a message block
 *			chain as defined by mblk.h
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;       /* Always L2CA_DATAWRITE_REQ */
    cid_t               cid;        /* Local channel ID */
    uint16_t            length;     /* Length of data  - See note above */
    void	            *data;      /* Pointer to data - See note above */

} L2CA_DATAWRITE_REQ_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Confirmation of L2CAP packet sent              
 *      
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;           /* Always L2CA_DATAWRITE_CFM */
    phandle_t           phandle;        /* Destination phandle */
    cid_t               cid;            /* Local channel ID */
    uint16_t            length;         /* length of data sent */
    result_t            result;         /* Successful or not */

} L2CA_DATAWRITE_CFM_T;


/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Indication of new L2CAP packet (with the data)
 *      
 *	NB, if result is not READ_SUCCESS, this indicates that invalid packets
 *	have been received. Under these circumstances L2CAP will discard 
 *	the invalid data and the length and data parameters will be zero/NULL.
 *      
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;           /* Always L2CA_DATAREAD */
    phandle_t           phandle;        /* Destination phandle */
    cid_t               cid;            /* Local channel ID */
    psm_t               cl_psm;         /* PSM number for connectionless rx */
    uint16_t            length;         /* Length of buffer */
    uint8_t             *data;          /* Pointer to buffer */
    result_t            result;         /* Successful or not */

} L2CA_DATAREAD_IND_T;


/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Request to disconnect L2CAP channel              
 *      
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;       /* Always L2CA_DISCONNECT_REQ */
    cid_t               cid;        /* Local channel ID */

} L2CA_DISCONNECT_REQ_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Response to request to disconnect L2CAP channel              
 *      Disconnection is non-negotiable
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;       /* Always L2CA_DISCONNECT_RSP */
    identifier_t        identifier; /* Used to identify the disconnect signal */
    cid_t               cid;        /* Local channel ID */

} L2CA_DISCONNECT_RSP_T;


/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Indication of request to disconnect L2CAP channel              
 *      
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;           /* Always L2CA_DISCONNECT_IND */
    phandle_t           phandle;        /* Destination phandle */
    identifier_t        identifier;     /* Used to identify the disconnect signal */
    cid_t               cid;            /* Local channel ID */

} L2CA_DISCONNECT_IND_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Confirmation of request to disconnect L2CAP channel              
 *      
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;           /* Always L2CA_DISCONNECT_CFM */
    phandle_t           phandle;        /* Destination phandle */
    cid_t               cid;            /* Local channel ID */
    uint16_t            result;         /* Successful or timed out */

} L2CA_DISCONNECT_CFM_T;


/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Indicates that the rtx or ertx timer has expired.              
 *      This will occur an implementation dependant number of times
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;           /* Always L2CA_TIMEOUT_IND */
    phandle_t           phandle;        /* Destination phandle */
    cid_t               cid;            /* Local channel ID */

} L2CA_TIMEOUT_IND_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Quality of service violation indication.              
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;           /* Always L2CA_QOSVIOLATION_IND */
    phandle_t           phandle;        /* Destination phandle */
    BD_ADDR_T           bd_addr;        /* Bluetooth device address that violated */

} L2CA_QOSVIOLATION_IND_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Ping request to remote device (send echo request signal)              
 *      Need the phandle otherwise do not know where to send
 *      the confirmation to
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;       /* Always L2CA_PING_REQ */
    BD_ADDR_T           bd_addr;    /* Bluetooth device address */
    uint8_t             *data;      /* data to be transmitted (and returned in PING_CFM command) */
    uint16_t            length;     /* number of bytes pointed to by data */
    psm_t               psm_local;  /* psm of local requesting protocol */

} L2CA_PING_REQ_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Ping confirm to previous request              
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;       /* Always L2CA_PING_CFM */
    phandle_t           phandle;    /* Destination phandle */
    uint8_t             *data;      /* data to be transmitted (and returned in PING_CFM command) */
    uint16_t            size;       /* number of bytes pointed to by data (spec calls 'length' a
                                     * different name here! */
    uint16_t            result;     /* Success or timeout */

} L2CA_PING_CFM_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Request info from remote device              
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;       /* Always L2CA_GETINFO_REQ */
    BD_ADDR_T           bd_addr;    /* Bluetooth device address */
    uint16_t            info_type;  /* Type of information requested */
    psm_t               psm_local;  /* psm of local requesting protocol */

} L2CA_GETINFO_REQ_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Confirmation to information request  
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;           /* Always L2CA_GETINFO_CFM */
    phandle_t           phandle;        /* Destination phandle */
    uint16_t            result;         /* Success, failure, etc */
    uint8_t             *info_data;     /* Data payload */
    uint16_t            size;           /* length of data in info_data */
} L2CA_GETINFO_CFM_T;

/*----------------------------------------------------------------------------*
 *  PURPOSE
 *
 *  To create a group so that connetionless data can be sent using the returned 
 *  CID
 *
 *  Change History
 *
 *  Added SP2 2/10/2000 To deal with TCS needing to page all devices in the net
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;       /* Always L2CA_GROUP_CREATE_REQ */
    psm_t               psm_local;  /* psm of local requesting protocol */

} L2CA_GROUP_CREATE_REQ_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *  To close a group 
 *      
 *  Change History
 *
 *  Added SP2 2/10/2000 To deal with TCS needing to page all devices in the net
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;           /* Always L2CA_GROUP_CREATE_CFM */
    phandle_t           phandle;        /* Destination phandle */
    cid_t               cid;            /* Channel identifier */
    psm_t               psm;            /* Copy of PSM from request */
    response_t          response;       /* Logical connection setup codes */                                       

} L2CA_GROUP_CREATE_CFM_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Group Close request              
 *      
 *  Change History
 *
 *  Added SP2 2/10/2000 To deal with TCS needing to page all devices in the net
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;           /* Always L2CA_GROUP_CLOSE_REQ */
    cid_t               cid;            /* Channel identifier */
    psm_t               psm_local;      /* psm of local requesting protocol */

} L2CA_GROUP_CLOSE_REQ_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Confirmation to earlier Group Close request              
 *      
 *  Change History
 *
 *  Added SP2 2/10/2000 To deal with TCS needing to page all devices in the net
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;           /* Always L2CA_GROUP_CLOSE_CFM */
    phandle_t           phandle;        /* Destination phandle */
    psm_t               psm;            /* Copy of PSM from request */
    response_t          response;       /* Logical connection setup codes */                                       

} L2CA_GROUP_CLOSE_CFM_T;



/*----------------------------------------------------------------------------*
 *  PURPOSE
 *
 *  To enable the reception of connectionless data
 *
 *  Change History
 *
 *  Added SP2 2/10/2000 To deal with TCS needing to page all devices in the net
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;       /* Always L2CA_ENABLE_CONNECTIONLESS_REQ */
    psm_t               psm;        /* psm of Sent Data to allow reception of */
    phandle_t           phandle;    /* Handle for the response */

} L2CA_ENABLE_CONNECTIONLESS_REQ_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Confirmation to earlier enable connectionless data request              
 *      
 *  Change History
 *
 *  Added SP2 2/10/2000 To deal with TCS needing to page all devices in the net
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;           /* Always L2CA_ENABLE_CONNECTIONLESS_CFM */
    phandle_t           phandle;        /* Destination phandle */
    response_t          response;       /* Logical connection setup codes */                                       

} L2CA_ENABLE_CONNECTIONLESS_CFM_T;

/*----------------------------------------------------------------------------*
 *  PURPOSE
 *
 *  To disable the reception of connectionless data
 *
 *  Change History
 *
 *  Added SP2 2/10/2000 To deal with TCS needing to page all devices in the net
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;       /* Always L2CA_DISABLE_CONNECTIONLESS_REQ */
    psm_t               psm;        /* psm of Sent Data to Disallow reception of */
    phandle_t           phandle;    /* Handle for the response */

} L2CA_DISABLE_CONNECTIONLESS_REQ_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Confirmation to earlier disable connectionless data request              
 *      
 *  Change History
 *
 *  Added SP2 2/10/2000 To deal with TCS needing to page all devices in the net
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
    l2cap_prim_t        type;           /* Always L2CA_DISABLE_CONNECTIONLESS_CFM */
    phandle_t           phandle;        /* Destination phandle */
    response_t          response;       /* Logical connection setup codes */                                       

} L2CA_DISABLE_CONNECTIONLESS_CFM_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Fields common to all "upstream" L2CAP primitives               
 *      
 *  Change History
 *
 *  Added JMG 9/5/01 for use by L2CAP proxy tasks.
 *
 *----------------------------------------------------------------------------*/

typedef struct
{
	l2cap_prim_t	type ;
	phandle_t		phandle ;
} L2CA_UPSTREAM_COMMON_T ;


/*------------------------------------------------
 Union of the primitives 
-------------------------------------------------*/

typedef union
{
    l2cap_prim_t                type;
	L2CA_UPSTREAM_COMMON_T		l2ca_upstream_common ;

    L2CA_REGISTER_REQ_T         l2ca_register_req;
    L2CA_REGISTER_CFM_T         l2ca_register_cfm;
    L2CA_CONNECT_REQ_T          l2ca_connect_req;
    L2CA_CONNECT_RSP_T          l2ca_connect_rsp;
    L2CA_CONNECT_IND_T          l2ca_connect_ind;
    L2CA_CONNECT_CFM_T          l2ca_connect_cfm;
    L2CA_CONFIG_REQ_T           l2ca_config_req;
    L2CA_CONFIG_RSP_T           l2ca_config_rsp;
    L2CA_CONFIG_IND_T           l2ca_config_ind;
    L2CA_CONFIG_CFM_T           l2ca_config_cfm;
    L2CA_DATAWRITE_REQ_T        l2ca_datawrite_req;
    L2CA_DATAWRITE_CFM_T        l2ca_datawrite_cfm;
    L2CA_DATAREAD_IND_T         l2ca_dataread_ind;
    L2CA_DISCONNECT_REQ_T       l2ca_disconnect_req;
    L2CA_DISCONNECT_RSP_T       l2ca_disconnect_rsp;
    L2CA_DISCONNECT_IND_T       l2ca_disconnect_ind;
    L2CA_DISCONNECT_CFM_T       l2ca_disconnect_cfm;
    L2CA_TIMEOUT_IND_T          l2ca_timeout_ind;
    L2CA_QOSVIOLATION_IND_T     l2ca_qosviolation_ind;
    L2CA_PING_REQ_T             l2ca_ping_req;
    L2CA_PING_CFM_T             l2ca_ping_cfm;
    L2CA_GETINFO_REQ_T          l2ca_getinfo_req;
    L2CA_GETINFO_CFM_T          l2ca_getinfo_cfm;

    L2CA_GROUP_CREATE_REQ_T             l2ca_group_create_req;
    L2CA_GROUP_CREATE_CFM_T             l2ca_group_create_cfm;

    L2CA_GROUP_CLOSE_REQ_T              l2ca_group_close_req;
    L2CA_GROUP_CLOSE_CFM_T              l2ca_group_close_cfm;

    L2CA_ENABLE_CONNECTIONLESS_REQ_T    l2ca_enable_connectionless_req;
    L2CA_ENABLE_CONNECTIONLESS_CFM_T    l2ca_enable_connectionless_cfm;

    L2CA_DISABLE_CONNECTIONLESS_REQ_T   l2ca_disable_connectionless_req;
    L2CA_DISABLE_CONNECTIONLESS_CFM_T   l2ca_disable_connectionless_cfm;

} L2CA_UPRIM_T;

/*============================================================================*
 *                               PUBLIC FUNCTIONS
 *============================================================================*/
/* None */


/*=============================== END OF FILE ================================*/
#endif /* ndef _L2CAP_PRIM_H */
