/*******************************************************************************

    (C) COPYRIGHT Cambridge Consultants Ltd 1998

    CCL Project Reference C6066 - Lindisfarne

FILE
        $Workfile: sds_prim.h $
        $Revision: #7 $
        $Author: bcsw $
    

ORIGINAL AUTHOR
        Rob Davis

CONTAINS
        SDS Interface Primitives.

DESCRIPTION
    The interface between the application and the SDS sub-system.
        
NOTES/LIMITATIONS
    Based on Draft Version 1.0b of the Bluetooth specification


REFERENCES
    c6066-s-001 Bluetooth Software Architecture
    c6066-s-003 L2CAP Subsystem design
    c6066-s-008 Service Discovery Subsystem design
  

*******************************************************************************/
#ifndef BLUESTACK__SDS_PRIM_H     /* Once is enough */
#define BLUESTACK__SDS_PRIM_H

#ifdef RCS_STRINGS
static const char* const sds_prim_h_id = 
"$Header: //depot/bc/main/interface/app/bluestack/sds_prim.h#7 $";
#endif

/*============================================================================*
                    Header files on which this one depends
 *============================================================================*/
#include "sdc_prim.h" 

/*============================================================================*
                       Interface #defined parameters
 *============================================================================*/

/*------------------------------------------------
     Request(CFM) result codes
--------------------------------------------------*/
#define REQ_ERROR    0x0001
#define REQ_PASS     0x0000

/*-------------------------------------------------------------
 Start of SDS primitives 
---------------------------------------------------------------*/

#define SDS_PRIM_BASE    ENUM_SEP_SDC_LAST_PRIM

/*-------------------------------------------------------------
 SDS Primitives
---------------------------------------------------------------*/

typedef enum sds_prim_tag
{
    ENUM_SEP_SDS_FIRST_PRIM = SDS_PRIM_BASE,
    
    ENUM_SDS_REGISTER_REQ,
    ENUM_SDS_REGISTER_CFM,
    ENUM_SDS_UNREGISTER_REQ,
    ENUM_SDS_UNREGISTER_CFM,
	ENUM_SDS_CONFIG_REQ,
    
    ENUM_SEP_SDS_LAST_PRIM
} SDS_PRIM_T;

#define SDS_REGISTER_REQ              ((sds_prim_t)(ENUM_SDS_REGISTER_REQ))
#define SDS_REGISTER_CFM              ((sds_prim_t)(ENUM_SDS_REGISTER_CFM))
#define SDS_UNREGISTER_REQ            ((sds_prim_t)(ENUM_SDS_UNREGISTER_REQ))
#define SDS_UNREGISTER_CFM            ((sds_prim_t)(ENUM_SDS_UNREGISTER_CFM))
#define SDS_CONFIG_REQ                ((sds_prim_t)(ENUM_SDS_CONFIG_REQ))

#define SDS_BASE	               (ENUM_SEP_SDS_FIRST_PRIM + 1)
#define SDS_MAX                    (ENUM_SEP_SDS_LAST_PRIM - SDS_PRIM_BASE - 1)

/*---------------------------------- 
 end of #define of primitives 
------------------------------------*/

/*----------------------------------
 primitive type for sds 
------------------------------------*/
typedef uint16_t        sds_prim_t;             /* same as l2cap_prim.h */

/*----------------------------------
 other parameter typedefs 
------------------------------------*/

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Request to register a service with the service discovery server.
 *
 *---------------------------------------------------------------------------*/
typedef struct
{
    sds_prim_t              type;          /* Always SDS_REGISTER_REQ */
	    phandle_t           phandle;       /* routing handle */
        uint8_t             *service_rec;  /* Pointer to service record data */
        uint16_t            num_rec_bytes; /* Number of bytes in the service */
		                                   /* record data */
} SDS_REGISTER_REQ_T;


/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Confirm the registration of a service with the service discovery 
 *      server.
 *
 *---------------------------------------------------------------------------*/
typedef struct
{
    sds_prim_t              type;         /* Always SDS_REGISTER_CFM */
	    phandle_t           phandle;      /* routing handle */
        uint32_t            svc_rec_hndl; /* ServiceRecordHandle of the */
		                                  /* registered service */
        uint16_t            result;       /* Success or failure */
} SDS_REGISTER_CFM_T;


/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Request to unregister a service with the service discovery server.
 *
 *---------------------------------------------------------------------------*/
typedef struct
{
    sds_prim_t              type;         /* Always SDS_UNREGISTER_REQ */
	    phandle_t           phandle;      /* routing handle */
        uint32_t            svc_rec_hndl; /* ServiceRecordHandle of the */
		                                  /* registered service */
} SDS_UNREGISTER_REQ_T;


/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Confirm the unregistration of a service with the service discovery 
 *      server.
 *
 *---------------------------------------------------------------------------*/
typedef struct
{
    sds_prim_t              type;         /* Always SDS_UNREGISTER_CFM */
	    phandle_t           phandle;      /* routing handle */
        uint32_t            svc_rec_hndl; /* ServiceRecordHandle of the */
		                                  /* unregistered service */
        uint16_t            result;       /* Success or failure */
} SDS_UNREGISTER_CFM_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *       Configure L2CAP parameters
 *
 *			This currently only applies to the MTU size.
 *			Note that this request has no confirmation
 *			(if the value supplied is invalid, the DEFAULT_MTU value is used).
 *
 *---------------------------------------------------------------------------*/
typedef struct
{
        sds_prim_t          type;   /* Always SDS_CONFIG_REQ */
        uint16_t			mtu;	/* L2CAP MTU value */
} SDS_CONFIG_REQ_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *		Common elements of "upstream" SDS primitives
 *
 *---------------------------------------------------------------------------*/
typedef struct
{
    sds_prim_t              type;         /* Whatever */
	    phandle_t           phandle;      /* routing handle */
} SDS_UPSTREAM_COMMON_T;

/*------------------------------------------------
 Union of the primitives 
-------------------------------------------------*/

typedef union 
{
    sds_prim_t              type;
	SDS_UPSTREAM_COMMON_T	sds_upstream_common;
	
    SDS_REGISTER_REQ_T      sds_register_req;
    SDS_REGISTER_CFM_T      sds_register_cfm;
    SDS_UNREGISTER_REQ_T    sds_unregister_req;
    SDS_UNREGISTER_CFM_T    sds_unregister_cfm;
	SDS_CONFIG_REQ_T		sds_config_req;

} SDS_UPRIM_T;

/*============================================================================*
 *                               PUBLIC FUNCTIONS
 *===========================================================================*/
/* None */


/*=============================== END OF FILE ===============================*/
#endif /* ndef _SDS_PRIM_H */
