/****************************************************************************
FILE
        bccmdpdu.h  -  bluecore command interpreter packet structure

DESCRIPTION
	There are two bidirectional "private channels" that carry messages
	between the host and the host controller (aka the bc01 chip) over the
	BCSP (or USB) link:

	 1) One carries commands from a control entity on the host (possibly
	    called bcmgr) to the bccmd task on the host.  The bccmd task is a
	    command interpreter, carrying out local actions in response to
	    the host's commands.  The channel's reverse path carries command
	    responses back to the control entity on the host.

	    This channel will allow the host to configure, control and
	    monitor the CSR/bluecore Bluetooth implementation.

	 2) The second channel carries commands from the hq (host querier)
	    task on the host controller to a command interpreter on the
	    host.  The reverse channel carries back the host's command
	    responses.

	    This channel is expected to find only very limited use.  At
	    present, the only messages are from fault() and delayed_panic().

	This file describes the protocol pdus carried on the first channel.
	The second channel's protocol is described in hqpdu.h.

	In addition to the host<->host_controller path, it is anticipated
	that one or more alternative means of passing commands into the bccmd
	will be provided, e.g., from the chip's SPI interface.  It is
	expected that the same protocol/pdus will be used on all interfaces
	to bccmd, with command responses being returned to the corresponding
	command sources.

	It is also anticipated that the set of commands held in the pdus may
	have different behaviours depending on the interface on which the
	commands are submitted.  Notably, for security reasons, some commands
	will be rejected if they reach bccmd via the BCSP channel.

	As these pdus are to pass through the BCSP, they are structured to
	fit CCL's "bluecore friendly format", i.e. the pdus' wire structures
	will match the xap's memory structure.  The lower byte of each uint16
	travels over the wire first (confirmed by jrrk).  Thus, the pdus can
	be defined as C structures on the understanding that the wire format
	will be a (byte) serialisation of the way in which the types will be
	laid out by the xap C compiler.

	To allow a possible future optimisation in the way in which hostio
	copies pdus to/from BUFFERS, all pdus defined here are an even number
	of bytes.  (This is also implied by use of the "bluecore friendly
	format".)

	The protocol is influenced by snmp.  The host (client) views the
	services presented by the bccmd (server) as a set of values to be
	read and/or written.  There are only two types of transaction:

	    To read a value, the host sends a GET-REQUEST pdu to the bccmd.
	    The bccmd reads the value and returns it in a GET-RESPONSE pdu to
	    the host.

	    To set a value, the host sends a SET-REQUEST pdu to the bccmd.
	    The bccmd sets the value and returns the new value in a
	    GET-RESPONSE pdu to the host.

	There are only three basic types of PDU:

	    GET-REQUEST - (called BCCMDPDU_GETREQ below) asks for the value
	        of a variable to be returned to the host.

	    GET-RESPONSE - (called BCCMDPDU_GETRESP below) returns the value
	        of a variable to the host.

	    SET-REQUEST - (called BCCMDPDU_SETREQ below) asks to set the
	        value of a variable.

	The protocol may be extended to provide a GET-NEXT-REQUEST in the
	future.  This would work in a manner similar to snmp.  There is no
	current need for this functionality.

	The host commands the bccmd Bluetooth chip to perform an action by
	writing to a particular variable.  For example, the chip might be
	rebooted by writing to a variable called "reboot".  Abstracting the
	bccmd's functionality as reading and writing variables promotes a
	simple protocol.

	Alas, not all of the protocol's transactions fit this simple
	read/write pattern cleanly - some variables are special in that they
	need extra parameters, e.g., accessing blocks of RAM.

	No attempt is made to support snmp's SMI - the way in which snmp
	names variables.  This protocol uses a single uint16 to identify each
	variable.

	Every GET-REQUEST and SET-REQUEST received by the bccmd causes it to
	(attempt to) return one GET-RESPONSE.  However, in exceptional
	circumstances, e.g. provoking a reboot, the host cannot guarantee to
	receive the GET_RESPONSE.  Thus, the host must use a timeout.

	The GET-REQUEST PDU is of fixed minimum size; it consists of an
	instance of the typedef BCCMDPDU, described below.  This effectively
	asks "return me the value of the variable with identifier ID".  Even
	though this PDU carries no data to the server it still provides
	data storage sized to allow the server's response.  This peculiarity
	is purely to simplify implementation.

	The two other PDU types consist of an instance of BCCMDPDU,
	optionally followed by extra data:

	 1) The SET_REQUEST BCCMDPDU effectively says "set the value of the
	    variable with identifier ID to be ..."  The PDU carries new value
	    in a union at the end of the BCCMDPDU.

	    However, some variables don't have values, notably "variables"
	    which cause an action when written, e.g., reboot.

	 2) The BCCMDPDU part of the GET_RESPONSE PDU effectively says
	    "the value of the variable with identifier ID is ...".  The
	    PDU carries the item's value in the union at the end of the
	    BCCMDPDU.

	    However, if the data cannot be returned to the host then the data
	    part of the PDU is still returned - it is just not initialised by
	    the server.  The status code in the header indicates the cause of
	    the caller's disappointment.

	Most variables' values fit within the union at the end of the 
	BCCMDPDU struct.  This is wasteful in that the largest data struct
	sets the size of all BCCMDPDUs, but it is simple to work with, and
	this traffic is rare, so we don't care too much about wasting
	bandwidth or RAM.

	However, a handful of data types have variable sized data - values
	for RAM, ps and buffers.  Here life becomes ugly.  The union at the
	end of the BCCMDPDU has a (small) fixed size array for carrying these
	data, but it will be acceptable for the array to be bigger than
	advertised.  The actual size of the data can be determined by the
	pdulen field.

	Example Transactions
	--------------------

	1) To read the value of variable BCCMDVARID_BC01_STATUS

	  host->bccmd BCCMDPDU_GETREQ

	    uint16 field 		value 
	    1	   BCCMDPDU.type	0      (BCCMDPDU_GETREQ)
	    2	   BCCMDPDU.pdulen	9      (number of uint16s in PDU)
	    3	   BCCMDPDU.seqno	1234   (value chosen by host)
	    4	   BCCMDPDU.varid	10241  (BCCMDVARID_BC01_STATUS)
	    5	   BCCMDPDU.status 	0      (BCCMDPDU_STAT_OK)
	    6	   BCCMDPDU.d.u16 	0      (empty)
	    7-9	   ignored              0      (zero padding)

	  bccmd->host BCCMDPDU_GETRESP

	    uint16 field 		value 
	    1	   BCCMDPDU.type	1      (BCCMDPDU_GETRESP)
	    2	   BCCMDPDU.pdulen	9      (number of uint16s in PDU)
	    3	   BCCMDPDU.seqno	1234
	    4	   BCCMDPDU.varid	10241  (BCCMDVARID_BC01_STATUS)
	    5	   BCCMDPDU.status 	0      (BCCMDPDU_STAT_OK)
	    6	   BCCMDPDU.d.u16 	0x8642 (value of bc01_status)
	    7-9	   ignored              0      (zero padding)
  
	2) To set the value of variable BCCMDVARID_CONFIG_UART to the
	   value 0x1234:

	  host->bccmd BCCMDPDU_SETREQ

	    uint16 field 		value 
	    1	   BCCMDPDU.type	2      (BCCMDPDU_SETREQ)
	    2	   BCCMDPDU.pdulen	9      (number of uint16s in PDU)
	    3	   BCCMDPDU.seqno	1235   (value chosen by host)
	    4	   BCCMDPDU.varid	26626  (BCCMDVARID_CONFIG_UART)
	    5	   BCCMDPDU.status 	0      (BCCMDPDU_STAT_OK)
	    6	   BCCMDPDU.d.u16 	0x1234 (data)
	    7-9	   ignored              0      (zero padding)

	  bccmd->host BCCMDPDU_GETRESP

	    uint16 field 		value 
	    1	   BCCMDPDU.type	1      (BCCMDPDU_GETRESP)
	    2	   BCCMDPDU.pdulen	9      (number of uint16s in PDU)
	    3	   BCCMDPDU.seqno	1235
	    4	   BCCMDPDU.varid	26626  (BCCMDVARID_CONFIG_UART)
	    5	   BCCMDPDU.status 	0      (BCCMDPDU_STAT_OK)
	    6	   BCCMDPDU.d.u16 	0x1234 (data)
	    7-9	   ignored              0      (zero padding)

	3) Failing to set the value of non-existent uint8 variable
	   BCCMDVARID_UTTERCRUD to the value 0x23:

	  host->bccmd BCCMDPDU_SETREQ

	    uint16 field 		value 
	    1	   BCCMDPDU.type	2      (BCCMDPDU_SETREQ)
	    2	   BCCMDPDU.pdulen	9      (number of uint16s in PDU)
	    3	   BCCMDPDU.seqno	1236   (value chosen by host)
	    4	   BCCMDPDU.varid	9066   (BCCMDVARID_UTTERCRUD)
	    5	   BCCMDPDU.status 	0      (BCCMDPDU_STAT_OK)
	    6	   BCCMDPDU.d.u16 	0x0023 (dummy data)
	    7-9	   ignored              0      (zero padding)

	  bccmd->host BCCMDPDU_GETRESP

	    uint16 field 		value 
	    1	   BCCMDPDU.type	1      (BCCMDPDU_GETRESP)
	    2	   BCCMDPDU.pdulen	9      (number of uint16s in PDU)
	    3	   BCCMDPDU.seqno	1236
	    4	   BCCMDPDU.varid	9066   (BCCMDVARID_UTTERCRUD)
	    5	   BCCMDPDU.status 	1      (BCCMDPDU_STAT_NO_SUCH_VARID)
	    6	   BCCMDPDU.d.u16 	0x0000 (dummy data - no match needed.)
	    7-9	   ignored              0      (zero padding)

	4) To read the 3 words of memory from data address 0xb374.

	  host->bccmd BCCMDPDU_GETREQ

	    uint16 field 		value 
	    1	   BCCMDPDU.type	0      (BCCMDPDU_GETREQ)
	    2	   BCCMDPDU.pdulen	11     (number of uint16s in PDU)
	    3	   BCCMDPDU.seqno	1237   (value chosen by host)
	    4	   BCCMDPDU.varid	4097   (BCCMDVARID_MEMORY)
	    5	   BCCMDPDU.status 	0      (BCCMDPDU_STAT_OK)
	    6	   BCCMDPDU.d.mem.base_addr 0xb374  (memory address)
	    7	   BCCMDPDU.d.mem.len   3      (n words to be read)
	    8	   BCCMDPDU.d.mem.dummy 0      (padding)
	    9	   BCCMDPDU.d.mem.mem[0] 0     (dummy data)
	    10	   BCCMDPDU.d.mem.mem[1] 0     (dummy data)
	    11	   BCCMDPDU.d.mem.mem[2] 0     (dummy data)

	  bccmd->host BCCMDPDU_GETRESP

	    uint16 field 		value 
	    1	   BCCMDPDU.type	1      (BCCMDPDU_GETRESP)
	    2	   BCCMDPDU.pdulen	11     (number of uint16s in PDU)
	    3	   BCCMDPDU.seqno	1237   (value chosen by host)
	    4	   BCCMDPDU.varid	4097   (BCCMDVARID_MEMORY)
	    5	   BCCMDPDU.status 	0      (BCCMDPDU_STAT_OK)
	    6	   BCCMDPDU.d.mem.base_addr 0xb374
	    7	   BCCMDPDU.d.mem.len   3      (n words read)
	    8	   BCCMDPDU.d.mem.dummy 0      (padding)
	    9	   BCCMDPDU.d.mem.mem[0] 0x1357 (contents of 0xb374)
	    10	   BCCMDPDU.d.mem.mem[1] 0x9ace (contents of 0xb375)
	    11	   BCCMDPDU.d.mem.mem[2] 0x2468 (contents of 0xb376)

	5) To set the value of the persistent store variable PSKEY_BDADDR,
	   the local device's Bluetooth address, to {lap=0x123456,
	   uap=0x78, nap=0x9abc}.  The address is held in the ps as type
	   bdaddr, which is typedefed to be {uint32 lap; uint8 uap; 
	   uint16 nap;}, so two bytes are wasted in memory - one because only
	   24 bits of the uint32 are used, the other because of the XAP
	   treating the uint8 as a uint16.

	  host->bccmd BCCMDPDU_SETREQ

	    uint16 field 		value 
	    1	   BCCMDPDU.type	2      (BCCMDPDU_SETREQ)
	    2	   BCCMDPDU.pdulen	12     (number of uint16s in PDU)
	    3	   BCCMDPDU.seqno	1238   (value chosen by host)
	    4	   BCCMDPDU.varid	0x7003 (BCCMDVARID_PS)
	    5	   BCCMDPDU.status 	0      (BCCMDPDU_STAT_OK)
	    6	   BCCMDPDU.d.ps.id     1      (PSKEY_BDADDR)
	    7	   BCCMDPDU.d.ps.len    4      (sizeof(bdaddr))
	    8	   BCCMDPDU.d.ps.stores 0      (PS_STORES_DEFAULT)
	    9	   BCCMDPDU.d.ps.psmem[0] 0x0056  (lap - high word of uint32)
	    10	   BCCMDPDU.d.ps.psmem[1] 0x1234  (lap - low word of uint32)
	    11	   BCCMDPDU.d.ps.psmem[2] 0x0078  (uap - in low byte of word)
	    12	   BCCMDPDU.d.ps.psmem[3] 0x9abc  (nap)

	  <Delay, while ps value is written.>

	  bccmd->host BCCMDPDU_GETRESP

	    uint16 field 		value 
	    1	   BCCMDPDU.type	1      (BCCMDPDU_GETRESP)
	    2	   BCCMDPDU.pdulen	12     (number of uint16s in PDU)
	    3	   BCCMDPDU.seqno	1238   (value chosen by host)
	    4	   BCCMDPDU.varid	0x7003 (BCCMDVARID_PS)
	    5	   BCCMDPDU.status 	0      (BCCMDPDU_STAT_OK)
	    6	   BCCMDPDU.d.ps.id     1      (PSKEY_BDADDR)
	    7	   BCCMDPDU.d.ps.len    4      (sizeof(bdaddr))
	    8	   BCCMDPDU.d.ps.stores 0      (PS_STORES_DEFAULT)
	    9	   BCCMDPDU.d.ps.psmem[0] 0x0056  (lap - high word of uint32)
	    10	   BCCMDPDU.d.ps.psmem[1] 0x1234  (lap - low word of uint32)
	    11	   BCCMDPDU.d.ps.psmem[2] 0x0078  (uap - in low byte of word)
	    12	   BCCMDPDU.d.ps.psmem[3] 0x9abc  (nap)

REFERENCES
	bc01-s-024   -  bccmd protocol.
	bc01-s-026   -  bccmd basic command set.
	bc01-s-027   -  bccmd basic test commands.
	bc01-s-028   -  bccmd security.
	bc01-sp-040  -  bccmd ps commands.
	bc01-sp-045  -  bccmd radiotest commands.
	bc01-sp-047  -  spi bccmd protocol.

MODIFICATION HISTORY
	1.1   20:apr:01   ajh	Moved file from src/csr/bccmd to
				interface/host/bccmd, and sets the version
				number back to 1.1, hence the "OLD" section
				below.
	1.2   25:apr:01   cjo	Corrected comment.
	1.3   25:apr:01   cjo	Added support for commands
				BCCMDVARID_CRYPTKEYLEN and
				BCCMDVARID_PICONET_INSTANT.
				Replaced bloody C++ comment with C style
				as former breaks lint.
	1.4   18:may:01   cjo	Added support for commands
				BCCMDVARID_DEFRAG_RESET.
	1.5   8:jun:01    cjo   Added support for commands BCCMDVARID_RAND
				and BCCMDVARID_DEFAULT_TX_POWER.
	1.6   11:jun:01   cjo	Comments.
	1.7   11:jun:01   cjo	Comments.
	1.8   13:jun:01   cjo	Comments.
	1.9   29:jun:01   cjo	Extended BCCMDVARID_PS and BCCMDVARID_PSSIZE
				to allow stores to be specified.
	1.10  14:sep:01   cjo	Added BCCMDVARID_GET_CLR_EVT.
	1.11  14:sep:01   cjo	Changed BCCMDVARID_TESTA_AMUX and
				BCCMDVARID_TESTB_AMUX from read-only to
				read-write.  This changes the varids.
	1.12  20:sep:01   cjo	Added BCCMDVARID_MAXCRYPTKEYLEN.
	1.13  03:oct:01   pws   Add BCCMDVARID_DISABLE_VM.
	1.14  03:oct:01   cjo   Renamed BCCMDVARID_DISABLE_VM to be
				BCCMDVARID_KILL_VM_APPLICATION.
	1.15  03:oct:01   pws   Really changed it.
	1.16  07:dec:01   cjo   Added BCCMDVARID_GET_NEXT_BUILDDEF.
	1.17  17:jan:02   at    Added PS_STORES_T*, BCCMDVARID_PSCLRS and
				BCCMDVARID_PSCLRALLS to support psram.
	1.18  29:jan:02   pws   hci-15 #10: BCCMDVARID_SINGLE_CHAN,
	                        BCCMDVARID_HOPPING_ON.
	1.19  25:feb:02   pws   H15.47: add BCCMDVARID_CANCEL_PAGE.
	1.20  08:apr:02   pws   H15.106: add BCCMDVARID_CHECK_ROM,
				BCCMDVARID_CHECK_RAM,
				BCCMDVARID_CHECK_RAM_RESULT.
	1.21  26:jun:02   pws   H16.12: add BCCMDVARID_PCM_GAIN (uint16, r/w).
	1.22  26:jun:02   pws   H16.12: Changed to *_ATTENUATION.
	1.23  26:jun:02   pws   H16.12: Fix comment.
	1.24  01:jul:02   at    H16.16: Add I2C/EEPROM access commands.
	1.25  27:jul:02   at    H16.16: Removed i2c field from union.
	1.26  27:jul:02   at    H16.16: Renumbered BCCMDVARID_E2_APP_SIZE
				to avoid clash with BCCMDVARID_PCM_ATTENUATION.
	1.27  01:aug:02   cjo   H16.58: Add varids BCCMDVARID_STACK_CHECK and
				BCCMDVARID_STACK_CHECK_CFM to support
				rom/flash integrity check.
	1.28  09:aug:02   pws   H16.66: Add BCCMDVARID_BYPASS_UART placeholder.
	1.29  09:aug:02   pws   H16.67: Add BCCMDVARID_CHIPANAVER.
	1.30  29:aug:02   pws   T.579: Added BCCMDVARID_SYNC_CLOCK,
	                        BCCMDVARID_ANA_FTRIM.
	1.31  18:sep:02   at    H16.130: Added BCCMDVARID_BUILDID_LOADER.
	1.34  08:nov:02   at    Updated BCCMDVARID_STACK_CECK* description.
	1.35  08:nov:02   cjo   H17.46: Changed BCCMD command names.
				STACK_CHECK becomes FIRMWARE_CHECK, and
				STACK_CHECK_CFM becomes FIRMWARE_CHECK_MASK.
	1.36  08:nov:02   cjo   H17.46: Undo screwed up change control
				comments.
	1.37  13:nov:02   at    H17.52: Added BCCMDVARID_ENABLE_AFH.
	1.38  21:nov:02   pws   H17.71: BCCMDVARID_BER_THRESHOLD placeholder.
	1.39  06:dec:02   pws   H17.98: Add BCCMDVARID_PIO_STRONG_BIAS.
				Also correct comments on PIO register width.
	1.40  24:mar:03   cjo   Following advice from Simon Harrison, 
				corrected "* 64" comments for the Red-M call
				handoff command.
	1.41  20:may:03   npm   B-528 - bcccmd access to bootmode 
	1.42  06:jun:03   at    B-598: Add BCCMDVARID_PS_MEMORY_TYPE.
	1.43  03:sep:03   cjo   B-941: Commented that varids for RECAL,
				INIT_BT_STACK, ACTIVATE_BT_STACK, PSCLR,
				PSCLRALL, BC01_STATUS and CANCEL_PAGE are
				redundant.
	#44   18:sep:03   cjo   B-1031: Add BCCMDVARID_PSNEXT_ALL.
	#45   23:sep:03   pws   B-1055: scatternet/SCO data workaround.
	#46   01:oct:03   mm	B-1135: Added L2CAP CRC code.
	#47   20:nov:03   sms   B-1423: Reserve PDUs for medium rate testing.
	#48   08:jan:04   tg    B-1497: Added BCCMDVARID_KALDEBUG.
	#49   22:jan:04   doc   B-1809: No change to this file.
	#50   26:jan:04   tg    B-1497: Commented BCCMDVARID_KALDEBUG.
	#51   26:jan:04   sms   B-1799: Add medium rate debugging PDUs.
	#53   01:mar:04   pws   B-1716: Remote limits for sniff.
	#54   01:mar:04   ckl   B-1943: Babel / Casirack BCCMD VARIDs.

OLD MODIFICATION HISTORY
	1.1   29:nov:99	  cjo	Created.
	1.2   30:nov:99	  cjo	Corrected error in comment showing example
				ps transaction.
				Removed "vartype" and "pdulen" fields.
				Added memory block read example.
	1.3   30:nov:99	  cjo	Added extra varids, mostly for the adc.
	1.12  4:apr:00    cjo	BCCMDVARID_PS had the same value as another
				bccmdvarid.
	1.13  5:apr:00    cjo	Updated comments.
				Returned the pdulen field to BCCMDPDU_HDR.
				Added BCCMDVARID_RADIOTEST.
				Almost #included bccmdradiotestpdu.h.
	1.15  10:apr:00   cjo	Added serial number.
	1.16  11:apr:00   cjo	Converted to use BCCMDPDU.
				Removed BCCMDVARID_MODULE_ID.
	1.17  16:apr:00   cjo	Added BCCMDVARID_PSCLR. 
	1.21  17:apr:00   cjo	Added BCCMDVARID_PSCLRMOST and
				BCCMDVARID_PSCLRALL. 
				Overhauled read/write permissions.
	1.22  3:may:00    cjo	Corrected BCCMDVARID_BC01_STATUS example.
	1.23  3:may:00    cjo	Corrected more examples.
	1.24  3:may:00    cjo	Added BCCMDVARID_BUILDID,
				BCCMDVARID_CHIPVER and BCCMDVARID_CHIPREV.
	1.25  4:may:00    cjo	BCCMDVARID_CONFIG_UART to read/write.
	1.26  5:may:00    cjo	Added BCCMDVARID_MAP_SCO_PCM and
				BCCMDVARID_SPI_BUFSIZE.
	1.27  6:may:00    cjo	Added BCCMDVARID_COLD_HALT and
				BCCMDVARID_WARM_HALT.
				Added BCCMDPDU_STAT_NO_ACCESS.
	1.29  22:may:00   cjo	Added padding uint16 to BCCMDPDU_MEMORY
				and BCCMDPDU_PS to even up sizes of
				commands that use extensible arrays.
	1.30  22:may:00   cjo   Corrected ps and mem examples in comments.
	1.31  23:may:00   cjo   Improved commenting of the RECAL command.
	1.32  30:may:00   cjo   Added pio port access commands.
	1.33  31:may:00   cjo   Added BCCMDVARID_BT_CLOCK.
	1.35  6:jun:00    cjo   Corrected comments.
	1.36  12:jul:00   cjo   Added calls to provoke (test) calls to
				panic(), delayed_panic() and fault().
	1.37  19:jul:00   cjo   Changed BCCMDVARID_PIO from RO to RW.
	1.38  21:jul:00   cjo   BCCMDVARID_RECAL from WO ot RW.
	1.39  21:jul:00   cjo   Changed BCCMDVARID_RECAL back again.
	1.42  3:aug:00    cjo   Added BCCMDVARID_PIO_PROTECT_MASK.
	1.43  9:aug:00    cjo   Added 57k6 baud.
	1.45  21:sep:00   cjo   BCCMDVARID_PSSIZE.
	1.47  25:sep:00   cjo   Corrected value of UART_BAUD_19200.
	1.48  25:sep:00   cjo   Removed obsolete #define BCCMDPDU_HDR.
	1.52  16:oct:00   cjo   Corrected comment.
	1.53  27:oct:00   pws   Comment out PSCLRMOST differently so as not
				not to confuse PSCLRMOST.
	1.54  31:oct:00   cjo   Corrected comments.
	1.59  29:nov:00   cjo   Added BCCMDVARID_MAX_TX_POWER.
	1.60  23:jan:01   cjo   BCCMDVARID_PSNEXT.
	1.62  29:jan:01   cjo   Moved BCCMDVARID_PSSIZE BCCMDVARID_PSNEXT
				to the specials block.
	1.62  5:mar:01    cjo   Changed interface to adc.
*/

#ifndef	__BCCMDPDU_H__
#define	__BCCMDPDU_H__
 
#ifdef RCS_STRINGS
static const char bccmdpdu_h_id[]
  = "$Id: //depot/bc/main/interface/host/bccmd/bccmdpdu.h#57 $";
#endif

/****************************************************************************
BCCMDPDU_MEMORY
===============

Data used to control a read from, or write to, the bc01's memory space. 

The bccmd treats the value of "base_addr" as a uint16*.  It specifies the
base address of the block to be written or read.

When used to specify a "write" operation or used to report on a "read"
operation, the "len" field sets the number of uint16s to be written or read.
This block of uint16s is held in mem[].

The mem[] array is declared as being of length 1, but the length is actually
arbitrary.  The actual length can be derived from the "pdulen" field.  The
limit implied by BCCMDPDU_MAXLEN must be respected.  We can pull this nasty
trick because the mem[] is the last element of the BCCMDPDU struct.

The bccmd performs some basic filtering of the address range.  The machine
can't be hung by reading/writing arbitrary locations, but all sorts of
unpleasant effects can be provoked - e.g., buffer page exception interrupts,
mysterious flash memory effects, etc.   Actual restrictions are TBD. HERE */

typedef struct {
	uint16		base_addr;	/* Start address. */
	uint16		len;		/* Number of uint16s. */
	uint16		dummy;		/* Padding word - unused. */
	uint16		mem[1];		/* Data. */
	} BCCMDPDU_MEMORY;


/****************************************************************************
BCCMDPDU_PS 
===========

Data used to control a read or write to the bc01's persistent store.

The "id" is the same identifier used in the psbc_data, where it identifies an
item of data.

When used to specify an item of data to be read, the "len" field specifies
the (maximum) length of the data to be returned.  If the bccmd is able to
determine that the data stored under the identifier "id" is larger than "len"
then a BCCMDPDU_STAT_TOO_BIG value is given in the reply's "status" field.
The "stores" field describes which of the four ps stores to examine and
the order in which to search them.

When used to specify an item of data to be written, the "len" field specifies
the data's length in uint16s.  The data is passed to the bccmd in the "len"
element array psmem[]. The "stores" field describes which of the three
writable ps stores to modify.

Comments for the mem[] array in BCCMDPDU_MEMORY also apply to psmem[].

Writing to the bc01's ps hangs the machine for the (significant) duration of
the operation.  This should avoided while the processor is supporting a
Bluetooth link.

The requirement that the data is passed as a number of uint16s presumes that
the ps interface handles data in uint16s.  If we ever use uint8-wide
persistent storage this is likely to be problematic. */

typedef struct {
	uint16		id;		/* PS identifier (pskey). */
	uint16		len;		/* Length of data. */

/* The "stores" parameter is a bitmask.  Only these entries work. */
#define	PS_STORES_DEFAULT (0x0000)	/* default stores for command. */
#define	PS_STORES_I       (0x0001)	/* implementation (normal psi.h). */
#define	PS_STORES_F       (0x0002)	/* factory-set (psf.h). */
#define	PS_STORES_R       (0x0004)	/* rom (psrom.h). */
#define	PS_STORES_T       (0x0008)	/* transient (psram.h). */
#define	PS_STORES_IF      (0x0003)	/* psi.h->psf.h. */
#define	PS_STORES_IFR     (0x0007)	/* psi.h->psf.h->psrom.h. */
#define	PS_STORES_TI      (0x0009)	/* psram.h->psi.h. */
#define	PS_STORES_TIF     (0x000b)	/* psram.h->psi.h->psf.h. */
#define	PS_STORES_TIFR    (0x000f)	/* psram.h->psi.h->psf.h->psrom.h. */

	uint16		stores;		/* PS store(s) for read only. */
	uint16		psmem[1];	/* Data. */
	} BCCMDPDU_PS;

/*
BCCMDPDU_PSNEXT
===============

Support for the BCCMDVARID_PSNEXT command. */

typedef struct {
	uint16		id;		/* PS identifier (pskey). */
	uint16		stores;		/* PS store(s) to examine. */
	uint16		nextid;		/* Next PS identifier (pskey). */
	} BCCMDPDU_PSNEXT;

/*
BCCMDPDU_PSNEXT_ALL
===================

Support for the BCCMDVARID_PSNEXT_ALL command. */

typedef struct {
	uint16		id;		/* PS identifier (pskey). */
	uint16		nextid;		/* Next PS identifier (pskey). */
	} BCCMDPDU_PSNEXT_ALL;


/*
BCCMDPDU_PSSIZE
===============

Support for the BCCMDVARID_PSSIZE command.  In the getreq the "id" specifies
the pskey to be examined and the "stores" describes the stores to examine
and the order in which to search them.  In the corresponding successful
getresp the "len" gives the length of data stored under "id" measured in
uint16s. */

typedef struct {
	uint16		id;		/* PS identifier (pskey). */
	uint16		len;		/* Length of data stored under id. */
	uint16		stores;		/* PS store(s) to examine. */
	} BCCMDPDU_PSSIZE;


/*
BCCMDPDU_PSCLRS
===============

Support for the BCCMDVARID_PSCLRS command. */

typedef struct {
	uint16		id;		/* PS identifier (pskey). */
	uint16		stores;		/* PS store(s) to examine. */
	} BCCMDPDU_PSCLRS;


/*
BCCMDPDU_PS_MEMORY_TYPE
=======================

Support for the BCCMDVARID_PS_MEMORY_TYPE command. */

typedef struct {
	uint16		stores;		/* PS store(s) to examine. */

/* The "memorytype" return value will be one of the following. */
#define PSMEMORY_FLASH    (0x0000)  /* Flash memory */
#define PSMEMORY_EEPROM   (0x0001)  /* EEPROM */
#define PSMEMORY_RAM      (0x0002)  /* RAM (transient) */
#define PSMEMORY_ROM      (0x0003)  /* ROM (or read-only flash memory) */

	uint16		memorytype;	/* The underlying memory type. */
	} BCCMDPDU_PS_MEMORY_TYPE;


/****************************************************************************
BCCMDPDU_BUFFER
===============

Data used to control a read or write to one of the bc01's buffers. 

The bccmd treats the "handle" field as the bufhandle of the buffer to be
accessed.

The "start" field specifies the first byte (offset) in the buffer to be
accessed.

The "length" field specifies the number of bytes to be read or written.

Data to be read or written is held in the bufmem[] array.

The bufmem[] array is declared as being of length 1, but the length is
actually arbitrary.  The actual length can be derived from the "pdulen"
field.  The limit implied by BCCMDPDU_MAXLEN must be respected.  We can pull
this nasty trick because the bufmem[] is the last element of the BCCMDPDU
struct.

bufmem[] is declared to be uint8[], so the upper byte of each location is
wasted.  These extra bytes are passed over the wire, but must be ignored.
I.e., the wire data is *not packed* - each buffer byte lies in the lower
byte of each of the "length" unint16s.

The "length" field is the actual size of the array bufmem[]. */

typedef struct {
	uint16		handle;		/* bufhandle of buffer */
	uint16		start;		/* First byte to be accessed. */
	uint16		len;		/* Number of bytes to be accessed. */
	uint8		bufmem[1];	/* Data. */
	} BCCMDPDU_BUFFER;


/****************************************************************************
BCCMDPDU_ADC
============

Writing to BCCMDVARID_ADC provokes the firmware into attempting to read from
and ADC channel.  The result may subsequently be obtained by reading from
BCCMDVARID_ADCRES, which obtains a BCCMDPDU_ADCRES.

The result of reading the ADC is held in the "result" field.  The host should
accept this 8 bit value only if "valid" is TRUE (1) and if "channel" matches
the value written to BCCMDVARID_ADC.

The ADC reading will fail if the value written to BCCMDVARID_ADC does not
describe a valid ADC input, or if the processor is extremely heavily laden.

Under heavy load conditions the ADC can take several tens of milliseconds to
make the reading, hence the value of the "valid" flag.

Only one ADC read attempt should be attempted at a time; the "channel"
field helps guard against this. */

typedef struct {
	uint16		channel;	/* ADC channel read. */
	bool		valid;		/* Is "result" valid? */
	uint16		result;		/* Value read from ADC. */
	} BCCMDPDU_ADCRES;


/****************************************************************************
BCCMDPDU_CRYPTKEYLEN
====================

When encryption is applied to an ACL connection the LMs on the local and
remote machines negotiate the (effective) encryption key length.  This
structure supports a command that allows the host to discover the key length,
measured in bytes.

The getreq must hold the hci_handle, and keylen must be zero.

If the command is successful getresp holds the (same) hci_handle and the
effective key length negotiated for the connection.  This may not be
the link's current encryption strength: it is possible that encryption
was first negotiated and applied to the link (deriving the link's "keylen"
value) then encryption was removed - but the negotiated "keylen" value
will still reflect the earlier negotiation. */

typedef struct {
	uint16		hci_handle;	/* ACL/HCI handle. */
	uint16		keylen;		/* Effective key length, in bytes. */
	} BCCMDPDU_CRYPTKEYLEN;


/****************************************************************************
BCCMDPDU_PICONET_INSTANT
========================

The local device's Bluetooth clock is available via one of the uint32
commands.  However, if the local device is a slave on a connection then that
connection's master's Bluetooth clock can be of interest.  This structure
supports a command that obtains the Bluetooth clock that applies to an
established ACL.  If the local device is the master of the ACL then the value
is (much) the same as the value returned by the uint32 command.

The getreq sets the hci_handle, and sets h_instant to zero.

If the command is successful getresp holds the (same) hci_handle and 
h_instant holds HALF! the value of the ACL link's master's Bluetooth clock.

(Reporting half the link's clock is mr01's choice; it seems Byzantine to me.)
*/

typedef struct {
	uint16		hci_handle;	/* ACL/HCI handle. */
	uint32		h_instant;	/* Timestamp. */
	} BCCMDPDU_PICONET_INSTANT;


/****************************************************************************
BCCMDPDU_GET_CLR_EVT
====================

The firmware includes a set of counters that are used to monitor certain
events.  Each time the event occurs a counter is incremented.

This varid obtains the value of the counter identified by "evtcntid" and
clears the counter to zero.

The VM also has access to these same event counters, so care must be taken if
the VM and host access a single counter - this is unlikely to be a
significant problem for real applications.

The monitoring mechanisms are not precise - it is possible to overcount or
undercount.  This is because the underlying code deliberately does not bother
to block interrupts when these counters are accessed so as to avoid blizzards
of block/unblock trains, which could upset the radio's timing.  Since this
function is likely to be used just to waggle LEDs it doesn't seem to be worth
fixing - more accurately, it seems to be worth not fixing.

Each counter is held in a uint16, so the values can wrap.

Reading and clearing the BCCMDPDU_GCE_NONE counter has no effect; it accesses
no counter and the cnt value returned is always zero.

The PIODEBOUNCE, USRADC, BCSTAT and DEEPSLEEP counters should not be read by
the host - if the host (reads and) clears these counters an element of the
firmware can break. */

typedef struct {
/* Event counter identifier and acceptable values. */
	uint16			   evtcntid;
#define	BCCMDPDU_GCE_NONE	   (0)   /* No counter. */
#define	BCCMDPDU_GCE_TXACL	   (1)   /* Tx acl event started. */
#define	BCCMDPDU_GCE_RXACL	   (2)   /* Rx acl event started. */
#define	BCCMDPDU_GCE_TXSCO	   (3)   /* Tx sco event started. */
#define	BCCMDPDU_GCE_RXSCO	   (4)   /* Rx sco event started. */
#define	BCCMDPDU_GCE_TXHOST	   (5)   /* Tx host event started. */
#define	BCCMDPDU_GCE_RXHOST	   (6)   /* Rx host event started. */
#define	BCCMDPDU_GCE_RADIOACTIVE   (7)   /* Any radio activity. */
#define	BCCMDPDU_GCE_PIODEBOUNCE   (8)   /* PIO debounce value change. */
#define	BCCMDPDU_GCE_USRADC	   (9)   /* ADC converstion finished. */
#define	BCCMDPDU_GCE_BCSTAT	   (10)  /* Chip status changed. */
#define	BCCMDPDU_GCE_DEEPSLEEP	   (11)  /* Chip woken from deep sleep. */

/* Number of events.  0 in getreq, counter value in getresp. */
	uint16			   cnt;
	} BCCMDPDU_GET_CLR_EVT;


/****************************************************************************
BCCMDPDU_GET_NEXT_BUILDDEF
==========================

The elements included by each firmware build is controlled by a set of
top-level #defines, passed to the build system.  The GET_NEXT_BUILDDEF
command allows the host to determine the list of these #defines, and hence
the characteristics/elements of the build.

The values of "id" and "nextid" are from builddefs.h, a header that can be
used by firmware and host code.  This file's entries directly map to the
build system's top-level #defines.

The command is build on the common "get next" mechanism.  The normal manner
of use is that the host first calls the command with "id" set to zero.  The
command returns the BCCMDPDU_GET_NEXT_BUILDDEF with the "nextid" set to the
first (numerically sorted) identifier.  The host records this identifer, then
places it into the "id" field of the next call to the command, which obtains
the second identifier.  This process is continued until "nextid" is returned
holding BUILDDEF_NONE. */

typedef struct {
	uint16			   id;
	uint16			   nextid;
	} BCCMDPDU_GET_NEXT_BUILDDEF;


/****************************************************************************
BCCMDPDU_I2C_TRANSFER
=====================

Data used to control a composite write/read transfer on the I2C-bus.

The "address" field specifies the 7bit slave address (in bits 7 to 1).

The "tx_octets" field specifies the number of bytes to be written.

The "rx_octets" field specifies the number of bytes to be read.

The "restart" field should normally be TRUE, but may be set to FALSE to
force a change of direction without an intervening repeated start
condition (Sr).

The "octets" field must be zero for the getreq/setreq, and is set to the
number of bytes transferred and acknowledged for the getresp. This includes
both the slave address byte(s) and the final byte read, even though the
latter is followed by a NACK.

The "data" array is used for both the data to be written and the data read.
It is declared as being of length 1, but the length is actually artibtrary.
The first "tx_octets" elements contain the bytes to be written, and the
following "rx_octets" elements are overwritten by the bytes read. The limit
imposed by BCCMDPDU_MAXLEN must be respected. "data" is declared to be
uint8[], so the upper byte of each location is wasted.

See the description of i2c_transfer() in i2c.h for a more comprehensive
explanation; the "data" array corresponds to the "tx_data" and "rx_data"
parameters, and the return value is written to "octets". */

typedef struct {
	uint16			   address;
	uint16			   tx_octets;
	uint16			   rx_octets;
	bool			   restart;
	uint16			   octets;
	uint8			   data[1];
	} BCCMDPDU_I2C_TRANSFER;


/****************************************************************************
BCCMDPDU_E2_DEVICE
==================

A header is stored at the start of I2C EEPROM devices used with BlueCore to
indicate the device characteristics. This is required to enable the correct
addressing scheme to be used.

The "log2_bytes" field specifies the size of the device as (2 ^ log2_bytes)
bytes.

The "addr_mask" field specifies which, if any, bits of the slave address
are used to extend the array address byte(s). Any of the three least
significant bits may be set to 1 to indicate that the corresponding slave
address bit is used.
*/

typedef struct {
	uint16			   log2_bytes;
	uint16			   addr_mask;
	} BCCMDPDU_E2_DEVICE;


/****************************************************************************
BCCMDPDU_E2_APP_DATA
=====================

Data used to control a read or write of part of a VM application stored
in EEPROM.

The "offset" field specifies the offset in words from the start of the
region reserved for a VM application at which to start reading or writing.

The "words" field specifies the maximum number of words to read or write.
In the getresp this indicates the actual number of words read or written,
which may be less than the number requested if the reserved size is
exceeded.

The "dummy" field is not currently used; it must be set to 0.

The "data" array is used for the data to be read or written. It is declared
as being or length 1, but the length is actually arbitrary. The actual
length is given by the "words" field. The limit imposed by BCCMDPDU_MAXLEN
must be respected.
*/

typedef struct {
	uint16			   offset;
	uint16			   words;
	uint16			   dummy;
	uint16			   data[1];
	} BCCMDPDU_E2_APP_DATA;


/****************************************************************************
BCCMDPDU_ENABLE_AFH
====================

Adaptive frequency hopping (AFH) is used by default if both devices
support it (i.e. their AFH_capable feature bit is set).  This varid allows
use of AFH to be explicitly enabled or disabled, on either a global or
per-link basis.

The setreq must hold the hci_handle specifying the link to control, and the
required state.  If the hci_handle is zero then this sets the state for new
connections, and for any active links where the local device is master.  If
the hci_handle is non-zero then it must specify an active link where the
local device is master.  Attempting to enable AFH for a link where one or
both devices does not support AFH, or for a link where the local device is
a slave, results in an error.

The getreq must hold the hci_handle, and enable must be zero.  If the
command is successful then the getresp holds the (same) hci_handle and
enable indicates the state of the specified link.  If hci_handle specifies
an active link where the local device is master then the current state for
that link is returned, otherwise the state used for new connections is
returned.

Note that having use of AFH enabled for a link or future connections does
not necessarily mean that AFH will be used.  The LM may decide to operate
a link with AFH disabled for other reasons, for example to avoid use of
multiple park beacon structures.
*/

typedef struct {
	uint16		hci_handle;	/* ACL/HCI handle, or 0. */
	bool		enable;		/* Disable (0) or enable (1). */
	} BCCMDPDU_ENABLE_AFH;

/****************************************************************************
BCCMDPDU_L2CAP_CRC
==================

This BCCMD is used to turn on/off the automatic generation of CRC's on
transmit and to turn on the checking of CRC's on reception.  */

typedef struct {
    uint16          hci_handle;     /* ACL/HCI handle. */
    bool            tx_cid;         /* TRUE if CID is a transmit channel */
    uint16          cid;            /* The CID */
    bool            enable;         /* TRUE to enable this CID. */
} BCCMDPDU_L2CAP_CRC;

/****************************************************************************
BCCMDPDU_ENABLE_MEDIUM_RATE
==========================

*/

typedef struct {
	uint16		hci_handle;	/* ACL/HCI handle */
	bool		enable;		/* Disable (0) or enable (1). */
	} BCCMDPDU_ENABLE_MEDIUM_RATE;

/****************************************************************************
BCCMDPDU_ALTER_SCO_SETTINGS
===========================

*/

typedef struct {
	uint16		hci_handle;	/* ACL/HCI handle, or 0. */
	bool		rx;		/* Tx (0) or rx (1) */
        uint16		packet;		/* Packet type and table */
	uint16		length;		/* Packet length */
	} BCCMDPDU_ALTER_SCO_SETTINGS;

/****************************************************************************
BCCMDPDU_RADIOTEST
==================

Complicated and growing, so held in a separate header file. */

#include	"bccmdradiotestpdu.h"

/****************************************************************************
BCCMDPDU_KALDEBUG
=================

Commands for debugging kalimba.
*/

#include "bccmdkaldebugpdu.h"

/* Red-M Change Starts */
/****************************************************************************
BCCMDPDU_BER_THRESHOLD
======================

Setting a non-zero threshold means that when raw_ber hits 32*threshold, an
HQ packet is emitted to indicate a fall in signal quality.
These events are one-shot: the threshold is zeroed after triggering.
Only the lower 8 bits are used in the threshold. */

typedef struct {
	uint16		hci_handle;	/* ACL/HCI handle. */
	uint16		ber_threshold;	/* point at which ber trigger fires.
					   8 bits used, premultiplied by 32. */
	} BCCMDPDU_BER_THRESHOLD;

/* Red-M Change Ends */

/****************************************************************************
BCCMDPDU
========

A PDU always starts with a BCCMDPDU.  If the data is larger than will fit in
the BCCMDPDU the extra data follows the BCCMDPDU.  (E.g., the BCCMDPDU could
carry a large BCCMDPDU_PS value.)

The host sends PDUs of types BCCMDPDU_GETREQ and BCCMDPDU_SETREQ to the bccmd
on the chip.  The bccmd sends PDUs of type BCCMDPDU_GETRESP back to the
host.

The PDU "type" field is first in the structure to match CCL's conventions.

The PDU "pdulen" field gives the total length of the PDU counted in uint16s.
(The use of "Bluecore Friendly Format" implies the PDU must be an even number
of bytes, so we might as well count in uint16s.)

Each PDU carries a sequence number.  The bccmd (on chip) copies the "seqno"
from each PDU it receives into the corresponding BCCMDPDU_GETRESP(s).  This
allows the host to match questions to answers.  bccmd makes no use of
"seqno".

The "varid" field defines the variable to be read or written.  The value zero
must not be used to identify a variable.

Both the host and bccmd maintain knowledge of each varid's type; this implies
the variable's size.   (Though, incidently, this information can normally be
derived from a varid's value as these values are allocated in bands matching
variables' sizes.)

The "status" field allows the bccmd to proffer pathetic excuses for not
fulfilling the host's commands.  By default the "status" field takes value
BCCMDPDU_STAT_OK.  All PDUs of type BCCMDPDU_GETREQ and BCCMDPDU_SETREQ must
have a "status" of BCCMDPDU_STAT_OK.

Not all "varid" values are defined.  If the bccmd doesn't recognise the
"varid" of a variable it will reply with a "status" of
BCCMDPDU_STAT_NO_SUCH_VARID.

The bccmd may recognise the value of a "varid", but it may have been told not
to allow the variable to be accessed (from a given command source - security
reasons).  In this case it will also reply with a "status" of
BCCMDPDU_STAT_NO_SUCH_VARID.

Other values of the "status" field should be self-evident.  The "status"
value BCCMDPDU_STAT_ERROR is a catch-all - provided when none of the other
codes fits the situation.

If this protocol is extended to provide a GETNEXT-REQ pdu then it is likely
that a "vartype" field, describing the varid's type, will be added to
BCCMDPDU.  (A "vartype" field was provided in the first draft of this
definition.)

(Although it would be natural to use enums to describes some of the elements
of BCCMDPDU, we cannot do so as they are really ints, and ints have different
sizes and representations on different machines.) */

typedef struct {
	/* PDU type. */
	uint16		type;

/* Acceptable values for "type". */
#define	BCCMDPDU_GETREQ		(0)
#define	BCCMDPDU_GETRESP	(1)
#define	BCCMDPDU_SETREQ		(2)
/* #define BCCMDPDU_GETNEXTREQ	(3)  may be added. */

	/* Total PDU length in uint16s. */
	uint16		pdulen;

/* Maximum total length of a PDU. */
#define	BCCMDPDU_MAXLEN		(64)	

	/* Sequence number. */
	uint16		seqno;

	/* Identifier of variable to be accessed. */
	uint16		varid;

	/* How the bccmd managed to perform the operation. */
	uint16		status;

/* Acceptable values for "status". */
#define	BCCMDPDU_STAT_OK	(0)	/* Default.  No problem found. */
#define	BCCMDPDU_STAT_NO_SUCH_VARID (1)	/* varid not recognised. */
#define	BCCMDPDU_STAT_TOO_BIG	(2)	/* Data exceeded pdu/mem capacity. */
#define	BCCMDPDU_STAT_NO_VALUE  (3)	/* Variable has no value. */
#define	BCCMDPDU_STAT_BAD_PDU	(4)	/* Dodgy value(s) in PDU. */
#define	BCCMDPDU_STAT_NO_ACCESS	(5)	/* Value of varid inaccessible. */
#define	BCCMDPDU_STAT_READ_ONLY	(6)	/* Value of varid unwritable. */
#define	BCCMDPDU_STAT_WRITE_ONLY (7)	/* Value of varid unreadable. */
#define	BCCMDPDU_STAT_ERROR	(8)	/* The useless catch-all error. */
#define BCCMDPDU_STAT_PERMISSION_DENIED (9) /* Request not allowed. */


/***************************************************************************
That's the end of the common header of a bccmdpdu.  Now follows stuff that
varies according to the settings of the "type" and "varid" fields.
***************************************************************************/

        union {
		/* Useless placeholder.  Nothing is written into BCCMDPDU.d
		if "type" is BCCMDPDU_GETREQ or if "varid", stripped of its
		access control bits, is between BCCMDVARID_COMMANDS_START and
		(BCCMDVARID_UINT8_START-1). */

                uint16  	dummy;

		/* For use if "varid", stripped of its access control bits,
		is between BCCMDVARID_UINT8_START and
		(BCCMDVARID_UINT16_START-1). */

                uint8   	u8;

		/* For use if "varid", stripped of its access control bits,
		is between BCCMDVARID_UINT16_START and
		(BCCMDVARID_UINT32_START-1). */

                uint16  	u16;

		/* For use if "varid", stripped of its access control bits,
		is between BCCMDVARID_UINT32_START and
		(BCCMDVARID_SPECIAL_START-1). */

                uint32  	u32;

		/* If "varid", stripped of its access control bits, is
		BCCMDVARID_SPECIAL_START or above then one of a set of ugly
		data forms follows. */

		BCCMDPDU_MEMORY mem;	/* If "varid" is BCCMDVARID_MEMORY. */ 
		BCCMDPDU_PS	ps;	/* If "varid" is BCCMDVARID_PS. */
		BCCMDPDU_BUFFER bufmem; /* If "varid" is BCCMDVARID_BUFFER. */
		BCCMDPDU_RADIOTEST radiotest;  /* If "varid" is 
					    BCCMDVARID_RADIOTEST. */
		BCCMDPDU_PSNEXT psn;	/* If "varid" is BCCMDVARID_PSNEXT. */
		BCCMDPDU_PSNEXT_ALL psna; /* If "varid" is
					    BCCMDVARID_PSNEXT_ALL. */
		BCCMDPDU_PSSIZE pss;	/* If "varid" is BCCMDVARID_PSSIZE. */
		BCCMDPDU_PSCLRS psc;	/* If "varid" is BCCMDVARID_PSCLRS. */
		BCCMDPDU_PS_MEMORY_TYPE psmt;/* If "varid" is
					    BCCMDVARID_PS_MEMORY_TYPE. */
		BCCMDPDU_ADCRES adcres; /* If "varid" is BCCMDVARID_ADCRES. */
		BCCMDPDU_CRYPTKEYLEN ckl; /* BCCMDVARID_CRYPTKEYLEN. */
		BCCMDPDU_PICONET_INSTANT pi; /* BCCMDVARID_PICONET_INSTANT. */
		BCCMDPDU_GET_CLR_EVT gce; /* BCCMDVARID_GET_CLR_EVT. */
		BCCMDPDU_GET_NEXT_BUILDDEF gnbd; /* If "varid" is
					    BCCMDVARID_GET_NEXT_BUILDDEF. */
	        /* The struct for BCCMDVARID_I2C_TRANSFER is larger than the
		minimum BCCMD PDU size, so is omitted from this union:
		    BCCMDPDU_I2C_TRANSFER i2c;
		*/
		BCCMDPDU_E2_DEVICE e2d; /* BCCMDVARID_E2_DEVICE. */
		BCCMDPDU_E2_APP_DATA e2ad; /* BCCMDVARID_E2_APP_DATA. */
		BCCMDPDU_ENABLE_AFH ea; /* BCCMDVARID_ENABLE_AFH. */
	        BCCMDPDU_L2CAP_CRC l2cc; /* BCCMDPDU_L2CAP_CRC */
        BCCMDPDU_KALDEBUG kaldebug;  /* BCCMDVARID_KALDEBUG */
/* Red-M Change Starts */
		BCCMDPDU_BER_THRESHOLD bert;	/* BCCMDVARID_BER_THRESHOLD */
/* Red-M Change Ends */
		BCCMDPDU_ENABLE_MEDIUM_RATE emr; /* BCCMDVARID_MEDIUM_RATE. */
		BCCMDPDU_ALTER_SCO_SETTINGS ass;
					/* BCCMDVARID_ALTER_SCO_SETTINGS. */
        	} d;
	} BCCMDPDU;


/***************************************************************************/

/* The varids are listed in blocks.  Each block holds variables of a given
type.  This allows bccmd to determine a variable's type by seeing which
range it falls in.

Commands from the host, received via the bcsp/hci_extn private channel, are
subject to some restrictions.  These bitmasks/offsets apply to varids to
restrain the host's ambitions. */

#define BCCMDVARID_NOACCESS	(0)
#define BCCMDVARID_READONLY	(0x2000)
#define BCCMDVARID_WRITEONLY	(0x4000)
#define BCCMDVARID_READWRITE	(0x6000)


/***************************************************************************/
/* COMMAND VARIDS (variables which invoke commands when written to. */

/* Marker of start of the command varid block. */
#define	BCCMDVARID_COMMANDS_START (0)

/* Dummy variable.  No value written or read.  No command action performed. */
#define	BCCMDVARID_NO_VARIABLE	(BCCMDVARID_COMMANDS_START+ \
					BCCMDVARID_READWRITE+0)

/* bc01's best attempt at imitating a hardware reset. */
#define	BCCMDVARID_COLD_RESET	(BCCMDVARID_COMMANDS_START+ \
					BCCMDVARID_WRITEONLY+1)

/* Restart software, but attempt to preserve some values from existing state. */
#define	BCCMDVARID_WARM_RESET	(BCCMDVARID_COMMANDS_START+ \
					BCCMDVARID_WRITEONLY+2)

/* bc01's best attempt at imitating a hardware halt.  This currently 
doesn't work (insufficiently strong sleeping pills). */
#define	BCCMDVARID_COLD_HALT	(BCCMDVARID_COMMANDS_START+ \
					BCCMDVARID_WRITEONLY+3)

/* Halt software, but attempt to preserve some values from existing state. 
This currently doesn't work (insufficiently strong sleeping pills). */
#define	BCCMDVARID_WARM_HALT	(BCCMDVARID_COMMANDS_START+ \
					BCCMDVARID_WRITEONLY+4)

/* Initialise the bc01's Bluetooth stack.  (Start stack's tasks.) */
/* Obsolete command. */
#define	BCCMDVARID_INIT_BT_STACK (BCCMDVARID_COMMANDS_START+ \
					BCCMDVARID_WRITEONLY+5)

/* Activate the bc01's Bluetooth stack.  (Controlled init sequencing.) */
/* Obsolete command. */
#define	BCCMDVARID_ACTIVATE_BT_STACK (BCCMDVARID_COMMANDS_START+ \
					BCCMDVARID_WRITEONLY+6)

/* Enable the radio to transmit. */
#define	BCCMDVARID_ENABLE_TX	(BCCMDVARID_COMMANDS_START+ \
					BCCMDVARID_WRITEONLY+7)

/* Prevent the radio from transmitting. */
#define	BCCMDVARID_DISABLE_TX	(BCCMDVARID_COMMANDS_START+ \
					BCCMDVARID_WRITEONLY+8)

/* Force recalibration of the synthesiser's look up table.  This may take a
significant time (2 seconds?). 

If the chip's boot time calibration fails then a message will have been sent
to the host via the hq channel.  The host might then reasonably invoke this
command to force a further attempt at recalibration.  Until the synth has
been calibrated successfully the lc prevents traffic travelling over the
radio.

Once the synth has successfully been calibrated any attempt to recalibrate
will probably destroy any active radio communcation and cause the calibration
either to fail or to scramble the look up table - either way, all future
radio communication will be unlikely to succeed.  The only safe way out of
such a mess will be to reboot.

Returns status of BCCMDPDU_STAT_ERROR if the calibration fails. */
/* Obsolete command. */
#define	BCCMDVARID_RECAL	(BCCMDVARID_COMMANDS_START+ \
					BCCMDVARID_WRITEONLY+9)

/* Remove almost all values from the bc01's persistent store (ps). */
/* This command is obsolete - it is no longer supported. */
/* #define	BCCMDVARID_PSCLRMOST   	     (BCCMDVARID_COMMANDS_START+ \
					BCCMDVARID_WRITEONLY+10) */

/* Remove all values from the bc01's persistent store (ps).  This varid is
deprecated - use BCCMDVARID_PSCLRALLS instead. */
/* Obsolete command. */
#define	BCCMDVARID_PSCLRALL    	     (BCCMDVARID_COMMANDS_START+ \
					BCCMDVARID_NOACCESS+11)

/* Take a snapshot of the values in the ps. */
#define	BCCMDVARID_PS_FACTORY_SET    (BCCMDVARID_COMMANDS_START+ \
					BCCMDVARID_NOACCESS+12)

/* Restore all ps values from the last BCCMDVARID_PS_FACTORY_SET command. */
#define	BCCMDVARID_PS_FACTORY_RESTORE  (BCCMDVARID_COMMANDS_START+ \
					BCCMDVARID_WRITEONLY+13)

/* As BCCMDVARID_PS_FACTORY_RESTORE, but remove all other ps values. */
#define	BCCMDVARID_PS_FACTORY_RESTORE_ALL (BCCMDVARID_COMMANDS_START+ \
					BCCMDVARID_WRITEONLY+14)

/* Arrange that next reboot will perform ps defrag, then warm reboot. */
#define	BCCMDVARID_DEFRAG_RESET	(BCCMDVARID_COMMANDS_START+ \
					BCCMDVARID_WRITEONLY+15)

/* Stop whatever the VM is doing, if anything; it is not restartable. */
#define BCCMDVARID_KILL_VM_APPLICATION (BCCMDVARID_COMMANDS_START+ \
					BCCMDVARID_WRITEONLY+16)

/* Turn hopping back on.  This is only useful after using
BCCMDVARID_SINGLE_CHAN to enable transmit and receive on a single channel. */

#define BCCMDVARID_HOPPING_ON (BCCMDVARID_COMMANDS_START+ \
			       BCCMDVARID_WRITEONLY+17)

/* Cancel any page the lc happens to be doing.  If it isn't, don't care. */
/* Obsolete command. */
#define BCCMDVARID_CANCEL_PAGE (BCCMDVARID_COMMANDS_START+ \
				BCCMDVARID_WRITEONLY+18)

/*
 * Arrange to test system RAM.  The test is destructive and this causes
 * a system reset.  See the description for BCCMDVARID_CHECK_RAM_RESULT
 * for more details.
 *
 * May not be available on all builds; if not available, getting or setting
 * will return BCCMD_NO_SUCH_VARID.
 */
#define BCCMDVARID_CHECK_RAM (BCCMDVARID_COMMANDS_START+ \
			      BCCMDVARID_WRITEONLY+19)

/*
 * From chips starting with bc02-kato only, enable UART bypass mode.
 * This redirects the UART I/O lines to PIO pins 4 to 7 inclusive.
 * The assumption is that in fact no UART is connected to these pins
 * and the chip will enter deep sleep until a reset is forced by
 * hardware.  This takes immediate effect so no response will be received.
 */
#define BCCMDVARID_BYPASS_UART (BCCMDVARID_COMMANDS_START+ \
				BCCMDVARID_WRITEONLY+20)

/*
 * Synchronise the internal clock to correspond to a remote master.
 * Has no effect if there is no remote master.  This does not store
 * the result, which lasts until the next reset.  The value set can
 * be retrieved via BCCMDVARID_ANA_FTRIM; this is suitable for storing in
 * PSKEY_ANA_FTRIM.  Obviously, how useful the value is depends on the
 * accuracy of the remote master's clock.
 *
 * The command returns immediately with a successful status indication
 * if there is a remote master, but the operation takes several
 * seconds after that, up to a maximum of roughly 5.  No further status
 * indication is returned; examining changes to BCCMDVARID_ANA_FTRIM is
 * the only way of testing the effect of the command.
 */
#define BCCMDVARID_SYNC_CLOCK  (BCCMDVARID_COMMANDS_START+ \
				  BCCMDVARID_WRITEONLY+21)

/*
 * Enable Device Under Test Mode.
 * Do just the same as the HCI command of the same name.
 */
#define BCCMDVARID_ENABLE_DUT_MODE (BCCMDVARID_COMMANDS_START+ \
				    BCCMDVARID_WRITEONLY+22)
/*
Babel commands.
 
The babel hardware is a USB to SPI converter.  It's based around a BlueCore, 
with PIO lines driving the SPI as Master.  
 
In order to get the Babel's BlueCore to execute the spi transactions, we
have these BCCMDs.  They basically encapsulate the spifns and spicommon APIs, 
so we have such things as block read and write, verified operations, and 
some stop and go calls.

Eventually we will implement the interface between the Host and the Babel's 
BlueCore in terms of a USB mass storage device. 

*/


/*
The following Babel commands are executed as standard commands.  
They return the success of the operation using 
BCCMDPDU_STAT_ERROR or BCCMDPDU_STAT_OK.  
*/
#define BCCMDVARID_BABEL_SPI_RESET_AND_GO   (BCCMDVARID_COMMANDS_START+ \
                                        BCCMDVARID_WRITEONLY + 23)

#define BCCMDVARID_BABEL_SPI_RESET_AND_STOP (BCCMDVARID_COMMANDS_START+ \
                                        BCCMDVARID_WRITEONLY + 24)

#define BCCMDVARID_BABEL_SPI_GO             (BCCMDVARID_COMMANDS_START+ \
                                        BCCMDVARID_WRITEONLY + 25)

#define BCCMDVARID_BABEL_SPI_STOP           (BCCMDVARID_COMMANDS_START+ \
                                        BCCMDVARID_WRITEONLY + 26)


/***************************************************************************/
/* Variables of type [u]int8.  The variable's value is in BCCMDPDU.d.u8. */

/* Marker of start of the [u]int8 varid block. */
#define	BCCMDVARID_UINT8_START (1024)

/* <empty list> */


/***************************************************************************/
/* Variables of type [u]int16.  The variable's value is in BCCMDPDU.d.u16. */

/* Marker of start of the [u]int16 varid block. */
#define	BCCMDVARID_UINT16_START (2048)


/* BC01 status. */
/* Obsolete command. */
#define	BCCMDVARID_BC01_STATUS	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+1)

/* UART configuration.  Writing this value has immediate effect and the
value is not written to the ps.  To make a permanent configuration change
write directly to the ps. */
#define BCCMDVARID_CONFIG_UART       (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READWRITE+2)

/* The value of the UART config variable is a bitfield, formed by ORing the
following values.  For example, to set one stop bit, 38.4 kbaud and even parity
the value is ( UART_BAUD_38400 | UART_ONE_STOP_BIT | UART_EVEN_PARITY ). */

#define UART_BAUD_9600               (0x0027)
#define UART_BAUD_19200              (0x004f)
#define UART_BAUD_38400              (0x009d)
#define UART_BAUD_57600              (0x00ec)
#define UART_BAUD_115200             (0x01d8)
#define UART_BAUD_921600             (0x0ebf)
#define UART_ONE_STOP_BIT    	     (0x0000)
#define UART_TWO_STOP_BITS   	     (0x2000)
#define UART_NO_PARITY               (0x0000)
#define UART_ODD_PARITY              (0x4000)
#define UART_EVEN_PARITY             (0xc000)

/* The bc01 holds a pair of analogue multiplexors (rotary switches) that
route the voltage from a point within the chip to one of a pair of external
test pins.  These varids select the switches' positions.  Acceptable
values 0->15. */

#define	BCCMDVARID_TESTA_AMUX	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READWRITE+3)
#define	BCCMDVARID_TESTB_AMUX	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READWRITE+4)

/* The argument to the last call made to panic().  Because panic() tears down
the machine in a hurry, the returned value may be suspect. */
#define BCCMDVARID_PANIC_ARG         (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READWRITE+5)

/* The argument to the last call made to fault().  This value may be
from before the current boot, so its value may be suspect. */
#define BCCMDVARID_FAULT_ARG         (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READWRITE+6)

/* The validity of the checksum over the preserve memory area as determined
by init_preserve_memory(), called during system start up.  This indicates the 
integrity of BCCMDVARID_PANIC_ARG and BCCMDVARID_FAULT_ARG.  The value is
really a bool, so zero means failure. */

#define BCCMDVARID_PRESERVE_VALID    (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+7)


/* These ADC varids are no longer used - see below. */
#define	BCCMDVARID_ADC0	     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+8)
#define	BCCMDVARID_ADC1	     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+9)
#define	BCCMDVARID_ADC2	     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+10)
#define	BCCMDVARID_ADC3	     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+11)
#define	BCCMDVARID_ADC4	     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+12)
#define	BCCMDVARID_ADC5	     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+13)
#define	BCCMDVARID_ADC6	     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+14)
#define	BCCMDVARID_ADC7	     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+15)
#define	BCCMDVARID_ADC8	     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+16)
#define	BCCMDVARID_ADC9	     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+17)
#define	BCCMDVARID_ADC10     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+18)
#define	BCCMDVARID_ADC11     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+19)
#define	BCCMDVARID_ADC12     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+20)
#define	BCCMDVARID_ADC13     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+21)
#define	BCCMDVARID_ADC14     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+22)
#define	BCCMDVARID_ADC15     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+23)

/* Remove a value from the bc01's persistent store (ps).  The value is the
key (pskey) of the item to be removed, as defined in psbc_data.  This varid
is deprecated - use BCCMDVARID_PSCLRS instead. */
/* Obsolete command. */
#define	BCCMDVARID_PSCLR     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_WRITEONLY+24)

/* Obtain the stack firmware's build identifier. */
#define	BCCMDVARID_BUILDID     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+25)

/* BlueCore hardware's major hardware version. */
#define	BCCMDVARID_CHIPVER     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+26)

/* BlueCore's hardware's minor hardware version. */
#define	BCCMDVARID_CHIPREV     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+27)

/* Normal hci calls to add sco connections to acl links cause the sco data
stream to route over hci.  If this variable is set to "1" then the next hci
call to add a sco link will route the sco channel's data over the bc01's
first pcm interface rather than over hci.  bc01b has only one pcm interface,
but bc02 and later versions of the chip may have up to 3, so setting this
variable may route to pcm channel 2 or 3 in future.  Setting this variable to
zero clears a preceding request to route over a pcm channel.  */

#define	BCCMDVARID_MAP_SCO_PCM       (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_WRITEONLY+28)

/* This command is no longer supported ...  See bccmd_spi.h.

It is possible to pass commands to bccmd over spi, using a fixed-size ram
buffer in the bc01.  This variable obtains the buffer's size.   The size
limits the length of some bccmd commands' payloads. */

#define	BCCMDVARID_SPI_BUFSIZE 	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+29)

/* Sets whether the individual bits of the bc01's PIO port are used as inputs
or outputs.  The PIO is 16 bits wide.  If a bit is 1 then the
corresponding bit position of the chip's PIO port is set to be an
output, else it is set as an input.

This command must be used with care: chip code using the pio port probably
depends on knowledge of the bits' directions. */

#define	BCCMDVARID_PIO_DIRECTION_MASK 	     	(BCCMDVARID_UINT16_START+ \
						BCCMDVARID_READWRITE+30)
/* Old name. */
#define	BCCMDVARID_PIO_MASK			BCCMDVARID_PIO_DIRECTION_MASK

/* Access the value of the chip's 16-bit pio port.
Whether a pio bit is input (on read) or output (on write)
depends on the value of BCCMDVARID_PIO_MASK. */

#define	BCCMDVARID_PIO 	     		(BCCMDVARID_UINT16_START+ \
						BCCMDVARID_READWRITE+31)

/* Provoke a panic() call within the chip so causing an immediate shutdown
and reboot of the chip.  The value written to this varid is the value passed
to the panic() call.  This is provided for testing only and should not be
used as a normal mechanism for restarting the module. */

#define	BCCMDVARID_PROVOKE_PANIC 	(BCCMDVARID_UINT16_START+ \
						BCCMDVARID_WRITEONLY+32)

/* Provoke a delayed_panic() call within the chip so causing an shutdown and
reboot of the chip after a delay.  The value written to this varid is the
value passed to the delayed_panic() call.  This is provided for testing only
and should not be used as a normal mechanism for restarting the module. */

#define	BCCMDVARID_PROVOKE_DELAYED_PANIC (BCCMDVARID_UINT16_START+ \
						BCCMDVARID_WRITEONLY+33)

/* Provoke a fault() call within the chip.  This should cause the chip to
attempt to send a fault report back to the host.  The value written to this
varid is the value passed to the fault() call.  This is provided for testing
only. */

#define	BCCMDVARID_PROVOKE_FAULT 	(BCCMDVARID_UINT16_START+ \
						BCCMDVARID_WRITEONLY+34)

/* The chip's PIO port (confusingly called the "aux" io port by the hal and
below) is supposed to be a general purpose provision.  Alas, lots of bits of
hardware and software bag various pins for their own purposes, e.g. external
power amp control (hw) and usb support (firmware).  This effectively means
that for any given application/configuration only a subset of the 16 pins may
be available to an application.  This applies whether the application is on
chip or on the host.  In either case the application needs some sort
constraint on its attempts to change arbitrary pio pins.  For example, we
cannot allow the host to use bccmd commands to set and clear arbitrary pio
pins because this may interfere with, say, usb operation.  We certainly can't
block interrupts while a pair of bccmds perform a read-modify-write.

The low uint8 of this uint16 acts as a crude access control mask.  The bits
in the mask map to the corresponding bits in the pio port.  Each bit set to 1
in the mask cannot be set by bccmd/application. */

#define	BCCMDVARID_PIO_PROTECT_MASK	(BCCMDVARID_UINT16_START+ \
						BCCMDVARID_READONLY+35)

/* Old location/value for the pssize varid.  Now in the specials block.
#define	BCCMDVARID_PSSIZE     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+36) */

/* The version number of the bccmd interface, as defined by the RCS version
number of the file that defines the interface (bccmdpdu.h).  This will change
when commands are added/changed/removed.  This will not change if the
implementation of a command changes, e.g. a bug fix. 

Value 0 means the information is unavailable.

The code that reports the version of this file needs this RCS keyword:
$Revision: #57 $ */

#define	BCCMDVARID_INTERFACE_VERSION  (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+37)


/* The version number of the bccmdradiotest interface (a sub-element of the
bccmd interface), as defined by the RCS version number of the file that
defines the interface (bccmdradiotestpdu.h).  This will change when commands
are added/changed/removed.  This will not change if the implementation of a
command changes, e.g. a bug fix.

Value 0 means the information is unavailable.

The code that reports the version of this file needs the "Revision" RCS
keyword to be in file bccmdradiotestpdu.h. */

#define	BCCMDVARID_RADIOTEST_INTERFACE_VERSION  (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+38)


/* Set the maximum transmit power.  A ram value is set from pskey
PSKEY_LC_MAX_TX_POWER when the system boots; this limits the maximum transmit
power of the radio.  This bccmd varid writes to this ram value, allowing the
maximum transmit power to be altered after booting.

This limit is only referenced when the lc considers increasing the transmit
power, so if the transmit power on a link is currently above this level the
new value won't take effect until an attempt is made to increase the power.
Values as for the pskey - dBm. */

#define	BCCMDVARID_MAX_TX_POWER		(BCCMDVARID_UINT16_START+ \
					  BCCMDVARID_READWRITE+39)


/* Old location/value for the psnext varid.  Now in the specials block.
#define	BCCMDVARID_PSNEXT     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+40) */


/* The bc01 includes an ADC that can read voltages from points within the
chip.  In front of the ADC is a MUX (another rotary switch) that selects the
point to be monitored.  The next two varids are used to make ADC readings.

The interface is a little awkward - a consequence of the way the request is
scheduled within the firmware.  The ADC channel to be read is first written
to BCCMDVARID_ADC; this provokes the firmware to start to make the ADC
reading.  The result may subsequently be picked up by reading
BCCMDVARID_ADCRES.

We deliberately do not describe the ADC channels here as this reveals too
much of the innards of the radio, and this file has been known to leak
outside CSR.  The only channel that might legitimately interest host code is
channel 1 - this measures the chip's Vdd against an internal reference.  This
is used by the on-chip battery monitor. */

#define	BCCMDVARID_ADC     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_WRITEONLY+41)


/* The bc01 uses a random number generator to support pairing, authentication
and encryption.  This varid obtains a single uint16 of random data from the
generator.

The bc01 uses a hardware mechanism to produce a slow trickle of random
bits.  These are collected in a store.  Requests for random numbers
take from this store.  If the rate of bit consumption exceeds generation
(roughly 500 bits/s) then reading fails. */

#define	BCCMDVARID_RAND     	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+42)


/* The pskey PSKEY_LC_DEFAULT_TX_POWER sets the device's default transmit
power, used for page, inquiry, and their responses, and as the initial power
for new acl links (before power control takes over).  The lc takes the value
from ps at system boot.

The value is normally constrained by the Bluetooth power class, though
under most circumstances it should be between 0 and 4 dBm.

This command provides allows the default transmit power to be adjusted. 
The value is a signed int16, expressed in dBm.

The value is constrained by the unit's power table and maximum transmit
power value. */

#define	BCCMDVARID_DEFAULT_TX_POWER  (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READWRITE+43)

/* The maximum effective encryption key length is 16 bytes, limited by the
hardware and BT spec.  Some governments' anti-privacy demands have forced us
to add a smaller limit in some firmware builds.  This varid returns the
maximum effective encryption key length imposed by the firmware, measured
in bytes.   Note that the value of PSKEY_ENC_KEY_LMAX may further
undermine the user's security. */

#define	BCCMDVARID_MAXCRYPTKEYLEN    (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+44)

/* Remove all values from the bc01's persistent store (ps).  The value is
the stores to clear.  This is an extension of BCCMDVARID_PSCLRALL. */
#define	BCCMDVARID_PSCLRALLS         (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_NOACCESS+45)

/* Transmit and receive on a single channel.  Turns off hopping and
whitening. */
#define BCCMDVARID_SINGLE_CHAN	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_WRITEONLY+46)

/*
 * Return a 16-bit checksum of ROM contents.  This only works with real rom.
 * May not be available on all builds; if not available, getting or setting
 * will return BCCMD_NO_SUCH_VARID.
 */
#define BCCMDVARID_CHECK_ROM	     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+47)

/*
 * Return a 16-bit checksum of RAM contents.
 * As this trashes the entire contents of RAM it is handled in two
 * stages.  Writing to the variable BCCMDVARID_CHECK_RAM causes a
 * destructive RAM test to be made; the result of the test is then
 * saved, and a warm reset forced.  After the reset, reading from
 * the variable BCCMDVARID_CHECK_RAM_RESULT will return the result
 * of the test.
 *	0  Test succeeded
 *      1  Test failed at stage 1
 *      2  Test failed at stage 2
 *      3  Result of test was not valid.
 *
 * May not be available on all builds; if not available, getting or setting
 * will return BCCMD_NO_SUCH_VARID.
 */
#define BCCMDVARID_CHECK_RAM_RESULT     (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+49)

/*
 * Set or get the PCM gain.  This is a three bit number which uses
 * the top three bits of the PCM stream.  It is common to all PCM
 * slots.  A value outside the range 0 to 7 will be rejected.
 */
#define BCCMDVARID_PCM_ATTENUATION	(BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READWRITE+50)

/* Set or get the number of 16-bit words reserved for a VM application in
EEPROM. Setting the size erases any VM application that was present
previously. */
#define BCCMDVARID_E2_APP_SIZE       (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+51)

/* Two varids are used to perform an integrity check of the firmware:

BCCMDVARID_FIRMWARE_CHECK returns a bitfield indicating which components of
the firmware were successfully tested, i.e. for which the checksum is
correct.

BCCMDVARID_FIRMWARE_CHECK_MASK returns a bitfield indicating which components
of the firmware can be tested, i.e. that the components are present and
contain the information required to verify the checksum.

If the two varids return the same value then all of the checks that could
be performed resulted in the expected checksum, i.e. no errors were
detected.

The bitfield returned by both varids has the following bits defined:
    bit 0 - Stack
    bit 1 - Loader
    bit 2 - VM application
*/

#define BCCMDVARID_FIRMWARE_CHECK       (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+52)

#define BCCMDVARID_FIRMWARE_CHECK_MASK   (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+53)

/* The version of the analogue hardware on the BlueCore chip */

#define BCCMDVARID_CHIPANAVER	(BCCMDVARID_UINT16_START+ \
				   BCCMDVARID_READONLY+54)

/* 
 * Return the internal value corresponding to the configuration setting
 * PSKEY_ANA_FTRIM.  The value will usually be the same but may have
 * been altered by BCCMDVARID_SYNC_CLOCK.  In that case, the value
 * read is suitable for storing in PSKEY_ANA_FTRIM; the bccmd command
 * does not do this.
 */
#define BCCMDVARID_ANA_FTRIM	(BCCMDVARID_UINT16_START+ \
				   BCCMDVARID_READONLY+55)

/* Obtain the loader firmware's build identifier. */
#define	BCCMDVARID_BUILDID_LOADER    (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READONLY+56)

/*
For hardware starting with Kato, it is possible for a PIO used as input
to be configured with a weak bias (the default), or a strong bias.  A
strong bias is a constant current source of a few tens of microamps.
This value sets or returns the register
*/
#define BCCMDVARID_PIO_STRONG_BIAS	(BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READWRITE+57)

/*
We desire to be able to read the current bootmode via bccmd.  More than 
this, we wish to be able to boot into a new mode at the behest of the host.  
Note that a bootmode SET_REQUEST may well not generate a GET_RESPONSE 
since it will trigger an immediate warm reset
*/

#define BCCMDVARID_BOOTMODE	        (BCCMDVARID_UINT16_START+ \
					BCCMDVARID_READWRITE+58)

/*
Determine how we handle a pathology in the specification with data
transfer when we are in a scatternet with one HV3 link (either as master
or slave) and a second link on which we are slave.  We need to override
the SCO link to receive an acknowledgement from the master of the second
link otherwise data transfer to the master will stall.  By default,
we only do this for control packets, since allowing bulk L2CAP data
has a severe impact on the SCO link.  With this set to TRUE, we will
allow ACL links to override the SCO link in this case.

This only applies to ACL links on which we are slave, other than
the link with the HV3 connection.
*/

#define BCCMDVARID_SCATTERNET_OVERRIDE_SCO	(BCCMDVARID_UINT16_START+ \
						BCCMDVARID_READWRITE+59)

/*
This is a debug PDU to lock all ACL transmissions to the same packet type.
It requires FIX_PACKET_TYPE to be defined to work. By default, that's not
defined. Also, the wiring is not yet present on the main line.
*/

#define BCCMDVARID_LOCK_ACL_PACKET_TYPE    (BCCMDVARID_UINT16_START+ \
                                            BCCMDVARID_WRITEONLY+60)

/* a uint16 operation
 * The shift delay is in BCCMDPDU.d.u16
 */
#define BCCMDVARID_BABEL_SPI_SHIFT_PERIOD  (BCCMDVARID_UINT16_START+ \
                                        BCCMDVARID_READWRITE + 61)

/* a uint16 read only operation.
 * A spi_run_state_t will be returned in BCCMDPDU.d.u16
 */
#define BCCMDVARID_BABEL_SPI_RUN_STATE (BCCMDVARID_UINT16_START+ \
                                        BCCMDVARID_READONLY + 62)



/***************************************************************************/
/* Variables of type [u]int32.  The variable's value is in BCCMDPDU.d.u32.
(jrrk confirms that the XAP stores a uint32 with the more significant uint16
in the lower address.  Thus, presuming the bcsp carries data in ascending
addr. order, the more significant uint16 will travel over the wire first.) */

#define	BCCMDVARID_UINT32_START (3072)

/* The local device's bluetooth clock, held in the low 28 bits of the
returned value.  This increments every 312.5 microseconds, to the returned
value will be a little stale. */

#define BCCMDVARID_BT_CLOCK	(BCCMDVARID_UINT32_START+ \
					BCCMDVARID_READONLY+0)


/***************************************************************************/
/* Variables of special (i.e. awkward) types.  PDUs with this type always
use a special struct within the the BCCMDPDU to carry the data.  The type of
struct depends on the value of "varid". */

#define	BCCMDVARID_SPECIAL_START (4096)

/* A block memory read or written.  The data is held in the BCCMDPDU_MEMORY
in the BCCMDPDU.d. */

#define	BCCMDVARID_MEMORY     	(BCCMDVARID_SPECIAL_START+ \
					BCCMDVARID_NOACCESS+1)

/* Read or write from/to a bc01 buffer.  The data is carried in the
BCCMDPDU_BUFFER struct in the BCCMDPDU.d.  Use this command only with great
care. */

#define	BCCMDVARID_BUFFER     	(BCCMDVARID_SPECIAL_START+ \
					BCCMDVARID_NOACCESS+2)

/* Read or write from/to the bc01's persistent store.  The ps data is held in
the BCCMDPDU_PS in the BCCMDPDU.d.  ps keys values have individual read/write
permissions which add to the access controls in varids. */

#define	BCCMDVARID_PS     	(BCCMDVARID_SPECIAL_START+ \
					BCCMDVARID_READWRITE+3)

/* Run one of the (jdc-defined) radio tests.  These are detailed in
bccmdradiotestspdu.h - described in a separate file because the list seems
to grow hourly. */

#define	BCCMDVARID_RADIOTEST   	(BCCMDVARID_SPECIAL_START+ \
					BCCMDVARID_WRITEONLY+4)


/* Finds the next higher key in the ps.  Notionally this command numerically
sorts all of the keys in the ps into ascending order and returns the first
key that that is greater than the pskey value in the u16.  If the u16 is
0 then the lowest valued key in the store is returned.

This function deals only with the values of keys in the persistent
store - it ignores the data stored with keys.

The command returns the next key in the store with value greater than the
id or 0 if there is no such key.

The "stores" parameter describes which of the three ps stores to examine
and the order in which to search them.

The getreq holds the initial pskey in BCCMDPDU_PSNEXT.id and the stores to be
examined in BCCMDPDU_PSNEXT.stores.  A successful getresp holds the next
pskey in BCCMDPDU_PS.nextid. */

#define	BCCMDVARID_PSNEXT     	     (BCCMDVARID_SPECIAL_START+ \
					BCCMDVARID_READONLY+5)


/* How much storage is held under a specified pskey?   The getreq holds the
pskey in BCCMDPDU_PSSIZE.id and the stores to examine in
BCCMDPDFU_PSSIZE.stores.  A successful getresp holds the size, counted in
uint16s, in BCCMDPDU_PSSIZE.len. */

#define	BCCMDVARID_PSSIZE     	     (BCCMDVARID_SPECIAL_START+ \
					BCCMDVARID_READONLY+6)


/* Result of attempting to read the ADC. */
#define	BCCMDVARID_ADCRES     	     (BCCMDVARID_SPECIAL_START+ \
					BCCMDVARID_READONLY+7)


/* How strong is the encryption on an established ACL? */
#define	BCCMDVARID_CRYPTKEYLEN 	     (BCCMDVARID_SPECIAL_START+ \
					BCCMDVARID_READONLY+8)


/* Half the BT clock on an established ACL. */
#define	BCCMDVARID_PICONET_INSTANT   (BCCMDVARID_SPECIAL_START+ \
					BCCMDVARID_READONLY+9)


/* Get and clear an evt event counter. */
#define	BCCMDVARID_GET_CLR_EVT	     (BCCMDVARID_SPECIAL_START+ \
					BCCMDVARID_READONLY+10)

/* Get the next build definition identifier. */
#define	BCCMDVARID_GET_NEXT_BUILDDEF (BCCMDVARID_SPECIAL_START+ \
					BCCMDVARID_READONLY+11)

/* Remove a value from the bc01's persistent store (ps).  The setreq holds
the key (pskey) of the item to be removed in BCCMDPDU_PSCLRS.id, and the
stores to remove it from in BCCMDPDU_PSCLRS.stores.  This is an extension
of BCCMDVARID_PSCLR. */
#define	BCCMDVARID_PSCLRS            (BCCMDVARID_SPECIAL_START+ \
					BCCMDVARID_WRITEONLY+12)

/* Perform a composite write/read transfer on the I2C-bus. */
#define BCCMDVARID_I2C_TRANSFER      (BCCMDVARID_SPECIAL_START+ \
					BCCMDVARID_NOACCESS+13)

/* Read or write the chracteristics of an I2C EEPROM device. */
#define BCCMDVARID_E2_DEVICE         (BCCMDVARID_SPECIAL_START+ \
					BCCMDVARID_READONLY+14)

/* Read or write part of a VM application stored in EEPROM. */
#define BCCMDVARID_E2_APP_DATA       (BCCMDVARID_SPECIAL_START+ \
					BCCMDVARID_READONLY+15)

/* Enable or disable use of adaptive frequency hopping (AFH) for either
existing links or new connections where the local device is master. */
#define BCCMDVARID_ENABLE_AFH        (BCCMDVARID_SPECIAL_START+ \
					BCCMDVARID_READWRITE+16)

/* Red-M Change Starts */

/* This value is used to control a "bad link" trigger, used for handoff.
The first word is an HCI handle.  The second is the BER trigger value;
setting it to zero disables the trigger. Otherwise the value is from 1->255
where the trigger is (val * 32) and compared to raw BER. */
#define	BCCMDVARID_BER_THRESHOLD     	(BCCMDVARID_SPECIAL_START+ \
					BCCMDVARID_READWRITE+17)

/* Red-M Change Ends */

/* Read the type of memory underlying the BCCMDPDU_PS_MEMORY_TYPE.stores
store. The result is returned in the BCCMDPDU_PS_MEMORY_TYPE.memorytype
field. */
#define BCCMDVARID_PS_MEMORY_TYPE	(BCCMDVARID_SPECIAL_START+ \
					BCCMDVARID_READONLY+18)

/* The firmware contains a list of pskeys that are used directly by the
firmware (e.g., PSKEY_BDADDR) or that can reasonably be set for use with/by
the firmware (e.g., PSKEY_USR0).  This command searches this list.

The command returns the first key from the list that that is numerically
greater than "id" and writes this to "nextid".  If "id" is zero then the
lowest numbered key is returned.  If "id" is the highest numbered pskey then
"nextid" is set to zero.

This should allow host tools, e.g., pstool.exe, to display only pskeys
relevent to the attached firmware build. */

#define	BCCMDVARID_PSNEXT_ALL  	     (BCCMDVARID_SPECIAL_START+ \
					BCCMDVARID_READONLY+19)

/* Enable or disable the generation or checking of a CRC on an L2CAP
   channel. */
#define BCCMDVARID_L2CAP_CRC            (BCCMDVARID_SPECIAL_START+ \
                                        BCCMDVARID_READWRITE+20)

/*
The following two bccmds are for medium rate testing on the branch
main-medium-rate. They are here just to reserve the numbers. Full
documentation will appear when they're wired in.
*/

#define BCCMDVARID_ENABLE_MEDIUM_RATE  (BCCMDVARID_SPECIAL_START+ \
                                        BCCMDVARID_WRITEONLY+21)

#define BCCMDVARID_ALTER_SCO_SETTINGS  (BCCMDVARID_SPECIAL_START+ \
                                        BCCMDVARID_WRITEONLY+22)

/* Command to debug the kalimba. See bccmdkaldebugpdu.h for more details */
#define BCCMDVARID_KALDEBUG             (BCCMDVARID_SPECIAL_START+ \
                                         BCCMDVARID_READWRITE+23)

/* Reserved for VM debugging commands */
#define BCCMDVARID_VM_DEBUG         (BCCMDVARID_SPECIAL_START+ \
                                     BCCMDVARID_READWRITE + 24)

/*
Get or set the limits which will be applied by BlueCore to sniff
requests initiated by a remote device.  The information consists of six
16-bit words in the order:

Max Tsniff	)  Even, 0 to 0xfffe
Min Tsniff	)
Max attempts	  ) 1 to 0x7fff inclusive
Min attempts	  )
Max timeout	) 0 to 0x7fff inclusive
Min timeout	)

The quantities correspond to the values in the HCI sniff specification.
(Note in particular this means Tsniff is in baseband slots while
attempts and timeout are in receive slots.)
If the initiating device requests values out of the range given,
a reply will be sent offering the nearest values inside the range.

Any value can be 0xffff to specify `don't care'.  This is an
exception to the limits shown above.

Note that the ability to negotiate the `attempts' and `timeout'
parameters is not explicit in the Bluetooth specification, hence
attempts to alter these values might be rejected by the remote side.
Typical values of attempts and timeouts are small, no more than
a dozen or so.
*/
#define BCCMDVARID_REMOTE_SNIFF_LIMITS	(BCCMDVARID_SPECIAL_START+ \
					 BCCMDVARID_READWRITE + 25)

/* 
BABEL_SPI_RAW is a read / write operation using unverified spi.
BABEL_SPI_VERIFIED is a read / write operation using verified spi.
They use the BCCMDPDU_MEMORY pdu
*/
#define BCCMDVARID_BABEL_SPI_RAW       (BCCMDVARID_SPECIAL_START+ \
                                        BCCMDVARID_READWRITE + 26)
#define BCCMDVARID_BABEL_SPI_VERIFIED  (BCCMDVARID_SPECIAL_START+ \
                                        BCCMDVARID_READWRITE + 27)

#endif	/* __BCCMDPDU_H__ */
