/****************************************************************************
FILE
	hqpdu.h  -  bluecore host querier packet structure

DESCRIPTION
	There are two bidirectional "private channels" that carry messages
	between the host and the host controller (aka the bc01 chip) over the
	BCSP link:
	
	1) One carries commands from a control entity on the host (possibly
	   called bcmgr) to the bccmd task on the host.  The bccmd task is a
	   command interpreter, carrying out local actions in response to
	   the host's commands.  The channel's reverse path carries command
	   responses back to the control entity on the host.
	
	   This channel will allow the host to configure, control and
	   monitor the CSR/bluecore Bluetooth implementation.

	2) The second channel carries commands from the host querier (hq)
	   task on the host controller to a command interpreter on the
	   host.  The reverse channel carries back the host's command
	   responses.
	   
	   This channel is expected to find only very limited use.  At
	   present, the main messages are from fault(), delayed_panic() and
	   a "booted" message from onchipbcmgr.
	
	This file describes the protocol pdus carried on the second channel.
	The first channel's protocol is described in bccmdpdu.h.

	These pdus are structured to fit CCL's "bluecore friendly format",
	i.e. the pdus' wire structures match the xap's memory structure.  The
	lower byte of each uint16 travels over the wire first (confirmed by
	jrrk).  Thus, the pdus can be defined as C structures on the
	understanding that the wire format will be a (byte) serialisation of
	the way in which the types will be laid out by the xap C compiler.
	The pdus defined here are an even number of bytes as implied by use
	of the "bluecore friendly format".

	The hq protocol is similar in style to that of the command
	interpreter, bccmd and is similarly influenced by snmp.  The host
	controller (client) views the services presented by the host (server)
	as a set of values to be read and/or written.  Only two types of
	transaction are defined:

	    To get a value, the client sends a GET-REQUEST pdu to the
	    server.  The server reads the value and returns it in a
	    GET-RESPONSE pdu to the host.

	    To set a value, the client sends a SET-REQUEST pdu to the
	    server.  The server sets the value and returns the new value in a
	    GET-RESPONSE pdu to the host.

	There are only three basic types of PDU:

            GET-REQUEST - (called HQPDU_GETREQ below) asks for the value
               of a variable to be returned to the host.

            GET-RESPONSE - (called HQPDU_GETRESP below) returns the value
                of a variable to the host.

            SET-REQUEST - (called HQPDU_SETREQ below) asks to set the
                value of a variable. 

	The host controller commands the host to perform an action or gives a 
	status report by writing to a particular variable.  Abstracting the
	bccmd's functionality as reading and writing variables promotes a
	simple protocol.

	No attempt is made to support snmp's SMI - the way in which snmp
	names variables.  This protocol uses a single uint16 to identify each
	variable.

	Every REQUEST received by the host causes it to (attempt to) return
	one RESPONSE.  However, in exceptional circumstances a response may
	not be sent or received.  Thus, the host controller must use a
	timeout.

	The GET-REQUEST PDU is always the same size as the corresponding
	GET-RESPONSE PDU.  Similarly, the SET-REQUEST PDU is always the same
	size as the corresponding GET-RESPONSE PDU.

	The type HQPDU is of fixed size.  However, it is acceptable for
	payload part of a message to be longer than this.  The actual length
	of the PDU is given in HQPDU.pdulen.  Awkward payloads may overhang
	the sizeof(HQPDU) in the manner of bccmdpdu.h, however for all
	current messages there is no overhang - all messages fit within the
	HQPDU.  The result is that all HQPDU messages are currently the
	same size.

        Example Transactions
        --------------------  

        1) To signal that the bc01 has just been booted.

          hq->host HQPDU_SETREQ

            uint16 field             value
            1      HQPDU.type        2      (HQPDU_SETREQ)
            2      HQPDU.pdulen      12     (number of uint16s in PDU)
            3      HQPDU.seqno       1235   (value chosen by bc01)
            4      HQPDU.varid       1      (HQVARID_BOOTED)
            5      HQPDU.status      0      (HQPDU_STAT_OK)
            6      HQPDU.d.dummy     0      (no payload)
            7->12  HQPDU.d  (unused) 0      (unused space - set to zero)

          host->hq HQPDU_GETRESP

            uint16 field             value
            1      HQPDU.type        1      (HQPDU_GETRESP)
            2      HQPDU.pdulen      12     (number of uint16s in PDU)
            3      HQPDU.seqno       1235
            4      HQPDU.varid       1      (HQVARID_BOOTED)
            5      HQPDU.status      0      (HQPDU_STAT_OK)
            6      HQPDU.d.dummy     0      (no payload)
            7->12  HQPDU.d  (unused) 0      (unused space - set to zero)

        2) To signal that the bc01 is about to reboot itself following
	   an irrecoverable error.  The response may not reach the bc01
	   in time, coz' it may be rebooting.

          hq->host HQPDU_SETREQ

            uint16 field             value
            1      HQPDU.type        2      (HQPDU_SETREQ)
            2      HQPDU.pdulen      12     (number of uint16s in PDU)
            3      HQPDU.seqno       1236   (value chosen by bc01)
            4      HQPDU.varid       2049   (HQVARID_DELAYED_PANIC)
            5      HQPDU.status      0      (HQPDU_STAT_OK)
            6      HQPDU.d.u16       31     (panicid - deathbed confession)
	    7-12   HQPDU.d (unused)  0	    (unused space - set to zero)

          host->hq HQPDU_GETRESP

            uint16 field             value
            1      HQPDU.type        1      (HQPDU_GETRESP)
            2      HQPDU.pdulen      12     (number of uint16s in PDU)
            3      HQPDU.seqno       1236
            4      HQPDU.varid       2049   (HQVARID_DELAYED_PANIC)
            5      HQPDU.status      0      (HQPDU_STAT_OK)
            6      HQPDU.d.u16       31     (panicid - deathbed confession)
	    7-12   HQPDU.d (unused)  0	    (unused space - set to zero)

        3) To pass a fault report to the host.

          hq->host HQPDU_SETREQ

            uint16 field             value
            1      HQPDU.type        2      (HQPDU_SETREQ)
            2      HQPDU.pdulen      12     (number of uint16s in PDU)
            3      HQPDU.seqno       1237   (value chosen by bc01)
            4      HQPDU.varid       4096   (HQVARID_FAULT)
            5      HQPDU.status      0      (HQPDU_STAT_OK)
            6      HQPDU.d.fault.f   10     (faultid - failure code)
            7-8    HQPDU.d.fault.t   999999 (TIME is uint32, low word 1st)
            9      HQPDU.d.fault.n   3      (Number of times fault occurred)
            10-12  HQPDU.d  (unused) 0      (unused space - set to zero)

          host->hq HQPDU_GETRESP

            uint16 field             value
            1      HQPDU.type        1      (HQPDU_GETRESP)
            2      HQPDU.pdulen      12     (number of uint16s in PDU)
            3      HQPDU.seqno       1237
            4      HQPDU.varid       4096   (HQVARID_FAULT)
            5      HQPDU.status      0      (HQPDU_STAT_OK)
            6      HQPDU.d.fault.f   10     (faultid - failure code)
            7-8    HQPDU.d.fault.t   999999 (TIME is uint32, low word 1st)
            9      HQPDU.d.fault.n   3      (Number of times fault occurred)
            10-12  HQPDU.d  (unused) 0      (unused space - set to zero)

        4) Host refuses request to read unknown value.

          hq->host HQPDU_GETREQ

            uint16 field             value
            1      HQPDU.type        0      (HQPDU_GETREQ)
            2      HQPDU.pdulen      12     (number of uint16s in PDU)
            3      HQPDU.seqno       1238   (value chosen by bc01)
            4      HQPDU.varid       12345  (HQVARID_TWADDLE)
            5      HQPDU.status      0      (HQPDU_STAT_OK)
            6      HQPDU.d.dummy     0      (no payload)
            7->12  HQPDU.d  (unused) 0      (unused space - set to zero)

          host->hq HQPDU_GETRESP

            uint16 field             value
            1      HQPDU.type        1      (HQPDU_GETRESP)
            2      HQPDU.pdulen      12     (number of uint16s in PDU)
            3      HQPDU.seqno       1238
            4      HQPDU.varid       12345  (HQVARID_TWADDLE)
            5      HQPDU.status      1      (HQPDU_STAT_NO_SUCH_VARID)
            6      HQPDU.d.dummy     0      (no payload)
            7->12  HQPDU.d  (unused) 0      (unused space - set to zero)

MODIFICATION HISTORY
	1.1  20:apr:01  ajh	Moved to interface/host/hq.
				Removed dependency on bt.h.
	1.2  13:jun:01  cjo	Corrected comment - bcmon task removed.
	1.3  02:oct:01  pws     Added HQPDU_RADIOTEST_STATUS_ARRAY.
	1.4.branched    ajh     Changed HQPDU_ARRAYn to more meaningfully
				named types.  Corrected a few comments.
	1.6  21:nov:02  pws     H17.71: HQVARID_BER_TRIGGER placeholder.
	1.7  22:nov:02  pws     H17.71: Rationalise names.
	#10  10:feb:04  mm      B-1494: Added HQPDU_DRAIN_CALIBRATE.

OLD MODIFICATION HISTORY
	1.1  10:dec:99	sm	Created.
	1.5  6:apr:00	cjo	Reworked.
	1.6  10:apr:00	cjo	Restructured.
	1.8  11:apr:00	cjo	Added radiotest reports.
	1.9  19:apr:00  cjo     Additions agreed between James and jyb's
				client.  Extra vals in HQPDU_RXPKTSTATS.
	1.10 11:jul:00  pws     New PDU's for returning radiotest data.
	1.11 29:aug:00  pws     Note ordering of 32-bit integers in bit err.
	1.12 18:oct:00  cjo     Corrected comment.
	1.13 18:oct:00  cjo     Corrected more comments - pws increased the
				size of HQPDU, so all of the above example
				transactions were broken.
	1.14 15:dec:00  pws     Add HQVARID_RADIO_STATUS.
	1.15 19:jan:01  cjo     Made comments correcter.
				Aligned HQPDU_STAT* codes with bccmdpdu.h.
	1.16 22:jan:01  pws     Say `(0 to 8)'.
	1.17 01:mar:01  pws     New argument for hq_radio_status().
	1.18 01:mar:01  pws     Change PDU length in examples to 12.
	1.19 12:may:02  npm     Add HQ_PDU_PROTOCOL_ERROR .
*/

#ifndef	__HQPDU_H__
#define	__HQPDU_H__
 
#ifdef RCS_STRINGS
static char hqpdu_h_id[]
  = "$Id: //depot/bc/main/interface/host/hq/hqpdu.h#10 $";
#endif


/***************************************************************************
HQPDU_FAULT
===========

A report of a set of fault() calls on the bc01. */

typedef struct {
	uint16		f;	/* A fault identifier. */
	uint32		t;	/* TIME of first report. */
	uint16		n;	/* Number of f reports. */
	} HQPDU_FAULT;

/***************************************************************************
HQPDU_PROTOCOL_ERROR
====================

A report of an error in an air protocol 1. */

typedef struct {
     uint16		layer;	/* Layer which detected problem */
     uint16		entity;	/* remote or local entity*/
     uint16		error;	/* error code*/
     uint16             info0 ; /* optional additional information */
     uint16             info1 ; /* optional additional information */
     uint16             info2 ; /* optional additional information */
	} HQPDU_PROTOCOL_ERROR;


/**************************************************************************
HQPDU_RXPKTSTATS
================

A report of a set of received packets and correctable received packets from
one of the bc01-specific radio tests in the radiotest subsystem. */

typedef struct {
	uint16		n_pkts;			/* Total no. pkts rxed. */
	uint16		n_good_pkts;		/* HERE wtf is a "good" pkt? */
	uint16		n_correctable_pkts;	/* No. correctable pkts rxed */
	uint16		rssi;			/* avg rssi value.  */
	bool		rssi_valid;		/* Is "rssi" valid? */
	} HQPDU_RXPKTSTATS;

/**************************************************************************
HQPDU_LUT_ENTRY
===============

A part of an array report with a single uint16 value. */

typedef struct {
	uint16		index;			/* Index of array element */
	uint16		trim;			/* Value to return */
	} HQPDU_LUT_ENTRY;

/**************************************************************************
HQPDU_SYNTH_SETTLE
==================

A part of an array report with a single uint16 value. */

typedef struct {
	uint16		index;			/* Index of array element */
	uint16		voltage;		/* Value to return */
	} HQPDU_SYNTH_SETTLE;

/**************************************************************************
HQPDU_RADIO_STATUS_ARRAY
========================

A part of an array report with a single uint16 value. */

typedef struct {
	uint16		index;			/* Index of array element */
	uint16		value;			/* Value to return */
	} HQPDU_RADIO_STATUS_ARRAY;

/**************************************************************************
HQPDU_FILTER_RESP
=================

A part of an array report with two uint16 values. */

typedef struct {
	uint16		index;			/* Index of array element */
	uint16		freq;			/* First value to return */
	uint16		rssi;			/* Second value to return */
	} HQPDU_FILTER_RESP;

/**************************************************************************
HQPDU_IQ_MATCH
==============

A part of an array report with three uint16 values. */

typedef struct {
	uint16		index;			/* Index of array element */
	uint16		trim;			/* First value to return */
	uint16		rssi_chan;		/* Second value to return */
	uint16		rssi_image;		/* Third value to return */
	} HQPDU_IQ_MATCH;

/**************************************************************************
HQPDU_BIT_ERR
=============

One measurement for a bit error report.  The type of measurement is given
by the index.  These are as listed for radiotest_bit_err1 in radiotest.h.
The two values are a count over the last report period (default 1 second),
and a cumulative amount to date (until the target count is reached, when
it is reset). 

Note that 32-bit integers have their words in XAP order.  This means
that the data is effectively a series of 16-bit words as follows:
  index (0 to 8)
  high word of value_last
  low word of value_last
  high word of value_tot
  low word of value_tot
*/

typedef struct {
	uint16		index;			/* Type of measurement */
	uint32		value_last;		/* Value over last period */
	uint32		value_tot;		/* Value since reset */
	} HQPDU_BIT_ERR;

/**************************************************************************
HQPDU_RADIO_STATUS
==================

A set of six numbers returned by radiotest_get_radio_report.  They are the
values of the registers ANA_TX_LVL, ANA_TX_EXT_LVL, ANA_RX_LVL, ANA_LO_LVL
an ANA_IQ_TRIM, if the IQ trim was updated, a value of the signal
to image ratio, and the register RX_ATTEN.  If you don't already know what
those are, you aren't interested.
*/

typedef struct {
	uint16		tx_lvl;		/* Internal transmission level */
	uint16		tx_ext_lvl;	/* External transmission level */
	uint16		rx_lvl;		/* Reception level */
	uint16		lo_lvl;		/* Local oscillator level */
	uint16		iq_trim;	/* IQ trim */
	uint16		sir;		/* Signal/image ratio for IQ trim */
	uint16		rx_atten;	/* Receiver attentuation */
} HQPDU_RADIO_STATUS;


/* Red-M Change Starts */

/**************************************************************************
HQPDU_BER_TRIGGER
===================

Sent when a configured bit error rate has been hit. Used to trigger
handoff operations.
*/

typedef struct {
	uint16 hci_handle;
	uint16 avg_ber;
	uint16 raw_ber;
	uint16 ber_threshold;		/* Only 8 bits used */
} HQPDU_BER_TRIGGER;

/* Red-M Change Ends */


/**************************************************************************
HQPDU_DRAIN_CALIBRATE
=====================

This is sent when we are configuring the drain sub-system.
*/

typedef struct {
    uint32	awake_time;
    uint32      sleep_time;
    uint32      half_slots;
} HQPDU_DRAIN_CALIBRATE;


/**************************************************************************
HQPDU
=====

The protocol's pdu structure. */

typedef struct {
	/* PDU type. */
	uint16		type;		

/* Acceptable values for "type". */
#define	HQPDU_GETREQ		(0)
#define	HQPDU_GETRESP		(1)
#define	HQPDU_SETREQ		(2)

	/* Length of PDU in uint16s. */
	uint16		pdulen;			

/* Maximum total length of a PDU. */
#define HQPDU_MAXLEN		(64)

	/* Sequence number. */
	uint16		seqno;		

	/* Id of variable to be accessed. */
	uint16		varid;		

	/* How host managed to perform operation. */
	uint16		status;		

/* Acceptable values for "status". */
#define	HQPDU_STAT_OK		(0)	/* Default.  No problem found. */
#define	HQPDU_STAT_NO_SUCH_VARID (1)	/* varid not recognised. */
#define	HQPDU_STAT_TOO_BIG	(2)	/* Data exceeded pdu/mem capacity.*/
#define	HQPDU_STAT_NO_VALUE	(3)	/* Variable has no value. */
#define	HQPDU_STAT_BAD_PDU	(4)	/* Dodgy value(s) in PDU. */
#define	HQPDU_STAT_NO_ACCESS	(5)	/* Dodgy value(s) in PDU. */
#define	HQPDU_STAT_READ_ONLY	(6)	/* Value of varid inaccessible. */
#define	HQPDU_STAT_WRITE_ONLY	(7)	/* Dodgy value(s) in PDU. */
#define	HQPDU_STAT_ERROR	(8)	/* The useless catch-all error. */
#define HQPDU_STAT_PERMISSION_DENIED (9) /* Request not allowed. */


/**************************************************************************
That's the end of the common header of an hqpdu.  Now follows stuff that
varies according to the settings of the "type" and "varid" fields.

Not all of the types in the following union occupy all of the union's space.
All unused space must be set to zero.
**************************************************************************/

	union {
		/* Useless placeholder.  Nothing is written into HQPDU.d if
		"type" is HQPDU_GETREQ or if "varid" is between
		HQVARID_COMMANDS_START and (HQVARID_UINT8_START-1). */

		uint16	dummy;

		/* For use if "varid" is between HQVARID_UINT8_START and
		(HQVARID_UINT16_START-1). */

		uint8	u8;

		/* For use if "varid" is between HQVARID_UINT16_START and
		(HQVARID_UINT32_START-1). */

		uint16	u16;

		/* For use if "varid" is between HQVARID_UINT32_START and
		(HQVARID_SPECIAL_START-1). */

		uint32	u32;

		/* If "varid" is HQVARID_SPECIAL_START or above then
		one of a set of ugly data structures follows. */

		HQPDU_FAULT              fault;	   /* HQVARID_FAULT          */
		HQPDU_PROTOCOL_ERROR     protocol_error;
                                                   /* HQVARID_PROTOCOL_ERROR */
		HQPDU_RXPKTSTATS         rxpkt_stats;
                                                   /* HQVARID_RXPKTSTATS     */
		HQPDU_LUT_ENTRY          lut_entry;/* HQVARID_LUT_ENTRY      */
		HQPDU_SYNTH_SETTLE       synth_settle;
                                                   /* HQVARID_LUT_ENTRY      */
		HQPDU_RADIO_STATUS_ARRAY radio_status_array;
                                                   /* HQVARID_LUT_ENTRY      */
		HQPDU_FILTER_RESP        filter_resp;
                                                   /* HQVARID_FILTER_RESP    */
		HQPDU_IQ_MATCH           iq_match; /* HQVARID_IQ_MATCH       */
		HQPDU_BIT_ERR            bit_err;  /* HQVARID_BIT_ERR        */
	        HQPDU_RADIO_STATUS       radio_status;
                                                   /* HQVARID_RADIO_STATUS   */
		HQPDU_BER_TRIGGER	 ber_trigger; /* HQVARID_BER_TRIGGER */
	        HQPDU_DRAIN_CALIBRATE    drain; /* HQVARID_DRAIN_CALIBRATE */
		} d;

	} HQPDU;


/***************************************************************************
The varids are listed in blocks.  Each block holds variables of a given
type.  This allows the host to determine a variable's type by seeing which
range it falls in. */


/***************************************************************************
COMMAND VARIDS (variables which invoke commands when written to). */

#define	HQVARID_COMMANDS_START 	(0)

/* HQVARID_NO_VARIABLE harmlessly and silently does nothing. */
#define	HQVARID_NO_VARIABLE	(HQVARID_COMMANDS_START+0)

/* Message to the host that the bc01 has just booted itself. */
#define	HQVARID_BOOTED		(HQVARID_COMMANDS_START+1)


/***************************************************************************
Variables with type [u]int8.  The variable's value is held in HQPDU.d.u8. */

#define	HQVARID_UINT8_START 	(1024)


/****************************************************************************
Variables with type [u]int16.  The variable's value is held in HQPDU.d.u16. */

#define	HQVARID_UINT16_START 	(2048)

/* Msg to the host that the bc01 is about to reboot.  Reason in the u16. */
#define	HQVARID_DELAYED_PANIC	(HQVARID_UINT16_START+0)

/* An rssi report from one of the bc01's radio's test modes. */
#define	HQVARID_RSSI		(HQVARID_UINT16_START+1)

/****************************************************************************
Variables with type [u]int32.  The variable's value is held in HQPDU.d.u32. */

#define	HQVARID_UINT32_START 	(3072)


/***************************************************************************
Variables of special (i.e., awkward) types. */

#define	HQVARID_SPECIAL_START 	(4096)

/* Report in HQPDU.d.fault. */
#define	HQVARID_FAULT		(HQVARID_SPECIAL_START+0)

/* Report in HQPDU.d.rxpkt_stats. */
#define	HQVARID_RXPKTSTATS	(HQVARID_SPECIAL_START+1)

/* Report in HQPDU.d.lut_entry */
#define HQVARID_LUT_ENTRY	(HQVARID_SPECIAL_START+2)

/* Report in HQPDU.d.synth_settle */
#define HQVARID_SYNTH_SETTLE	(HQVARID_SPECIAL_START+3)

/* Report in HQPDU.d.filter_resp */
#define HQVARID_FILTER_RESP	(HQVARID_SPECIAL_START+4)

/* Report in HQPDU.d.iq_match */
#define HQVARID_IQ_MATCH	(HQVARID_SPECIAL_START+5)

/* Report in HQPDU.d.bit_err */
#define HQVARID_BIT_ERR		(HQVARID_SPECIAL_START+6)

/* Report in HQPDU.d.radio_status */
#define HQVARID_RADIO_STATUS	(HQVARID_SPECIAL_START+7)

/* Report in HQPDU.d.radio_status_array */
#define HQVARID_RADIO_STATUS_ARRAY	(HQVARID_SPECIAL_START+8)

/* Report in HQPDU.d.protocol_error. */
#define	HQVARID_PROTOCOL_ERROR		(HQVARID_SPECIAL_START+9)

/* Red-M Change Starts */
/* A handoff trigger event: report in HQPDU.d.bert. */
#define	HQVARID_BER_TRIGGER	(HQVARID_SPECIAL_START+10)
/* Red-M Change Ends */

/* Report in HQPDU.d.drain */
#define HQVARID_DRAIN_CALIBRATE       (HQVARID_SPECIAL_START+11)

#endif	/* __HQPDU_H__ */
