/****************************************************************************
FILE
	io_bt.h  -  high-level definitions required for accessing ps

DESCRIPTION
	Definitions of enums widely used in the firmware, but not tied
	to the hardware.  Presented here since they need to be accessed
	from host tools for persistent store values.

MODIFICATION HISTORY
	1.1  11:jan:02  pws     Created.
	1.2  17:jan:02  pws     hci-14 #4: limit CPU clock reduction for PCM.
	1.4  08:feb:02  pws     H15.29: add PSKEY_CPU_IDLE_MODE
	1.5  08:feb:02  pws     H15.29: improve comments.
	1.6  05:apr:02  pws     H15.104: ROM sleep (cpu_idle_rom).
	1.7  09:apr:02  sm	H15.107: add support for phys_bus_none.
	1.8  21:nov:02  mm01    H17.70: add support for phys_bus_h5.
	1.9  25:nov:02  pws     H17.70: copied from mm01 and committed.
	1.10 02:sep:03  sm	B-932: Merge h4+ onto the mainline.
	#12  31:mar:04  sm	B-2400: Merge h4ds onto the mainline.
*/

#ifndef IO_BT_H
#define IO_BT_H

#ifdef RCS_STRINGS
static char io_bt_h_id[]
  = "$Id: //depot/bc/main/interface/host/io/io_bt.h#12 $";
#endif


/* A device's country code and its allowed values.   These values are
the wire values, i.e. as specified by hci.  (This value is not carried
by lmp.) */

enum bdcountrycode_enum {
	bdcc_na_and_e = 0x00,	/* North America and Yerp, except ... */
	bdcc_france = 0x01,	/* France. */
	bdcc_spain = 0x02,	/* Spain. */
	bdcc_japan = 0x03	/* Japan. */
	};
typedef enum bdcountrycode_enum	bdcountrycode;


/* The physical interface present between chip and the host. */
enum phys_bus_enum {
	phys_bus_none = 0,	/* No host controller <-> host connection */
	phys_bus_bcsp = 1,	/* UART link running BCSP */
	phys_bus_usb = 2,	/* USB link */
	phys_bus_h4 = 3,	/* UART link running H4 */
	phys_bus_usr = 4,	/* vm access to the uart */
        phys_bus_h4plus = 5,    /* H4+ */
	phys_bus_h5 = 6,	/* UART link running H5 (like BCSP) */
 	phys_bus_h4ds = 7	/* UART link running H4DS */
};
typedef enum phys_bus_enum phys_bus;

/* States for the host - for the case where the host can go to sleep. */
enum hwakeup_state_enum {
    HOST_AWAKE,         /* Host is operating normally */
    HOST_ASLEEP,        /* Host requires waking up before communication */
    HOST_WAKING,        /* We are waking the host up */
    HOST_WAIT,          /* Lets the host doze after waking up before we send data */
    HOST_NEVER_SLEEPS   /* Host never sleeps */
};
typedef enum hwakeup_state_enum hwakeup_state;

/*
 * Whether the lc (and radiotest) should perform trimming of the IQ block.
 */
typedef enum {
    iq_trim_off,		/* Never */
    iq_trim_start,		/* Only at start */
    iq_trim_full		/* Regular 10,000 mile service schedule */
} iq_trim_enable_flag;

/* Definitions for choosing whether to enter deep sleep (dorm module) */
enum deep_sleep_state_enum {
    DEEP_SLEEP_NEVER = 0,	/* Never enter deep sleep. */
    DEEP_SLEEP_ALWAYS = 1,	/* Enter deep sleep whenever possible. */
    DEEP_SLEEP_INACTIVE = 2	/* Enter deep sleep if no connections. */
};

typedef enum deep_sleep_state_enum deep_sleep_state;

/*
 * How slow to slow the clock down to.
 *
 * There is no particular constraint on the processor itself about how
 * far we can go.  However, we have to worry about the effect on UART, USB and
 * PCM activity.  PCM should allow us to go down to 1 MHz, although 1.024MHz
 * is similar and may be more accurate; with UART, we need to keep
 * an oversampling margin of 8 times.  With standard 115200 bps, 1 MHz is OK.
 */
enum cpu_tardiness_enum {
    CPU_FAST,			/* Must correspond to zero */
    CPU_SLOW_4M,
    CPU_SLOW_2M,
    CPU_SLOW_1p024M,
    CPU_SLOW_1M,
    CPU_SLOW_p5M,
    CPU_SLOW_p25M,
    CPU_SLOW_p125M,
    CPU_DEEP_SLEEP
};

typedef enum cpu_tardiness_enum cpu_tardiness;

/*
 * Enum used for bc02 only to decide what to do when the radio is
 * active, or we are in test mode, but the processor is idle.
 * Not used for bc01 but defined anyway --- the host tools like
 * to be able to see all possible enums before deciding what
 * chip they are talking too.
 */
enum cpu_idle_mode_enum {
    /* Control passes straight back to the scheduler; no idle code. */
    cpu_idle_none,
    /* Execute a long loop, which does not fit in the chip's cache */
    cpu_idle_long_loop,
    /*
     * Execute a tight loop, which does fit in the chip's instruction cache.
     * Some makes of flash take a while to rouse when we do this, so
     * it is not always appropriate.
     */
    cpu_idle_short_loop,
    /*
     * Go into an idle state in RAM.  This minimises current consumption
     * and is used if the radio is not in use.  However, if the radio is
     * active it can cause the current to change too much.
     */
    cpu_idle_ram,
    /*
     * Sleep from ROM when idle.
     */
    cpu_idle_rom
};

typedef enum cpu_idle_mode_enum cpu_idle_mode;

#endif /* IO_BT_H */
