/****************************************************************************
FILE
    ps.h  -  persistent store subsystem

CONTAINS
    init_ps1  -  first stage initialisation before pmalloc
    init_ps2  -  second stage initialisation after pmalloc
    ps_get  -  get a value from the persistent store
    ps_set  -  write a value to the persistent store
    ps_clr  -  remove a value from the persistent store
    ps_size  -  how big is a value in the persistent store
    ps_location_size  -  how big is each location of the store
    ps_memory_type  -  determine underlying memory type for persistent store
    ps_match  -  compare persistent store value
    ps_next_key  -  find next higher key in the store
    ps_free_space  -  how much unused capacity has the store
    ps_clear_all  -  remove all values from the ps
    
DESCRIPTION
    The persistent store is layered over the local implementation
    of the normal store (defined in psi.h), the ROM store (defined
    in psrom.h) and the RAM store (defined in psram.h).

MODIFICATION HISTORY
    1.1   18:aug:99     cjo    Created.
    1.2   19:aug:99     cjo    Rearranged ps_get() parameters.
    1.3   15:sep:99     cjo    Changed ps_init() to init_ps().
    1.4   16:sep:99     cjo    Updated comments.
    1.5   16:sep:99     cjo    Updated comments.
    1.6   21:jan:00     cjo    Added ps_get_default().
                               Extended ps_get().
    1.7   10:mar:00     cjo    Updated comments.
    1.8   16:mar:00     cjo    Added ps_clear_all().
    1.9   25:mar:00     cjo    Made ps_get_default() void.
    1.10  04:apr:00     cjo    Added the "really" flag to ps_clear_all().
    1.11  05:apr:00     cjo    Updated comments.
    1.12  20:jun:00     sms    Added ps_size().
    1.13  04:sep:00     cjo    Corrected comments.
    1.14  04:sep:00     cjo    Updated comments.
    1.15  08:sep:00     cjo    Added ps_next_key() and ps_free_space().
    1.16  15:sep:00     cjo    Major overhaul...
    1.17  15:sep:00     cjo    ...
    1.18  10:sep:00     cjo    ... role of ps.h moved to psi.h and ps.h now
                               layers above psi.h and psrom.h. Added pstores
                               argument to some functions.
    1.19  10:oct:00     cjo    Added pstores_f.
    1.20  26:jan:01     cjo    Changed pstores_enum to bitfield.
    1.21  21:jun:01     at     Corrected ps_next_key to handle pstores_f.
    1.22  25:jun:01     at     Added ps_match().
    1.23  02:jul:01     at     Added pstores_default.
    1.24  28:jan:02     at     Added pstores_t* to access psram.
    1.25  27:jul:02     at     H16.16: Added init_ps1() and init_ps2().
    1.27  06:jun:03     at     B-598: Added ps_memory_type().
    #28   03:dec:03     at     B-1544: Added psmemory_none.
*/

#ifndef	__PS_H__
#define	__PS_H__
 
#ifdef RCS_STRINGS
static char ps_h_id[]
  = "$Id: //depot/bc/main/dev/src/csr/ps/ps.h#28 $";
#endif


#include "bt.h"

/* An key to identify an item in the persistent store */
typedef uint16  pskey;


/* Value denoting "not a pskey" */
#define	PSKEY_NULL ((pskey) (0))


/* Which stores to work with, and the order in which to use them. Unless
there is a strong requirement to specify particular store(s) it is strongly
recommended that pstores_default is used, especially for set/clr operations,
since the most appropriate stores may depend on the current hardware. */
enum pstores_enum
{
    pstores_default = 0x0,	/* default stores for operation. */
    /* Single stores */
    pstores_i       = 0x1,	/* implementation (normal psi.h). */
    pstores_f       = 0x2,	/* factory-set (psf.h). */
    pstores_r       = 0x4,	/* rom (psrom.h). */
    pstores_t       = 0x8,	/* transient (psram.h). */
    /* Useful combinations of stores */
    pstores_fr      = 0x6,	/* psf.h -> psrom.h. */
    pstores_if      = 0x3,	/* psi.h -> psf.h. */
    pstores_ifr     = 0x7,	/* psi.h -> psf.h -> psrom.h. */
    pstores_ti      = 0x9,	/* psram.h -> psi.h. */
    pstores_tif     = 0xb,	/* psram.h -> psi.h -> psf.h. */
    pstores_tifr    = 0xf,	/* psram.h -> psi.h -> psf.h -> psrom.h. */

    pstores_raw     = 0x80      /* use with caution - the actual store in flash
				   is only 4 bits.  However, stores passed in via
				   bccmd can be larger.  The raw view is only used from
				   offchip (via bccmd) so having a 'raw' store is a 
				   useful concept.  See psbc_bootmode for more details
				*/
};
typedef enum pstores_enum pstores;


/* Underlying memory type used to implement persistent store */
enum psmemory_enum
{
    psmemory_flash  = 0x0,
    psmemory_eeprom = 0x1,
    psmemory_ram    = 0x2,
    psmemory_rom    = 0x3,
    psmemory_none   = 0x4
};
typedef enum psmemory_enum psmemory;


/* mask out 'raw' bit from store */

#define fixstores(s) ((pstores) ((s) & (~pstores_raw)))


/****************************************************************************
NAME
    init_ps1  -  first stage initialisation before pmalloc

FUNCTION
    Initialise the persistent store. This function must be called before
    all others in this file, and before initialisation of the pmalloc
    subsystem. This must be followed by a call to init_ps2().
*/
extern void init_ps1(void);


/****************************************************************************
NAME
    init_ps2  -  second stage initialisation after pmalloc

FUNCTION
    Initialise the persistent store. This function must be called after
    the pmalloc subsystem has been initialised, but before buffer memory
    is used.
*/
extern void init_ps2(void);


/****************************************************************************
NAME
    ps_get  -  get a value from the persistent store

FUNCTION
    Reads the last value written into the persistent store under
    identifier "key" and copies this into the buffer at "data".
    
    The "stores" parameter determines where this call looks for data
    and the order in which the sources are searched. Any combination
    of stores may be specified. If pstores_default is specified then
    all valid stores are searched in order (pstores_tifr).
    
    The number of bits that can be stored at each location of "data"
    must match the value returned by ps_location_size(). (Normally
    "data" will be a uint8* or a uint16*.)
        
    The length of the buffer at "data" must be at least "maxlen"
    locations long. If len is not NULL the actual length of the data
    written into "data" is written into *len.
    
    It is acceptable for a data item to be of zero size.

RETURNS
    TRUE if the data item was found in the store and its size was did not
    exceed "maxlen", else FALSE.
    
    If "data" is not NULL then the value of the data item is written
    into the buffer at "data".

    If "len" is not NULL then the length of the data item is written into
    "len".

    If the function returns FALSE the memory at "data" and "len" are not
    changed.

NOTE
   This function is not thread safe. The persistent store may be made
   inaccessible by a call to ps_set(), so it is highly advisable to
   call ps_set() and ps_get() from the same thread. In practice
   these functions should really be called from the system's background
   code only.
*/
extern bool ps_get(pstores stores, pskey key, void *data, uint16 maxlen, uint16 *len);


/****************************************************************************
NAME
    ps_set  -  write a value to the persistent store

FUNCTION
    Copies "len" locations of data from "data" to the persistent store
    specified by "stores" and records the stored data under the access
    identifier "key". This  will overwrite any previous data stored
    under that key, i.e. the next call to ps_get() will obtain the fresh
    data.
    
    The value of "key" is arbitrary, but it must not be 0x0000 or 0xffff.
    (There may be other unacceptable values - this depends on the value
    returned by psfl_erase_value() - but for almost any system these are
    the only two unacceptable values for "key". psfl_erase_value() is
    described in psfl.h.)
    
    The "stores" parameter determines where this call writes the value.
    Only a single store may be specified, and that store must be
    writable, i.e. not psrom. If pstores_default is specified then
    this will normally write to the implementation store (pstores_i),
    but if that is not supported by the current hardware it will write
    to the RAM store instead (pstores_t).
    
    The number of bits in each location of the buffer at "data" is the
    value returned by ps_location_size(). (Normally "data" will be a
    uint8* or a uint16*.)
    
    It is acceptable for "len" to be zero.
    
    Writing to the persistent store will normally take a significant
    period, and this function blocks interrupts during the write.

    The function will be emit an error report (fault()) if the
    persistent store's memory is perceived to have filled or failed.
    
    Some implementations (EEPROM?) of this function may fail if the size
    of the data exceeds the value last stored under the same "key".
    
RETURNS
    TRUE if the value was written to the store, else FALSE.

    The function returns FALSE if there are insufficient resources to
    write the value or if any parameter is unacceptable.
    
NOTES
    This function is not thread safe. The persistent store may be made
    inaccessible by a simultaneous call to ps_set(), so it is highly
    advisable to call ps_set() (and ps_get()) from the same thread. In
    practice these functions should really be called from the system's
    background code only.

    This function checks that the active value of the key is not already
    the same as the value being written. This checks across all the
    stores.
*/
extern bool ps_set(pstores stores, pskey key, void *data, uint16 len);


/****************************************************************************
NAME
    ps_clr  -  remove a value from the persistent store

FUNCTION
   Attempts to remove a data item stored under the identifier "key"
   from the persistent store.

   The "stores" parameter determines which store(s) this call removes
   the key from. Multiple stores may be specified, but they must all
   be writable, i.e. not psrom. If pstores_default is specified then
   this will clear the key from the RAM and implementation stores
   (pstores_ti), leaving the factory store unaffected.
   
   If a call to ps_clr() succeeds then a subsequent call to ps_get()
   using the same key would fail for the same stores.
   
RETURNS
   TRUE if a data item stored under the identifier "key" was removed
   from at least one of the specified stores, else FALSE.

NOTE
   This function is not thread safe.
*/
extern bool ps_clr(pstores stores, pskey key);


/****************************************************************************
NAME
    ps_size  -  how big is a value in the persistent store

FUNCTION
    Finds whether any data is stored under the given persistent store
    "key", and if so determines it's length.

    The "stores" parameter determines where this call looks for data
    and the order in which the sources are searched. Any combination
    of stores may be specified. If pstores_default is specified then
    all valid stores are searched in order (pstores_tifr).
    
RETURNS
    TRUE if the data item was found in the ps, else FALSE.

    If the function returns TRUE and if "len" is not NULL then the
    length of the value is stored at *len.

    If the "key" is identifies zero length data then the function returns
    TRUE and, if len is not NULL, *len is set to zero.
    
    If the function returns FALSE the memory at "len" is not changed.
        
NOTE
    This function is not thread safe.
*/
extern bool ps_size(pstores stores, pskey key, uint16 *len); 


/****************************************************************************
NAME
    ps_location_size  -  how big is each location of the store

FUNCTION
    Most processors have byte-addressable data memory: each address
    accesses one byte. Alas, not all machine architectures are like
    this; one notorious processor makes a 16 bit word accessible at
    each data address. This irregularity can apply to memory used by
    the persistent storage library.
    
    The "stores" parameter determines which store the information is
    returned for. Only a single store may be specified.
    
RETURNS
   The number of data bits at each accessible location of the local
   machine's persistent storage memory.

NOTE
   The initialisation code ensures that all of the stores have the
   same location size. Hence, it is safe (and quicker) to always use
   the value for the psrom store.
*/
/* extern uint16 ps_location_size(pstores stores); */
#define ps_location_size(stores) \
    psrom_location_size()


/****************************************************************************
NAME
    ps_memory_type  -  determine underlying memory type for persistent store

RETURNS
    The type of memory underlying the store specified by "stores".
    Only a single store may be specified. If pstores_default is specified
    then the return value will be for the default store used by ps_set().

    It should not normally be necessary to use this function, as the
    persistent store libraries are deliberately designed to provide
    compatible functionality regardless of the underlying storage mechanism.
    However, it may be useful for verifying that the software has selected
    the expected store, or to improve performance.
*/
extern psmemory ps_memory_type( pstores stores );


/****************************************************************************
NAME
    ps_match  -  compare persistent store value

RETURNS
    TRUE if the persistent store holds an entry stored under "key" of
    size "len" that exactly matches the data stored at "data", else
    FALSE.
    
    The "stores" parameter determines where this call looks for data
    and the order in which the sources are searched. Any combination
    of stores may be specified. If pstores_default is specified then
    all valid stores are searched in order (pstores_tifr).

NOTE
    This function is not thread safe.
*/
extern bool ps_match(pstores stores, pskey key, void *data, uint16 len);


/****************************************************************************
NAME
    ps_next_key  -  find next higher key in the store

FUNCTION
    This function deals only with the values of keys in the persistent
    store - it ignores the data stored with keys.
    
    Notionally the function numerically sorts all of the keys in the
    store(s) into ascending order and returns the first key that that is
    greater than "key".
    
    If "key" is 0 then the lowest valued key in the store is returned.
    
    The "stores" parameter determines where this call looks for data
    and the order in which the sources are searched. Any combination
    of stores may be specified. If pstores_default is specified then
    all valid stores are searched in order (pstores_tifr).

RETURNS
    The next key in the store with value greater than "key", or 0 if
    there is no such key.

NOTE
    This function is not thread safe.
*/
extern pskey ps_next_key(pstores stores, pskey key);  


/****************************************************************************
NAME
    ps_free_space  -  how much unused capacity has the store

FUNCTION
    Determines how many keys of size "len" can be written to persistent
    store. This takes into account any overhead associated with each
    value written.

    The "stores" parameter determines which store the information is
    returned for. Only a single store may be specified. If
    pstores_default is specified then this will normally check the
    implementation store (pstores_i), but if that is not supported by
    the current hardware it will check the RAM store instead
    (pstores_t).

RETURNS
    The number of keys that can be written.

NOTE
    The psrom subsystem is always full.

    This function is not thread safe.
*/
extern uint16 ps_free_space(pstores stores, uint16 len);


/****************************************************************************
NAME
    ps_clear_all  -  remove all values from the ps

FUNCTION
    Removes all values from the persistent store.

    The "stores" parameter determines which store(s) this call removes
    keys from. Multiple stores may be specified, but they must all be
    writable, i.e. not psrom. If pstores_default is specified then
    this will clear the keys from all writable stores (pstores_tif).

NOTE
    This function is not thread safe.
*/
extern void ps_clear_all(pstores stores);


#endif
