/****************************************************************************
FILE
        psbc.h  -  persistent storage on bluecore

CONTAINS
	init_psbc  -  initialise the psbc subsystem
        psbc_location_size  -  how big is each locn of the persistent store
	psbc_size  -  how big is a persistent store item
	psbc_set  -  write a value to the persistent store
	psbc_get  -  get a value from the persistent store
        psbc_getx  -  get a value from the persistent store
        psbc_get_default  -  read a required value from the persistent store
        psbc_get_defaultx  -  read a required value from the store
	psbc_clr  -  remove a value from the persistent store
	psbc_clear_all  -  clear all values from the persistent store
        psbc_next_key  -  find next higher key in the store
        psbc_next_key_all  -  find next higher valid key
        psbc_free_space  -  how much unused capacity has the store
        psbc_known_key  -  does the ps know about this key
        psbc_key_permissions  -  get a key's access permissions
        psbc_user_key  -  is this a user/application key
        psbc_factory_set  -  establish pskey values for psbc_factory_restore
        psbc_factory_restore  -  restore a set of pskey values
        psbc_dfu_protected  -  is the key protected against dfu
        psbc_dfu_stack  -  is key associated with the stack
        psbc_dfu_application  -  is key associated with the application
        psbc_dfu_private  -  convert a key number to internal representation
        psbc_dfu_public  -  convert a key number to external representation

        pssize  -  macro to get size of a persistent store item
        psset  -  macro to write a value to the persistent store
        psget  -  macro to get a value from the persistent store
        psget_default  -  macro to get a required value from the store
        psclr  -  macro to remove a value from the persistent store
	psfree_space  - macro to get unused capacity of the store
        psnext  -  macro to find next higher key in the store

DESCRIPTION
	The ps subsystem attempts to provide a general purpose name/value
	database in flash or eeprom memory.  To a first approximation it
	knows nothing about the requirements of bluecore.

	The psbc subsystem is layered above ps and extends it to support
	bluecore's requirements.

	Many of this subsystem's functions map almost directly to the
	corresponding ps.h calls.

	This subsystem holds psbckeys.h, the list of pskeys used on
	bluecore.  (This used to be called psbc01.h - renamed because this
	code may also be used on bc02.)

	This subsystem also holds, the set of keys' default (psrom) values.
	This information used to be held in the psrom subsystem (in file
	psdata), but it is really project-specific, so it belongs here.

	The subsystem builds a list of these keys into a const data array to
	support other functions, e.g. bccmd's filtering of allowable pskeys.

	Earlier versions of the ps subsystems had a design error: the pskey
	values included the keys' (bccmd) access permissions.  These keys
	were used directly in ps.h and psrom.h stores.  Consequently we could
	never change a key's access permissions.  This error is addressed by
	this subsystem's initialisation function, init_psbc().  This scans
	the list of pskeys, rewriting them with the access bits stripped and
	an extra (no_perms) bit set.  This should result in the ps.h store
	losing knowledge of the access bits.

	This header file provides macros: pssize(), psset(), psget() and
	psget_default().  These should be the normal way for the bulk of
	bluecore code to access the store.

MODIFICATION HISTORY
	1.1   11:sep:00	 cjo	Created.
	1.5   20:sep:00	 cjo	Added psbc_key_permissions() and
				psbc_known_key().
	1.6   20:sep:00	 cjo	Added psbc_user_key().
	1.8   27:sep:00  cjo    Moved psbc_basekey() and psbc_fixkey() to
				psbc_private.h
	1.9   11:oct:00  cjo    Added support for psf.h and removed
				psbc_factory_clear().
	1.13  15:jan:01	 at	Added psbc_dfu_protected().
	1.16  2:mar:01	 cjo	Removed psv from the ps stack.  Replaced psv
				calls replaced with ps calls.
	1.17  12:mar:01  at	Added psbc_dfu_stack() and
				psbc_dfu_application().
	1.18  20:mar:01  at     Added psbc_dfu_private() and
	                        psbc_dfu_public().
	1.19  18:may:01  sms    Minor mod to definition of psbc_factory_set.
	1.20  3:jul:01   at     Added stores parameter to psbc_size() and
	                        psbc_get() plus derivatives.
	1.21  28:jan:02  at     Added stores parameter to psbc_set(),
	                        psbc_clr(), psbc_clear_all() and
				psbc_free_space() plus derivatives.
	1.22  26:jun:02  at     H16.13: Removed (void *) casts from macros.
	1.23  04:jul:02  sms    H16.30: New PS access functions.
	#25   18:sep:03  cjo    B-1031: add psbc_next_key_all().
	#26   06:oct:03  cjo    B-1168: Moved declaration of psbc_all_keys[]
				and attendant baggage from psbc_private.h to
				psbc.h.
	#27   26:feb:04 shm     B-1392: Port flash interlock code.
*/

#ifndef	__PSBC_H__
#define	__PSBC_H__
 
#ifdef RCS_STRINGS
static const char psbc_h_id[]
  = "$Id: //depot/bc/main/dev/src/csr/psbc/psbc.h#27 $";
#endif

#include "bt.h"
#include "ps.h"
#include "psf.h"
#include "psbckeys.h"
#include <stddef.h> /* For size_t and offsetof() */

#include "psflbc01_interlock.h"   /* interlocks on flash operations */

/* A more descriptive alias. */
#define	pesky		(pskey)


/* Maximum size of a ps value on bluecore. */
#define MAX_PSBC_VALSIZE	(64)

/* Structure containing a list of pskeys to get with psgroup_get */

struct psgroup {
    pskey key;
    size_t offset;
    uint16 size;
};

/* This might be handy when using X macros to build struct psgroup arrays */

#define PSGROUP_X1(key, type, str) \
    { PSKEY_ ## key , offsetof(struct str, key), sizeof(type) },

/* Handy array termination macro */

#define PSGROUP_END { PSKEY_NULL, 0, 0 }


/* Ranges of keys protected against dfu. Terminated by a PSKEY_NULL pair. */
typedef struct
{
    pskey start;                 /* Start of range (inclusive) */
    pskey end;                   /* End of range (inclusive) */
} pskey_range;
extern const pskey_range psbc_dfu_keys[];

/* Ranges of keys that are "valid" with this build.  Here, "valid" means
that the pskey is used by the firmware (e.g., PSKEY_BDADDR) or that it is
reasonable for the firmware to accept the pskey's value in the ps (e.g.,
PSKEY_USR0). */

extern const pskey_range psbc_all_keys[];


/****************************************************************************
NAME
        init_psbc  -  initialise the psbc subsystem

FUNCTION
        Initialises the bluecore's persistent store.

        This function must be called before all others in this file.

	This does *not* call init_ps().
*/

extern void init_psbc( void );


/****************************************************************************
NAME
        psbc_size  -  how big is a persistent store item

FUNCTION
        Finds whether any data is stored under the given persistent store
        "key", and if so determines it's length.

	The "stores" parameter determines where this call looks for data
	and the order in which the sources are searched.  It is usually
	sufficient to specify pstores_default to search all valid stores.

RETURNS
        TRUE if the data item was found in the ps, else FALSE.

        If the function returns TRUE and if "len" is not NULL then the
        length of the value is stored at *len.

        If the "key" is identifies zero length data then the function returns
        TRUE and, if len is not NULL, *len is set to zero.

        If the function returns FALSE the memory at "len" is not changed.

NOTE
        This function is not thread safe.
*/

extern bool psbc_size( pstores stores, pskey key, uint16 *len );

/* On bc01 users will only be interested in next key in the normal ps. */
#define	pssize(key,len)	psbc_size(pstores_default,(key),(len))


/****************************************************************************
NAME
        psbc_set  -  write a value to the persistent store

FUNCTION
        Copies "len" locations of data from "data" to the persistent store
        specified by "stores" and records the stored data under the access
	identifier "key".  This  will overwrite any previous data stored
	under that key, i.e. the next call to psbc_get() will obtain the
	fresh data.

        The value of "key" is arbitrary, but it must not be 0x0000 or 0xffff.
        (There may be other unacceptable values - this depends on the value
        returned by psfl_erase_value() - but for almost any system these are
        the only two unacceptable values for "key".  psfl_erase_value() is
        described in psfl.h.  Also see the constraints imposed by psf.h.)

	The "stores" parameter determines where this call writes the value.
	It is usually sufficient to specify pstores_default to write to the
	default store for the current hardware.

	The number of bits in each location of the buffer at "data" is the
	value returned by psbc_location_size().  (Normally "data" will be a
	uint8* or a uint16*.)

	It is acceptable for "len" to be zero.  The value of "len" must not
	exceed MAX_PSBC_VALSIZE.

	Writing to the persistent store will normally take a significant
	period, and this function blocks interrupts during the write.

	The function will be emit an error report (fault()) if the
	persistent store's memory is perceived to have filled or failed.

	Some implementations (EEPROM?) of this function may fail if the size
	of the data exceeds the value last stored under the same "key".

RETURNS
	TRUE if the value was written to the store, else FALSE.

	The function returns FALSE if there are insufficient resources to
	write the value, or if any parameter is unacceptable.

NOTE
	This function is not thread safe.  The persistent store may be made
	inaccessible by a simultaneous call to psbc_set(), so it is highly
	advisable to call psbc_set() (and psbc_get()) from the same thread.
	In practice these functions should really be called from the system's
	background code only.
*/

extern bool psbc_set( pstores stores, pskey key, void *data, uint16 len );

/* Write a value to the ps, using sizeof() to determine the size. */
#define psset(key,data)	\
    psbc_set(pstores_default,(key),&(data),sizeof(data))


/****************************************************************************
NAME
        psbc_get  -  get a value from the persistent store

FUNCTION
        Reads the last value written into the persistent store under
        identifier "key" and copies this into the buffer at "data".

	The "stores" parameter determines where this call looks for data
	and the order in which the sources are searched.  It is usually
	sufficient to specify pstores_default to search all valid stores.

        The number of bits that can be stored at each location of "data" must
        match the value returned by psbc_location_size().  (Normally "data"
        will be a uint8* or a uint16*.)

	The length of the buffer at "data" must be at least "maxlen"
	locations long.  If len is not NULL the actual length of the data
	written into "data" is written into *len.

	It is acceptable for a data item to be of zero size.

RETURNS
        TRUE if the data item was found in the store and its size was did not
        exceed "maxlen", else FALSE.

        If "data" is not NULL then the value of the data item is written
        into the buffer at "data".

        If "len" is not NULL then the length of the data item is written into
        "len".

        If the function returns FALSE the memory at "data" and "len" are not
        changed. 

NOTE
	This function is not thread safe.  The persistent store may be made
	inaccessible by a call to psbc_set(), so it is highly advisable to
	call psbc_set() and psbc_get() from the same thread.   In practice
	these functions should really be called from the system's background
	code only.
*/

extern bool psbc_get( pstores stores, pskey key, void *data, uint16 maxlen,
		uint16 *len );


/****************************************************************************
NAME
        psbc_getx  -  get a value from the persistent store

FUNCTION
	As psbc_get(), except the length of the data item is not returned.

NOTE
        This function exists purely to support the macro psget().
*/

extern bool psbc_getx( pstores stores, pskey key, void *data, 
		uint16 datasize );

/* Read a value from the ps, ignoring the returned length. */
#define psget(key,data,maxlen) \
    psbc_getx(pstores_default,(key),(data),(maxlen))


/****************************************************************************
NAME
        psbc_get_default  -  read a required value from the persistent store

FUNCTION
	Exactly as psbc_get(), except that the function makes a panic()
	call if no value can be found.

NOTE
	This call is expected to be used for values that are required to be
	in the store for the system to run, e.g. the radio's default jitter
	value.
*/

extern void psbc_get_default( pstores stores, pskey key, void *data,
		uint16 maxlen, uint16 *len );


/****************************************************************************
NAME
        psbc_get_defaultx  -  read a required value from the store

FUNCTION
	As psbc_get_default(), except the length of the item obtained is
	not returned.

NOTE
        This exists purely to support the macro psget_default().
*/

void psbc_get_defaultx( pstores stores, pskey key, void *data,
		uint16 datasize );

/* Read a value from the ps, ignoring the returned length. */
#define psget_default(key,data,maxlen) \
    psbc_get_defaultx(pstores_default,(key),(data),(maxlen))


/****************************************************************************
NAME
        psbc_clr  -  remove a value from the persistent store

FUNCTION
	Attempts to remove a data item stored under the identifier "key"
	from the persistent store.

	The "stores" parameter determines which store(s) this call removes
	the key from.  It is usually sufficient to specify pstores_default
	to clear the key from the stores usually written to.

	If a call to psbc_clr() succeeds then a subsequent call to psbc_get()
	using the same key will fail for the same stores.

RETURNS
        TRUE if a data item stored under the identifier "key" was removed
        from at least one of the specified stores, else FALSE.

NOTE
        This function is not thread safe.
*/

extern bool psbc_clr( pstores stores, pskey key );

/* A more terse and consistent way for bc code to access the function. */
#define	psclr(key) psbc_clr(pstores_default,(key))


/****************************************************************************
NAME
        psbc_clear_all  -  clear all values from the persistent store

FUNCTION
	Clears all values from the persistent store.  This effectively
	calls psbc_clr() for every key.
*/

/* Fall back onto ps.h function.
extern void psbc_clear_all( pstores stores ); */

#define psbc_clear_all(stores) ps_clear_all(stores)

/* A more terse and consistent way for bc code to access the function. */
#define psclear_all() psbc_clear_all(pstores_default)


/****************************************************************************
NAME
        psbc_location_size  -  how big is each locn of the persistent store

FUNCTION
        Most processors have byte-addressable data memory: each address
        accesses one byte.  Alas, not all machine architectures are like
        this; one notorious processor makes a 16 bit word accessible at
        each data address.  This irregularity can apply to memory used by
        the persistent storage library.

	The "stores" parameter determines which store the information is
	returned for.  It is usually sufficient to specify pstores_default
	since hopefully all the stores have the same location size.

RETURNS
	The number of data bits at each accessible location of the local
	machine's persistent storage memory.
*/

/* Macro - fall back on ps.h function.
extern uint16 psbc_location_size( pstores stores ); */

#define	psbc_location_size(stores) ps_location_size(stores)

/* A more terse and consistent way for bc code to access the function. */
#define pslocation_size() psbc_location_size(pstores_default)


/****************************************************************************
NAME
        psbc_next_key  -  find next higher key in the store

FUNCTION
        This function deals only with the values of keys in the persistent
        store - it ignores the data stored with keys.

        Notionally the function numerically sorts all of the keys in the
        store(s) into ascending order and returns the first key that that is
        greater than "key".

        If "key" is 0 then the lowest valued key in the store is returned.

	The "stores" parameter determines where this call looks for data
	and the order in which the sources are searched.  It is usually
	sufficient to specify pstores_default to search all valid stores.

RETURNS
        The next key in the store with value greater than "key", or 0 if
        there is no such key.

NOTE
        This function is not thread safe.
*/

extern pskey psbc_next_key( pstores stores, pskey key );

/* On bc01 users will only be interested in next key in the normal ps. */
#define psnext(k) psbc_next_key(pstores_default,(k))


/****************************************************************************
NAME
        psbc_next_key_all  -  find next higher valid key

FUNCTION
	the firmware (e.g., PSKEY_BDADDR) or that can reasonably be set for
	use with/by the firmware (e.g., PSKEY_USR0).  This function searches
	this list.

	The function returns the first key that that is numerically greater
	than "key".  If "key" is 0 then the lowest valued key is returned.

RETURNS
	The next key in the store with value greater than "key", or 0 if
	there is no such key.

NOTE
        This function is not thread safe.
*/

extern pskey psbc_next_key_all(pskey key);


/****************************************************************************
NAME
        psbc_free_space  -  how much unused capacity has the store

FUNCTION
	Determines how many keys of size "len" can be written to persistent
	store.  This takes into account any overhead associated with each
	value written.

	The "stores" parameter determines which store the information is
	returned for.  It is usually sufficient to specify pstores_default
	to check the default store for the current hardware.

RETURNS
        The number of keys that can be written.

NOTE
        The psrom subsystem is always full.

        This function is not thread safe.
*/

/* Macro - fall back on ps.h function.
extern uint16 psbc_free_space( pstores stores, uint16 len ); */

#define psbc_free_space(stores,len) ps_free_space((stores),(len))

/* On bc01 users will only be interested in the capacity of the normal ps. */
#define psfree_space(len) psbc_free_space(pstores_default,(len))


/****************************************************************************
NAME
        psbc_known_key  -  does the ps know about this key

RETURNS
	TRUE if the "key" has been named/defined on the Bluecore system, else
	FALSE.
*/

extern bool psbc_known_key( pskey key );


/****************************************************************************
NAME
        psbc_key_permissions  -  get a key's access permissions

FUNCTION
	Each pskey on Bluecore has a pair of access permissions: whether the
	key is readable and whether the key is writable.  These permissions
	are intended to be used to restrain the ambitions of the application,
	probably on the host, when the device is part of a user product.

	This function obtains these two permissions bits for "key".

RETURNS
	TRUE if the "key" has been named/defined on the system, else FALSE.

	If the function returns TRUE and if "readable" is not NULL then 
	*readable is set to the key's read permission.

	If the function returns TRUE and if "writable" is not NULL then 
	*writable is set to the key's write permission.

	If the function returns FALSE then *readable and *writable are not
	changed.
*/

extern bool psbc_key_permissions( pskey key, bool *readable, bool *writable );


/****************************************************************************
NAME
        psbc_user_key  -  is this a user/application key

RETURNS
	TRUE if "key" is in the range dedicated to user/application, else
	FALSE.
*/

extern bool psbc_user_key( pskey key );


/****************************************************************************
NAME
        psbc_factory_set  -  establish pskey values for psbc_factory_restore

FUNCTION
        Marks each key value in the psi store as being the "factory set"
        value.  This allows a subsequent call to psbc_factory_restore() to
        restore these keys' values.

RETURNS
	TRUE if the conversion was performed, else FALSE.  The function may
	return FALSE from pmalloc() exhaustion or from filling the ps.
*/

/* Macro - fall back on psf.h function.
extern bool psbc_factory_set( void ); */

#define	psbc_factory_set		psf_factory_set


/****************************************************************************
NAME
        psbc_factory_restore  -  restore a set of pskey values

FUNCTION
	If "all" is FALSE this removes every value from the variable ps that
	has a corresponding "factory set" value.

	If "all" is TRUE this removes all values from the variable ps except
	"factory set" values.

RETURNS
        TRUE if the removal was performed, else FALSE.  The function may
        return FALSE from filling the ps.
*/

/* Macro - fall back on psf.h function.
extern bool psbc_factory_restore( bool all ); */

#define	psbc_factory_restore(a)		psf_factory_restore(a)


/****************************************************************************
NAME
        psbc_dfu_protected  -  is the key protected against dfu

RETURNS
	TRUE if "key" is in the range protected against dfu upload or
	download, else FALSE.
*/

extern bool psbc_dfu_protected( pskey key );


/****************************************************************************
NAME
        psbc_dfu_stack  -  is key associated with the stack

RETURNS
	TRUE if "key" is in the range considered to be specific to the
	stack for the purposes of dfu, else FALSE.
*/

extern bool psbc_dfu_stack( pskey key );


/****************************************************************************
NAME
        psbc_dfu_stack  -  is key associated with the application

RETURNS
	TRUE if "key" is in the range considered to be specific to the
	application running under the vm for the purposes of dfu, else
	FALSE.
*/

extern bool psbc_dfu_application( pskey key );


/****************************************************************************
NAME
        psbc_dfu_private  -  convert a key number to internal representation

RETURNS
        Key number "key" with the appropriate bit set to indicate that
	it does not contain permission bits.
*/

extern pskey psbc_dfu_private( pskey key );


/****************************************************************************
NAME
        psbc_dfu_public  -  convert a key number to external representation

RETURNS
        Key number "key" with all permission related bits cleared.
*/

extern pskey psbc_dfu_public( pskey key );


/****************************************************************************
NAME
	psgroup_get  -  get lots of values from persistent store

FUNCTION
	Given an array of type struct psgroup which contains a list of
	keys, offsets and sizes, it gets the values of each key using
	psget_default() and writes them at the specified offset from the
	base address give.

	The list is terminated by an entry for PSKEY_NULL.

	If any of the keys cannot be found, the underlying psget_default()
	panics.
*/

extern void psgroup_get(void *base, const struct psgroup *psg);


/****************************************************************************
NAME
	psget_variable  -  get a variable length pskey

RETURNS
	A pointer to a freshly pmalloced block of memory containing the
	value of the key, or NULL if the key could not be found.
*/

extern uint16 *psget_variable(pskey key, uint16 *len);


/****************************************************************************
NAME
	psget_default_uint16  -  get a uint16 pskey

RETURNS
	The value of the specified key in the default selection of stores.

NOTES
	If the key cannot be found, the underlying psget_default() panics.
*/

extern uint16 psget_default_uint16(pskey key);

/*
 * If I recall correctly the underlying PS is defined so that each key is
 * one or more uint16s. In case this changes, the following macros are
 * defined. Use the correct one for the type you want.
 */

#define psget_default_bool(key) ((bool) psget_default_uint8(key))
#define psget_default_uint8(key) ((uint8) psget_default_uint16(key))
#define psget_default_int8(key) ((int8) psget_default_uint8(key))
#define psget_default_int16(key) ((int16) psget_default_uint16(key))


/****************************************************************************
NAME
	psget_default_uint32  -  get a uint32 pskey

RETURNS
	The value of the specified key in the default selection of stores.

NOTES
	If the key cannot be found, the underlying psget_default() panics.
*/

extern uint32 psget_default_uint32(pskey key);

/* More type compatability functions */

#define psget_default_int32(key) ((int32) psget_default_uint32(key))
#define psget_default_TIME(key) ((TIME) psget_default_uint32(key))



/****************************************************************************
NAME
	psbc_bootmode_init1  -  stage 1 initialisation of psbc_bootmode
*/
extern void psbc_bootmode_init1(void) ;

/****************************************************************************
NAME
	psbc_bootmode_init2  -  stage 2 initialisation of psbc_bootmode
*/
extern void psbc_bootmode_init2(void) ;

/****************************************************************************
NAME
	psbc_read_initial_bootmode  -  retrieve the bootmode from PS

NOTES
        It may seem odd to do this in a dedicated function but this is the
        only key which needs to be read before bootmode mapping can take 
        place.
*/

extern void psbc_read_initial_bootmode(uint16 * mode);
#endif	/* __PSBC_H__ */
