/****************************************************************************
FILE
        psf.h  -  factory-set values in the persistent store

CONTAINS
	init_psf  -  initialise the persistent storage library
	psf_factory_set  -  establish pskey values for psf_factory_restore
	psf_factory_restore  -  restore a set of pskey values
	psf_get  -  read a value from the persistent store
	psf_set  -  write a value to the persistent store
	psf_clr  -  remove a value from the persistent store
	psf_size  -  find size of value stored under a key
    psf_memory_type  -  determine underlying memory type for persistent store
	psf_match  -  compare persistent store value
        psf_next_key  -  find next higher key in the factory-set ps
        psf_clear_all  -  remove all values from the factory-set ps

DESCRIPTION
	This subsystem's two main functions are psf_factory_set() and
	psf_factory_restore().  The first effectively takes a snapshot of the
	state of the ps store - this will normally be called as part of the
	manufacturing process.  The second function attempts to restore the
	ps to the state it was in when psf_factory_set() was called - this
	attempts to return the ps back to the state it was in when
	psf_factory_set() was called.

	The factory-set store acts as a separate store of ps information.
	It is shaped very like psi.h and psrom.h stores, however it takes
	a slab of the psi.h storage to hold its values.  Values in the
	psf.h store are signified by having a high pskey bit set to 1
	(mask PS_FACTORY_SET).

	The system's main psget() function (a macro mapping to something like
	psbc_get()) searches, in sequence, the normal ps store (psi.h), the
	factory-set ps store (psf.h - this file) then the psrom store
	(psrom.h).  A psget( PSKEY_BDADDR ) will thus:

	    Look for a value stored under key PSKEY_BDADDR in psi.h.

	    If this fails it looks in psf.h, which seeks a value stored under
	    key ( PSKEY_BDADDR | PS_FACTORY_SET ).

	    If this fails it looks in psrom.h under PSKEY_BDADDR.

	Values are placed in the factory-store with the function
	psf_factory_set().  This moves every value stored under psi.h to
	new key (key|PS_FACTORY_SET).  This effectively takes a snapshot of
	the system's non-psrom ps values, but psget() calls will behave as
	before the snapshot.

	Subsequent calls to psset() will then write to the normal psi.h
	store, so overlaying (masking) any psf.h values.  This may occur in a
	user device - e.g., the local name or BT address may be changed.

	The user device may get into a mess.  If so a call to
	psf_factory_restore() may help untangle the mess.  This removes all
	psi.h values that mask a corresponding psf.h value, so unmasking
	the factory-set values.  An option to this function discards
	all ps values that have no factory-set bit.

MODIFICATION HISTORY
	1.1   10:oct:00	 cjo	Created.
	1.5   25:jun:01  at	Added psf_match().
	1.6   28:jan:02  at	Added psf_clear_all().
	1.7   06:jun:03  at	B-598: Added psf_memory_type().
*/

#ifndef __PSF_H__
#define __PSF_H__
 
#ifdef	RCS_STRINGS
static  char    psf_h_id[]
  = "$Id: //depot/bc/main/dev/src/csr/psf/psf.h#7 $";
#endif

#include	"bt.h"

/* Not nice - but we need the typedef of pskey. */
#include	"ps.h"		


/****************************************************************************
NAME
	init_psf  -  initialise the persistent storage library

FUNCTION
	Initialises the factory-store persistent storage implementation.

	This function must be called before all others in this file.
*/

/* extern void init_psf( void );  Stub away. */
#define	init_psf()


/****************************************************************************
NAME
	psf_factory_set  -  establish pskey values for psf_factory_restore

FUNCTION
	Marks each key value in the psi store as being the "factory set"
	value.  This allows a subsequent call to psf_factory_restore() to
	restore these keys' values.

RETURNS
	TRUE if the conversion was performed, else FALSE.  The function may
	return FALSE from pmalloc() exhaustion or from filling the ps.
*/

extern bool psf_factory_set( void );


/****************************************************************************
NAME
	psf_factory_restore  -  restore a set of pskey values

FUNCTION
	If "all" is TRUE this removes all values from the ps except "factory
	set" values.  This should restore the whole of the ps to the state
	it was in when psf_factory_set() was last called.

	If "all" is FALSE this removes every value from the ps that has a
	corresponding "factory set" value or psrom value.  This should
	restore the whole of the ps to the state it was in when
	psf_factory_set() was last called except that ps values that are not
	masking psf or psrom values are left in place, e.g. keys in the user
	block.

RETURNS
	TRUE if the removal was performed, else FALSE.  The function may
	return FALSE from filling the ps.
*/

extern bool psf_factory_restore( bool all );


/****************************************************************************
NAME
	psf_get  -  read a value from the persistent store

FUNCTION
	Internally converts "key" to its factory-set form (ORs the key with
	PS_FACTORY_SET) then attempts to read the value from the persistent
	store held under this new key, placing this into the buffer at
	"data".

	The number of bits that can be stored at each location of "data" must
	match the value returned by psi_location_size().   (Normally "data"
	will be a uint8* or a uint16*.)

	The length of the buffer at "data" must be at least maxlen locations
	long.  The actual length of the data written into "data" is written
	into *len.

	It is acceptable for a data item to be of zero size.

RETURNS
	TRUE if the data item was found in the store and its size was did not
	exceed "maxlen", else FALSE.

	If "data" is not NULL then the value of the data item is written
	into the buffer at "data".

	If "len" is not NULL then the length of the data item is written into
	"len".

	If the function returns FALSE the memory at "data" and "len" are not
	changed.

NOTE
	This function is not thread safe.  The persistent store may be made
	inaccessible by a call to psf_set(), so it is highly advisable to call
	psf_set() and psf_get() from the same thread.   In practice these
	functions should really be called from the system's background code
	only.
*/

extern bool psf_get( pskey key, void *data, uint16 maxlen, uint16 *len );


/****************************************************************************
NAME
	psf_set  -  write a value to the persistent store

FUNCTION
	Internally converts "key" to its factory-set form (ORs the key with
	PS_FACTORY_SET) then attempts to write the "len" locations of data
	from "data" to the persistent store and records the stored data under
	the new pskey.  This will overwrite any previous data stored under
	that key, i.e. the next call to psf_get() will obtain the fresh
	data.

	The value of "key" is almost arbitrary.  It must must not be 0x0000 or
	0xffff.  (There may be other unacceptable values - this depends on
	the value returned by flash_erase_value() - but for almost any system
	these are the only two unacceptable values for "key".
	flash_erase_value() is described in psfl.h.)  "key" must also not
	be a value with PS_FACTORY_SET at 1, else this function is a
	nonsense.

	The number of bits in each location of the buffer at "data" is the
	value returned by ps_location_size().   (Normally "data" will be a
	uint8* or a uint16*.)

	It is acceptable for "len" to be zero.

	Writing to the persistent store will normally take a significant
	period, and this function blocks interrupts during the write.

	An error report will be generated if the persistent store's memory
	has filled.  This error report will typically be an hci "hardware
	failure event".

	Some implementations (eeprom?) of this function may fail if the size
	of the data exceeds the value last stored under the same "key".

RETURNS
	TRUE if the value was written to the store, else FALSE.

	The function returns FALSE if there are insufficient resources to
	write the value, or if any parameter is unacceptable.

NOTES
	This function is not thread safe.  The persistent store may be made
	inaccessible by a simultaneous call to psf_set(), so it is highly
	advisable to call psf_set() and psf_get()) from the same thread.   In
	practice these functions should really be called from the system's
	background code only.

	This function is intended to support psf_factory_set().  Any other
	use is probably a mistake.
*/

extern bool psf_set( pskey key, void *data, uint16 len );


/****************************************************************************
NAME
	psf_clr  -  remove a value from the persistent store

FUNCTION
	Internally converts "key" to its factory-set form (ORs the key with
	PS_FACTORY_SET) then attempts to remove the data item stored under
	that key from the persistent store.

	If a call to psf_clr() succeeds then a subsequent call to psf_get()
 	for the same key would fail.

RETURNS
	TRUE if a data item stored under the identifier "key" was removed
	from the store, else FALSE.

NOTE
	This function is provided for completeness.  Any use is probably a
	mistake.
*/

extern bool psf_clr( pskey key );


/****************************************************************************
NAME
	psf_size  -  find size of value stored under a key

FUNCTION
	Internally converts "key" to its factory-set form (ORs the key with
	PS_FACTORY_SET) then attempts to determine the length of the data
	item stored under that key in the persistent store.

RETURNS
	TRUE if the data item was located, else FALSE.  

	If the function returns TRUE and if len is not NULL, the length of
	the data item is written to *len.

	If the key identifies a data item of length zero then the function
	returns TRUE and *len (if len is not NULL) is set to zero.

	If the function returns FALSE the memory at "len" is not changed.

NOTE
	No attempt has been made to make this function thread safe.
*/

extern bool psf_size( pskey key, uint16 *len );


/****************************************************************************
NAME
    psf_memory_type  -  determine underlying memory type for persistent store

RETURNS
    The type of memory underlying the factory store.

    It should not normally be necessary to use this function, as the
    persistent store libraries are deliberately designed to provide
    compatible functionality regardless of the underlying storage mechanism.
    However, it may be useful for verifying that the software has selected
    the expected store, or to improve performance.
*/

extern psmemory psf_memory_type( void );


/****************************************************************************
NAME
	psf_match  -  compare persistent store value

RETURNS
	TRUE if the persistent store holds an entry stored under "key" of
	size "len" that exactly matches the data stored at "data", else
	FALSE.
*/

extern bool psf_match( pskey key, void *data, uint16 len );


/****************************************************************************
NAME
        psf_next_key  -  find next higher key in the factory-set ps

FUNCTION
        This function deals only with the values of keys in the factory set
	persistent store - it ignores the data stored with keys.

	Internally converts "key" to its factory-set form (ORs the key with
	PS_FACTORY_SET).  It then, notionally, numerically sorts all of the
	keys in the store into ascending order and returns the first key that
	that is greater than the converted "key".

        If "key" is PSKEY_NULL then the lowest valued key in the store is
        returned.

RETURNS
	The next key in the store with value greater than the converted form
	of "key", or PSKEY_NULL if there is no such key.

NOTE
	This function is almost certainly useless.
*/

extern pskey psf_next_key( pskey key );


/****************************************************************************
NAME
        psf_clear_all  -  remove all values from the factory-set ps

FUNCTION
        Removes all values from the psf.h store.
*/

extern void psf_clear_all( void );


#endif	/* __PSF_H__ */
