/****************************************************************************
FILE
    psi.h  -  persistent storage implementation

CONTAINS
    init_psi1  -  first stage initialisation before pmalloc
    init_psi2  -  second stage initialisation after pmalloc
    psi_is_writable  -  is the implementation persistent store modifiable
    psi_defrag  -  defragment persistent store if appropriate
    psi_flood  -  fill persistent store to force defrag on reboot
    psi_location_size  -  how big is each locn of the persistent store
    psi_memory_type  -  determine underlying memory type for persistent store
    psi_get  -  read a value from the persistent store
    psi_set  -  write a value to the persistent store
    psi_clr  -  remove a value from the persistent store
    psi_size  -  find size of value stored under a key
    psi_match  -  compare persistent store value
    psi_next_key  -  find next higher key in the store
    psi_free_space  -  how much unused capacity has the store
    psi_clear_all  -  remove all values from the ps

DESCRIPTION
    The persistent storage implementation library may be built using many
    types of non-volatile storage. This library abstracts one or more
    potential implementations, e.g. using flash or EEPROM, so that client
    code does not need to know the implementation being used. The
    implementation may be selected at either compile-time (by including
    just a single implementation) or at run-time (by detecting the
    available hardware).

    At the time of writing there are two implementations, in ../psfl and
    ../pse2, but alternative implementations could be easily added.
    Support for these implementations is controlled by the INSTALL_FLASH
    and INSTALL_EEPROM macros. If only one of these is defined then the
    implementation is hard-coded at compile time, otherwise a run-time
    decision is made:

        if (known flash detected)
            use 'psfl'
        else if (known EEPROM detected)
            use 'pse2'
        else
            use 'psfl' in read-only mode

    Different implementations of this interface will have their own
    characteristics. For example, a flash implementation will use storage
    storage memory in a transaction-log manner. When the area of unused
    flash is exhausted, the system must take abortive action. This means
    that code using this interface must take care not to write many *times*
    to the store.

    An EEPROM implementation is different - this typically has much less
    memory, but it is also much slower. Also, resizing items may require
    a defrag mechanism which is likely to further slow writes.

MODIFICATION HISTORY
    1.1   14:sep:00     cjo    Created from a copy of ps.h.
    1.2   14:spe:00     cjo    Removed parameter from psi_clear_all.
    1.3   14:feb:01     at     Made defrag externally visible to allow use
                               use by dfu.
    1.4   13:mar:01     cjo    Added psi_match().
    1.5   18:may:01     at     Added psi_flood().
    1.6   28:jan:02     at     Added length parameter to psi_free_space()
                               and added psi_is_writable().
    1.7   27:jul:02     at     H16.16: Added init_ps1() and init_ps2().
    1.8   06:jun:03     at     B-598: Added psi_memory_type().
    1.9   14:aug:03     at     B-848: Void cast PSI_IMPLEMENTATION_NONE.
    #10   03:dec:03     at     B-1544: Allow build without any implementation.
*/

#ifndef __PSI_H__
#define __PSI_H__
 
#ifdef  RCS_STRINGS
static  char    psi_h_id[]
  = "$Id: //depot/bc/main/dev/src/csr/psi/psi.h#10 $";
#endif


#include "bt.h"
#include "ps.h" /* for 'pskey' typedef */


/* Use a thin veneer for a single (or no) implementation */
#if defined(INSTALL_FLASH) && !defined(INSTALL_EEPROM)
#include "psfl.h"
#define PSI_IMPLEMENTATION psfl
#elif !defined(INSTALL_FLASH) && defined(INSTALL_EEPROM)
#include "pse2.h"
#define PSI_IMPLEMENTATION pse2
#elif !defined(INSTALL_FLASH) && !defined(INSTALL_EEPROM)
#define PSI_IMPLEMENTATION psnone
#endif

/* Simplify forwarding to a particular implementation */
#define PSI_PASTE(x, y) PSI_PASTE2(x, y)
#define PSI_PASTE2(x, y) x ## _ ## y
#define PSI_IMPLEMENTATION_FUNC(x) PSI_PASTE(PSI_IMPLEMENTATION, x)
#define PSI_IMPLEMENTATION_VOID ((void) 0)


/****************************************************************************
NAME
    init_psi1  -  first stage initialisation before pmalloc

FUNCTION
    Initialise the persistent storage implementation. This function must be
    called before all others in this file, and before initialisation of the
    pmalloc subsystem. This must be followed by a call to init_psi2().
*/
#ifdef PSI_IMPLEMENTATION
#ifdef INSTALL_EEPROM
#define init_psi1() init_pse2_1();
#elif defined(INSTALL_FLASH)
#define init_psi1() init_psfl();
#else
#define init_psi1() PSI_IMPLEMENTATION_VOID
#endif
#else
extern void init_psi1(void);
#endif


/****************************************************************************
NAME
    init_psi2  -  second stage initialisation after pmalloc

FUNCTION
    Initialise the persistent storage implementation. This function must be
    called after the pmalloc subsystem has been initialised, but before
    buffer memory is used.
*/
#ifdef PSI_IMPLEMENTATION
#ifdef INSTALL_EEPROM
#define init_psi2() init_pse2_2();
#else
#define init_psi2() PSI_IMPLEMENTATION_VOID
#endif
#else
extern void init_psi2(void);
#endif


/****************************************************************************
NAME
    psi_is_writable  -  is the implementation persistent store modifiable

RETURNS
   TRUE if the persistent store is writable, otherwise FALSE.

   The store may be read-only, for example if the psfl subsystem is
   being used on a mask ROM part. This could be useful to support
   custom persistent store defaults in different mask versions without
   requiring a recompile of the firware (as would be required for
   changes to psrom).
*/
#ifdef PSI_IMPLEMENTATION
#define psnone_is_writable() (FALSE)
#define psi_is_writable() PSI_IMPLEMENTATION_FUNC(is_writable)()
#else
extern bool psi_is_writable(void);
#endif


/****************************************************************************
NAME
    psi_defrag  -  defragment persistent store if appropriate

FUNCTION
    Perform any defragmentation of the persistent store that may be
    required. This would normally only be called by init_psi() during
    initialisation, but may also be used to force the defrag operation
    to occur at other times.
    
    For persistent store implementations that do not require any
    defragmentation this function returns immediately.
*/
#ifdef PSI_IMPLEMENTATION
#define psnone_defrag() PSI_IMPLEMENTATION_VOID
#define psi_defrag() PSI_IMPLEMENTATION_FUNC(defrag)()
#else
extern void psi_defrag(void);
#endif


/****************************************************************************
NAME
    psi_flood  -  fill persistent store to force defrag on reboot

FUNCTION
    Write a dummy persistent store key entry to force a defragment
    on the next reboot.

    This is intended primarily to support automated testing.

RETURNS
    TRUE if the store was left in a state that would require a defrag,
    else FALSE.

    The function returns FALSE if a flash write failed to verify, but
    not if the persistent store page is corrupt since that would
    trigger a defrag itself.

NOTE
    This function is not thread safe. The persistent store may be made
    inaccessible by a simultaneous call to psi_set(), so it is highly
    advisable to call psi_set() and psi_flood()) from the same thread.
    In practice these functions should really be called from the system's
    background code only.
*/
#ifdef PSI_IMPLEMENTATION
#define pse2_flood() (FALSE)
#define psnone_flood() (FALSE)
#define psi_flood() PSI_IMPLEMENTATION_FUNC(flood)()
#else
extern bool psi_flood(void);
#endif


/****************************************************************************
NAME
    psi_location_size  -  how big is each locn of the persistent store

FUNCTION
    Most processors have byte-addressable data memory: each address
    accesses one byte. Alas, not all machine architectures are like
    this; one notorious processor makes a 16 bit word accessible at each
    data address. This irregularity can apply to memory used by the
    persistent storage library.
    
    This function returns the number of bits of memory accessible at each
    addressable data location of the local machine's persistent storage.

RETURNS
    The number of data bits at each accessible location of the local
    machine's persistent storage memory.
*/
#ifdef PSI_IMPLEMENTATION
#define psnone_location_size() (0)
#define psi_location_size() PSI_IMPLEMENTATION_FUNC(location_size)()
#else
extern uint16 psi_location_size(void);
#endif


/****************************************************************************
NAME
    psi_memory_type  -  determine underlying memory type for persistent store

RETURNS
    The type of memory underlying the implementation store.

    It should not normally be necessary to use this function, as the
    persistent store libraries are deliberately designed to provide
    compatible functionality regardless of the underlying storage mechanism.
    However, it may be useful for verifying that the software has selected
    the expected store, or to improve performance.
*/
#ifdef PSI_IMPLEMENTATION
#define psnone_memory_type() (psmemory_none)
#define psi_memory_type() PSI_IMPLEMENTATION_FUNC(memory_type)()
#else
extern psmemory psi_memory_type(void);
#endif


/****************************************************************************
NAME
    psi_get  -  read a value from the persistent store

FUNCTION
    Reads the last value written into the persistent store under
    identifier "key" and copies this into the buffer at "data".
    
    The number of bits that can be stored at each location of "data" must
    match the value returned by psi_location_size(). (Normally "data"
    will be a uint8* or a uint16*.)
    
    The length of the buffer at "data" must be at least maxlen locations
    long. The actual length of the data written into "data" is written
    into *len.
    
    It is acceptable for a data item to be of zero size.

RETURNS
    TRUE if the data item was found in the store and its size was did not
    exceed "maxlen", else FALSE.

    If "data" is not NULL then the value of the data item is written
    into the buffer at "data".
    
    If "len" is not NULL then the length of the data item is written into
    "len".
    
    If the function returns FALSE the memory at "data" and "len" are not
    changed.

NOTES
    This function is not thread safe. The persistent store may be made
    inaccessible by a call to psi_set(), so it is highly advisable to call
    psi_set() and psi_get() from the same thread. In practice these
    functions should really be called from the system's background code
    only.
    
    An early version of this function looked first in the normal ps then,
    if that drew a blank, looked in the psrom. This functionality has
    been removed.
*/
#ifdef PSI_IMPLEMENTATION
#define psnone_get(key, data, maxlen, len) (FALSE)
#define psi_get(key, data, maxlen, len) \
    PSI_IMPLEMENTATION_FUNC(get)((key), (data), (maxlen), (len))
#else
extern bool psi_get(pskey key, void *data, uint16 maxlen, uint16 *len);
#endif


/****************************************************************************
NAME
    psi_set  -  write a value to the persistent store

FUNCTION
    Copies "len" locations of data from "data" to the persistent store
    and records the stored data under the access identifier "key". This
    will overwrite any previous data stored under that key, i.e. the next
    call to psi_get() will obtain the fresh data.
    
    The value of "key" is arbitrary, but it must not be 0x0000 or 0xffff.
    (There may be other unacceptable values - this depends on the value
    returned by psflbc01_erase_value() - but for almost any system these
    are the only two unacceptable values for "key". psflbc01_erase_value()
    is described in psflbc01.h.)
    
    The number of bits in each location of the buffer at "data" is the
    value returned by psi_location_size(). (Normally "data" will be a
    uint8* or a uint16*.)
    
    It is acceptable for "len" to be zero.
    
    Writing to the persistent store will normally take a significant
    period, and this function blocks interrupts during the write.
    
    An error report will be generated if the persistent store's memory
    has filled. This error report will typically be an hci "hardware
    failure event".
    
    Some implementations of this function may fail if the size of the data
    exceeds the value last stored under the same "key".

RETURNS
    TRUE if the value was written to the store, else FALSE.

    The function returns FALSE if there are insufficient resources to
    write the value, or if any parameter is unacceptable.
    
NOTE
    This function is not thread safe. The persistent store may be made
    inaccessible by a simultaneous call to psi_set(), so it is highly
    advisable to call psi_set() and psi_get()) from the same thread. In
    practice these functions should really be called from the system's
    background code only.
*/
#ifdef PSI_IMPLEMENTATION
#define psnone_set(key, data, len) (FALSE)
#define psi_set(key, data, len) \
    PSI_IMPLEMENTATION_FUNC(set)((key), (data), (len))
#else
extern bool psi_set(pskey key, void *data, uint16 len);
#endif


/****************************************************************************
NAME
    psi_clr  -  remove a value from the persistent store

FUNCTION
    Attempts to remove a data item stored under the identifier "key"
    from the persistent store.

    If a call to psi_clr() succeeds then a subsequent call to psi_get()
    for the same key would fail.

RETURNS
    TRUE if a data item stored under the identifier "key" was removed
    from the store, else FALSE.
*/
#ifdef PSI_IMPLEMENTATION
#define psnone_clr(key) (FALSE)
#define psi_clr(key) PSI_IMPLEMENTATION_FUNC(clr)(key)
#else
extern bool psi_clr(pskey key);
#endif


/****************************************************************************
NAME
    psi_size  -  find size of value stored under a key

FUNCTION
    Finds whether any data is stored under the given persistent store
    key, and if so determines it's length.

RETURNS
    TRUE if the data item was located, else FALSE.

    If the function returns TRUE and if len is not NULL, the length of
    the data item is written to *len.
    
    If the key identifies a data item of length zero then the function
    returns TRUE and *len (if len is not NULL) is set to zero.

    If the function returns FALSE the memory at "len" is not changed.
        
NOTE
    No attempt has been made to make this function thread safe.
*/
#ifdef PSI_IMPLEMENTATION
#define psnone_size(key, len) (FALSE)
#define psi_size(key, len) PSI_IMPLEMENTATION_FUNC(size)((key), (len))
#else
extern bool psi_size(pskey key, uint16 *len);
#endif


/****************************************************************************
NAME
    psi_match  -  compare persistent store value

RETURNS
    TRUE if the persistent store holds an entry stored under "key" of
    size "len" that exactly matches the data stored at "data", else
    FALSE.
*/
#ifdef PSI_IMPLEMENTATION
#define psnone_match(key, data, len) (FALSE)
#define psi_match(key, data, len) \
    PSI_IMPLEMENTATION_FUNC(match)((key), (data), (len))
#else
extern bool psi_match(pskey key, void *data, uint16 len);
#endif


/****************************************************************************
NAME
    psi_next_key  -  find next higher key in the persistent store

FUNCTION
    This function deals only with the values of keys in the persistent
    store - it ignores the data stored with keys.

    Notionally the function numerically sorts all of the keys in the
    store into ascending order and returns the first key that that is
    greater than "key".
    
    If "key" is PSKEY_NULL then the lowest valued key in the store is
    returned.

RETURNS
    The next key in the store with value greater than "key", or
    PSKEY_NULL if there is no such key.
*/
#ifdef PSI_IMPLEMENTATION
#define psnone_next_key(key) (PSKEY_NULL)
#define psi_next_key(key) PSI_IMPLEMENTATION_FUNC(next_key)(key)
#else
extern pskey psi_next_key(pskey key);
#endif


/****************************************************************************
NAME
    psi_free_space  -  how much unused capacity has the store

FUNCTION
    Determines how much many keys of size "len" that can be written to
    the persistent store. This takes into account any overhead
    associated with each value written.

RETURNS
    The number of keys that can be written.
*/
#ifdef PSI_IMPLEMENTATION
#define psnone_free_space(len) (0)
#define psi_free_space(len) PSI_IMPLEMENTATION_FUNC(free_space)(len)
#else
extern uint16 psi_free_space(uint16 len);
#endif


/****************************************************************************
NAME
    psi_clear_all  -  remove all values from the ps

FUNCTION
    Removes all values from the psi.h store.
*/
extern void psi_clear_all(void); /* Always implemented by 'psi' */


#endif
