/****************************************************************************
FILE
        psv.h  -  virtual persistent store subsystem

CONTAINS
	init_psv  -  initial the virtual persistent store
	psv_get  -  get a value from the virtual persistent store
	psv_set  -  write a value to the virtual persistent store
	psv_clr  -  remove a value from the virtual persistent store
	psv_size  -  how big is a value in the virtual persistent store
	psv_location_size  -  how big is each location of the store
        psv_next_key  -  find next higher key in the store
        psv_free_space  -  how much unused capacity has the store
        psv_is_virtual  -  is a key virtual

DESCRIPTION
	The virtual persistent store is layered over the normal store,
	defined in ps.h, and over the psrom store, defined in psrom.h.

	The psv allows "virtual" persistent store values to be formed from
	parts of other persistent store values - the virtual keys use (part
	of) the underlying (usually larger normal) ps values.

	Example problem:

	    Martin Cooper wrote the first version of hostio.  Since the
	    hostio was the only part of the system that used the uart at the
	    time, it made perfect sense for his code to control the uart.  He
	    held uart initialisation information in the ps.  Unfortunately,
	    he held it in a single ps struct that was also used to configure
	    bcsp.

	    We now have three potential users of the uart - bcsp, h4 and user
	    code - so it makes sense to try to split the uart config ps info
	    from the bcsp config ps info.  This would be easy, except that
	    lots of frisbees have been sent to customers, and that some host
	    config tools know about the original large structure.

	    The vps allows Martin's single large struct to appear to be a set
	    of smaller elements - separate pskeys for the uart baud rate and
	    settings, etc.

	When a call is made into the vps the key is matched against the set
	of virtual pskeys.  If no match is made the call is passed straight
	through to the normal corresponding ps.h function.  If the vps
	recognises the key as being virtual it makes a re-entrant call to
	obtain the key on which the virtual key is based, then reads or
	read/modify/writes the underlying key as needed.

	The virtual key mechanism requires these restraints on the general
	ps interface:

	  - All virtual pskeys must be of fixed size.

	  - All virtual pskeys must be layered over a pskey that is of fixed
	    size.

	  - All virtual pskeys must be layered over a pskey that has a
	    (default) value in psrom.  (Setting a virtual key does a
	    read/modify/write on the underlying key.  If there is no value in
	    psrom we can't sensibly make up values for the portions of the
	    underlying key that are not set by the virtual key.)

MODIFICATION HISTORY
	1.1   13:sep:00	 cjo	Created.
*/

#ifndef	__PSV_H__
#define	__PSV_H__
 
#ifdef RCS_STRINGS
static char psv_h_id[]
  = "$Id: //depot/bc/main/dev/src/csr/psv/psv.h#2 $";
#endif

#include "ps.h"


/* The storage of a virtual pskey, carved from the storage of an
underlying pskey.  The "key" is the virtual pskey which identifies a
fixed-size stored data block.  The "offset" and "size" carve storage
from the underlying pskey.  An array of PSVKEYs, terminated by an element
with a "key" of 0, is pointed to by the "vkeys" entry in a PSVKEYBASE. */

typedef struct {
	pskey		key;	/* Virtual pskey. */
	uint16		offset;	/* of virtual storage in underlying key. */ 
	uint16		size;	/* Size of storage for virtual key. */
	} PSVKEY;


/* A set of virtual keys in "vkeys" layered over the fixed sized storage
("size") of the underlying "key". */

typedef struct {
	pskey		key;	/* Underlying key. */
	uint16		size;	/* Size of storage for underlying key. */
	const PSVKEY	*vkeys;	/* Virtual pskey storage. */
	} PSVKEYBASE;


/****************************************************************************
NAME
	init_psv  -  initial the virtual persistent store

FUNCTION
	Initialise the virtual persistent store.

	This calls init_ps().

	This must be called before any other function defined in this file.
*/

extern void init_psv( void );


/****************************************************************************
NAME
	psv_get  -  get a value from the virtual persistent store

FUNCTION
	The "stores" parameter determines where this call looks for data and
	the order in which the sources are searched.

	The function writes into the buffer at "data" the value last written
	into the persistent store with identifier "key".

	The number of bits stored at each location at "data" must match the
	value returned by psbc_location_size().   (Typically, "data" will
	really be a uint8* or a uint16*.)

	The length of the buffer at "data" must be at least "maxlen"
	locations long.  The actual length of the data written into "data" is
	written into *len.

	It is acceptable for a data item to be of zero length.

RETURNS
	TRUE if the data item was located, if the value of the data item was
	written into the buffer at "data" and its length at "len", else
	FALSE.

	The function returns FALSE if a value for "key" is not held in the
	store.

	The function returns FALSE if the stored data exceeds size maxlen.

	If the function returns FALSE the memory at "data" and "len" are not
	changed.

NOTE
        This function is not thread safe.  The persistent store may be made
        inaccessible by a call to psv_set(), so it is highly advisable to
        call psv_set() and psv_get() from the same thread.   In practice
        these functions should really be called from the system's background
        code only.
*/

extern bool psv_get( pstores stores, pskey key, void *data, uint16 maxlen,
		uint16 *len );


/****************************************************************************
NAME
	psv_set  -  write a value to the virtual persistent store

FUNCTION
        Copies "len" locations of data from "data" to the persistent store
        and records the stored data under the access identifier "key".  This
        will overwrite any previous data stored under that key, i.e. the next
        call to psbc_get() will obtain the fresh data.

        The value of "key" is arbitrary, but it must not be 0x0000 or 0xffff.
        (There may be other unacceptable values - this depends on the value
        returned by flash_erase_value() - but for almost any system these are
        the only two unacceptable values for "key".  flash_erase_value() is
        described in psfl.h.)

        The number of bits in each location of the buffer at "data" is the
        value returned by psbc_location_size().   (Typically, "data" will
        really be a uint8* or a uint16*.)

        It is acceptable for "len" to be zero.

        Writing to the persistent store will normally take a significant
        period, and this function blocks interrupts during the write.

        The function will be emit an error report (fault()) if the
        persistent store's memory is perceived to have filled or failed.

        Some implementations (eeprom?) of this function may fail if the size
        of the data exceeds the value last stored under the same "key".

RETURNS
        TRUE if the value was written to the store, else FALSE.

        The function returns FALSE if there are insufficient resources to
        write the value or if any parameter is unacceptable.

NOTES
        This function is not thread safe.  The persistent store may be made
        inaccessible by a simultaneous call to psbc_set(), so it is highly
        advisable to call psbc_set() (and psbc_get()) from the same thread.
        In practice these functions should really be called from the system's
        background code only.

	This function does not access the psrom store.
*/

extern bool psv_set( pskey key, void *data, uint16 len );


/****************************************************************************
NAME
	psv_clr  -  remove a value from the virtual persistent store

FUNCTION
        Attempts to remove a data item stored under the identifier "key"
        from the persistent store.

	If a call to psv_clr() succeeds then a subsequent call to psv_get()
	using the same key would fail.

RETURNS
        TRUE if a data item stored under the identifier "key" was removed
        from the store, else FALSE.

BUG
	If "key" is a virtual key then this function will fail, i.e. this
	function can only be used to remove normal ps.h keys.

NOTE
	This function does not access the psrom store.
*/

extern bool psv_clr( pskey key );


/****************************************************************************
NAME
	psv_size  -  how big is a value in the virtual persistent store

FUNCTION
        Finds whether any data is stored under the given persistent store
        "key", and if so determines it's length.

	The function searches each of the stores defined by "stores" in
	turn and returns information on the first match of "key".

RETURNS
        TRUE if the data item was found in the ps, else FALSE.

        If the function returns TRUE and if "len" is not NULL then the
        length of the value is stored at *len.

        If the "key" is identifies zero length data then the function returns
        TRUE and *len is set to zero.

        If the function returns FALSE the memory at "len" is not changed.

NOTE
        No attempt has been made to make this function thread safe.
*/

extern bool psv_size( pstores stores, pskey key, uint16 *len ); 


/****************************************************************************
NAME
	psv_location_size  -  how big is each location of the store

FUNCTION
        Most processors have byte-addressable data memory: each address
        accesses one byte.  Alas, some machine architectures make different
        amounts of memory accessible at each address.  This can apply to
        memory used by the persistent storage library.

        This function returns the number of bits of memory accessible at each
        addressable data location of the local machine's persistent storage
	- presumed to be the same for the ps.h and psrom.h subsystems.

RETURNS
        The number of data bits at each accessible location of the local
        machine's persistent storage memory.
*/

/* Macro - fall back on ps.h function.
extern uint16 psv_location_size( void ); */

#define psv_location_size()    ps_location_size()


/****************************************************************************
NAME
        psv_next_key  -  find next higher key in the store

FUNCTION
	The "stores" parameter determines where this call looks for keys and
	the order in which the sources are searched.

        This function deals only with the values of keys in the persistent
        store - it ignores the data stored with keys.

	Notionally the function numerically sorts all of the keys in the
	store(s) into ascending order and returns the first key that that is
	greater than "key".

        If "key" is 0 then the lowest valued key in the store is returned.

RETURNS
        The next key in the store with value greater than "key", or 0 if
        there is no such key.
*/

extern pskey psv_next_key( pstores stores, pskey key );  


/****************************************************************************
NAME
        psv_free_space  -  how much unused capacity has the store

FUNCTION
	Determines how much ununsed capacity remains in the persistent
	store(s) specified by "stores".  This deals with gross capacity -
	some capacity is consumed by each value written.

RETURNS
        The size of the unused space in the persistent store.

NOTE
	The psrom subsystem is always full.
*/

/* Macro - fall back on ps.h function.
extern uint16 psv_free_space( pstores stores ); */

#define psv_free_space(s)	ps_free_space(s)


/****************************************************************************
NAME
	psv_is_virtual  -  is a key virtual

RETURNS
	TRUE if "key" refers to a virtual key, else FALSE.
*/

extern bool psv_is_virtual( pskey key );


#endif	/* __PSV_H__ */
