///////////////////////////////////////////////////////////////////////
//
//  FILE   :  HCIDeviceController.h
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  HCIDeviceController
//
//  PURPOSE:  Provide a C++ object API.
//            Advertise the services of the HCIDeviceController dll.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/HCIDeviceController.h#22 $
//
///////////////////////////////////////////////////////////////////////

#ifndef __HCIDEVICECONTROLLER_H__
#define __HCIDEVICECONTROLLER_H__

#include "HCI/HCILinkage.h"
#include "HCITransport.h"
#include "thread/signal_box.h"

#if defined _WIN32 && defined _WINCE
#define HCI_TRANS_CALLING_CONVENTION __stdcall
#else
#define HCI_TRANS_CALLING_CONVENTION
#endif 

extern "C"
{
#include "HCIPacker.h"
#include "packer.h"
}

class CommandHandler;
class ACLHandler;
class ConnectionList;
class CriticalSection;
class pduList;
class Threadable;

#include <stdlib.h> /* size_t */

//  Handles cannot be zero, so set the default value to 0.
#define DEFAULT_CALL_BACK_HANDLE (0x0000)

typedef struct _DataCallBackListEntry
{
    struct _DataCallBackListEntry * next;
    uint16 connectionHandle;
    void (*function) ( uint16 connectionHandle , 
                       bool packetBoundaryFlag , 
                       uint8 broadcastFlag , 
                       uint8 *dataStart , 
                       uint16 dataLength, 
                       void * parameters);
    void * parameters;
}
DataCallBackListEntry;

//  There is no zero event, so set the default value to 0.
#define DEFAULT_CALL_BACK_EVENT (0x0000)

typedef struct _EventCallBackListEntry
{
    struct _EventCallBackListEntry * next;
    uint16 eventCode;
    void (*function) ( uint8 eventCode, 
                       PARAMETER_ARRAY eventParameters , 
                       void * parameters);
    void * parameters;
}
EventCallBackListEntry;

class DEVICECONTROLLER_API_CLASS HCIDeviceController
{
public:
	HCIDeviceController ( HCITransport * aTransport = 0 , bool usingBlueStack = false );

    //  Destructor
    virtual ~HCIDeviceController();

    //  Transport related functions
    void setTransport ( HCITransport * transport );
    HCITransport * getTransport( void );
    bool isTransportReady( void );

    //  when data comes along, we send it here.  0 sets the "default" handle.
    void setCallBackOnData ( uint16 aHandle ,
                             void (HCI_TRANS_CALLING_CONVENTION *dataCallBackFn) ( uint16 connectionHandle , 
                                                      bool packetBoundaryFlag , 
                                                      uint8 broadcastFlag , 
                                                      uint8 *dataStart , 
                                                      uint16 dataLength, 
                                                      void * parameters),
                             void * dataCallBackParameters);
    void unsetCallBackOnData ( uint16 aHandle );

    //  when an event comes along, we send it here.  0 sets the "default" handle.
    void setCallBackOnEvent ( uint16 event,
                              void (HCI_TRANS_CALLING_CONVENTION *eventCallBackFn) (uint8 eventcode, 
                                                       PARAMETER_ARRAY eventParameters ,
                                                       void * parameters),
                              void * eventCallBackParameters);
    void unsetCallBackOnEvent ( uint16 event );

    //  However the threading is implemented, calling "doWork" gives
    //  processor time to the object to allow it to send, receive etc.
    //
    //  doWork will return when it has done any pending work it can do.
    void doWork( void );

    //  Wait for something to happen and then do it and wait some more...
    //  createWorker will create a worker thread which calls workerThread
    //  which calls waitForWork, which will return only when the object
    //  is deleted, but it will do all background processing that would
    //  have to be implemented by repeated calls to doWork otherwise.
    void createWorker ( void );
    static void workerThread ( void * aObject );
    //  or just give wait for work a thread...
    void waitForWork( void );

    bool sendHCICommand ( PARAMETER_ARRAY parameters );

    bool sendHCIData ( uint16 connectionHandle ,
	 	       bool packetBoundaryFlag ,
		       uint8 broadcastFlag ,
		       uint8* dataStart ,
		       uint16 dataLength );
    // return the amount flushed from that handle.
    size_t flushHCIData ( uint16 connectionHandle );

    //  additional call which returns the amount of data in bytes currently
    //  awaiting transmission in the ACL queues.
    size_t getSizeOfBufferContent ();
    size_t getSizeOfDataWaitingOnHandle ( uint16 connectionHandle );
    uint16 getNumberOfPacketsOutstandingOnHandle ( uint16 connectionHandle );

    //  Functions to get events and data direct from the queues without the call backs.
    //  they only work if the call backs aren't set.
    bool getHCIEvent ( PARAMETER_ARRAY * parameters );
    bool getHCIData  ( uint16* connectionHandle ,
	 	       bool* packetBoundaryFlag ,
		       uint8* broadcastFlag ,
		       uint8** dataStart ,
		       uint16* dataLength );

    //  Extra calls to add and get Bad pdus - ones that fail the packer unpacking routines in some way.
    bool getBadPDU ( BadPDUReason* pWhy , uint8 ** ppdu , uint32* pLength );
    void setCallBackOnBadPDU ( void (HCI_TRANS_CALLING_CONVENTION * callBackFn)( BadPDUReason aWhy ,
                                                   uint8 * apdu ,
                                                   uint32 aLength ,
                                                   void * parameters ) ,
                               void * parameters );
    static void registerBadPDU ( BadPDUReason aWhy , uint8 * apdu , uint32 aLength , void * aThis );

protected:
    bool sendHCICommand ( PARAMETER_ARRAY parameters , bool internallyGeneratedCommand );
    void onHCIEvent ( uint8 evCode , PARAMETER_ARRAY parameters );
    void onHCIData ( uint16 connectionHandle ,
		bool packetBoundaryFlag ,
		uint8 broadcastFlag ,
	        uint8 *dataStart ,
	        uint16 dataLength );

    void checkQueues ( void ) ;

    SignalBox*			mSignals ;
    SignalBox::Item*	mComSentSignal ;
    SignalBox::Item*	mEvtRecvSignal ;
    SignalBox::Item*	mACLSentSignal ;
    SignalBox::Item*	mACLRecvSignal ;
    SignalBox::Item*	mBeingDeletedSignal ;
    SignalBox::Item*	mBadPDUSignal ;

    SignalBox*			mThreadCloseDown ;
    SignalBox::Item*	mThreadExitedSignal ;
    Threadable *		mThread;

    CommandHandler * ch;
    ACLHandler * ah;
	
private:
    bool interpretEvent ( PARAMETER_ARRAY aAnswer );
	bool withBlueStack;

    ConnectionList * cl;

    class DEVICECONTROLLER_API_CLASS sco_handler_workaround
    {
    public:
        sco_handler_workaround() : sco_conns ( 0 ) , sample_size ( 16 ) {}
        void conn_sco() { sco_conns ++; }
        void disc_sco() { sco_conns --; }
        void set_sample_size( uint8 size ) { sample_size = size; }
        void set_potential_sample_size( uint8 size ) { p_sample_size = size; }
        void potential_was_good() { sample_size = p_sample_size; }
        uint32 get_bandwidth() const { return sco_conns * sample_size * 1000; }
    private:
        uint8 sample_size;
        uint8 p_sample_size;
        uint8 sco_conns;
    } sh;

    HCITransport *tr;
    bool mTransportReady;
    bool deleteTransportWithDeviceController;

    bdaddr localDevice;

    DataCallBackListEntry * mDataCallBackListStart;
    DataCallBackListEntry * mDefaultDataCallBack;
    CriticalSection * mDataCallBackProtection;
    DataCallBackListEntry * getCallBackOnData ( uint16 aHandle );
    DataCallBackListEntry * getCallBackOnData_nocs ( uint16 aHandle );

    EventCallBackListEntry * mEventCallBackListStart;
    EventCallBackListEntry * mDefaultEventCallBack;
    CriticalSection * mEventCallBackProtection;
    EventCallBackListEntry * getCallBackOnEvent ( uint16 aEventCode );
    EventCallBackListEntry * getCallBackOnEvent_nocs ( uint16 aEventCode );

    //  Common creation code to the two creators
    void creationInit ();

    void useCommandComplete ( PARAMETER_ARRAY aAnswer , bool passingOn );

    // bad pdu logging.
    pduList * BadPDUList;
    void (HCI_TRANS_CALLING_CONVENTION * BadPDUCallBackFn)( BadPDUReason aWhy , uint8 * apdu , uint32 aLength , void * parameters );
    void * BadPDUCallBackParameters;

#ifdef _INDEBUG
    //  release debugging!
public:
    indebug * mDebug;
#else
    //  keep the size of the object the same.
    void * dummy;
#endif
};

#endif // __HCIDEVICECONTROLLER_H__
