#ifndef __INCTestEngineh
#define __INCTestEngineh

#define RSSI_MAX_SAMPLE (100)
#define SETTLE_MAX (10)


#define BER_BIT_COUNT 0
#define BER_BIT_ERRORS 1
#define BER_ACCESS_CODE 2 /* Not used */
#define BER_RCVD_PKTS 3
#define BER_EXP_PKTS 4
#define BER_HDR_ERRORS 5
#define BER_CRC_ERRORS 6
#define BER_UNCORR_ERRORS 7
#define BER_SYNC_ERRORS 8
#define BER_MAX 9

#define LIBAPI(T) extern "C" T _stdcall

/*************************************************************************************
	VERSION INFORMATION
*************************************************************************************/
/*************************************************************************************
	
	Function Name : char * getTestEngineVersion(void);

	Parameters :	A pointer to a preallocated string that will have the
	                version string copied to it.

	Returns :		Always 1.

	Description :	Retrieve or displays the current version string.

*************************************************************************************/
LIBAPI(int) getTestEngineVersion(char * versionStr);

/*************************************************************************************
	LINK MANAGEMENT FUNCTIONS
*************************************************************************************/

#define BCSP 0x1
#define USB 0x2
#define H4 0x4

/*************************************************************************************
	
	Function Name : uint32 initTestEngine(int transport, char * transportDevice, unsigned long int dataRate, int usbTimeOut);

	Parameters :
					transport - defines the protocol to be used when setting up the 
								communication with BlueCore.
								USB = 2
								H4 = 4
								BCSP = 1

					transportDevice - Defines the physical port to use. A string which
								for comm ports is "COMn" where 'n' is the number of the 
								port. 

					dataRate - defines the baud rate to be used. For UART connections 
								only.

					usbTimeout - where transport = 2, this parameter will define the 
								timeout to allow the USB device to ennumerate if it
								has not already ennumerated. It has been found that some PC's 
								take a longer time than expected to ennumerate a USB device and, 
								in some cases, the PC's internal USB hub may have been reset 
								which can cause a further delay in ennumeration. This parameter 
								can be set to accomodate the extra time taken to reset USB devices.

	Returns :		This function will return a zero on failure or an unsigned integer which 
					defines the handle to the host / host controller connection. This 
					value is used as a parameter for all other function calls.

	Description :	This function has to called before any communication with BlueCore
					can begin. 

*************************************************************************************/
LIBAPI(uint32) initTestEngine(int transport, char * transportDevice, unsigned long int dataRate, int usbTimeOut);

/*************************************************************************************
	
	Function Name :	int closeTestEngine( uint32 handle);

	Parameters :
					handle - handle to the device.

	Returns :		Always 1.

	Description :	This function should always be called when terminating 
					communications with BlueCore. This function will NOT issue a reset.

					
*************************************************************************************/
LIBAPI(int) closeTestEngine( uint32 handle);

/*************************************************************************************
	
	Function Name :	uint32 getLastError(uint32 handle);

	Parameters :
					handle - handle to the device.

	Returns :		Last reported error code. If this value is zero then no error was 
					reported. A non-zero value signifies an error and the value is 
					decoded as follows:
						Byte 1 (0x000000ff) = BCCMD error
						Byte 2 (0x0000ff00) = Reserved
						Byte 3 (0x00ff0000) = HCI / DM error
						Byte 4 (0xff000000) = Reserved
					BCCMD errors are defined as:
						0 = No error
						1 = BCCMD command ID not supported
						2 = Data exceeded
						3 = Variable ahs no value
						4 = Get or set command held an error
						5 = Value inaccessable
						6 = Unwriteable
						7 = Unreadable
						8 = Other error
						9 = Request not allowed
					HCI / DM errors are defined in the Bluetooth specification.
					If the returned value is 0xffffffff then this signifies a transport 
					failure.

	Description :	This function can be called to obtain more detailed errors when a 
					function fails.

					
*************************************************************************************/
LIBAPI(uint32) getLastError(uint32 handle);

/*************************************************************************************
	
	Function Name : int bccmdSetColdReset( uint32 handle, int usbTimeout);

	Parameters :	
					handle - handle to the device.

					usbTimeout - where transport = 2, this parameter will define the 
						timeout to allow the USB device to ennumerate if it
						has not already ennumerated. It has been found that some PC's 
						take a longer time than expected to ennumerate a USB device and, 
						in some cases, the PC's internal USB hub may have been reset 
						which can cause a further delay in ennumeration. This parameter 
						can be set to accomodate the extra time taken to reset USB devices.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure.

	Description :	This command forces a hardware reset of BlueCore, deliberately 
					discarding all of its current state - the command emulates removing 
					power from the chip, then restoring it. It will attempt to 
					reinitialise communication using the same transport settings as 
					declared in initTestEngine(..). If the reset or the 
					re-establishment fails the objects associated with the 
					communication stack will be deleted.

*************************************************************************************/
LIBAPI(int) bccmdSetColdReset( uint32 handle, int usbTimeout);

/*************************************************************************************
	
	Function Name : int bccmdSetWarmReset( uint32 handle, int usbTimeout);

	Parameters :	
					handle - handle to the device.

					usbTimeout - where transport = 2, this parameter will define the 
						timeout to allow the USB device to ennumerate if it
						has not already ennumerated. It has been found that some PC's 
						take a longer time than expected to ennumerate a USB device and, 
						in some cases, the PC's internal USB hub may have been reset 
						which can cause a further delay in ennumeration. This parameter 
						can be set to accomodate the extra time taken to reset USB devices.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure.

	Description :	This forces a hardware reset of the BlueCore, arranging that 
					some elements of the chips current state may be available when 
					the chip is restarted, assuming the chip remains powered through 
					the reset. It will then attempt to reinitialise communication 
					using the same transport settings as declared in 
					initTestEngine(). If the reset or the re-establishment fails the 
					objects associated with the communication stack will be deleted.

*************************************************************************************/
LIBAPI(int) bccmdSetWarmReset( uint32 handle, int usbTimeout);

/*************************************************************************************
	MISC. TESTS
*************************************************************************************/

/*************************************************************************************
	
	Function Name :	int radiotestPause( uint32 handle);

	Parameters :	handle - handle to the device
	
	Returns :		-1 on invalid handle, 1 on success, 0 on failure.

	Description :	This function will cause any tests running to terminate.

*************************************************************************************/
LIBAPI(int) radiotestPause( uint32 handle);

/*************************************************************************************
	
	Function Name :	int radiotestDeepSleep(uint32 handle);

	Parameters :	handle - handle to the device
	
	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function will cause the DUT to enter deep sleep.

*************************************************************************************/
LIBAPI(int) radiotestDeepSleep(uint32 handle);

/*************************************************************************************
	
	Function Name :	int radiotestPcmExtLb(uint32 handle, uint16 pcm_mode);

	Parameters :	handle - handle to the device

					pcm_mode - can be
						0  Slave in normal 4-wire configuration
						1  Master in normal 4-wire configuration
						2  Master in Alcatel-specific 2-wire configuration
	
	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This command enables PCM external loopback test mode in which a 
					block of random data is written to the PCM output port and is read 
					back again on the PCM input port. A check is made that the data 
					matches. External wiring must be provided between the corresponding 
					pins. The pcm_mode parameter can take the following values:
						0 Slave in normal four-wire configuration
						1 Master in normal four-wire configuration
						2 Master in Manchester encoded, two-wire configuration
					For a normal loopback configuration pcm_mode should be 1.

*************************************************************************************/
LIBAPI(int) radiotestPcmExtLb(uint32 handle, uint16 pcm_mode);

/*************************************************************************************
	
	Function Name :	int radiotestPcmLb(uint32 handle, uint16 pcm_mode);

	Parameters :	handle - handle to the device

					pcm_mode - can be
						0  Slave in normal 4-wire configuration
						1  Master in normal 4-wire configuration
						2  Master in Alcatel-specific 2-wire configuration
	
	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This command enables PCM LOOPBACK test mode in which data read from 
					the PCM input port is written back out again via the pulse code 
					modulation (PCM) output port after a delay. The pcm_mode parameter 
					can take the following values:
						0 Slave in normal 4-wire configuration
						1 Master in normal 4-wire configuration
						2 Master in Manchester encoded, 2-wire configuration
					For a normal loopback configuration pcm_mode should be 1.

*************************************************************************************/
LIBAPI(int) radiotestPcmLb(uint32 handle, uint16 pcm_mode);

/*************************************************************************************
	
	Function Name :	int radiotestPcmTimingIn(uint32 handle, uint16 pio_out, uint16 pcm_in);

	Parameters :	handle - handle to the device

					pio_out - 0 to 7, PIO pin used for writing

					pcm_in - 0 to 1, PCM pin used for reading
	
	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This command is similar to PCM_EXT_LB: this makes a series of 
					writes to a PIO pin and tests that the result is readable on one 
					of the PCM pins used for timing in slave mode, PCM_SYNC and PCM_CLK. 
					A check is made that the data matches. External wiring must be 
					provided between the corresponding pins. The pio_out parameter can 
					take the values 0 to 7, corresponding to the PIO pin to be used 
					for output. The pcm_in parameter can take the following values:
						0 Test input on PCM_SYNC
						1 Test input on PCM_CLK

*************************************************************************************/
LIBAPI(int) radiotestPcmTimingIn(uint32 handle, uint16 pio_out, uint16 pcm_in);

/*************************************************************************************
	
	Function Name :	int radiotestPcmTone(uint32 handle, uint16 freq, uint16 ampl, uint16 dc);

	Parameters :	handle - handle to the device

					freq - controls the frequency of the sine wave. The value 0 
						corresponds to 250Hz; each increment of 1 doubles the frequency. 
						Hence, 1 corresponds to 500Hz, 2 to 1kHz and so on. No bounds 
						checking is performed on the parameter, but large values will 
						not be useful owing to hardware limitations.

					ampl - controls the amplitude of the sine wave. 8 is the full 
						volume; each decrement of 1 reduces the amplitude by a factor 
						of 2. 0 is valid and causes the hardware to be activated with 
						constant audio data.

					dc - constant offset to be added to the audio data.
	
	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This command enables PCM external loopback test mode in which a 
					block of random data is written to the PCM output port and is read 
					back again on the PCM input port. A check is made that the data 
					matches. External wiring must be provided between the corresponding 
					pins. The pcm_mode parameter can take the following values:
						0 Slave in normal four-wire configuration
						1 Master in normal four-wire configuration
						2 Master in Manchester encoded, two-wire configuration
					For a normal loopback configuration pcm_mode should be 1.

*************************************************************************************/
LIBAPI(int) radiotestPcmTone(uint32 handle, uint16 freq, uint16 ampl, uint16 dc);

/*************************************************************************************
	
	Function Name :	int radiotestCtsRtsLb(uint32 handle);

	Parameters :	handle - handle to the device

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This command performs a series of writes to the UART CTS (clear to 
					send) line and attempts to read back the value written on the UART 
					RTS (ready to send) line. External wiring must be provided between 
					the corresponding pins.

*************************************************************************************/
LIBAPI(int) radiotestCtsRtsLb(uint32 handle);

/*************************************************************************************
	
	Function Name :	int radiotestRadioStatus(uint32 handle);

	Parameters :	handle - handle to the device

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function will send request BlueCore to send the current radio 
					status information. Once the command is issued BlueCore will need 
					to be polled for the status using the hqGetRadioStatus(..) function.

				
*************************************************************************************/
LIBAPI(int) radiotestRadioStatus(uint32 handle);

/*************************************************************************************
	
	Function Name :	int hqGetRadioStatus(uint32 handle, uint16 * r, int timeout)

	Parameters :	handle - handle to the device

					r - pointer to an array of 7 unsigned 16-bit integers. These will 
						be used to store the radio status array as follows:
							Index Parameter	Type of Status Report
							0				Internal transmission level
							1				External transmission level
							2				Reception level
							3				Local oscillator level
							4				IQ trim
							5				Signal to image ratio in dB
							6				Receivers attenuation
							7				Local oscillator amplitude (LO_AMPL) used for table look-up

					timeout - Used to specify a timeout, in ms, which the software will 
						wait for the HQ radio status packet to be returned from BlueCore.

	Description :	This function will wait for a time period specified by "timeout" 
					for the HQ radio status packet.
				
*************************************************************************************/
LIBAPI(int) hqGetRadioStatus(uint32 handle, uint16 * r, int timeout);

/*************************************************************************************
	
	Function Name :	int bccmdGetBuildId(uint32 handle, uint16 * buildid);

	Parameters :	handle - handle to the device

					buildid - pointer to a variable to hold the returned firmware build ID

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function will return the firmware's build ID

*************************************************************************************/
LIBAPI(int) bccmdGetBuildId(uint32 handle, uint16 * buildid);

/*************************************************************************************
	
	Function Name :	int bccmdGetChipVersion(uint32 handle, uint16 * chipver);

	Parameters :	handle - handle to the device

					chipver - pointer to a variable to hold the returned chip version

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function will return the chip major version number

*************************************************************************************/
LIBAPI(int) bccmdGetChipVersion(uint32 handle, uint16 * chipver);

/*************************************************************************************
	
	Function Name :	int bccmdGetChipRevision(uint32 handle, uint16 * chiprev);

	Parameters :	handle - handle to the device

					chiprev - pointer to a variable to hold the returned chip revision.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function will return the chip revision number

*************************************************************************************/
LIBAPI(int) bccmdGetChipRevision(uint32 handle, uint16 * chiprev);

/*************************************************************************************
	
	Function Name :	int bccmdGetInterfaceVersion(uint32 handle, uint16 * interfaceVersion);

	Parameters :	handle - handle to the device

					interfaceVersion - pointer to a variable to hold the returned 
						BCCMD interface version.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function will return the firmware BCCMD interface version.

*************************************************************************************/
LIBAPI(int) bccmdGetInterfaceVersion(uint32 handle, uint16 * interfaceVersion);

/*************************************************************************************
	
	Function Name :	int bccmdGetRadiotestInterfaceVersion(uint32 handle, uint16 * interfaceVersion);

	Parameters :	handle - handle to the device

					interfaceVersion - pointer to a variable to hold the returned 
						radiotest interface version.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function will return the firmware radiotest BCCMD interface 
					version.

*************************************************************************************/
LIBAPI(int) bccmdGetRadiotestInterfaceVersion(uint32 handle, uint16 * interfaceVersion);

/*************************************************************************************
	
	Function Name :	int bccmdSetPio(uint32 handle, uint16 mask, uint16 port);

	Parameters :	handle - handle to the device

					mask - read / write bit mask of PIO port.
					
					port - bit settings of PIO port.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function will set the pio port.

*************************************************************************************/
LIBAPI(int) bccmdSetPio(uint32 handle, uint16 mask, uint16 port);

/*************************************************************************************
	
	Function Name :	int bccmdGetPio(uint32 handle, uint16 * mask, uint16 * port);

	Parameters :	handle - handle to the device

					mask - read / write bit mask of PIO port.
					
					port - bit settings of PIO port.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function will get the pio port.

*************************************************************************************/
LIBAPI(int) bccmdGetPio(uint32 handle, uint16 * mask, uint16 * port);

/*************************************************************************************
	
	Function Name :	int bccmdGetAio(uint32 handle, uint16 aio, uint16 * result);

	Parameters :	handle - handle to the device

					aio - AIO pin which will be read 0 to 3.
					
					result - reference to 32 bit unsigned int, holds the result of the 
					ADC voltage reading. This value is from 0 to 255 where 0 = 0V and 
					255 = VDD Therefore:
						voltage_reading = (result/255)*(VDD)

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function will perform a voltage reading on the specified AIO
					pin.

*************************************************************************************/
LIBAPI(int) bccmdGetAio(uint32 handle, uint16 aio, uint16 * result);

/*************************************************************************************
	
	Function Name :	int bccmdGetFirmwareCheckMask(uint32 handle, uint16 * mask);

	Parameters :	handle - handle to the device

					mask - pointer to mask which states which firmware component is 
						present AND if that component contains a pre-calculated checksum.

						Bit			Component
						=====================================
						0			Stack
						1			Loader
						3			VM application
					
					
	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function will check if each firmware component is present and 
					that it contains a valid checksum. This function should be called 
					before bccmdGetFirmwareCheck.

*************************************************************************************/
LIBAPI(int) bccmdGetFirmwareCheckMask(uint32 handle, uint16 * mask);

/*************************************************************************************
	
	Function Name :	int bccmdGetFirmwareCheck(uint32 handle, uint16 * check);

	Parameters :	handle - handle to the device

					check - pointer to a mask which states which firmware component is 
						present AND if that component contains the correct checksum

						Bit			Component
						=====================================
						0			Stack
						1			Loader
						3			VM application
					
					
	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function will check if each firmware component checksum is correct.

*************************************************************************************/
LIBAPI(int) bccmdGetFirmwareCheck(uint32 handle, uint16 * check);

/*************************************************************************************
	TRANSMITTER TESTS
*************************************************************************************/

/*************************************************************************************
	
	Function Name :	int radiotestTxstart(uint32 handle, uint16 frequency, uint16 intPA, uint16 extPA, int16 modulation);

	Parameters :	handle - handle to the device.

					frequency - Local Oscillator frequency

					intPA - Drive level for internal PA (0 to 63)

					extPA - Drive level for external PA

					modulation - modulation offset - 4096 = 1MHz
	
	Returns :		-1 on invalid handle, 1 on success, 0 on failure.

	Description :	This function initiates TXSTART

*************************************************************************************/
LIBAPI(int) radiotestTxstart(uint32 handle, uint16 frequency, uint16 intPA, uint16 extPA, int16 modulation);

/*************************************************************************************
	
	Function Name :	int radiotestTxdata1( uint32 handle, uint16 frequency, uint16 intPA, uint16 extPA);

	Parameters :	handle - handle to the device

					frequency - Local Oscillator frequency

					intPA - Drive level for internal PA (0 to 63)

					extPA - Drive level for external PA

	Returns :		-1 on invalid handle, 1 on success, 0 on failure.

	Description :	This function initiates TXDATA1

*************************************************************************************/
LIBAPI(int) radiotestTxdata1(uint32 handle, uint16 frequency, uint16 intPA, uint16 extPA);

/*************************************************************************************
	
	Function Name :	int radiotestTxdata2(uint32 handle, uint16 countrycode, uint16 intPA, uint16 extPA);

	Parameters :	handle - handle to the device

					countrycode - Country code for hopping sequence

					intPA - Drive level for internal PA (0 to 63)

					extPA - Drive level for external PA

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function initiates TXDATA2

*************************************************************************************/
LIBAPI(int) radiotestTxdata2(uint32 handle, uint16 countrycode, uint16 intPA, uint16 extPA);

/*************************************************************************************
	
	Function Name :	int radiotestTxdata3(uint32 handle, uint16 frequency, uint16 intPA, uint16 extPA);

	Parameters :	handle - handle to the device

					frequency - Local Oscillator frequency

					intPA - Drive level for internal PA (0 to 63)

					extPA - Drive level for external PA

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function initiates TXDATA3

************************************************************************************/
LIBAPI(int) radiotestTxdata3(uint32 handle, uint16 frequency, uint16 intPA, uint16 extPA);

/*************************************************************************************
	
	Function Name :	int radiotestTxdata4(uint32 handle, uint16 frequency, uint16 intPA, uint16 extPA);

	Parameters :	handle - handle to the device

					frequency - Local Oscillator frequency

					intPA - Drive level for internal PA (0 to 63)

					extPA - Drive level for external PA

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function initiates TXDATA4

*************************************************************************************/
LIBAPI(int) radiotestTxdata4(uint32 handle, uint16 frequency, uint16 intPA, uint16 extPA);

/*************************************************************************************
	RECIEVER TESTS
*************************************************************************************/
/*************************************************************************************
	
	Function Name :	int radiotestRxstart1(uint32 handle, uint16 frequency, int hiside, uint16 rx_attenuation);

	Parameters :	handle - handle to the device

					frequency - Local Oscillator frequency

					hiside - 

					rx_attenuation - 

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function initiates RXSTART1.

*************************************************************************************/
LIBAPI(int) radiotestRxstart1(uint32 handle, uint16 frequency, int hiside, uint16 rx_attenuation);

/*************************************************************************************
	
	Function Name :	int radiotestRxstart2(uint32 handle, uint16 frequency, int hiside, uint16 rx_attenuation, int sample_size);

	Parameters :	handle - handle to the device

					frequency - Local Oscillator frequency

					hiside - 

					rx_attenuation - 

					sample_size - Number of RSSI reports that will be averaged to 
						achieve the final RSSI value.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function initiates RXSTART2. 

*************************************************************************************/
LIBAPI(int) radiotestRxstart2(uint32 handle, uint16 frequency, int hiside, uint16 rx_attenuation, int sample_size);

/*************************************************************************************
	
	Function Name :	hqGetRssi(uint32 handle, int timeout, int max_size, uint16 * r);

	Parameters :	handle - handle to the device
					
					timeout - time, in ms, which the function will wait for valid results. 
						If set to zero the function will just check for completion.

					max_size - size of the array allocated to hold the RSSI samples

					r - Array containing the RSSI samples

	Return :		-1 on invalid handle, 0 on failure, sample_size or max_size on success.

	Description :	This function obtains the last complete RSSI results from 
					BlueCore. The results are collected by calling do_rxstart2.

*************************************************************************************/
LIBAPI(int) hqGetRssi(uint32 handle, int timeout, int max_size, uint16 * r);

/*************************************************************************************
	
	Function Name :	int radiotestBer1(uint32 handle, uint16 frequency, int hiside, uint16 rx_attenuation, uint32 sampleSize);

	Parameters :	handle - handle to the device

					frequency - Local Oscillator frequency

					hiside - 

					rx_attenuation - 

					sampleSize - Number of bits to be sampled.

	Returns :		-1 on invalid handle, 0 on failure, 1 on success

	Description :	This function initiates BER1.

*************************************************************************************/
LIBAPI(int) radiotestBer1(uint32 handle, uint16 frequency, int hiside, uint16 rx_attenuation, uint32 sampleSize);

/*************************************************************************************
	
	Function Name :	int radiotestBer2(uint32 handle, uint16 country_code, int hiside, uint16 rx_attenuation, uint32 sampleSize);

	Parameters :	handle - handle to the device

					country_code - Valid country code

					hiside - 

					rx_attenuation - 

					sampleSize - Number of bits to be sampled.

	Returns :		-1 on invalid handle, 0 on failure, 1 on success

	Description :	This function initiates BER1.

*************************************************************************************/
LIBAPI(int) radiotestBer2(uint32 handle, uint16 country_code, int hiside, uint16 rx_attenuation, uint32 sampleSize);

/*************************************************************************************
	
	Function Name :	int radiotestBerLoopback(uint32 handle, uint16 frequency, uint16 intPA, uint16 extPA, uint32 sampleSize);

	Parameters :	handle - handle to the device

					frequency - Local Oscillator frequency

					intPA - Drive level for internal PA (0 to 63)

					extPA - Drive level for external PA

					sampleSize - Number of bits to be sampled.

	Returns :		-1 on invalid handle, 0 on failure, 1 on success.

	Description :	This function initiates BER_LOOPBACK

*************************************************************************************/
LIBAPI(int) radiotestBerLoopback(uint32 handle, uint16 frequency, uint16 intPA, uint16 extPA, uint32 sampleSize);

/*************************************************************************************
	
	Function Name :	radiotestLoopback(uint32 handle, uint16 frequency, uint16 intPA, uint16 extPA);

	Parameters :	handle - handle to the device

					frequency - Local Oscillator frequency

					intPA - Drive level for internal PA (0 to 63)

					extPA - Drive level for external PA

	Returns :		-1 on invalid handle, 0 on failure, 1 on success

	Description :	This function initiates LOOPBACK

*************************************************************************************/
LIBAPI(int) radiotestLoopback(uint32 handle, uint16 frequency, uint16 intPA, uint16 extPA);

/*************************************************************************************
	
	Function Name :	int hqGetBer( uint32 handle, int timeout, uint32 * r);

	Parameters :	handle - handle to the device
					
					timeout - time, in ms, which the function will wait for valid results. 
						If set to zero the function will just check for completion.

					r - Array containing culmative BER report. Defined as follows:
						r[0] = cumulative bit count
						r[1] = cumulative bit errors
						r[2] = NOT USED
						r[3] = cumulative recieved packets
						r[4] = cumulative expected packets
						r[5] = cumulative header errors
						r[6] = cumulative CRC errors
						r[7] = cumulative uncorrected errors
						r[8] = cumulative sync errors

	Return :		-1 on invalid handle, 0 on failure, BER_MAX on success.

	Description :	This function obtains the last complete BER results samlpe from 
					BlueCore.

*************************************************************************************/
LIBAPI(int) hqGetBer( uint32 handle, int timeout, uint32 * r);

/*************************************************************************************
	
	Function Name :	int radiotestRxdata1(uint32 handle, uint16 frequency, int hiside, uint16 rx_attenuation);

	Parameters :	handle - handle to the device

					frequency - Local Oscillator frequency

					hiside - 

					rx_attenuation - 

	Returns :		-1 on invalid handle, 0 on failure, BER_MAX on success

	Description :	This function initiates RXDATA1.

*************************************************************************************/
LIBAPI(int) radiotestRxdata1(uint32 handle, uint16 frequency, int hiside, uint16 rx_attenuation);

/*************************************************************************************
	
	Function Name :	int radiotestRxdata2(uint32 handle, uint16 countrycode, int hiside, uint16 rx_attenuation);

	Parameters :	handle - handle to the device

					countrycode -

					hiside - 

					rx_attenuation - 

	Returns :		-1 on invalid handle, 0 on failure, BER_MAX on success

	Description :	This function initiates RXDATA2.

*************************************************************************************/
LIBAPI(int) radiotestRxdata2(uint32 handle, uint16 countrycode, int hiside, uint16 rx_attenuation);

/*************************************************************************************
	
	Function Name :	int hqGetRxdata(uint32 handle, uint16 * r);

	Parameters :	handle - handle to the device

					r - Pointer to an array which will receive the following:
						r[0] = Number of packets received
						r[1] = Number of good packets
						r[2] = Number of corrected packets
						r[3] = RSSI value
						r[4] = Boolean flag showing if RSSI value is valid.

	Returns :		-1 on invalid handle, 0 on failure, array size on success

	Description :	This function will return the last packet generated by either 
					RXDATA1 or RXDATA2. Once the packet has been read it cannot be 
					read again until the next packet is generated.

*************************************************************************************/
LIBAPI(int) hqGetRxdata( uint32 handle, uint16 * r);

/*************************************************************************************
	CONFIGURATION COMMANDS
*************************************************************************************/
/*************************************************************************************
	
	Function Name :	int radiotestCfgFreq(uint32 handle, uint16 txrxinterval, uint16 loopback, uint16 report);

	Parameters :	handle - handle to the device

					txrxinterval - sets the period in microseconds between TX and RX 
						events in RXDATA, TXDATA, BIT ERR and LOOP BACK test modes. 
						Default is 1250 (20 slots), maximum 65536. If passed as 0, 
						current value unchanged.
					
					loopback - Sets the offset in microseconds between a reception 
						event and retransmission of the data in loopback. Default is 
						1875 (two slots later), must be less than txrxinterval. If 
						passed as zero current value unchanged.

					report - Sets the time in seconds between reports to host sent by 
						RXDATA and BIT ERR functions. Default 1, if passed as 0 
						current value unchanged.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Sets three values used in deciding timing details of tests.

*************************************************************************************/
LIBAPI(int) radiotestCfgFreq(uint32 handle, uint16 txrxinterval, uint16 loopback, uint16 report);

/*************************************************************************************
	
	Function Name :	int radiotestCfgPkt(uint32 handle, uint16 type, uint16 size);

	Parameters :	handle - handle to the device

					type - The standard Bluetooth packet type, 0-15 (12-13 not 
						allowed). Any other number sets default: 
							DM5 for TXDATA1/2
							DH5 for TXDATA3/4

					size - The size of data in packet, from one to maximum for type. 
						If zero sets default: 
							20 bytes for TXDATA1/2
							192 bytes for TXDATA3/4.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Sets packet type and size for transmitter tests. It has no effect 
					on RX or LOOP BACK tests.

*************************************************************************************/
LIBAPI(int) radiotestCfgPkt(uint32 handle, uint16 type, uint16 size);

/*************************************************************************************
	
	Function Name :	int radiotestCfgBitError(uint32 handle, uint32 sampleSize, int reset);

	Parameters :	handle - handle to the device

					sampleSize - If non-zero, the target for total counters is set to 
						this and total count resets at this value. If passed as 0 
						current value, unchanged. 
					
					reset - if not false and BIT ERR/2 is active, immediately resets
						the counters for the total statistics, but not over the last 
						report period.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Sets two values used in bit error measurements.

*************************************************************************************/
LIBAPI(int) radiotestCfgBitError(uint32 handle, uint32 sampleSize, int reset);

/*************************************************************************************
	
	Function Name :	int radiotestCfgXtalFtrim(uint32 handle, uint16 ftrim);

	Parameters :	ftrim - a number between 0 and 63 inclusive. This is not a 
						permanent change.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Timing for BlueCore01 is controlled by a crystal. This requires
					trimming for new hardware. This command can be used to set a new 
					trim value either before a radiotest command is started or while 
					a test is already in operation; the change takes effect 
					immediately.

*************************************************************************************/
LIBAPI(int) radiotestCfgXtalFtrim(uint32 handle, uint16 ftrim);

/*************************************************************************************
	
	Function Name :

	Parameters :

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :

*************************************************************************************/
LIBAPI(int) radiotestCfgUapLap(uint32 handle, uint16 uap, uint32 lap);

/*************************************************************************************
	
	Function Name :	int radiotestCfgIqTrim(uint32 handle, uint16 trim);

	Parameters :	handle - handle to the device

					trim - value to be written to the register, ANA_IQ_TRIM.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	The IQ block may be trimmed by setting a register, ANA_IQ_TRIM.
					This allows the variable to be set for the test modes.  The value
					"trim", 0 to 511 inclusive, takes effect immediately, is not 
					saved, and must be reset on each test, as the IQ is calibrated at 
					the start.

*************************************************************************************/
LIBAPI(int) radiotestCfgIqTrim(uint32 handle, uint16 trim);

/*************************************************************************************
	
	Function Name :	radiotestCfgTxIf(uint32 handle, int16 offset);

	Parameters :	handle - handle to the device

					offset - value to be written to the register, ANA_IQ_TRIM.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	The bc01 transmitter uses an additional offset in the IF part of 
					the transmitter, which needs to be configured for optimum 
					performance. The target is to be able to set this to zero. 
					However, the stack currently uses an offset of 1MHz 1.5MHz.
					"offset_half_mhz" sets the value of this in units of 0.5MHz for 
					the radiotests. The default is the value used in standard 
					operation, currently 3 units, i.e. 1.5MHz offset.  Note this is a 
					signed integer.  The range is -5 to +5. This must be set before 
					the start of a radio transmitter test.

*************************************************************************************/
LIBAPI(int) radiotestCfgTxIf(uint32 handle, int16 offset);

/*************************************************************************************
	
	Function Name :	int radiotestCfgTxTrim(uint32 handle, uint16 am_addr);

	Parameters :	handle - handle to the device

					am_addr - active member address.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Sets the AM_ADDR used in the header of test transmissions, and
					expected in receiving test transmissions.  It is a number between 
					0 and 7, inclusive.  The default value is 7.  This takes effect
					immediately and lasts until a new test is restarted, at which
					point a default read from persistent store is used.  The defaults
					are: PSKEY_RADIOTEST_TX_TRIM1 for PRBS data (which is assumed
					for all reception routines), PSKEY_RADIOTEST_TX_TRIM2 for alternating
					bits (1010...), PSKEY_RADIOTEST_TX_TRIM3 for alternating nybbles
					(11110000).

*************************************************************************************/
LIBAPI(int) radiotestCfgTxTrim(uint32 handle, uint16 am_addr);

/*************************************************************************************
	
	Function Name :	int radiotestCfgLoLvl(uint32 handle, uint16 lo_lvl);

	Parameters :	handle - handle to the device

					lo_lvl - analogue LO output level.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This command sets the value of the Analogue Local Oscillator 
					output level to lvl, over-riding the value calculated by the 
					internal calibration algorithm.

					Important Note:
						Use of this command is unnecessary for all normal users.

*************************************************************************************/
LIBAPI(int) radiotestCfgLoLvl(uint32 handle, uint16 lo_lvl);

/*************************************************************************************
	MISC TESTS
*************************************************************************************/

/*************************************************************************************
	
	Function Name :

	Parameters :

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :

*************************************************************************************/
LIBAPI(int) radiotestSettle(uint32 handle, uint16 start, uint16 end);

/*************************************************************************************
	
	Function Name :

	Parameters :

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :

*************************************************************************************/
LIBAPI(int) hqGetSettle(uint32 handle, uint16 * r);

/*************************************************************************************
*	
*	Function Name :	int get_freq_offset(uint32 handle, double * offset, int sample_size);
*
*	Parameters :	handle - handle to the device
*
*					offset - pointer to the offset. Measured in parts per million (PPM), 
*						this value is an average of sample_size readings from the 
*						BlueCore receiver.
*				
*					sample_size - determines how many readings of offset are averaged.
*						The software will only accept values in the range of 20 to 200, 
*						any values outside this range will be forced up to 20 or 200. 
*						The choice of value will determine the accuracy and speed. The 
*						lower the value the quicker but less accurate the results, the 
*						higher the value the more accurate but slower the function. 
*						The optimal value is 60 to 100.
*
*	Returns :		-1 on error
*					0 on success
*					1 on fail
*
*	Description :	This command will read the offset of a received signal.
*
*************************************************************************************/
LIBAPI(int) get_freq_offset(uint32 handle, double * offset, int sample_size);

/*************************************************************************************
	PS ACCESS COMMANDS
*************************************************************************************/

/*************************************************************************************
	PS_STORES - the BlueCore persistent store is conceptually a four layer model. When 
	the BlueCore firmware initialises or reads a persistent store value it will always 
	read it from the top most layer where that PS entry is present. When writing or 
	reading to the persistent store in production it is possible to select the PS 
	store layer or let the firmware decide. This selection is carried out by the use of
	the 'stores' parameter of all the PS read and write functions. This parameter is a 
	bit mask so multiple layers can be selected. The layers are defined as follows 
	(bottom to top):

		1. ROM layer - the bottom layer in the model, basically these are the hard 
		coded firmware defaults. This is a read-only layer so all PS writes to this 
		layer will fail. Note that not all PS entries will have a default value so in 
		some cases a PS read will also fail.

		2. Factory-set layer - this layer is not present on BlueCore ROM devices unless
		an external EEPROM is present. PS keys are moved to this layer when 
		"psFactorySet()" is called. This layer can also be written to and read from 
		directly by selecting this layer as the 'stores' parameter.

		3. Implementation - this layer is not present on BlueCore ROM devices unless
		an external EEPROM is present. This layer is where most PS keys are written 
		during production for BlueCore Flash, External and ROM devices with an EEPROM.
		In these cases calling "psFactorySet()" will move the PS values from this 
		layer to the factory-set layer. Calling "psFactoryRestoreAll()" will erase 
		the implemention layer and expose the factory-set layer. When using BlueCore 
		Flash, External and ROM devices with an EEPROM if this layer is selected then
		the function will fail if the flash or EEPROM is read-only, for example if 
		there is a fault on the flash write-enable.

		4. Transient - this layer is the BlueCore RAM. By definition a cold reset or 
		power down reset will clear this layer. A warm reset will retain this layer.
		This layer exists on all BlueCore devices.

	The bitmasks to define these layers are shown below. If no bits are set (0), then
	the firmware will select the layer. If an external storage device is present and 
	functioning correctly then the firmware will write to the implemention layer (this 
	includes BlueCore Flash), otherwise the firmware will write to the transient layer.
**************************************************************************************/
#define	PS_STORES_DEFAULT (0x0000)	/* default */
#define	PS_STORES_I       (0x0001)	/* implementation (normal) */
#define	PS_STORES_F       (0x0002)	/* factory-set */
#define	PS_STORES_R       (0x0004)	/* rom (read-only) */
#define	PS_STORES_T       (0x0008)	/* transient (RAM) */
#define PS_STORES_FR      (0x0006)
#define PS_STORES_IF      (0x0003)
#define PS_STORES_IFR     (0x0007)

/*************************************************************************************
	
	Function Name :	int bccmdSetEeprom(uint32 handle, uint16 log2bytes, uint16 addrMask);

	Parameters :	handle - handle to the device

					log2bytes - Specifies the size of the EEPROM (log2 bytes). That is 
						the number of bits required to address any byte of the memory 
						array.

					addrMask - A bitmap specifying which, if any, of the slave address 
						bits are used to specify high-order bits of the array address.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Write the header to the I2C EEPROM to allow the device to be used 
					to store PS values and VM applications.

*************************************************************************************/
LIBAPI(int) bccmdSetEeprom(uint32 handle, uint16 log2bytes, uint16 addrMask);

/*************************************************************************************
	
	Function Name : int psReadBdAddr( uint32 handle, uint32 * lap, uint8 * uap, uint16 * nap);

	Parameters :	handle - handle to the device
					
					lap - pointer to a uint32 to hold the lap portion of the BD_ADDR
					
					uap - pointer to a uint8 to hold the uap portion of the BD_ADDR

					nap - pointer to a uint16 to hold the nap portion of the BD_ADDR

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Function to read the BD_ADDR from the persistent store.

*************************************************************************************/
LIBAPI(int) psReadBdAddr( uint32 handle, uint32 * lap, uint8 * uap, uint16 * nap);

/*************************************************************************************
	
	Function Name :	int psRead( uint32 handle, uint16 ps_key, uint16 store, uint16 arraylen, uint16 * ps, uint16 * len);

	Parameters :	handle - handle to the device

					ps_key - ps key ID of the value to be read.
					
					store - bit mask to identify the PS store layer to write to. See 
						comments above under PS_STORES.

					arraylen - size of the array allocated to hold the PS value

					ps - array of 16-bit words of size "arraylen" to hold the PS value

					len - Pointer to a 16-bit word

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Used to read the value of the PS key specified

*************************************************************************************/
LIBAPI(int) psRead( uint32 handle, uint16 ps_key, uint16 store, uint16 arraylen, uint16 * ps, uint16 * len);

/*************************************************************************************
	
	Function Name :	int psClear( uint32 handle, uint16 ps_key, uint16 store);

	Parameters :	handle - handle to the device

					ps_key - ps key value of the entry to be removed

					store - bit mask to identify the PS store layer(s) from which the PS 
					entry will be erased.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function removes from the ps the value stored in the  
					layer specified by the stores parameter. Clearing values from the ps can have 
					subtle effects. The description of the Layered Model of the ps 
					stores above shows how clearing a value from the psfl layer can 
					(re)expose a value in the psf or psrom layer.

*************************************************************************************/
LIBAPI(int) psClear( uint32 handle, uint16 ps_key, uint16 store);

/*************************************************************************************
	
	Function Name :	int psClearAll(uint32 handle, uint16 store);

	Parameters :	handle - handle to the device

					store - bit mask to identify the PS store layer(s) from which the PS 
					entries will be erased.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This removes all values from the ps flash; only values in the 
					psrom layer remain. This command should be used with caution.

*************************************************************************************/
LIBAPI(int) psClearAll(uint32 handle, uint16 store);

/*************************************************************************************
	
	Function Name :	int psFactorySet( uint32 handle);

	Parameters :	handle - handle to the device

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This moves all values in the psfl layer to the psf layer.

*************************************************************************************/
LIBAPI(int) psFactorySet( uint32 handle);

/*************************************************************************************
	
	Function Name :	int psFactoryRestoreAll( uint32 handle);

	Parameters :	handle - handle to the device

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This command removes all values from the psfl store, so restoring 
					the chips ps to the state it was in when the PS_FACTORY_SET 
					command was invoked.

*************************************************************************************/
LIBAPI(int) psFactoryRestoreAll( uint32 handle);

/*************************************************************************************
	
	Function Name :	int psFactoryRestore(uint32 handle);

	Parameters :	handle - handle to the device

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This is identical to PS_FACTORY_RESTORE_ALL except that values in 
					the psfl layer that do not obscure values in the psf and/or psrom 
					layers are left in place.

*************************************************************************************/
LIBAPI(int) psFactoryRestore(uint32 handle);

/*************************************************************************************
	
	Function Name :	int psSize(uint32 handle, uint16 pskey, uint16 store, uint16 * size);

	Parameters :	handle - handle to the device

					pskey - PS entry to return the size of.

					store - PS store to access

					size - reference to size of PS entry.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	returns the internal amount of data, in uint16's, held by a 
					particular PS entry. Failure may also denote the the store mask 
					selected does not contain a valid PS entry.

*************************************************************************************/
LIBAPI(int) psSize(uint32 handle, uint16 pskey, uint16 store, uint16 * size);

/*************************************************************************************
	
	Function Name :	int psWrite( uint32 handle, uint16 ps_key, uint16 store, uint16 ps_length, uint16 *ps);

	Parameters :	handle - handle to the device

					ps_key - ps key value of the entry to be written

					store - PS store to access

					ps_length - length of the data to be written.

					ps - pointer to an array to hold the key contents

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	writes the PS setting "ps_key" array "ps" of length "ps_length" 
					to the persistent store layer identified by "store".

*************************************************************************************/
LIBAPI(int) psWrite( uint32 handle, uint16 ps_key, uint16 store, uint16 ps_length, uint16 *ps);

/*************************************************************************************
	
	Function Name :	int psWriteVerify( uint32 handle, uint16 ps_key, uint16 store, uint16 ps_length, uint16 *ps);

	Parameters :	handle - handle to the device

					ps_key - ps key value of the entry to be written

					store - PS store to access

					ps_length - length of the data to be written.

					ps - pointer to an array to hold the key contents

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	writes the PS setting "ps_key" array "ps" of length "ps_length" 
					to the persistent store layer PS_STORE_I or PS_STORE_F if one those 
					values are identified by "store". The function will then perform a 
					PS read and verify that the PS key has been written correctly.

					This function will fail if a store mask does not contain PS_STORE_I
					or PS_STORE_F.

*************************************************************************************/
LIBAPI(int) psWriteVerify(uint32 handle, uint16 ps_key, uint16 store, uint16 ps_length, uint16 *ps);

/*************************************************************************************
	
	Function Name :	int psWriteBdAddr( uint32 handle, uint32 lap, uint32 uap, uint32 nap);

	Parameters :

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :

*************************************************************************************/
LIBAPI(int) psWriteBdAddr( uint32 handle, uint32 lap, uint32 uap, uint32 nap);

/*************************************************************************************
	
	Function Name :	int psReadModuleId( uint32 handle, uint32 * module_id);

	Parameters :	handle - handle to the device

					*module_id - pointer to a variable that will hold the returned 
							module ID.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function will attempt to read the module ID stored in 
					BlueCore. The default for this value is 0 (zero)

*************************************************************************************/
LIBAPI(int) psReadModuleId( uint32 handle, uint32 * module_id);

/*************************************************************************************
	
	Function Name :	int psReadXtalFtrim(uint32 handle, uint16 * ftrim);

	Parameters :	handle - handle to the device

					*ftrim - pointer to a variable that will hold the returned 
							xtal ftrim.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function will attempt to read the xtal ftrim stored in 
					BlueCore.

*************************************************************************************/
LIBAPI(int) psReadXtalFtrim(uint32 handle, uint16 * ftrim);

/*************************************************************************************
	
	Function Name :	int psWriteXtalFtrim( uint32 handle, uint16 ftrim);

	Parameters :	handle - handle to the device

					ftrim - Crystal frequency trim

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function will write the crystal trim value to the persistent 
					store.

*************************************************************************************/
LIBAPI(int) psWriteXtalFtrim( uint32 handle, uint16 ftrim);

/*************************************************************************************
	
	Function Name :	int psWriteModuleSecurityCode(uint32 handle, uint16 * code);

	Parameters :	handle - handle to the device

					code - pointer to 8 member unsigned 16 bit integer array holding 
					the 128 bit module security code.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function will write the module security code to the 
					persistent store.

*************************************************************************************/
LIBAPI(int) psWriteModuleSecurityCode(uint32 handle, uint16 * code);

/*************************************************************************************
	
	Function Name :	int psWriteModuleId( uint32 handle, uint32 module_id);

	Parameters :	handle - handle to the device

					module_id - Module ID to be stored in BlueCore persistent store

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function will attempt to wriet the module ID to BlueCore. 
					The default for this value is 0 (zero).

*************************************************************************************/
LIBAPI(int) psWriteModuleId( uint32 handle, uint32 module_id);

/*************************************************************************************
	
	Function Name :	int psWriteBaudrate(uint32 handle, uint16 value)

	Parameters :	handle - handle to the device

					value - Used determine the baudrate where:
						baudrate = value/0.004096

						Some common values are:
							38k4    baud    -   157   (0x009d)
							57k6    baud    -   236   (0x00ec)
							115k2   baud    -   472   (0x01d8)
							230k4   baud    -   944   (0x03b0)
							460k8   baud    -   1887  (0x075f)
							921k6   baud    -   3775  (0x0ebf)
							1382k4  baud    -   5662  (0x161e)

						The maximum rated speed for the UART hardware is 1.5 Mbaud, although
						this key can be set to a higher value.


	Returns :		-1 on invalid handle, 1 on success, 0 on failure. 

	Description :	This function will write a value corresponding to the baudrate to 
					the persistent store. This function can only be used in builds later 
					than 18.x. For earlier builds the PSKEY_HOSTIO_UART_PS_BLOCK should 
					be modified.

*************************************************************************************/
LIBAPI(int) psWriteBaudrate(uint32 handle, uint16 value);

/*************************************************************************************
	
	Function Name :	int psWriteRadiotestFirstTrimTime(uint32 handle, uint32 time)

	Parameters :	handle - handle to the device

					time - The time from the start of a test till the first IQ trim 
							auto-calibration is done. 

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function will write the time to the persistent store.A value 
					of 0 disables the automatic calibration completely during the 
					test. A calibration is still done before the test starts. 

*************************************************************************************/
LIBAPI(int) psWriteRadiotestFirstTrimTime(uint32 handle, uint32 time);

/*************************************************************************************
	
	Function Name :	int psReadRadiotestFirstTrimTime(uint32 handle, uint32 * time)

	Parameters :	handle - handle to the device

					time - The time from the start of a test till the first IQ trim 
							auto-calibration is done. 

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function will write the time to the persistent store.A value 
					of 0 disables the automatic calibration completely during the 
					test. A calibration is still done before the test starts. 

*************************************************************************************/
LIBAPI(int) psReadRadiotestFirstTrimTime(uint32 handle, uint32 * time);

/*************************************************************************************
	
	Function Name :	int psWriteRadiotestLoLvlTrimEnable(uint32 handle, uint16 state)

	Parameters :	handle - handle to the device

					state - Determine whether IQ trim is enabled. Basically this is a 
							boolean where 1 = true and 0 = false.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	This function enables trimming of the value for the register 
					ANA_LO_ENABLE; otherwise, has no effect. This trimming takes place 
					before the start of each test.  

*************************************************************************************/
LIBAPI(int) psWriteRadiotestLoLvlTrimEnable(uint32 handle, uint16 state);

/*************************************************************************************
	
	Function Name :	int psReadRadiotestLoLvlTrimEnable(uint32 handle, uint16 * state)

	Parameters :	handle - handle to the device

					state - Determine whether IQ trim is enabled. Basically this is a 
							boolean where 1 = true and 0 = false.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Reads value of PS setting

*************************************************************************************/
LIBAPI(int) psReadRadiotestLoLvlTrimEnable(uint32 handle, uint16 * state);

/*************************************************************************************
	
	Function Name :	int psWriteRadiotestSubsequentTrimTime(uint32 handle, uint32 time);

	Parameters :	handle - handle to the device

					time - The time between each IQ trim auto-calibration.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Once the first IQ trim auto calibration is done in test mode, then 
					subsequent trims are done at regular intervals with the period 
					being given by this key. If this key is 0 then only the first trim 
					is done. If PSKEY_RADIOTEST_FIRST_TRIM_TIME is 0 then no trims are 
					done during the test regardless of the setting of this key. 

*************************************************************************************/
LIBAPI(int) psWriteRadiotestSubsequentTrimTime(uint32 handle, uint32 time);

/*************************************************************************************
	
	Function Name :	int psReadRadiotestSubsequentTrimTime(uint32 handle, uint32 * time);

	Parameters :	handle - handle to the device

					time - The time between each IQ trim auto-calibration.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Read the PS setting

*************************************************************************************/
LIBAPI(int) psReadRadiotestSubsequentTrimTime(uint32 handle, uint32 * time);

/*************************************************************************************
	
	Function Name :	int psWriteHostInterface(uint32 handle, uint16 transport):

	Parameters :	handle - handle to the device

					transport - The choice of physical bus. Valid values are:
						1 - BCSP 
						2 - H2 - USB 
						3 - H4 - UART 
						4 - Raw UART (VM)

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Select the physical connection that is used to pass information 
					to and from the host - BCSP, H4, USB, etc. Only one of these can 
					be used at a time, not least because the USB interface uses some 
					of the same chip pins as the UART. If this key specifies use of 
					USB and the chip has no USB hardware support then it reverts to 
					using BCSP. Much of the chip's support for BCSP is performed by 
					hardware - this allows interrupt routines to deal with complete 
					BCSP packets. When the chip uses its UART in a more naive fashion, 
					e.g., for H4, this support must be disabled. The UART's hardware 
					support for BCSP must be enabled via PSKEY_UART_CONFIG. The H4 
					specification requires the use of UART hardware flow control and 
					no use of parity. These must also be set via PSKEY_UART_CONFIG. 

*************************************************************************************/
LIBAPI(int) psWriteHostInterface(uint32 handle, uint16 transport);

/*************************************************************************************
	
	Function Name :	int psReadHostInterface(uint32 handle, uint16 * transport):

	Parameters :	handle - handle to the device

					transport - The choice of physical bus. Valid values are:
						1 - BCSP 
						2 - H2 - USB 
						3 - H4 - UART 
						4 - Raw UART (VM)

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Reads the Host interface PS setting

*************************************************************************************/
LIBAPI(int) psReadHostInterface(uint32 handle, uint16 * transport);

/*************************************************************************************
	
	Function Name :	int psWriteUsbAttributes(uint32 handle, uint16 bmAttributes);

	Parameters :	handle - handle to the device

					bmAttributes - The bmAttributes of the Standard Configuration 
					Descriptor, described in Table 9.8 of USB specification, 
					version 1.1. 

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Configuration characteristics
						D7: Reserved (set to one)
						D6: Self-powered
						D5: Remote Wakeup
						D4...0: Reserved (reset to zero)
					D7 is reserved and must be set to one for historical reasons. A 
					device configuration that uses power from the bus and a local 
					source reports a non-zero value in MaxPower to indicate the 
					amount of bus power required and sets D6. The actual power 
					source at runtime may be determined using the GetStatus(DEVICE) 
					request (see Section 9.4.5 of USB specification 1.1). If a device 
					configuration supports remote wakeup, D5 is set to one. 

*************************************************************************************/
LIBAPI(int) psWriteUsbAttributes(uint32 handle, uint16 bmAttributes);

/*************************************************************************************
	
	Function Name :	int psWriteDPlusPullup(uint32 handle, uint16 pio_pin);

	Parameters :	handle - handle to the device

					pio_pin - The PIO line used to control the pull-up resistor on the 
					USB D+ line. 

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	The absence of this key in the ps indicates that this feature is 
					not in use. This value is only used if the chip is presenting its 
					USB interface. See the description of PSKEY_HOST_INTERFACE.

*************************************************************************************/
LIBAPI(int) psWriteDPlusPullup(uint32 handle, uint16 pio_pin);

/*************************************************************************************
	
	Function Name :	int psWriteUsbMaxPower(uint32 handle, uint16 max_power);

	Parameters :	handle - handle to the device

					pio_pin - The MaxPower field of the local USB device, as defined 
					in Table 9.8 of version 1.1 of the USB specification. Bizarrely, 
					this value is given in units of 2mA. 

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Function to write the pskey PSKEY_USB_MAX_POWER  to the persistent
					store. This value is only used if the chip is presenting its USB 
					interface. See the description of PSKEY_HOST_INTERFACE.

*************************************************************************************/
LIBAPI(int) psWriteUsbMaxPower(uint32 handle, uint16 max_power);

/*************************************************************************************
	
	Function Name :	int psWritePioProtectMask(uint32 handle, uint16 mask);

	Parameters :	handle - handle to the device

					mask - PIO mask to protect PIO lines from unwanted access. If a 
					bit of the pskey's value is set high then application code cannot 
					change the value of the corresponding PIO port pin. 

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	The chip's PIO port can function as a general purpose input and 
					output port. However, various optional hardware and software 
					configurations require some PIO pins for their own needs, e.g., an 
					optional external PA/LNA block is driven using PIO pins 0 and 1. 
					This pskey prevents applications changing PIO pins that are 
					required for lower level functions. This blocking action applies 
					to host-based applications requesting PIO changes via BCCMD and to 
					VM applications running on the chip. Bits 0 to 7 of this uint16 
					pskey map to PIO0 to PIO7 respectively. If a bit of the pskey's 
					value is set high then application code cannot change the value of 
					the corresponding PIO port pin. The default value, 0x03, suits 
					Casira as this has an external PA/LNA. 

*************************************************************************************/
LIBAPI(int) psWritePioProtectMask(uint32 handle, uint16 mask);

/*************************************************************************************
	
	Function Name :	int psReadPioProtectMask(uint32 handle, uint16 * mask);

	Parameters :	handle - handle to the device

					mask - PIO mask to protect PIO lines from unwanted access. If a 
					bit of the pskey's value is set high then application code cannot 
					change the value of the corresponding PIO port pin. 

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Reads the PIO protect mask PS setting

*************************************************************************************/
LIBAPI(int) psReadPioProtectMask(uint32 handle, uint16 * mask);

/*************************************************************************************
	
	Function Name :	int psWriteTxOffsetHalfMhz(uint32 handle, int16 offset);

	Parameters :	handle - handle to the device

					offset - transmit offset in units of 500kHz. For example -2 == -1MHz

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Sets the transmit offset.

*************************************************************************************/
LIBAPI(int) psWriteTxOffsetHalfMhz(uint32 handle, int16 offset);

/*************************************************************************************
	
	Function Name :	int psReadTxOffsetHalfMhz(uint32 handle, int16 * offset);

	Parameters :	handle - handle to the device

					offset - transmit offset in units of 500kHz. For example -2 == -1MHz

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	reads the transmit offset.

*************************************************************************************/
LIBAPI(int) psReadTxOffsetHalfMhz(uint32 handle, int16 * offset);

/*************************************************************************************
	
	Function Name :	int psWriteUsrValue(uint32 handle, uint16 user_no, uint16 length, uint16 * entry);

	Parameters :	handle - handle to the device

					user_no - Value between 0 and 49 which refers to 'n' in PSKEY_USRn

					length - length of PS key value in uint16's

					entry - pointer to an array containing the data to be written to 
					the PS.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Writes to PSKEY_USRn.

*************************************************************************************/
LIBAPI(int) psWriteUsrValue(uint32 handle, uint16 user_no, uint16 length, uint16 * entry);

/*************************************************************************************
	
	Function Name :	int psReadUsrValue(uint32 handle, uint32 user_no, uint16 maxLen, uint16 * length, uint16 * entry);

	Parameters :	handle - handle to the device

					user_no - Value between 0 and 49 which refers to 'n' in PSKEY_USRn

					maxLen - length of the array to hold the returned value

					length - pointer to the length of the returned ps entry

					entry[8] - pointer to an array to hold the returned PS value

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Reads the PSKEY_USRn.

*************************************************************************************/
LIBAPI(int) psReadUsrValue(uint32 handle, uint32 user_no, uint16 maxLen, uint16 * length, uint16 * entry);

/*************************************************************************************
	
	Function Name :	int psWritePowerTable(uint32 handle, int num_entries, uint8 * intPA, uint8 * extPA, int * power);

	Parameters :	handle - handle to the device

					num_entries - Number of entries in the power table

					intPA - pointer to an array of unsigned char of size num_entries 
							containing the intPA entries for the power table. 
					
					extPA - pointer to an array of unsigned char of size num_entries 
							containing the extPA entries for the power table.
 
					power - pointer to an array of signed integers of size num_entries 
							containing the Output power entries, in dBm, for the power 
					table. 
					
	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Writes to PSKEY_LC_POWER_TABLE

*************************************************************************************/
LIBAPI(int) psWritePowerTable(uint32 handle, uint16 num_entries, uint8 * intPA, uint8 * extPA, int * power);

/*************************************************************************************
	
	Function Name :	int psReadPowerTable(uint32 handle,  int maxSize, int * num_entries, uint8 * intPA, uint8 * extPA, int * power);

	Parameters :	handle - handle to the device

					maxSize - size of the arrays to hold the intPA, extPA and power

					num_entries - pointer to the number of entries in the power table

					intPA - pointer to an array of unsigned char of size num_entries 
							containing the intPA entries for the power table. 
					
					extPA - pointer to an array of unsigned char of size num_entries 
							containing the extPA entries for the power table.
 
					power - pointer to an array of signed integers of size num_entries 
							containing the Output power entries, in dBm, for the power 
					table. 
					
	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Reads the PSKEY_LC_POWER_TABLE

*************************************************************************************/
LIBAPI(int) psReadPowerTable(uint32 handle,  int maxSize, int * num_entries, uint8 * intPA, uint8 * extPA, int * power);

/*************************************************************************************
	
	Function Name :	int psWriteVmDisable(uint32 handle, uint8 vmDisable);

	Parameters :	handle - handle to the device

					vmDisable - Boolean value where non-zero indicates that the VM 
					will be disabled after the next reset.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Writes to PSKEY_VM_DISABLE

*************************************************************************************/
LIBAPI(int) psWriteVmDisable(uint32 handle, uint8 vmDisable);

/*************************************************************************************
	
	Function Name :	int psReadVmDisable(uint32 handle, uint8 * vmDisable);

	Parameters :	handle - handle to the device

					vmDisable - Boolean value where non-zero indicates that the VM 
					is disabled. 

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Writes to PSKEY_VM_DISABLE

*************************************************************************************/
LIBAPI(int) psReadVmDisable(uint32 handle, uint8 * vmDisable);

/*************************************************************************************
	HCI FUNCTION SET
*************************************************************************************/

/*************************************************************************************
	
	Function Name :	int hciSlave(uint32 handle);

	Parameters :	handle - handle to the device

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Performs four steps:
					1. Sets the event filter to auto accept a connection request. 
					Implements the command HCI_Set_Event_Filter with the following
					parameters:
						0x02 - Connection  setup filter type
						0x00 - Allow connections from all devices
						0x02 - Auto accept the connection
					
					2. Implements the command HCI_WriteScan_Enable with the following
					parameter:
						0x03 - Inquiry scan enabled, Page scan enabled.

*************************************************************************************/
LIBAPI(int) hciSlave(uint32 handle);

/*************************************************************************************
	
	Function Name :	int hciSetEventFilterAutoacceptConnection(uint32 handle);

	Parameters :	handle - handle to the device


	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Sets the event filter to auto accept a connection request. 
					Implements the command HCI_Set_Event_Filter with the following
					parameters:
						0x02 - Connection  setup filter type
						0x00 - Allow connections from all devices
						0x02 - Auto accept the connection

*************************************************************************************/
LIBAPI(int) hciSetEventFilterAutoacceptConnection(uint32 handle);

/*************************************************************************************
	
	Function Name :	int hciWriteInquiryScanActivity(uint32 handle, uint16 inquiryscan_interval, uint16 inquiryscan_window);

	Parameters :	handle - handle to the device

					inquiryscan_interval - defines the amount of time between consecutive 
						inquiry scans.

					inquiryscan_window - defines the amount of time for the duration of 
						the inquiry scan.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure 

	Description :	Implements the command HCI_Write_Inquiryscan_activity.

*************************************************************************************/
LIBAPI(int) hciWriteInquiryScanActivity(uint32 handle, uint16 inquiryscan_interval, uint16 inquiryscan_window);

/*************************************************************************************
	
	Function Name :	int hciWritePageScanActivity(uint32 handle, uint16 pagescan_interval, uint16 pagescan_window);

	Parameters :	handle - handle to the device

					pagescan_interval - defines the amount of time between consecutive 
						page scans.

					pagescan_window - defines the amount of time for the duration of 
						the page scan.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Implements the command HCI_Write_Pagescan_activity.

*************************************************************************************/
LIBAPI(int) hciWritePageScanActivity(uint32 handle, uint16 pagescan_interval, uint16 pagescan_window);

/*************************************************************************************
	
	Function Name :	int hciWriteScanEnable(uint32 handle, uint8 scan_enable);

	Parameters :	handle - handle to the device

					scan_enable - controls whether or not the Bluetooth device will 
						periodically scan for page attempts and/or inquiry requests 
						from other Bluetooth devices.
						
							0x00 = No Scans enabled.
							0x01 = Inquiry Scan enabled. Page Scan disabled.
							0x02 = Inquiry Scan disabled. Page Scan enabled.
							0x03 = Inquiry Scan enabled. Page Scan enabled.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Implements the command HCI_WriteScan_Enable.

*************************************************************************************/
LIBAPI(int) hciWriteScanEnable(uint32 handle, uint8 scan_enable);

/*************************************************************************************
	
	Function Name :	int hciInquiry(uint32 handle, uint8 inquiryLength, uint8 numResponses, uint32 lap);

	Parameters :	handle - handle to the device

					inquiryLength - Maximum amount of time before the inquiry is 
					halted, as defined by the BT spec.

					numResponses - Maximum number of responses before inquiry is 
					halted, as defined in the BT spec. 0x00 is unlimited.

					lap - The LAP from which the inquiry access code is derived, as 
					defined in the BT spec.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Causes the device to enter inquiry mode. 
					This function will NOT wait for any inquiry results.

*************************************************************************************/
LIBAPI(int) hciInquiry(uint32 handle, uint8 inquiryLength, uint8 numResponses, uint32 lap = 0x9e8b33);

/*************************************************************************************
	
	Function Name :	int hciGetInquiryResults(uint32 handle, uint32 * lap, uint8 * uap, uint16 * nap, int maxLen, int * len);

	Parameters :	handle - handle to the device

					lap - pointer to the array of lap's as defined by the returned value.

					uap - pointer to the array of uap's as defined by the returned value.

					nap - pointer to the array of nap's as defined by the returned value.

					maxLen - size of the arrays allocated for the lap, uap and nap.

					len - pointer to a variable to hold the actual data returned with 
						the lap, uap and nap.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Retrieves the bd_addrs of the devices found during the last 
					completed inquiry. If an inquiry is in progress then this function 
					will wait for a time based on the inquiryLength parameter passed 
					to do_inquiry().

*************************************************************************************/
LIBAPI(int) hciGetInquiryResults(uint32 handle, uint32 * lap, uint8 * uap, uint16 * nap, int maxLen, int * len);

/*************************************************************************************
	
	Function Name :	int hciInquiryCancel(uint32 handle);

	Parameters :	handle - handle to the device

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Causes the device to exit inquiry mode. The DLL will check if the 
					device is currently in inquiry before issuing this.Failure on 
					invalid handle or DM commands enabled.

*************************************************************************************/
LIBAPI(int) hciInquiryCancel(uint32 handle);

/*************************************************************************************
	
	Function Name :	int hciCreateConnection(uint32 handle, uint32 lap, uint8 uap, uint16 nap, uint16 pkt_type, uint16 * connectionHandle);

	Parameters :	handle - handle to the device

					lap - lap of bd_addr. Must be one of the bd_addr's returned by 
						get_inquiry_results.

					uap - uap of bd_addr. Must be one of the bd_addr's returned by 
						get_inquiry_results.

					nap - nap of bd_addr. Must be one of the bd_addr's returned by 
						get_inquiry_results.

					pkt_type - Or-ed combination of packet types as defined in the 
						Bluetooth specification.
							HCI_PKT_DM1 ((uint16)0x0008) SCO and ACL
							HCI_PKT_DH1 ((uint16)0x0010) ACL only
							HCI_PKT_DM3 ((uint16)0x0400) ACL only
							HCI_PKT_DH3 ((uint16)0x0800) ACL only
							HCI_PKT_DM5 ((uint16)0x4000) ACL only
							HCI_PKT_DH5 ((uint16)0x8000) ACL only
						BlueChat default is:
						(HCI_PKT_DH1|HCI_PKT_DH3|HCI_PKT_DH5|HCI_PKT_DM1|HCI_PKT_DM3|HCI_PKT_DM5)

					connectionHandle - connection handle to a Bluetooth connection. 
						NOTE: The connection handle is local to a device and not a link. 
						Therefore when you create a connection each member could have a 
						different connection handle for that link so the value returned by 
						this function should    only be used be this device.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure
					

	Description :	Causes the device to attempt to create a connection with a device 
					discovered during the inquiry phase. This function will wait for a 
					connection complete event with a 3 second time out. Success is 
					determined by:
						1. Valid handle
						2. Successful connection complete event
						3. Invalid BD_ADDR (not part of the inquiry results list).

*************************************************************************************/
LIBAPI(int) hciCreateConnection(uint32 handle, uint32 lap, uint8 uap, uint16 nap, uint16 pkt_type, uint16 * connectionHandle);

/*************************************************************************************
	
	Function Name :	int hciCreateConnectionNoInquiry(uint32 handle, uint32 lap, uint8 uap, uint16 nap, uint16 pkt_type, uint16 * connectionHandle);

	Parameters :	handle - handle to the device

					lap - lap of bd_addr.

					uap - uap of bd_addr.

					nap - nap of bd_addr.

					pkt_type - as do_create_connection(...)

					connectionHandle - connection handle to a Bluetooth connection. 
						NOTE: The connection handle is local to a device and not a link. 
						Therefore when you create a connection each member could have a 
						different connection handle for that link so the value returned by 
						this function should    only be used be this device.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Causes the device to attempt to create a connection with a remote 
					device. This function will wait for a connection complete event 
					with a 5 second time out. Success is 
					determined by:
						1. Valid handle
						2. Successful connection complete event
						3. Invalid BD_ADDR (not part of the inquiry results list).

*************************************************************************************/
LIBAPI(int) hciCreateConnectionNoInquiry(uint32 handle, uint32 lap, uint8 uap, uint16 nap, uint16 pkt_type, uint16 * connectionHandle);

/*************************************************************************************
	
	Function Name :	int hciCreateScoConnection(uint32 handle, uint16 aclConnectionHandle, uint16 pkt_type, uint16 * scoConnectionHandle);

	Parameters :	handle - handle to the device

					aclConnectionHandle - ACL connection handle.

					pkt_type - Or-ed combination of packet types as defined in the 
						Bluetooth specification.
							HCI_PKT_HV1 ((uint16)0x0020)
							HCI_PKT_HV2 ((uint16)0x0040)
							HCI_PKT_HV3 ((uint16)0x0080)

					scoConnectionHandle - pointer to a variable to hold the returned SCO 
						connection handle.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure 

	Description :	This command will cause the link manager to create a SCO connection 
					using the ACL connection specified by the Connection_Handle 
					command parameter. Success is determined by:
						1. Valid handle
						2. Successful connection complete event

*************************************************************************************/
LIBAPI(int) hciCreateScoConnection(uint32 handle, uint16 aclConnectionHandle, uint16 pkt_type, uint16 * scoConnectionHandle);

/*************************************************************************************
	
	Function Name :	int hciWriteLinkPolicySettings(uint32 handle, uint16 connection_handle, uint16 link_policy_settings);

	Parameters :	handle - handle to the device

					connection_handle - Bluetooth connection handle to remote device.

					link_policy_settings - determines the behavior of the local Link 
						Manager when it receives a request from a remote device or it 
						determines itself to change the master-slave role or to enter 
						the hold, sniff, or park mode.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Causes the device to attempt to write the link policy settings.
					Success determined by:
						1. Valid handle
						2. Valid connection handle

*************************************************************************************/
LIBAPI(int) hciWriteLinkPolicySettings(uint32 handle, uint16 connection_handle, uint16 link_policy_settings);

/*************************************************************************************
	
	Function Name :	int hciSniffMode(uint32 handle, uint16 connection_handle, uint16 sniff_max_interval, uint16 sniff_min_interval, uint16 sniff_attempt, uint16 sniff_timeout);

	Parameters :	handle - handle to the device

					connection_handle - Bluetooth connection handle to remote device.

					sniff_max_interval - Maximum acceptable number of Baseband slots 
						between each sniff period.

					sniff_min_interval - Minimum acceptable number of Baseband slots 
						between each sniff period.

					sniff_attempt - Number of Baseband receive slots for sniff attempt.

					sniff_timeout - Number of Baseband receive slots for sniff timeout.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Causes the device to attempt to enter Sniff Mode with a remote device.
					Success determined by:
						1. Valid handle
						2. Valid connection handle

*************************************************************************************/
LIBAPI(int) hciSniffMode(uint32 handle, uint16 connection_handle, uint16 sniff_max_interval, uint16 sniff_min_interval, uint16 sniff_attempt, uint16 sniff_timeout);

/*************************************************************************************
	
	Function Name :	int hciExitSniff(uint32 handle, uint16 connection_handle);

	Parameters :	handle - handle to the device

					connection_handle - Bluetooth connection handle to remote device.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Causes the device to attempt to exit Sniff Mode with a remote device.
					Success determined by:
						1. Valid handle
						2. Valid connection handle
						5. DM commands not in use

*************************************************************************************/
LIBAPI(int) hciExitSniff(uint32 handle, uint16 connection_handle);

/*************************************************************************************
	
	Function Name :	int hciDisconnect(uint32 handle, uint16 connection_handle);

	Parameters :	handle - handle to the device

					connection_handle - Bluetooth connection handle to remote device.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Causes the device to request a disconnect from a remote Bluetooth. 
					The DLL will wait, with a 5 second timeout, for the disconnect 
					event to be returned. The memory for all disconnected devices will 
					be freed for this handle. If the remote device is also connected 
					via TestEngine it is important to run hciPollDisconnectComplete(..) 
					to confirm the correct operation and to free any memory on that 
					handle.
					Success is determined by:
						1. Valid handle
						2. Valid active connection_handle
						3. Successful completion of disconnect.

*************************************************************************************/
LIBAPI(int) hciDisconnect(uint32 handle, uint16 connection_handle);

/*************************************************************************************
	
	Function Name :	int hciGetConnectionHandle(uint32 handle, uint32 lap, uint8 uap, uint16 nap, uint16 * connectionHandle);


	Parameters :	handle - handle to the device

					lap - lap of bd_addr. 

					uap - uap of bd_addr. 

					nap - nap of bd_addr. 

					connectionHandle - pointer to a variable which will hold the returned 
						connection handle.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Returns the connection handle, if existing, between the device 
					identified by the "handle" parameter and the device identified by 
					the Bluetooth address. Failure on invalid handle and DM commands 
					enabled

*************************************************************************************/
LIBAPI(int) hciGetConnectionHandle(uint32 handle, uint32 lap, uint8 uap, uint16 nap, uint16 * connectionHandle);

/*************************************************************************************
	
	Function Name :	int hciConnectionStatus(uint32 handle, uint16 connection_handle);


	Parameters :	handle - handle to the device

					connection_handle - Bluetooth connection handle to remote device.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Checks whether the connection handle points to a valid, active 
					Bluetooth connection.Failure on invalid handle and DM commands 
					enabled.

*************************************************************************************/
LIBAPI(int) hciConnectionStatus(uint32 handle, uint16 connection_handle);

/*************************************************************************************
	
	Function Name :	int hciEnableDeviceUnderTestMode(uint32 handle);

	Parameters :	handle - handle to the device

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Causes the device to send the enable device under test mode.
					Success is determined by:
						1. Valid handle
						2. Command complete within timeout

*************************************************************************************/
LIBAPI(int) hciEnableDeviceUnderTestMode(uint32 handle);

/*************************************************************************************
	
	Function Name :	int hciGetLinkQuality(uint32 handle, uint16 connection_handle, uint8 * quality);

	Parameters :	handle - handle to the device

					connection_handle - Bluetooth connection handle to remote device.

					quality - pointer to the link quality parameter returned by this function.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Reads the link quality

*************************************************************************************/
LIBAPI(int) hciGetLinkQuality(uint32 handle, uint16 connection_handle, uint8 * quality);

/*************************************************************************************
	
	Function Name :	int hciReadRemoteVersionInformation(uint32 handle, uint16 connection_handle, uint32 * r);

	Parameters :	handle - handle to the device

					connection_handle - Bluetooth connection handle to remote device.

					r - a 4 member array of unsigned 32 bit integers. On success r is:
						r[0] = connection_handle
						r[1] = lmp_version, as defined by the Bluetooth spec.
						r[2] = Manufacturer name.
						r[3] = lmp_subversion
					With BlueCore the lmp_subversion is the firmware build ID.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Reads the remoted version information.

*************************************************************************************/
LIBAPI(int) hciReadRemoteVersionInformation(uint32 handle, uint16 connection_handle, uint32 * r);

/*************************************************************************************
	
	Function Name :	int hciReadRemoteNameRequest(uint32 handle, uint32 lap, uint8 uap, uint16 nap, uint8 page_scan_repetition_mode, uint8 page_scan_offset, uint16 clock_offset, char * r);

	Parameters :	handle - handle to the device

					lap - lap of bd_addr. 

					uap - uap of bd_addr. 

					nap - nap of bd_addr. 
					
					page_scan_repetition_mode - page scan mode supported by the remote 
						device.

					page_scan_offset - page scan mode supported by the remote 
						device.

					clock_offset - The clock offset between the devices.

					r - pointer to the remote name returned by the command.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Reads the remoted name.

*************************************************************************************/
LIBAPI(int) hciReadRemoteNameRequest(uint32 handle, uint32 lap, uint8 uap, uint16 nap, uint8 page_scan_repetition_mode, uint8 page_scan_offset, uint16 clock_offset, char * r);

/*************************************************************************************
	DM ONLY COMMAND SET
*************************************************************************************/

/*************************************************************************************
	
	Function Name :	int dmRegisterReq(uint32 handle);

	Parameters :	handle - handle to the device

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Causes the device to register the application manager queue  
					command. No other DM commands will function unless this is called.
					Success is determined by:
						1. Valid handle
						2. Command complete within timeout
						3. non-HCI firmware

*************************************************************************************/
LIBAPI(int) dmRegisterReq(uint32 handle);

/*************************************************************************************
	
	Function Name :	int dmSlave(uint32 handle);

	Parameters :	handle - handle to the device

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Performs four steps:
					1. Sets the event filter to auto accept a connection request. 
					Implements the command DM_HCI_Set_Event_Filter with the following
					parameters:
						0x02 - Connection  setup filter type
						0x00 - Allow connections from all devices
						0x02 - Auto accept the connection
					
					2. Implements the command DM_HCI_WriteScan_Enable with the following
					parameter:
						0x03 - Inquiry scan enabled, Page scan enabled.

*************************************************************************************/
LIBAPI(int) dmSlave(uint32 handle);

/*************************************************************************************
	
	Function Name :	int dmEnableDeviceUnderTestMode(uint32 handle);

	Parameters :	handle - handle to the device

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Causes the device to send the enable device under test mode.
					Success is determined by:
						1. Valid handle
						2. Command complete within timeout

*************************************************************************************/
LIBAPI(int) dmEnableDeviceUnderTestMode(uint32 handle);

/*************************************************************************************
	
	Function Name :	int dmWritePageScanActivity(uint32 handle, uint16 pagescan_interval, uint16 pagescan_window);

	Parameters :	handle - handle to the device

					pagescan_interval - defines the amount of time between consecutive 
						page scans.

					pagescan_window - defines the amount of time for the duration of 
						the page scan.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Implements the command DM_HCI_Write_Pagescan_activity.

*************************************************************************************/
LIBAPI(int) dmWritePageScanActivity(uint32 handle, uint16 pagescan_interval, uint16 pagescan_window);

/*************************************************************************************
	
	Function Name :	int dmWriteInquiryScanActivity(uint32 handle, uint16 inquiryscan_interval, uint16 inquiryscan_window);

	Parameters :	handle - handle to the device

					inquiryscan_interval - defines the amount of time between consecutive 
						inquiry scans.

					inquiryscan_window - defines the amount of time for the duration of 
						the inquiry scan.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure 

	Description :	Implements the command DM_HCI_Write_Inquiryscan_activity.

*************************************************************************************/
LIBAPI(int) dmWriteInquiryScanActivity(uint32 handle, uint16 inquiryscan_interval, uint16 inquiryscan_window);

/*************************************************************************************
	
	Function Name :	int dmWriteScanEnable(uint32 handle, uint8 scan_enable);

	Parameters :	handle - handle to the device

					scan_enable - controls whether or not the Bluetooth device will 
						periodically scan for page attempts and/or inquiry requests 
						from other Bluetooth devices.
						
							0x00 = No Scans enabled.
							0x01 = Inquiry Scan enabled. Page Scan disabled.
							0x02 = Inquiry Scan disabled. Page Scan enabled.
							0x03 = Inquiry Scan enabled. Page Scan enabled.

	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Implements the command DM_HCI_WriteScan_Enable.

*************************************************************************************/
LIBAPI(int) dmWriteScanEnable(uint32 handle, uint8 scan_enable);

/*************************************************************************************
	
	Function Name :	int dmSetEventFilterAutoacceptConnection(uint32 handle);

	Parameters :	handle - handle to the device


	Returns :		-1 on invalid handle, 1 on success, 0 on failure

	Description :	Sets the event filter to auto accept a connection request. 
					Implements the command DM_HCI_Set_Event_Filter with the following
					parameters:
						0x02 - Connection  setup filter type
						0x00 - Allow connections from all devices
						0x02 - Auto accept the connection

*************************************************************************************/
LIBAPI(int) dmSetEventFilterAutoacceptConnection(uint32 handle);

#endif /* __INCTestEngineh */
