///////////////////////////////////////////////////////////////////////
//
//  FILE   :  blueCoreTransport.h
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  BlueCoreTransport
//
//  PURPOSE:  Provide a host-side API for the transport of HCI commands
//            /events, ACL packets, SCO packets and manufacturer
//            extension PDUs supported by BC01. This class inherits the
//            top API from the HCITransport class and adds optional
//            support for HCI tunnelling, controlled by the "tunnel"
//            member variable (disabled by default). It is recommended
//            that non-BCSP transports that use this as a base set this
//            variable within their constructor to enable tunnelling.
//
//            When tunnelling is enabled, manufacturer extension PDUs,
//            i.e. anything other than HCI/ACL/SCO data, are repackaged
//            as HCI commands using OGF = 0x3f and OCF = 0x00 (vendor-
//            specific debug commands). In addition, any Command
//            Complete events corresponding to this opcode are
//            discarded.
//
//            Regardless of whether tunnelling is enabled, any HCI
//            events received with code 0xFF (vendor specific debug
//            events) are decoded as HCI tunnelling from Host Controller
//            to Host, and the payload reprocessed as appropriate.
//
//            BCSP / HCI tunnelling channels are used as an enum to
//            allow the HCI, ACL, SCO and manufacturers extension
//            channels to be uniquely identified. A subset of these
//            constants are used directly within the payload descriptor
//            byte of HCI tunnelling commands/events, so the actual
//            numerical values are important.
//            
//            If the stack is started with out the callbacks in place,
//            then data arriving in the intervening time will be lost.
//            Since this is likely to be the HCI initialisation packet,
//            that would be bad...
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/TransportLayer/blueCoreTransport.h#21 $
//
//  MODIFICATION HISTORY:
//  1.5   7:Sep:00  at  Added modification history.
//                      Additional enum constants and send... functions
//                      included for all defined BCSP channels.
//                      BCSP channels can optionally be tunnelled via
//                      manufacturers extension HCI commands/events.
//                      Increased supported channels from 16 (BCSP) to
//                      64 (HCI tunnelling).
//
///////////////////////////////////////////////////////////////////////

#ifndef __BLUECORETRANSPORT_H__
#define __BLUECORETRANSPORT_H__

#include "HCITransport.h"
#include "tunnel.h"

enum BLUECORE_CHANNELS
{
    zero = 0,
    one = 1,
    le = 1,
    two = 2,
    bccmd = 2,
    h2hc = 2,
    three = 3,
    hq = 3,
    hc2h = 3,
    four = 4,
    deviceMgt = 4,
    five = 5,
    hciCommand = 5,
    six = 6,
    hciACL = 6,
    seven = 7,
    hciSCO = 7,
    eight = 8,
	l2cap = 8,
    nine = 9,
	rfcomm = 9,
    ten = 10,
	sdp = 10,
    eleven = 11,
    debug = 11,
    twelve = 12,
	upgrade = 12,
    thirteen = 13,
	tcs = 13,
    fourteen = 14,
    fifteen = 15,
    BLUECORE_CHANNEL_COUNT = 16,
	BLUECORE_HCI_TUNNEL_COUNT = 64
};

enum direction { upstream, downstream };

// HCI tunnelling command and event codes
#define HCI_TUNNEL_COMPLETE_EVENT (0x0E)

// HCI tunnelling payload descriptor flags and masks
#define HCI_TUNNEL_CHANNEL_MASK (0x3F)
#define HCI_TUNNEL_FIRST (1 << 6)
#define HCI_TUNNEL_LAST (1 << 7)
#define HCI_TUNNEL_MAX_PAYLOAD (254)

typedef struct TunnelSendInfo TunnelSendInfo;

typedef struct
{
	uint8 *pdu;
	uint32 length;
} TunnelRecvInfo;

class DEVICECONTROLLER_API_CLASS BlueCoreTransport : public HCITransport
{
public:
    BlueCoreTransport();
    virtual ~BlueCoreTransport();

    void setOnLinkFail( void (*aFunction) (void*) , void * aParameter );
    static void onLinkFail(void * aObject);

    void setupRecvHCReply ( void * aObject,
                            void (*aRecvCallBack) ( uint8 * , uint32 , void * ) );
    void setupRecvCHEvent ( void * aObject,
                            void (*aRecvCallBack) ( uint8 * , uint32 , void * ) );
    void setupRecvDebugData ( void * aObject,
                            void (*aRecvCallBack) ( uint8 * , uint32 , void * ) );
    void setupRecvData (BLUECORE_CHANNELS channelNumber , void * aObject,
                            void (*aRecvCallBack) ( uint8 * , uint32 , void * ) );
    void sendCommand  ( uint8 *apdu, uint16 aLength , CallBack *aCallBack ) ;
    void sendACLData  ( uint8 *apdu, uint32 aLength , CallBack *aCallBack ) ;
    void sendSCOData  ( uint8 *apdu, uint16 aLength , CallBack *aCallBack ) ;
    void sendbccmd    ( uint8 *apdu, uint16 aLength , CallBack *aCallBack ) ;
    void sendhq       ( uint8 *apdu, uint16 aLength , CallBack *aCallBack ) ;
    void senddeviceMgt( uint8 *apdu, uint32 aLength , CallBack *aCallBack ) ;
    void sendl2cap    ( uint8 *apdu, uint32 aLength , CallBack *aCallBack ) ;
    void sendrfcomm   ( uint8 *apdu, uint32 aLength , CallBack *aCallBack ) ;
    void sendsdp      ( uint8 *apdu, uint32 aLength , CallBack *aCallBack ) ;
    void senddebug    ( uint8 *apdu, uint32 aLength , CallBack *aCallBack ) ;
    void sendupgrade  ( uint8 *apdu, uint32 aLength , CallBack *aCallBack ) ;
    void sendtcs      ( uint8 *apdu, uint32 aLength , CallBack *aCallBack ) ;
	
    bool tunnel;
    bool discardDummyEvents;

    void setMagicSentThisCallBack ( void (*aCallBack) ( direction , BLUECORE_CHANNELS , const uint8* , uint32 , void* ) ,
                                    void * parameters );
    
protected:
    void recvData    ( BLUECORE_CHANNELS channel, uint8 *apdu, uint32 aLength );

    void *mCallBackObjects[BLUECORE_HCI_TUNNEL_COUNT];
    void  (*mCallBackFunctions[BLUECORE_HCI_TUNNEL_COUNT]) ( uint8 *, uint32 , void * );

    void (*mUserOnLinkFailFn) (void *);
    void *mUserOnLinkFailParams;
    bool linkFailed;

private:
    virtual void sendpdu     ( BLUECORE_CHANNELS channel, uint8 *apdu, uint32 aLength , CallBack *aCallBack ) = 0;
    void sendTunnel ( BLUECORE_CHANNELS channel, uint8 *apdu, uint32 aLength , CallBack *aCallBack );
    void recvTunnel ( uint8 *apdu, uint32 aLength );
    TunnelRecvInfo mRecvTunnel [BLUECORE_HCI_TUNNEL_COUNT];

    void (*MagicCallBack) ( direction , BLUECORE_CHANNELS , const uint8* , uint32 , void* );
    void * MagicParameters;
    inline void magicsendpdu ( BLUECORE_CHANNELS channel, uint8 *apdu, uint32 aLength , CallBack *aCallBack );
};

class TunnelledPacket
{
public:
    TunnelledPacket ( BlueCoreTransport *aTransport , uint32 length , CallBack *aCallBack )
    : transport ( aTransport ) , length_left ( length ) , callBack ( aCallBack )
    {}
    static void sendTunnelCallback ( bool done, void *aObject );
    void sendFragment (TunnelSendInfo *info);
private:
    BlueCoreTransport *transport;
    uint32 length_left;
    CallBack *callBack;
};

struct TunnelSendInfo
{
    uint8 *pdu_frag;
    uint32 length_sent;
    TunnelledPacket* p;
};

#endif // __BLUECORETRANSPORT_H__
