#ifndef __INCTestFlashh
#define __INCTestFlashh

#undef LIBAPI
#define LIBAPI(T) extern "C" T _stdcall

/*************************************************************************************
*	
*	Function Name :	int init_flash(int port, int xtal, int delats);
*
*	Parameters :
*					port - Number refering to the LPT port to be used
*
*					xtal - specifies module's crystal frequency. Acceptable values are 
*						10, 16, 26, 32. Higner values make code run slower, lower 
*						values may cause instability. (Casiras use 16Mhz).
*
*					delays - delay used in driving the SPI port.  1 = fastest SPI 
*						speed, but may give unreliablilty.
*						2 = reasonable value,
*						higer values may be better if chip is in deep sleep.
*	
*
*	Returns :		non-zero on failure, 0 on success.
*
*	Description :	This function is used to create the appropriate host side objects 
*					to enable flash programming via the SPI or gang programming 
*					interface.
*
*					No communication with BlueCore is carried out
*
*************************************************************************************/
LIBAPI(int) init_flash(int port, int xtal, int delays);

/*************************************************************************************
*	
*	Function Name :	int read_program_files(char * filename);
*
*	Parameters :
*					filename - string containing the directory and filename root. This
*						function will generate the XPV and XDV extensions.
*
*	Returns :		non-zero on failure, 0 on success.
*
*	Description :	This function is used to load the firmware files into memory.
*
*************************************************************************************/
LIBAPI(int) read_program_files(char * filename);

/*************************************************************************************
*	
*	Function Name :	int program_flash_block(void);
*
*	Parameters :	none
*
*	Returns :		0 on success, non-zero on failure where:
*						FLASH_ERROR_NONE = 0
*						FLASH_ERROR_DOWNLOAD_FAILED = 1
*						FLASH_ERROR_VERIFY_FAILED = 2
*						FLASH_ERROR_TIMED_OUT = 3
*						FLASH_ERROR_CRC_FAILED = 4
*						FLASH_ERROR_READBACK_FAILED = 5
*						FLASH_ERROR_COULD_NOT_DOWNLOAD_PROG = 6
*						FLASH_ERROR_COULD_NOT_STOP_XAP = 7
*						FLASH_ERROR_BOOT_PROG_HALTED = 8
*						FLASH_ERROR_ERASE_FAILED = 9
*						FLASH_ERROR_XAP_ERROR = 10
*
*	Description :	This function is used to download the firmware to the device under 
*					test. The function will block until the process has completed 
*					successfully or an error occurs.
*
*************************************************************************************/
LIBAPI(int) program_flash_block(void);

/*************************************************************************************
*	
*	Function Name :	int erase_flash(void);
*
*	Parameters :	none
*
*	Returns :		0 on success, non-zero on failure where:
*						FLASH_ERROR_NONE = 0
*						FLASH_ERROR_DOWNLOAD_FAILED = 1
*						FLASH_ERROR_VERIFY_FAILED = 2
*						FLASH_ERROR_TIMED_OUT = 3
*						FLASH_ERROR_CRC_FAILED = 4
*						FLASH_ERROR_READBACK_FAILED = 5
*						FLASH_ERROR_COULD_NOT_DOWNLOAD_PROG = 6
*						FLASH_ERROR_COULD_NOT_STOP_XAP = 7
*						FLASH_ERROR_BOOT_PROG_HALTED = 8
*						FLASH_ERROR_ERASE_FAILED = 9
*						FLASH_ERROR_XAP_ERROR = 10
*
*	Description :	This function is used to erase the flash.
*
*************************************************************************************/
LIBAPI(int) erase_flash(void);

/*************************************************************************************
*	
*	Function Name :	int gang_program_flash_block(unsigned short device_mask, bool eraseFirst, bool restartAfter);
*
*	Parameters :	device_mask - Specifies which devices will be programmed. Uses a 
*						bit-field so 5 = 0000000000000101 = devices 0 and 2. A value 
*						of 0 (zero) will auto-detect and programme all devices present.
*
*					eraseFirst - will erase the entire flash of all devices before 
*						programming.
*
*					restartAfter - will restart the XAP after completion of the flash process
*
*	Returns :		0 on success, non-zero on failure where:
*						FLASH_ERROR_NONE = 0
*						FLASH_ERROR_DOWNLOAD_FAILED = 1
*						FLASH_ERROR_VERIFY_FAILED = 2
*						FLASH_ERROR_TIMED_OUT = 3
*						FLASH_ERROR_CRC_FAILED = 4
*						FLASH_ERROR_READBACK_FAILED = 5
*						FLASH_ERROR_COULD_NOT_DOWNLOAD_PROG = 6
*						FLASH_ERROR_COULD_NOT_STOP_XAP = 7
*						FLASH_ERROR_BOOT_PROG_HALTED = 8
*						FLASH_ERROR_ERASE_FAILED = 9
*						FLASH_ERROR_XAP_ERROR = 10
*						FLASH_ERROR_UNKNOWN_CHIP_TYPE = 11
*						FLASH_ERROR_BROADCAST_MIXED_CHIP_TYPES = 12
*					
*					If all the specified devices were programmed correctly the function 
*					will return FLASH_ERROR_NONE, if there were 1 or more programming 
*					failures then the function will return 
*					FLASH_ERROR_COULD_NOT_DOWNLOAD_PROG. All other error returns denote 
*					a gross failure.
*
*	Description :	This function is used to download the firmware to up to 16 devices 
*					when using the gang programmer reference device. This function 
*					will block execution until the process completes successfully or 
*					an error occurs.
*
*					On completion with an error return of either FLASH_ERROR_NONE or 
*					FLASH_ERROR_COULD_NOT_DOWNLOAD_PROG the programmer should call 
*					get_bit_error_field() to determine which device has failed to 
*					programmme correctly.
*
*************************************************************************************/
LIBAPI(int) gang_program_flash_block(unsigned short device_mask, bool eraseFirst, bool restartAfter);

/*************************************************************************************
*	
*	Function Name :	unsigned short get_bit_error_field(void);
*
*	Parameters :	none
*
*	Returns :		Bitfield refering to the devices present on the gang programmer 
*					and those that have failed. Will return (failed_devices & device_mask). 
*					Therefore a return of zero (0) means all devices have been 
*					programmed successfully. A return value of five (5) denotes that 
*					devices 0 and 2 have failed and the remainder have successfully 
*					been programmed.
*
*	Description :	This function is used to determine which devices have failed to be 
*					programmed during the gang_programming process.
*
*************************************************************************************/
LIBAPI(unsigned short) get_bit_error_field(void);

/*************************************************************************************
*	
*	Function Name :	int program_flash_spawn(void);
*
*	Parameters :	none
*
*	Returns :		0 on success, non-zero on failure where:
*						FLASH_ERROR_NONE = 0
*						FLASH_ERROR_DOWNLOAD_FAILED = 1
*						FLASH_ERROR_VERIFY_FAILED = 2
*						FLASH_ERROR_TIMED_OUT = 3
*						FLASH_ERROR_CRC_FAILED = 4
*						FLASH_ERROR_READBACK_FAILED = 5
*						FLASH_ERROR_COULD_NOT_DOWNLOAD_PROG = 6
*						FLASH_ERROR_COULD_NOT_STOP_XAP = 7
*						FLASH_ERROR_BOOT_PROG_HALTED = 8
*						FLASH_ERROR_ERASE_FAILED = 9
*						FLASH_ERROR_XAP_ERROR = 10
*
*	Description :	This function is used to download the firmware to the device under 
*					test. The function will spawn the process which performs the flash 
*					programming. The programmer can then use further functions to 
*					check the progress and to retrieve the final error code.
*
*************************************************************************************/
LIBAPI(int) program_flash_spawn(void);



LIBAPI(int) gang_program_flash_spawn(unsigned short device_mask, bool eraseFirst, bool restartAfter);


/*************************************************************************************
*	
*	Function Name :	int get_spawned_progress(void);
*
*	Parameters :	none
*
*	Returns :		Progress so far in the range 0 to 100. Where 100 indicates the 
*					spawned programming process has returned with or without an error.
*
*	Description :	This function is used to check the progress of the process spawned 
*					by program_flash_spawn().
*
*************************************************************************************/
LIBAPI(int) get_spawned_progress(void);

/*************************************************************************************
*	
*	Function Name :	int get_spawned_error(void);
*
*	Parameters :	none
*
*	Returns :		0 on success, non-zero on failure where:
*						FLASH_ERROR_NONE = 0
*						FLASH_ERROR_DOWNLOAD_FAILED = 1
*						FLASH_ERROR_VERIFY_FAILED = 2
*						FLASH_ERROR_TIMED_OUT = 3
*						FLASH_ERROR_CRC_FAILED = 4
*						FLASH_ERROR_READBACK_FAILED = 5
*						FLASH_ERROR_COULD_NOT_DOWNLOAD_PROG = 6
*						FLASH_ERROR_COULD_NOT_STOP_XAP = 7
*						FLASH_ERROR_BOOT_PROG_HALTED = 8
*						FLASH_ERROR_ERASE_FAILED = 9
*						FLASH_ERROR_XAP_ERROR = 10
*
*	Description :	This function will return the last error generated by the flash 
*					programming stack. If called after get_spawned_progress() returns 
*					100 it can be used to check for programming errors.
*
*************************************************************************************/
LIBAPI(int) get_spawned_error(void);

/*************************************************************************************
*	
*	Function Name :	int reset_and_start(void);
*
*	Parameters :	none
*
*	Returns :		0 on success, -1 on failure
*
*	Description :	This function is used to reset the device after the spawned 
*					programming funtion has completed.
*
*************************************************************************************/
LIBAPI(int) reset_and_start(void);

/*************************************************************************************
*	
*	Function Name :	void close_flash(void);
*
*	Parameters :	none
*
*	Returns :		none
*
*	Description :	Deletes all the memory associated with the flash programming 
*					process.
*
*************************************************************************************/
LIBAPI(void) close_flash(void);

/*************************************************************************************
*	
*	Function Name :	void get_version(void);
*
*	Parameters :	none
*
*	Returns :		Version string for this DLL.
*
*	Description :	Returns the version string for this DLL.
*
*************************************************************************************/
LIBAPI(int) get_version(char * versionStr);

/************************************************************************************
	EXAMPLE CODE
************************************************************************************/

/************************************************************************************
1. Programming a single device via the SPI. Using LPT1 and a xtal frequence of 16 (in
keeping with a Casira) using the blocking function.

char * filename = "c:\\16_4\\bc02";
if(!init_flash(1, 16))
{
	if(!read_program_files(filename))
		error = program_flash_block(); // This will block until the programming completes
}
close_flash();

  ==========================================================================

2. Programming a single device via the SPI. Using LPT1 and a xtal frequence of 16 (in
keeping with a Casira) using the spawned function.

char * filename = "c:\\16_4\\bc02";
if(!init_flash(1, 16))
{
	if(!read_program_files(filename))
		error = program_flash_spawn();
	if(!error)
	{
		do
		{
			progress = get_spawned_progress();
			::Sleep(250); // Choose a suitable time slice
		}while(progress != 100);
		error = get_spawned_error();
		reset_and_start();
	}
}
close_flash();

  ==========================================================================

3. Using the gang programmer to programme all detected devices.

char * filename = "c:\\16_4\\bc02";
if(!init_flash(1, 16))
{
	if(!read_program_files(filename))
		error = gang_program_flash_block(); // This will block until the programming completes
	if(error == 0 || error == 6)
	{
		bitfield = get_bit_error_field();
		// Parse bitfield to find the failed devices
	}
}
close_flash();

**************************************************************************************/

#endif /* __INCTestFlashh */
