#ifndef __INCTestPSh
#define __INCTestPSh

#undef LIBAPI
#define LIBAPI(T) extern "C" T _stdcall

/*************************************************************************************
*	
*	Function Name :	int open_ps(int port, int device);
*
*	Parameters :
*					port - Number refering to the LPT port to be used
*
*					device - Number refering to the device to be communicated with, 0 
*						to 15 refering to the port on the CSR gang programmer. If the 
*						gang programmer is not used then this should be set to 0.
*
*	Returns :		non-zero on success, 0 on failure.
*
*	Description :	This function is used to create the appropriate host side objects 
*					to enable PS reads and writes via the SPI or gang programming 
*					interface. This function will have absolute control of the SPI 
*					interface so care should be taken that the interface has not been 
*					allocated elsewhere, for example by init_flash(..).
*
*************************************************************************************/
LIBAPI(int) open_ps(int port, int device);

/*************************************************************************************
*	
*	Function Name :	int close_ps();
*
*	Parameters :	none
*
*	Returns :		1.
*
*	Description :	This function is used to free any objects associated with open_ps().
*
*************************************************************************************/
LIBAPI(int) close_ps();

/*************************************************************************************
	PS_STORES - the BlueCore persistent store is conceptually a four layer model. When 
	the BlueCore firmware initialises or reads a persistent store value it will always 
	read it from the top most layer where that PS entry is present. When writing or 
	reading to the persistent store in production it is possible to select the PS 
	store layer or let the firmware decide. This selection is carried out by the use of
	the 'stores' parameter of all the PS read and write functions. This parameter is a 
	bit mask so multiple layers can be selected. The layers are defined as follows 
	(bottom to top):

		1. ROM layer - the bottom layer in the model, basically these are the hard 
		coded firmware defaults. This is a read-only layer so all PS writes to this 
		layer will fail. Note that not all PS entries will have a default value so in 
		some cases a PS read will also fail.

		2. Factory-set layer - this layer is not present on BlueCore ROM devices unless
		an external EEPROM is present. PS keys are moved to this layer when 
		"ps_factory_set()" is called. This layer can also be written to and read from 
		directly by selecting this layer as the 'stores' parameter.

		3. Implementation - this layer is not present on BlueCore ROM devices unless
		an external EEPROM is present. This layer is where most PS keys are written 
		during production for BlueCore Flash, External and ROM devices with an EEPROM.
		In these cases calling "ps_factory_set()" will move the PS values from this 
		layer to the factory-set layer. Calling "ps_factory_restore_all()" will erase 
		the implemention layer and expose the factory-set layer. When using BlueCore 
		Flash, External and ROM devices with an EEPROM if this layer is selected then
		the function will fail if the flash or EEPROM is read-only, for example if 
		there is a fault on the flash write-enable.

		4. Transient - this layer is the BlueCore RAM. By definition a cold reset or 
		power down reset will clear this layer. A warm reset will retain this layer.
		This layer exists on all BlueCore devices.

	The bitmasks to define these layers are shown below. If no bits are set (0), then
	the firmware will select the layer. If an external storage device is present and 
	functioning correctly then the firmware will write to the implemention layer (this 
	includes BlueCore Flash), otherwise the firmware will write to the transient layer.
**************************************************************************************/
#define	PS_STORES_DEFAULT (0x0000)	/* default */
#define	PS_STORES_I       (0x0001)	/* implementation (normal) */
#define	PS_STORES_F       (0x0002)	/* factory-set */
#define	PS_STORES_R       (0x0004)	/* rom (read-only) */
#define	PS_STORES_T       (0x0008)	/* transient (RAM) */


/*************************************************************************************
*	
*	Function Name :	void set_stores(const uint16 store);
*
*	Parameters :	store - bit mask to identify the PS store layer to write to. See 
*						comments above under PS_STORES.
*
*	Returns :		none
*
*	Description :	Sets the PS store to access.
*
*************************************************************************************/
LIBAPI(void) set_stores(const unsigned short store);

/*************************************************************************************
*	
*	Function Name :	int write_ps(const unsigned short key, const unsigned short *const data, const uint16 len );
*
*	Parameters :	key - The PS key entry id.
*
*					data - Pointer to an array holding the PS data to be written.
*					
*					len - The size of the array of PS data.
*
*	Returns :		0 on fail, 1 on success
*
*	Description :	This function is used to write to the persistent store.
*
*************************************************************************************/
LIBAPI(int) write_ps(const unsigned short key, const unsigned short *const data, const unsigned short len );

/*************************************************************************************
*	
*	Function Name :	int write_ps_verify(const unsigned short key, const unsigned short *const data, const uint16 len );
*
*	Parameters :	key - The PS key entry id.
*
*					data - Pointer to an array holding the PS data to be written.
*					
*					len - The size of the array of PS data.
*
*	Returns :		0 on fail, 1 on success, i.e. the key was veriffied as written.
*
*	Description :	This function is used to write and subsequenlty read and compare the persistent store.
*                   Prior to calling this, you could call set_stores with a bit pattern incorporating either 
*                   PS_STORES_I or PS_STORES_F, but the default for the stores 
*                   being (PS_STORES_I | PS_STORES_F | PS_STORES_R) means 
*					you need not worry about calling set_stores.
*
*************************************************************************************/
LIBAPI(int) write_ps_verify(const unsigned short key, const unsigned short *const data, const unsigned short len );

/*************************************************************************************
*	
*	Function Name :	int read_ps(const unsigned short key, unsigned short *const data, const unsigned short maxlen, unsigned short * const len );
*
*	Parameters :	key - The PS key entry id.
*
*					data - Pointer to an array to recieve the PS data to be read.
*					
*					maxlen - The size of the data array
*
*					len - The acrual size of the read key
*
*	Returns :		0 on fail, 1 on success
*
*	Description :	This function is used to write to the persistent store.
*
*************************************************************************************/
LIBAPI(int) read_ps(const unsigned short key, unsigned short *const data, const unsigned short maxlen, unsigned short * const len );

/*************************************************************************************
*	
*	Function Name :	int clear_ps(const pskey key);
*
*	Parameters :	key - The PS key entry id.
*
*	Returns :		0 on fail, 1 on success
*
*	Description :	This function is used to clear a persistent store entry.
*
*************************************************************************************/
LIBAPI(int) clear_ps(const unsigned short key);

/*************************************************************************************
*	
*	Function Name :	int factory_set(void);
*
*	Parameters :	none
*
*	Returns :		0 on fail, 1 on success
*
*	Description :	This function is used to move PS values to the factory layer.
*
*************************************************************************************/
LIBAPI(int) factory_set(void);

LIBAPI(int) e2_device(const unsigned short log2bytes, const unsigned short addrmask);

LIBAPI(int) get_firmware_id(int *const id, unsigned short *nameBuffer, const unsigned int length);

#endif /* __INCTestPSh */
