
/******************************************************************************
Project:		Flash.dll
File Name:		Flash.h
Description:	Implementation of the flash loader class.

Program your flash over spi with this class.  Follow these steps.
 * use dll's use_port(1) to select comm port 1.  
 * Construct a flash.  (Construct with a hwnd if you want to receive debug, title, progress and completion messages.)
 * Construct a FlashImage,  call FlashImage::ReadCodeFila and ReadDataFile to get your .xpv and .xdv files read in.
 * get the flash to use the image by calling flash::UseImage(FlashImage *)
 * Call flash_burn() to burn the image
 * Call flash_verify() to verify the contents of the flash against the image.
 * Call flash_dump(const char *filename) to read back contents of the flash.
 * Call flash_erase() if you want to lose your persistent store.
 * Call identify() to recognise the flash.  Results availabe after the MSG_FLASH_COMPLETE message in flash_id.
*/


// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the FLASHDLL_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// FLASHDLL_API functions as being imported from a DLL, wheras this DLL sees symbols
// defined with this macro as being exported.

// flash.h: interface for the flash class.

// $Id: //depot/bc/main/devHost/spi/flashdll/flash.h#69 $
//
//////////////////////////////////////////////////////////////////////

#if !defined(FLASHDLL_H_INCLUDED)
#define FLASHDLL_H_INCLUDED


#ifdef FLASHDLL_EXPORTS
#define FLASHDLL_API __declspec(dllexport)
#else
#define FLASHDLL_API __declspec(dllimport)
#endif


#include <bitset>
#include "thread/system_wide_mutex.h"


#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// MSG_FLASH_PROGRESS: wParam is the progress, lParam, is the total
#define MSG_FLASH_PROGRESS (WM_USER + 1)

// MSG_FLASH_COMPLETE: wparam is the status, one of FLASH_STATUS_DONE or FLASH_STATUS_TIMED_OUT
#define MSG_FLASH_COMPLETE (MSG_FLASH_PROGRESS + 1)

// MSG_FLASH_DEBUG: lParam is a LPCSTR containing the new caption
#define MSG_FLASH_DEBUG (MSG_FLASH_PROGRESS + 2)

// MSG_FLASH_TITLE: lParam is a LPCSTR containing the new caption
#define MSG_FLASH_TITLE (MSG_FLASH_PROGRESS + 3)


const int BLOCKSIZE =4 << 10;
const int MAX_SECTORS = 0x80;

#include "spi/FlashID.h"

// #include "spi/spi.h"
#include "FlashImg.h"

class FlashIDSolver;

const int flash_bc01_duff_area = 0x100;
const int NUM_FLASH_DEVICES = 16;
const int BROADCAST_DEVICE = 16;

typedef std::bitset<NUM_FLASH_DEVICES> FlashBitset;

#include "abstract/memory_reader.h"

class FLASHDLL_API flash: public IMemoryReader
{


public:

enum chip_type_t
{
	CHIP_TYPE_UNKNOWN = 0,
	CHIP_TYPE_BC01,
	CHIP_TYPE_BC02,
	CHIP_TYPE_PADDYWACK
};


enum flash_error_t
{
	FLASH_ERROR_NONE = 0,
	FLASH_ERROR_DOWNLOAD_FAILED,
	FLASH_ERROR_VERIFY_FAILED,
	FLASH_ERROR_TIMED_OUT,
	FLASH_ERROR_CRC_FAILED,
	FLASH_ERROR_READBACK_FAILED,
	FLASH_ERROR_COULD_NOT_DOWNLOAD_PROG,
	FLASH_ERROR_COULD_NOT_STOP_XAP,
	FLASH_ERROR_BOOT_PROG_HALTED,
	FLASH_ERROR_ERASE_FAILED,
	FLASH_ERROR_XAP_ERROR,
	FLASH_ERROR_UNKNOWN_CHIP_TYPE,
	FLASH_ERROR_BROADCAST_MIXED_CHIP_TYPES
};

enum flash_error_level_t {
	flash_error_level_info = 0,
	flash_error_level_warning = 1,
	flash_error_level_major = 2,
	flash_error_level_failure = 3
};


enum flash_status_t
{
	FLASH_STATUS_DONE = 0,
	FLASH_STATUS_BUSY,
	FLASH_STATUS_TIMED_OUT
};
	
// This should match up to the  OperationStrings in flash.cpp
enum flash_op_t
{
	FLASH_OP_DONE = 0,
	FLASH_OP_DUMPING,
	FLASH_OP_DOWNLOADING_BOOT,
	FLASH_OP_WRITING, 
	FLASH_OP_VERIFYING,
	FLASH_OP_ERASING,
	FLASH_OP_ACQUIRING_SPI 
};
	

// Copied from psfl.c
enum flash_id_t {
    ID_UNKNOWN = 0, 
    ID_AMD, 
    ID_PAIR, 
    ID_INTEL, 
    ID_BC02 = 0x20,
    ID_ERROR = -1};


enum flash_dump_t
{
	FLASH_DUMP_XPV = 0,
	FLASH_DUMP_RAW
};

// Construction
	flash(HWND CallbackWind = NULL);
	virtual ~flash();
	
	int flash_identify();
	int flash_burn();
	int flash_broadcast_burn(unsigned long devsToUse, bool eraseFirst, bool restartAfter);
	int flash_verify();
	int flash_erase();
	int flash_dump(const char *filename, const flash_dump_t format = FLASH_DUMP_XPV);
	
	bool AcquireMutex(DWORD timeout = INFINITE);  // Blocking.

	//	int read_code_file(const char *filename);
	//int read_data_file(const char *filename);

	flash_status_t GetStatus();
	int GetProgress();
	int GetProgressMax();
	flash_op_t GetCurrentOperation();
	const char *GetOperationString(flash_op_t op);
	const char *GetErrorString();
	flash_error_level_t GetErrorLevel() {return lastErrorLevel;};
	flash_error_t GetErrorCode();
	void UseImage(const FlashImage *image);
	bool UsingImage();
	bool IsBusy();
	void Terminate();
	int flash_size;
	long GetSupportedFlashDeviceSize();
	unsigned long GetFailedDevices();

	unsigned int broadcast_spi_delays_;


	bool GetFirmwareId(int *const id, char *const nameBuffer, const size_t sNameBuffer);
	virtual bool read_mem(uint16 addr, uint16 len, uint16 * data);

	int GetXapCrystalSpeedMhz(void);
	bool SetXapCrystalSpeedMhz(int speed);  // Will return true if value is appropriate.

	int xap_reset_and_stop(void);
	int xap_reset_and_go(void);
	int xap_go(void);
	int xap_stop (void);

	BOOL verbose;
	HWND CallbackWind;
	void (*CallbackFn)(void *Param);
	void *CallbackFnParam;
	flash_id_t flash_type;
	long manuf_id, device_id;


protected:

// Makes call to flash_write_noverify_spi()
// These values are used as commands to the boot_prog uploaded to the xap.
// Refer to psfl.c for the cannonical definition.
enum boot_prog_operation_t {
	BOOT_PROG_WAIT = 0, 
	BOOT_PROG_ERASE = 1, 
	BOOT_PROG_WRITE = 2, 
	BOOT_PROG_VERIFY = 3,
    BOOT_PROG_WRITABLE = 4, 
	BOOT_PROG_READ = 5, 
	BOOT_PROG_RAMCRC = 6, 
	BOOT_PROG_IDENTIFY = 7,
    BOOT_PROG_FLASHCRC = 8, 
	BOOT_PROG_CHIPERASE = 9, 
	BOOT_PROG_ALLCRC = 10, 
	BOOT_PROG_LAST_CMD
};

	SystemWideMutex *mutex_;

	int acquire_mutex_internal(void *extra);
	void flash_select_broadcast(void);
	void flash_select_chip(const unsigned int chipNum);
	void flash_select_none();
	FlashBitset get_active_chips(const FlashBitset &devsInUse, FlashBitset &failedDevs );
	int wait_for_chip(const unsigned int timeout);
	int wait_for_chips(DWORD timeout, const FlashBitset &devsInUse, FlashBitset &failedDevs );

	int dump(void * extra);
	bool dump_data_from_flash_to_file(const char *filename, int start, int end, int offset, flash_dump_t dumpFormat);

	int identify(void *extra);
	int burn(void *extra);
	int broadcast_burn(void *extra);
	bool do_crc();
	int verify(void *extra);
	int erase(void *extra);

	void emucon_debug(flash_error_level_t level, const char *fmt, ...);
	void title(const char *fmt, ...);
	void CallbackProgress(UINT pos);
	void CallbackComplete(flash_status_t status);

	enum {errorBufSize = 8192};
	char lastErrorStr[errorBufSize];
	flash_error_level_t lastErrorLevel;
	
	long crcbuf(unsigned long address);
	long crcdata(unsigned short *data, size_t len);
	long crcdata(unsigned long *data, size_t len);

	int xtal_freq_index;
	void start_thread();
	int go_wait_for_chip(const unsigned int timeout);
	int run_state_machine();
	static unsigned int __stdcall thread_proc(LPVOID f);
	void flash::run_in_thread(int (flash::* func)(void*), void *extra);
	static unsigned int __stdcall flash::threaded_run_proc(LPVOID x);

	unsigned short progress_sector, cmdset, intfc;
	BOOL *block_failed;
	BOOL *block_used;
	BOOL *block_changed;
	BOOL *block_blank;
	flash_op_t current_op;
	chip_type_t chip_type;
	flash_status_t status;
	flash_error_t lastErrorCode;

	bool Terminated;

	const FlashImage *image;

	unsigned long  failedDevsUL_;
	
	HANDLE hThread ;
};

FLASHDLL_API int use_port(int lpt_port_num);
FLASHDLL_API void use_multiplexer(int multiple_num);
FLASHDLL_API int xap_stopped();


#endif // FLASHDLL_H_INCLUDED


