/**********************************************************************
*
*  FILE   :  hi_res_clock.h
*
*            (C) CSR 2002
*
*  AUTHOR :  Mark Marshall
*
*  PURPOSE:  The interface to the HiResClock.  This should return the
*            time in a number of different resolutions.  It must NOT
*	     be used for busy waiting!
*
*  $Id: //depot/bc/main/devHost/util/time/hi_res_clock.h#7 $
*
***********************************************************************/

#ifndef HI_RES_CLOCK_H
#define HI_RES_CLOCK_H

#ifndef _WINCE
#include <time.h>
#endif
#include <limits.h>

#ifdef __cplusplus
extern "C" {
#endif


/* This is used to store the time.  It is the same as a 'struct
   timespec', but that type is not fully portable. */

typedef struct HiResClock_tag
{
    time_t tv_sec;
    long int tv_nsec;
} HiResClock;


/* This is the type returned by the 'HiResClockGetMilliSec' function. */

typedef unsigned long int HiResClockMilliSec;


/* This is the maximum amount of time that we can use without worrying 
   about wrap-around (half the range of the type above) */

#define HRC_MAX_MILLISEC (LONG_MAX)


/****************************************************************************
NAME
	HiResClockGetSec  -  Get the time in seconds

DESCRIPTION
	This will return the time in seconds.  This is the number of
	seconds since the Epoch.
*/

time_t HiResClockGetSec(void);


/****************************************************************************
NAME
	HiResClockGetMilliSec  -  Get the time in milli-seconds

DESCRIPTION
	This will return the time in milli seconds.  This is the
	number of seconds since the Epoch.  We attempt to use a method
	that will return the maximum resolution.
*/

HiResClockMilliSec HiResClockGetMilliSec(void);


/****************************************************************************
NAME
	HiResClockGetNanoSec  -  Get the time in nano-seconds

DESCRIPTION
	This will return the time in nano seconds.  This is the number
	of seconds since the Epoch.  We attempt to use a method that
	will return the maximum resolution.  The structure
	'HiResClock' is very simular to a 'struct timespec', but they
	are not as fully portable as we would like.
*/

void HiResClockGetNanoSec(HiResClock *ret);


/****************************************************************************
NAME
	HiResClockHMS  -  Get the time in seconds as a string

DESCRIPTION
	This will return the time in seconds.  The buffer is filled
	with a string containg hours, minuets, seconds and possibly
	milliseconds.  It is usefull for timestamping things.

	The buffer must be at least 13 bytes long.
*/

void HiResClockHMS(char *buf);


/****************************************************************************
NAME
	InitHiResClock  -  Initialise the Hi-Res clock

DESCRIPTION
	This must be called before any of the other functions in this
	library.  It can be called multiple times.  It will optionally
	return a rough guess at the accuracy of the clock.  This guess
	is almost certainly an overestimate.

	This function is also possibly not thread safe (it is on all
	Posix versions and WinCE, but not windows).  It is possible to
	call any of the other functions from multiple threads at the
	same time, and they should all work, but if this function is
	called from one thread while another HiResClock function is
	running it might go horribly wrong (basically dont call this
	function more than you need to).
*/

void InitHiResClock(HiResClock *res);


/****************************************************************************
NAME
	HiResClockSleepMilliSec  -  Block the current thread for n milliseconds

DESCRIPTION
	What it says on the tin.
*/

void HiResClockSleepMilliSec(HiResClockMilliSec n);


#ifdef __cplusplus
}
#endif

#endif /* HI_RES_CLOCK_H */
